
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const endpoint_cxx_Version =
    "$Id: endpoint.cxx,v 1.18 2001/01/17 01:52:24 liuhong Exp $";
#include "VTime.hxx"
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>

#include "endpoint.h"
#include "MgcpResponseCode.hxx"


extern Endpoint eps [];
extern CallData call [];
void processEvent (Endpoint*, int);


void
stateChange (Endpoint* ep, int newState)
{
    ep->state = newState;
}    /* stateChange */


int
initResponse (Endpoint* ep)
{
    stateChange (ep, Idle);
    return 1;
}    /* initResponse */


int
initRestart (Endpoint* ep)
{
    sprintf (ep->requestIdentifier, "%X", newRequestIdentifier());
    MgcNotificationRequest rqnt ("testID", ep->requestIdentifier);
    ep->gw->send (rqnt);

    if (rqnt.getResponseCode() == TransactionExecuted)
    {
        initResponse (ep);
    }
    return 1;
}    /* initRestart */


int
ringing (Endpoint* ep)
{
    stateChange (ep, Ringing);
    return 1;
}    /* ringing */


int
receiveCall (Endpoint* ep)
{
    stateChange (ep, IncomingCall);

    sprintf (ep->requestIdentifier, "%X", newRequestIdentifier());
    MgcNotificationRequest rqnt ("testID", ep->requestIdentifier);
    ep->gw->send (rqnt);
    if (rqnt.getResponseCode() == TransactionExecuted)
    {
        ringing (ep);
    }
    return 1;
}    /* receiveCall */


int
connectionReady (Endpoint* ep)
{
    stateChange (ep, RingBackState);

    CallData* call1 = ep->call;
    /* Warning: No locking on shared endpoint and call data */
    Endpoint* destEp = &eps[call1->endpoint[1].id];
    CallData* call2 = newCallData (destEp);
    strcpy (call2->callIdentifier, call1->callIdentifier);
    call2->endpoint[1].id = ep->id;
    strcpy (call2->endpoint[1].connectionId, call1->endpoint[0].connectionId);
    call2->endpoint[1].localConnection = call1->endpoint[0].localConnection;
    destEp->call = call2;

    notifyEndpoint (destEp, ReceiveCall);

    return 1;
}    /* connectionReady */


int
dialingComplete (Endpoint* ep)
{
    CallData* call = newCallData (ep);
    sprintf (call->callIdentifier, "%X", newCallIdentifier());
    int destEpId;
    if (ep->id == 0)
    {
        destEpId = 1;
    }
    else
    {
        destEpId = 0;
    }
    call->endpoint[1].id = destEpId;
    ep->call = call;

    stateChange (ep, NewCall);

    MgcCreateConnection crcx ("testID", call->callIdentifier, "sendrecv");

    ep->gw->send (crcx);

    if (crcx.getResponseCode() == TransactionExecuted)
    {
        strcpy (call->endpoint[0].connectionId,
                (crcx.getConnectionId ()).data());
        call->endpoint[0].localConnection = crcx.getLocalConnectionDescriptor ();
        connectionReady (ep);
    }
    else
    {
        cout << "failed TransactionExecuted\n";
    }
    return 1;
}    /* dialingComplete */


int
disconnectResponse (Endpoint* ep)
{
    stateChange (ep, Init);
    initRestart (ep);
    return 1;
}    /* disconnectResponse */


int
disconnect (Endpoint* ep)
{
    stateChange (ep, Disconnect);
    CallData* call = ep->call;
    assert (call);
    MgcDeleteConnectionAgent dlcx ("testID");
    dlcx.setCallId (call->callIdentifier);
    dlcx.setConnectionId (call->endpoint[0].connectionId);
    ep->gw->send (dlcx);

    if (dlcx.getResponseCode() == ConnectionDeleted ||
            dlcx.getResponseCode() == TransactionExecuted)
    {
        notifyEndpoint (&eps[call->endpoint[1].id], ClearCall);
        callDataInit (ep->call);
        ep->call = NULL;
        disconnectResponse (ep);
    }
    return 1;
}    /* disconnect */


int
clearCall (Endpoint* ep)
{
    stateChange (ep, Disconnect);

    CallData* call = ep->call;
    assert (call);
    MgcDeleteConnectionAgent dlcx ("testID");
    dlcx.setCallId (call->callIdentifier);
    dlcx.setConnectionId (call->endpoint[0].connectionId);
    ep->gw->send (dlcx);

    if (dlcx.getResponseCode() == ConnectionDeleted ||
            dlcx.getResponseCode() == TransactionExecuted)
    {
        callDataInit (ep->call);
        ep->call = NULL;
    }
    return 1;
}    /* clearCall */


int
connected (Endpoint* ep)
{
    CallData* call = ep->call;
    assert (call);
    MgcModifyConnection mdcx ("testID",
                              call->callIdentifier,
                              call->endpoint[0].connectionId,
                              "sendrecv");
    mdcx.setRemoteConnectionDescriptor (call->endpoint[1].localConnection);
    ep->gw->send (mdcx);

    if (mdcx.getResponseCode() == TransactionExecuted)
    {
        sprintf (ep->requestIdentifier, "%X", newRequestIdentifier());
        MgcNotificationRequest rqnt ("testID", ep->requestIdentifier);
        ep->gw->send (rqnt);

        if (rqnt.getResponseCode() == TransactionExecuted)
        {
            stateChange (ep, Connected);
        }
    }
    return 1;
}    /* connected */


int
answer (Endpoint* ep)
{
    CallData* call = ep->call;
    assert (call);
    MgcCreateConnection crcx ("testID", call->callIdentifier, "sendrecv");
    crcx.setRemoteConnectionDescriptor (call->endpoint[1].localConnection);
    ep->gw->send (crcx);

    if (crcx.getResponseCode() == TransactionExecuted)
    {
        strcpy (call->endpoint[0].connectionId,
                (crcx.getConnectionId ()).data());
        eps[call->endpoint[1].id].call->endpoint[1].localConnection = call->endpoint[0].localConnection
                = crcx.getLocalConnectionDescriptor ();
        notifyEndpoint (&eps[call->endpoint[1].id], Connect);

        sprintf (ep->requestIdentifier, "%X", newRequestIdentifier());
        MgcNotificationRequest rqnt ("testID", ep->requestIdentifier);
        ep->gw->send (rqnt);

        if (rqnt.getResponseCode() == TransactionExecuted)
        {
            stateChange (ep, Connected);
        }
    }
    return 1;
}    /* answer */


void
addDTMFSignal (MgcNotificationRequest* rqnt, char digit)
{
    switch (digit)
    {
#ifndef NCS_PROFILE
        case '0':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_0 ());
        }
        break;
        case '1':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_1 ());
        }
        break;
        case '2':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_2 ());
        }
        break;
        case '3':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_3 ());
        }
        break;
        case '4':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_4 ());
        }
        break;
        case '5':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_5 ());
        }
        break;
        case '6':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_6 ());
        }
        break;
        case '7':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_7 ());
        }
        break;
        case '8':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_8 ());
        }
        break;
        case '9':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_9 ());
        }
        break;
        case '#':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_Hash ());
        }
        break;
        case '*':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_Star ());
        }
        break;
        case 'A':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_A ());
        }
        break;
        case 'B':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_B ());
        }
        break;
        case 'C':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_C ());
        }
        break;
        case 'D':
        {
            rqnt->addSignalRequests (MgcDTMFPackage::signalDTMF_D ());
        }
        break;
#else   // NCS_PROFILE
case '0':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_0 ());
        }
        break;
        case '1':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_1 ());
        }
        break;
        case '2':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_2 ());
        }
        break;
        case '3':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_3 ());
        }
        break;
        case '4':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_4 ());
        }
        break;
        case '5':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_5 ());
        }
        break;
        case '6':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_6 ());
        }
        break;
        case '7':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_7 ());
        }
        break;
        case '8':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_8 ());
        }
        break;
        case '9':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_9 ());
        }
        break;
        case '#':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_Hash ());
        }
        break;
        case '*':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_Star ());
        }
        break;
        case 'A':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_A ());
        }
        break;
        case 'B':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_B ());
        }
        break;
        case 'C':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_C ());
        }
        break;
        case 'D':
        {
            rqnt->addSignalRequests (MgcNCSLinePackage::signalDTMF_D ());
        }
        break;
#endif
        default:
        {
            cerr << "Invalid digit " << digit << endl;
        }
    }    /* switch */
}    /* addDTMFSignal */


int
dtmfDigit (Endpoint* ep)
{
    CallData* call1 = ep->call;
    if (call1->playDtmfDigit != 0)
    {
        /* Warning: No locking on shared endpoint data */
        /* Receiving DTMF digit */
        /* Forward DTMF digit to the gateway */
        sprintf (ep->requestIdentifier, "%X", newRequestIdentifier());
        MgcNotificationRequest rqnt ("testID", ep->requestIdentifier);
        addDTMFSignal (&rqnt, call1->playDtmfDigit);
        ep->gw->send (rqnt);

        if (rqnt.getResponseCode() == TransactionExecuted)
        {
            call1->playDtmfDigit = 0;
        }
    }
    if (call1->dtmfDigits[0] != 0)
    {
        /* Warning: No locking on shared endpoint data */
        /* Sending DTMF digit */
        Endpoint* destEp = &eps[call1->endpoint[1].id];
        CallData* call2 = destEp->call;
        call2->playDtmfDigit = call1->dtmfDigits[0];
        call1->dtmfDigits[0] = 0;

        /* Forward DTMF digit to the other party */
        notifyEndpoint (destEp, DigitsCollected);
    }
    return 1;
}    /* dtmfDigit */


void
processEvent (Endpoint* ep, int event)
{
    switch (event)
    {
        case Restart:
        {
            initRestart (ep);
        }
        break;
        case OffHook:
        {
            if (ep->state == Idle)
            {
                dialingComplete (ep);
            }
            else
            {
                answer (ep);
            }
        }
        break;
        case DigitsCollected:
        {
            dtmfDigit (ep);
        }
        break;
        case ReceiveCall:
        {
            receiveCall (ep);
        }
        break;
        case Connect:
        {
            connected (ep);
        }
        break;
        case OnHook:
        {
            if (ep->state == Connected)
            {
                disconnect (ep);
            }
            else
            {
                disconnectResponse (ep);
            }
        }
        break;
        case ClearCall:
        {
            clearCall (ep);
        }
        break;
        default:
        cerr << "Unexpected Event" << endl;
    }    /* switch */
}   /* processEvent */


int
newSocket (struct sockaddr_in* socketAddress, int port)
{
    int sfd = -1;

    if ((sfd = socket (PF_INET, SOCK_DGRAM, 0)) == -1)
    {
        cerr << "newSocket: socket() failed - " << strerror (errno) << endl;
    }
    else
    {
        memset (socketAddress, 0, sizeof (sockaddr_in));

        socketAddress->sin_family = AF_INET;
        socketAddress->sin_addr.s_addr = INADDR_ANY;
        socketAddress->sin_port = htons (port);

        if (bind (sfd, (struct sockaddr *)socketAddress, sizeof (sockaddr_in)) == -1)
        {
            cerr << "newSocket: bind() failed - " << strerror (errno) << endl;
            shutdown (sfd, 2);
            sfd = -1;
        }
    }
    return sfd;
}    /* newSocket */


void
saveDigits (Endpoint* ep,
#ifndef NCS_PROFILE
            Sptr < MgcDTMFPackageEvent > event
#else  // NCS_PROFILE
            Sptr < MgcNCSLinePackageEvent > event
#endif
           )
{
    if (ep->call == NULL)
    {
        ep->call = newCallData (ep);
    }
    assert (ep->call);

    switch (event->getType())
    {
#ifndef NCS_PROFILE
        case MgcDTMFPackageDTMF_0:
        {
            sprintf (ep->call->dtmfDigits, "%c", '0');
        }
        break;
        case MgcDTMFPackageDTMF_1:
        {
            sprintf (ep->call->dtmfDigits, "%c", '1');
        }
        break;
        case MgcDTMFPackageDTMF_2:
        {
            sprintf (ep->call->dtmfDigits, "%c", '2');
        }
        break;
        case MgcDTMFPackageDTMF_3:
        {
            sprintf (ep->call->dtmfDigits, "%c", '3');
        }
        break;
        case MgcDTMFPackageDTMF_4:
        {
            sprintf (ep->call->dtmfDigits, "%c", '4');
        }
        break;
        case MgcDTMFPackageDTMF_5:
        {
            sprintf (ep->call->dtmfDigits, "%c", '5');
        }
        break;
        case MgcDTMFPackageDTMF_6:
        {
            sprintf (ep->call->dtmfDigits, "%c", '6');
        }
        break;
        case MgcDTMFPackageDTMF_7:
        {
            sprintf (ep->call->dtmfDigits, "%c", '7');
        }
        break;
        case MgcDTMFPackageDTMF_8:
        {
            sprintf (ep->call->dtmfDigits, "%c", '8');
        }
        break;
        case MgcDTMFPackageDTMF_9:
        {
            sprintf (ep->call->dtmfDigits, "%c", '9');
        }
        break;
        case MgcDTMFPackageDTMF_Hash:
        {
            sprintf (ep->call->dtmfDigits, "%c", '#');
        }
        break;
        case MgcDTMFPackageDTMF_Star:
        {
            sprintf (ep->call->dtmfDigits, "%c", '*');
        }
        break;
        case MgcDTMFPackageDTMF_A:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'A');
        }
        break;
        case MgcDTMFPackageDTMF_B:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'B');
        }
        break;
        case MgcDTMFPackageDTMF_C:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'C');
        }
        break;
        case MgcDTMFPackageDTMF_D:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'D');
        }
        break;
#else  // NCS_PROFILE
        case MgcNCSLinePackageDTMF_0:
        {
            sprintf (ep->call->dtmfDigits, "%c", '0');
        }
        break;
        case MgcNCSLinePackageDTMF_1:
        {
            sprintf (ep->call->dtmfDigits, "%c", '1');
        }
        break;
        case MgcNCSLinePackageDTMF_2:
        {
            sprintf (ep->call->dtmfDigits, "%c", '2');
        }
        break;
        case MgcNCSLinePackageDTMF_3:
        {
            sprintf (ep->call->dtmfDigits, "%c", '3');
        }
        break;
        case MgcNCSLinePackageDTMF_4:
        {
            sprintf (ep->call->dtmfDigits, "%c", '4');
        }
        break;
        case MgcNCSLinePackageDTMF_5:
        {
            sprintf (ep->call->dtmfDigits, "%c", '5');
        }
        break;
        case MgcNCSLinePackageDTMF_6:
        {
            sprintf (ep->call->dtmfDigits, "%c", '6');
        }
        break;
        case MgcNCSLinePackageDTMF_7:
        {
            sprintf (ep->call->dtmfDigits, "%c", '7');
        }
        break;
        case MgcNCSLinePackageDTMF_8:
        {
            sprintf (ep->call->dtmfDigits, "%c", '8');
        }
        break;
        case MgcNCSLinePackageDTMF_9:
        {
            sprintf (ep->call->dtmfDigits, "%c", '9');
        }
        break;
        case MgcNCSLinePackageDTMF_Hash:
        {
            sprintf (ep->call->dtmfDigits, "%c", '#');
        }
        break;
        case MgcNCSLinePackageDTMF_Star:
        {
            sprintf (ep->call->dtmfDigits, "%c", '*');
        }
        break;
        case MgcNCSLinePackageDTMF_A:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'A');
        }
        break;
        case MgcNCSLinePackageDTMF_B:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'B');
        }
        break;
        case MgcNCSLinePackageDTMF_C:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'C');
        }
        break;
        case MgcNCSLinePackageDTMF_D:
        {
            sprintf (ep->call->dtmfDigits, "%c", 'D');
        }
        break;
#endif
        default:
        cerr << "Invalid DTMF event " << (int)event->getType() << endl;
    }    /* switch */
}    /* saveDigits */


int
mapLinePackageEvent (
#ifndef NCS_PROFILE
    Sptr < MgcLinePackageEvent > event
#else  // NCS_PROFILE
Sptr < MgcNCSLinePackageEvent > event
#endif
)
{
    int epEvent = Unknown;

    switch (event->getType())
    {
#ifndef NCS_PROFILE
        case MgcLinePackageOffHookTransition:
        {
            epEvent = OffHook;
        }
        break;
        case MgcLinePackageOnHookTransition:
        {
            epEvent = OnHook;
        }
        break;
#else   // NCS_PROFILE
        case MgcNCSLinePackageOffHookTransition:
        {
            epEvent = OffHook;
        }
        break;
        case MgcNCSLinePackageOnHookTransition:
        {
            epEvent = OnHook;
        }
        break;
#endif
        default:
        //   cerr << "Unexpected Line Package event " << (int) event->getType() << endl;
        break;
    }    /* switch */
    return epEvent;
}    /* mapLinePackageEvent */


void
decodeNotify (Endpoint* ep, Sptr < MgcNotify > ntfy)
{
    int event = Unknown;

    Sptr < MgcEvent > observedEvent = ntfy->getItemObservedEvents (0);

    assert (observedEvent != 0);

    switch (observedEvent->getPackageType ())
    {
#ifndef NCS_PROFILE
        case MgcpDTMFPackageType:
        {
            Sptr < MgcDTMFPackageEvent > dtmf;
            dtmf.dynamicCast(observedEvent);
            saveDigits (ep, dtmf);
            event = DigitsCollected;
        }
        break;
        case MgcpLinePackageType:
        {
            Sptr < MgcLinePackageEvent > line;
            line.dynamicCast(observedEvent);
            event = mapLinePackageEvent(line);
        }
        break;
#else   // NCS_PROFILE
        case MgcpNCSLinePackageType:
        {
            Sptr < MgcNCSLinePackageEvent > line;
            line.dynamicCast(observedEvent);

            event = mapLinePackageEvent(line);

            if ( event == Unknown )
            {
                Sptr < MgcNCSLinePackageEvent > dtmf;
                dtmf.dynamicCast(observedEvent);
                saveDigits (ep, dtmf);
                event = DigitsCollected;
            }

        }
        break;
#endif
        default:
        cerr << "Unexpected package " << (int)observedEvent->getPackageType () << endl;
    }    /* switch */

    ntfy->setResponseCode (TransactionExecuted);
    ntfy->setResponseString (OKResponse);
    ep->gw->reply (ntfy);

    processEvent (ep, event);
}    /* decodeNotify */


void
stackEvent (Endpoint* ep)
{
    //    cout << "started decoding\n";
    Sptr < MgcCommand > cmd = ep->gw->receive ();

    //    cout << "finished decoding\n";
    switch (cmd->getType())
    {
        case MgcNotifyType:
        {
            Sptr < MgcNotify > ntfy;
            decodeNotify (ep, ntfy.dynamicCast(cmd));
            break;
        }
        default:
        {
            cerr << "Unexpected command " << static_cast < int > (cmd->getType()) << endl;
        }
    }
}    /* stackEvent */


void*
epStart (void* ptr)
{
    Endpoint* ep = (Endpoint*)ptr;
    int msgLen = 0;
    int event = -1;
    char buf[32];

    ep->sfd = newSocket (&(ep->socketAddress), UDP_PORT_MAX - ep->id);

    if (ep->sfd < 0)
    {
        cerr << "unable to open socket!\n";
    }

    fd_set fds;
    int retval;

    while (1)
    {
        //	cerr << "in loop\n";
        FD_ZERO (&fds);

        ep->gw->addToFdSet (&fds);
        FD_SET(0, &fds);
        FD_SET(ep->sfd, &fds);

        struct timeval tv;
        tv.tv_sec = 0;
        tv.tv_usec = 300;

        if ((retval = select(129, &fds, NULL, NULL, &tv)) > 0)
        {
            if (FD_ISSET (0, &fds))
            {
                char c;
                c = getchar();
                if (c == 'q')
                {
                    exit(0);
                }
            }
            if (FD_ISSET (ep->sfd, &fds))
            {
                //		cerr << "received message\n";
                if ((msgLen = recvfrom (ep->sfd, buf, 256, 0, NULL, NULL)) > 0)
                {
                    event = strtol (buf, NULL, 0);
                    processEvent (ep, event);
                }
                else
                {
                    break;
                }
            }    /* FD_ISSET */

            if (ep->gw->checkMsgReady (&fds) == true)
            {
                stackEvent (ep);
            }
        }    /* select */
        else if (retval == 0)
        {
            if (ep->gw->checkMsgReady (&fds) == true)
            {
                //		cout << "processing a message\n";
                stackEvent (ep);
            }
            else
            {
                //		cout << "timeout\n";
            }


        }
    }
    return NULL;
}    /* epStart */
