
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const endpoint_h_Version =
    "$Id: endpoint.h,v 1.11 2001/01/17 01:52:24 liuhong Exp $";
#include <sys/types.h>
#include <sys/socket.h>

#include "MgcGateway.hxx"

#ifndef ENDPOINT_H
#define ENDPOINT_H


///
#define UDP_PORT_MAX 9999

///
#define CONNECTION_ID_LEN_MAX 33

/**@name Endpoint Call Data
  * An endpoint of a call
  */
typedef struct endpointCallData
{
    //@{
    /// Endpoint Id
    int id;
    /// Connection Identifier
    char connectionId [CONNECTION_ID_LEN_MAX];
    /// Local Connection Descriptor
    SessionDescriptor localConnection;
    //@}
}

EndpointCallData;

///
#define CALL_ID_LEN_MAX 33
///
#define ENDPOINTS_PER_CALL 2
///
#define DIGITS_LEN_MAX 33

/**@name Call Data
  * Half Call Data
  */
typedef struct callData
{
    //@{
    /// Call Identifier
    char callIdentifier [CALL_ID_LEN_MAX];
    /// Endpoints involved in the call
    EndpointCallData endpoint [ENDPOINTS_PER_CALL];
    /// DTMF digits collected
    char dtmfDigits [DIGITS_LEN_MAX];
    /// DTMF digit to play
    char playDtmfDigit;
    //@}
}

CallData;

///
#define REQUEST_ID_LEN_MAX 33

/**@name Endpoint Data
  * Endpoint Data
  */
typedef struct endpoint
{
    //@{
    /// Endpoint Number (NOT Endpoint Identifier)
    int id;
    /// Thread Id
    VThread tid;
    /// Socket Descriptor that this endpoint is listening to
    int sfd;
    /// Socket Address of sfd
    struct sockaddr_in socketAddress;
    /// MGCP Gateway
    MgcGateway* gw;
    /// Current Request Identifier
    char requestIdentifier [REQUEST_ID_LEN_MAX];
    /// Current Call Data
    CallData* call;
    /// Current Call State
    int state;
    //@}
}

Endpoint;

/**@name Call States
  * Endpoint Call States
  */
typedef enum CallState
{
    //@{
    /// Init
    Init = 0,
    /// Idle
    Idle,
    /// New Outgoing Call
    NewCall,
    /// Ring Back
    RingBackState,
    /// New Incoming Call
    IncomingCall,
    /// Ringing
    Ringing,
    /// Connected
    Connected,
    /// Half Disconnect
    Disconnect,
    CallStateMax
    //@}
};

/**@name Call Events
  * Endpoint Call Events
  */
typedef enum CallEvent
{
    //@{
    /// Restart
    Restart = 0,
    /// Off Hook
    OffHook,
    /// DTMF Digits Collected
    DigitsCollected,
    /// Receive Call
    ReceiveCall,
    /// Connect
    Connect,
    /// On Hook
    OnHook,
    /// Clear Call
    ClearCall,
    /// Unknown
    Unknown,
    CallEventMax
    //@}
};

extern int newRequestIdentifier (void);
extern int newCallIdentifier (void);
extern void callDataInit (CallData*);
extern CallData* newCallData (Endpoint*);
extern void notifyEndpoint (Endpoint*, int);
extern void* epStart (void*);

#endif /* ENDPOINT_H */
