
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const gateway_cxx_Version =
    "$Id: gateway.cxx,v 1.24 2001/01/17 01:52:24 liuhong Exp $";
#include <signal.h>
#include "VTime.hxx"
#include <sys/types.h>
#include <unistd.h>
#include <queue>
#include <typeinfo>
#include <map>
#include "MgAgent.hxx"
#include "SdpSession.hxx"
#include "NullHardware.h"
#include "TpjackHardware.h"
#include "VThread.h"

// response codes, should be defined somewhere else
#define RSP_CODE_NORMAL ReasonTransactionExecuted

/**********************************************************************
 
Data structures
 
**********************************************************************/

class StateMachine;
class StateEvent;

// state handler and transition functions
typedef void (*StateHandler) (StateEvent* event, StateMachine* state);
void stateUninitialized (StateEvent* event, StateMachine* state);
void stateIdle (StateEvent* event, StateMachine* state);
void stateRingingIn (StateEvent* event, StateMachine* state);
void stateRingingOut (StateEvent* event, StateMachine* state);
void stateHalfOpenIn (StateEvent* event, StateMachine* state);
void stateHalfOpen (StateEvent* event, StateMachine* state);
void stateActiveNoNotifications (StateEvent* event, StateMachine* state);
void stateActive (StateEvent* event, StateMachine* state);
void stateWaitForClose (StateEvent* event, StateMachine* state);
void stateWaitOnHook (StateEvent* event, StateMachine* state);

// hardware events handled by state machine
enum StateEventStimulus
{
    StimulusNULL,
    StimulusNewMessage,
    StimulusResponse,
    StimulusOffHook,
    StimulusOnHook,
    StimulusDigit0,
    StimulusDigit1,
    StimulusDigit2,
    StimulusDigit3,
    StimulusDigit4,
    StimulusDigit5,
    StimulusDigit6,
    StimulusDigit7,
    StimulusDigit8,
    StimulusDigit9,
    StimulusDigitStar,
    StimulusDigitHash
};


/**********************************************************************
 
Class: State Machine
Description: A single endpoint, single connection, residential gateway.
 
**********************************************************************/
class StateMachine
{
    public:
        StateMachine ()
        {}
        ;

        MgAgent* callAgent;
        Hardware* hardware;
        StateHandler currentState;
        SdpSession localSD;
        SdpSession remoteSD;
        string endpointId;
        string currentRequestId;
        string callId;
        string connectionId;
        string connectionMode;

        void setState (StateHandler current)
        {
            char* string = "";
            if (current == stateUninitialized)
                string = "stateUninitialized";
            else if (current == stateIdle)
                string = "stateIdle";
            else if (current == stateRingingIn)
                string = "stateRingingIn";
            else if (current == stateRingingOut)
                string = "stateRingingOut";
            else if (current == stateHalfOpen)
                string = "stateHalfOpen";
            else if (current == stateActiveNoNotifications)
                string = "stateActiveNoNotifications";
            else if (current == stateActive)
                string = "stateActive";
            cout << "State changed to: " << string << "\n";

            currentState = current;
        };
};


/**********************************************************************
 
Class: State Event
Description: An incoming event into the residnetial gateway, may be
             an MGCP message from the call agent or an asynchronous 
             input from the endpoint hardware.
**********************************************************************/
class StateEvent
{
    public:
        StateEvent() : stimulus(StimulusNULL)
        {}

        StateEventStimulus getStimulus ()
        {
            return stimulus;
        }
        void setStimulus (StateEventStimulus stim)
        {
            stimulus = stim;
        }
        Sptr < MgCommand > getMessage ()
        {
            return newMsg;
        }
        void setMessage (Sptr < MgCommand > msg)
        {
            newMsg = msg;
            stimulus = StimulusNewMessage;
        }

    private:
        StateEventStimulus stimulus;
        Sptr < MgCommand > newMsg;
};


/*********************************************************************
 
Utility Functions
 
*********************************************************************/
char* getHost()
{
    static char buf[256];
#ifndef __vxworks
    gethostname(buf, 256);
#else
    strcpy(buf, "vovida.com");
#endif 
    return buf;
}

void wrongMessage()
{
    cerr << "received invalid message\n";
#if 0
    int parentpid = getpid();
    kill(parentpid, SIGSTOP);
#endif
    exit(1);
}

void fatalError(char* errorMsg)
{
    cerr << errorMsg << "\n";
    exit(1);
}


/*********************************************************************
 
State Handler Functions
 
*********************************************************************/
void stateUninitialized (StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgNotificationRequest > rqntPtr;

        switch (msgPtr->getType())
        {
            case MgNotificationRequestType:
            // state 3 -- receive notification request
            rqntPtr.dynamicCast(msgPtr);
            if (rqntPtr == NULL)
                wrongMessage();

            // store required paramters
            state->endpointId = rqntPtr->getEndpointId();
            state->currentRequestId = rqntPtr->getRequestIdentifier();

            // ignore all optional paramters

            // reply with successful response code and string,
            // transaction id handled by mgcp stack
            rqntPtr->setResponseCode(RSP_CODE_NORMAL);
            rqntPtr->setResponseString("OK");
            state->callAgent->reply(rqntPtr);
            state->setState(stateIdle);
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateIdle (StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusOffHook)
    {
        // create and send a Notify msg regarding the offHook event
        MgNotify ntfy(state->endpointId, state->currentRequestId,
#ifndef NCS_PROFILE
                      (MgLinePackage::notifyOffHookTransition())
#else   // NCS_PROFILE
                      (MgNCSLinePackage::notifyOffHookTransition())
#endif
                     );
        state->callAgent->send(ntfy);

        // set hardware
        state->hardware->sendSignal(SignalLocalRingbackStart);

        state->setState(stateRingingOut);
    }

    else if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgNotificationRequest > rqntPtr;

        switch (msgPtr->getType())
        {
            case MgNotificationRequestType:
            // state 3 -- receive notification request
            rqntPtr.dynamicCast(msgPtr);
            if (rqntPtr == NULL)
                wrongMessage();

            // store required paramters
            state->endpointId = rqntPtr->getEndpointId();
            state->currentRequestId = rqntPtr->getRequestIdentifier();

            // ignore all optional parameters

            // set hardware
            state->hardware->sendSignal(SignalRingStart);

            // reply with successful response code and string,
            // transaction id handled by mgcp stack
            rqntPtr->setResponseCode(RSP_CODE_NORMAL);
            rqntPtr->setResponseString("OK");
            state->callAgent->reply(rqntPtr);
            state->setState(stateRingingIn);
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


// the set is ringing and you may get the message offhook/onhook
void stateRingingIn(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusOffHook)
    {
        // stop ringing
        state->hardware->sendSignal(SignalRingStop);

        // create and send a Notify msg regarding the offHook event
        MgNotify notifyMsg(state->endpointId, state->currentRequestId,
#ifndef NCS_PROFILE
                           (MgLinePackage::notifyOffHookTransition())
#else   // NCS_PROFILE
                           (MgNCSLinePackage::notifyOffHookTransition())
#endif
                          );
        state->callAgent->send(notifyMsg);

        // wait for the create connection
        state->setState(stateHalfOpenIn);
    }
    else
        cout << "received message or hardware input for which no handling exists\n";
}

void stateRingingOut(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgCreateConnection > crcxPtr;

        switch (msgPtr->getType())
        {
            case MgCreateConnectionType:
            {
                // state 5 -- receive create connection
                crcxPtr.dynamicCast (msgPtr);
                if (crcxPtr == NULL)
                    wrongMessage();

                // store required paramters
                state->endpointId = crcxPtr->getEndpointId();
                state->currentRequestId = crcxPtr->getRequestIdentifier();
                state->callId = crcxPtr->getCallId();
                state->connectionMode = crcxPtr->getConnectionMode();
                if (state->connectionMode != "sendrecv")
                    wrongMessage();

                // ignore all optional parameters

                // setup local session descriptor
                state->localSD.setSessionName("test session");
                state->localSD.setUserName("testID");
                state->localSD.setAddress(getHost());
                state->localSD.setSessionId(128538);
                state->localSD.setVersion(0);

                SdpConnection* sessionConnection = new SdpConnection;
                sessionConnection->setUnicast ((state->localSD.getAddress()).c_str());
                state->localSD.setConnection (*sessionConnection);

                SdpMedia* media = new SdpMedia;
                media->setMediaType (MediaTypeAudio);
                media->setTransportType (TransportTypeRTP);
                media->setPort (6050);
                state->localSD.addMedia (media);

                // send a reply with connection id and local connection
                // description information; connection id is bogus (= callId)
                // and session descriptor is hardcoded
                state->connectionId = state->callId;
                crcxPtr->setConnectionId(state->connectionId);
                crcxPtr->setLocalConnectionDescriptor(state->localSD);

                // reply with successful response code and string,
                // transaction id handled by mgcp stack
                crcxPtr->setResponseCode(RSP_CODE_NORMAL);
                crcxPtr->setResponseString("OK");
                state->callAgent->reply(crcxPtr);

                state->setState(stateHalfOpen);
                break;
            }
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateHalfOpenIn(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgCreateConnection > crcxPtr;

        switch (msgPtr->getType())
        {
            case MgCreateConnectionType:
            {
                // state 5 -- receive create connection
                crcxPtr.dynamicCast(msgPtr);
                if (crcxPtr == NULL)
                    wrongMessage();

                // store required paramters
                state->endpointId = crcxPtr->getEndpointId();
                state->currentRequestId = crcxPtr->getRequestIdentifier();
                state->callId = crcxPtr->getCallId();
                state->connectionMode = crcxPtr->getConnectionMode();

                if (state->connectionMode != "sendrecv")
                    wrongMessage();

                // ignore all optional parameters except remoteSD
                state->remoteSD = crcxPtr->getRemoteConnectionDescriptor();

                // setup local session descriptor
                state->localSD.setSessionName("test session");
                state->localSD.setUserName("testID");
                state->localSD.setAddress(getHost());
                state->localSD.setSessionId(128538);
                state->localSD.setVersion(0);

                SdpConnection* sessionConnection = new SdpConnection;
                sessionConnection->setUnicast ((state->localSD.getAddress()).c_str());
                state->localSD.setConnection (*sessionConnection);

                SdpMedia* media = new SdpMedia;
                media->setMediaType (MediaTypeAudio);
                media->setTransportType (TransportTypeRTP);
                media->setPort (6060);
                state->localSD.addMedia (media);

                // send a reply with connection id and local connection
                // description information; connection id is bogus (= callId)
                // and session descriptor is hardcoded
                state->connectionId = state->callId;
                crcxPtr->setConnectionId(state->connectionId);
                crcxPtr->setLocalConnectionDescriptor(state->localSD);

                // reply with successful response code and string,
                // transaction id handled by mgcp stack
                crcxPtr->setResponseCode(RSP_CODE_NORMAL);
                crcxPtr->setResponseString("OK");
                state->callAgent->reply(crcxPtr);

                // setup audio
                int localPort = 0;
                vector < SdpMedia* > localMediaList = state->localSD.getMediaList();
                if (localMediaList.size() > 0)
                {
                    localPort = localMediaList[0]->getPort();
                }

                int remotePort = 0;
                vector < SdpMedia* > remoteMediaList = state->remoteSD.getMediaList();
                if (remoteMediaList.size() > 0)
                {
                    remotePort = remoteMediaList[0]->getPort();
                }

                state->hardware->audioStart
                (localPort,
                 state->remoteSD.getAddress().c_str(),
                 remotePort);

                state->setState(stateActiveNoNotifications);
            }
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateHalfOpen(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgModifyConnection > mdcxPtr;

        switch (msgPtr->getType())
        {
            case MgModifyConnectionType:
            {
                // state 7 -- receive create connection
                mdcxPtr.dynamicCast (msgPtr);
                if (mdcxPtr == NULL)
                    wrongMessage();

                // check for matching callId
                if (mdcxPtr->getCallId() != state->callId)
                    wrongMessage();

                // check connection mode
                //if (mdcxPtr->getConnectionMode() != state->connectionMode)
                //wrongMessage();

                // ignore all optional parameters except remoteSD
                state->remoteSD = mdcxPtr->getRemoteConnectionDescriptor();

                // stop ringback
                state->hardware->sendSignal(SignalLocalRingbackStop);

                // setup audio
                int localPort = 0;
                vector < SdpMedia* > localMediaList = state->localSD.getMediaList();
                if (localMediaList.size() > 0)
                {
                    localPort = localMediaList[0]->getPort();
                }

                int remotePort = 0;
                vector < SdpMedia* > mediaList = state->remoteSD.getMediaList();
                if (mediaList.size() > 0)
                {
                    remotePort = mediaList[0]->getPort();
                }

                state->hardware->audioStart
                (localPort,
                 state->remoteSD.getAddress().c_str(),
                 remotePort);

                // reply with successful response code and string,
                // transaction id handled by mgcp stack
                mdcxPtr->setResponseCode(RSP_CODE_NORMAL);
                mdcxPtr->setResponseString("OK");
                state->callAgent->reply(mdcxPtr);

                state->setState(stateActiveNoNotifications);
                break;
            }
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateActiveNoNotifications(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgNotificationRequest > rqntPtr;

        switch (msgPtr->getType())
        {
            case MgNotificationRequestType:
            rqntPtr.dynamicCast (msgPtr);
            if (rqntPtr == NULL)
                wrongMessage();

            // store required paramters
            state->endpointId = rqntPtr->getEndpointId();
            state->currentRequestId = rqntPtr->getRequestIdentifier();

            // ignore all optional paramters

            // reply with successful response code and string,
            // transaction id handled by mgcp stack
            rqntPtr->setResponseCode(RSP_CODE_NORMAL);
            rqntPtr->setResponseString("OK");
            state->callAgent->reply(rqntPtr);
            state->setState(stateActive);
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateActive(StateEvent* event, StateMachine* state)
{
    // wait for dlcx, onhook, or DTMF digits
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgDeleteConnectionAgent > dlcxPtr;
        Sptr < MgNotificationRequest > rqntPtr;

        switch (msgPtr->getType())
        {
            case MgNotificationRequestType:
            rqntPtr. dynamicCast (msgPtr);
            if (rqntPtr == NULL)
                wrongMessage();

            // store required paramters
            state->endpointId = rqntPtr->getEndpointId();
            state->currentRequestId = rqntPtr->getRequestIdentifier();

            // check for request to play DTMF tones
            if (rqntPtr->getNumSignalRequests())
            {
                Sptr < MgEvent > signalPtr = rqntPtr->getItemSignalRequests(0);
#ifndef NCS_PROFILE
                Sptr < MgDTMFPackageEvent > dtmfPtr;
#else // NCS_PROFILE
Sptr < MgNCSLinePackageEvent > dtmfPtr;
#endif
                dtmfPtr.dynamicCast(signalPtr);
                assert (dtmfPtr != 0);
                switch (dtmfPtr->getType())
                {
#ifndef NCS_PROFILE
                    case MgcDTMFPackageDTMF_0:
                    cerr << "received DTMF 0 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_1:
                    cerr << "received DTMF 1 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_2:
                    cerr << "received DTMF 2 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_3:
                    cerr << "received DTMF 3 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_4:
                    cerr << "received DTMF 4 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_5:
                    cerr << "received DTMF 5 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_6:
                    cerr << "received DTMF 6 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_7:
                    cerr << "received DTMF 7 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_8:
                    cerr << "received DTMF 8 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_9:
                    cerr << "received DTMF 9 from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_Hash:
                    cerr << "received DTMF # from far end\n";
                    break;
                    case MgcDTMFPackageDTMF_Star:
                    cerr << "received DTMF * from far end\n";
                    break;
#else  // NCS_PROFILE
                    case MgcNCSLinePackageDTMF_0:
                    cerr << "received DTMF 0 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_1:
                    cerr << "received DTMF 1 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_2:
                    cerr << "received DTMF 2 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_3:
                    cerr << "received DTMF 3 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_4:
                    cerr << "received DTMF 4 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_5:
                    cerr << "received DTMF 5 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_6:
                    cerr << "received DTMF 6 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_7:
                    cerr << "received DTMF 7 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_8:
                    cerr << "received DTMF 8 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_9:
                    cerr << "received DTMF 9 from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_Hash:
                    cerr << "received DTMF # from far end\n";
                    break;
                    case MgcNCSLinePackageDTMF_Star:
                    cerr << "received DTMF * from far end\n";
                    break;
#endif
                    default:
                    cerr << "received DTMF (unknown) from far end\n";
                    break;
                } // end switch
            }
            // reply with successful response code and string,
            // transaction id handled by mgcp stack

            rqntPtr->setResponseCode(RSP_CODE_NORMAL);
            rqntPtr->setResponseString("OK");
            state->callAgent->reply(rqntPtr);
            break;

            case MgDeleteConnectionAgentType:
            dlcxPtr.dynamicCast (msgPtr);
            if (dlcxPtr == NULL)
                wrongMessage();

            // check endpoint and connection id
            //if (state->endpointId != dlcxPtr->getEndpointId())
            //wrongMessage();
            //if (state->connectionId != dlcxPtr->getConnectionId())
            //wrongMessage();

            // ignore all optional paramters

            // reply with successful response code and string,
            // transaction id handled by mgcp stack
            dlcxPtr->setResponseCode(RSP_CODE_NORMAL);
            dlcxPtr->setResponseString("OK");
            state->callAgent->reply(dlcxPtr);

            state->setState(stateWaitOnHook);

            // close the audio channel
            state->hardware->sendSignal(SignalAudioStop);
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else if (event->getStimulus() == StimulusOnHook)
    {
        // create and send a Notify msg regarding the onHook event
        MgNotify ntfy(state->endpointId, state->currentRequestId,
#ifndef NCS_PROFILE
                      (MgLinePackage::notifyOnHookTransition())
#else  // NCS_PROFILE
                      (MgNCSLinePackage::notifyOnHookTransition())
#endif
                     );
        state->callAgent->send(ntfy);

        state->setState(stateWaitForClose);
    }
#ifndef NCS_PROFILE
    else if (event->getStimulus() == StimulusDigit0)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_0()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit1)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_1()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit2)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_2()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit3)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_3()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit4)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_4()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit5)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_5()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit6)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_6()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit7)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_7()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit8)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_8()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit9)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_9()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigitStar)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_Star()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigitHash)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgDTMFPackage::notifyDTMF_Hash()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

#else  // NCS_PROFILE
else if (event->getStimulus() == StimulusDigit0)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_0()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit1)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_1()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit2)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_2()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit3)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_3()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit4)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_4()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit5)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_5()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit6)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_6()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit7)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_7()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit8)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_8()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigit9)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_9()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigitStar)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_Star()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

    else if (event->getStimulus() == StimulusDigitHash)
    {
        MgNotify ntfy(state->endpointId, state->currentRequestId,
                      (MgNCSLinePackage::notifyDTMF_Hash()));
        state->callAgent->send(ntfy);
        // remain in current state
    }

#endif
    else
    {
        cout << "received hardware input for which no handling exists\n";
    }
}


void stateWaitForClose(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusNewMessage)
    {
        Sptr < MgCommand > msgPtr = event->getMessage();
        Sptr < MgDeleteConnectionAgent > dlcxPtr;

        switch (msgPtr->getType())
        {
            case MgDeleteConnectionAgentType:
            dlcxPtr. dynamicCast (msgPtr);
            if (dlcxPtr == NULL)
                wrongMessage();

            // check endpoint and connection id
            //if (state->endpointId != dlcxPtr->getEndpointId())
            //wrongMessage();
            //if (state->connectionId != dlcxPtr->getConnectionId())
            //wrongMessage();

            // ignore all optional paramters

            // reply with successful response code and string,
            // transaction id handled by mgcp stack
            dlcxPtr->setResponseCode(RSP_CODE_NORMAL);
            dlcxPtr->setResponseString("OK");
            state->callAgent->reply(dlcxPtr);

            state->setState(stateUninitialized);

            // close the audio channel
            state->hardware->sendSignal(SignalAudioStop);
            break;
            default:
            // ignore other messages
            cout << "received message for which no handling exists\n";
            break;
        } // end switch
    }

    else
        cout << "received hardware input for which no handling exists\n";
}


void stateWaitOnHook(StateEvent* event, StateMachine* state)
{
    if (event->getStimulus() == StimulusOnHook)
    {
        // create and send a Notify msg regarding the onHook event
        MgNotify ntfy(state->endpointId, state->currentRequestId,
#ifndef NCS_PROFILE
                      (MgLinePackage::notifyOnHookTransition())
#else  // NCS_PROFILE
                      (MgNCSLinePackage::notifyOnHookTransition())
#endif
                     );
        state->callAgent->send(ntfy);

        state->setState(stateUninitialized);
    }
    else
        cout << "received hardware input or message for which no handling exists\n";
}

void* process(void* foo)
{
    MgAgent* bar;

    //    cout << "entering process loop\n";

    bar = static_cast < MgAgent* > (foo);

    bar->threadProcess();
    //    cout << "exiting process loop\n";
    return NULL;
}

/*********************************************************************
 
main
Description: Single endpoint residential gateway.
 
*********************************************************************/
int main(int argc, char* argv[])
{
    StateEvent event;
    HardwareEvent hwEvent;
    int ret_val;

    // read configuration parameters
    if (argc != 4)
    {
        // this is an error
        cerr << "usage: gateway <device> <agentHostname> <port>\n";
        return 0;
    }

    string device(argv[1]);
    char* agent_host = argv[2];
    int port = atoi(argv[3]);

    // initialize gateway
    StateMachine state;
#if defined(__linux__)
    if (device == "tpjack")
    {
        // state.hardware = new TpjackHardware("/dev/ixj0");
        state.hardware = new TpjackHardware("/dev/phone0");
    }
    else
#endif
    {
        // state.hardware = new NullHardware("/dev/ixj0");
        state.hardware = new NullHardware("/dev/phone0");
    }

    fd_set test_fd;
    state.hardware->addToFdSet(&test_fd);

    int callAgentPort;
    if (port == 5050)
    {
        callAgentPort = 5010;
    }
    else
    {
        callAgentPort = 5011;
    }
    MgAgent callAgent(agent_host, callAgentPort, port);
    state.callAgent = &callAgent;
    state.setState(stateUninitialized);

    // set endpointId, this is just temporary, could be done in constructor
    if (port == 5050)
        state.endpointId = "testID@localhost:5050";
    else
        state.endpointId = "testID@localhost:5051";

    fd_set read_fds;
    srand(time(NULL));
    FD_SET(0, &read_fds);


    // spawn the looping thread
    VThread tid;
    // int errNum;

    tid.spawn(process, state.callAgent);

    // loop forever processing inputs from call agent and endpoint hardware
    for (; ; )
    {
        // reset file descriptor
        FD_ZERO(&read_fds);
        int maxFd = 16;

        state.hardware->addToFdSet(&read_fds);
        //	state.callAgent->addToFdSet(&read_fds);

        // temporarily perform a blocking wait that times out just to
        // make sure hardware does get processed
        struct timeval tv;
        tv.tv_sec = 0;
        tv.tv_usec = 300;

        // block on select, wait for input event from
        // endpoint hardware and/or call agent
        ret_val = select(maxFd, &read_fds, NULL, NULL, &tv);
        if (ret_val < 0)
        {
            cerr << "select() returned with an error\n";
        }
        else
        {
            if (ret_val == 0)
            {
                //cerr << "timeout... check hardware\n";
            }


            if (state.hardware->process(&read_fds))
            {
                //		cerr << "handle hardware I/O\n";

                // endpoint hardware interface
                while ((hwEvent = state.hardware->getEvent()) != EventNULL)
                {
                    switch (hwEvent)
                    {
                        case EventOffHook:
                        event.setStimulus(StimulusOffHook);
                        break;
                        case EventOnHook:
                        event.setStimulus(StimulusOnHook);
                        break;
                        case EventDigit0:
                        event.setStimulus(StimulusDigit0);
                        break;
                        case EventDigit1:
                        event.setStimulus(StimulusDigit1);
                        break;
                        case EventDigit2:
                        event.setStimulus(StimulusDigit2);
                        break;
                        case EventDigit3:
                        event.setStimulus(StimulusDigit3);
                        break;
                        case EventDigit4:
                        event.setStimulus(StimulusDigit4);
                        break;
                        case EventDigit5:
                        event.setStimulus(StimulusDigit5);
                        break;
                        case EventDigit6:
                        event.setStimulus(StimulusDigit6);
                        break;
                        case EventDigit7:
                        event.setStimulus(StimulusDigit7);
                        break;
                        case EventDigit8:
                        event.setStimulus(StimulusDigit8);
                        break;
                        case EventDigit9:
                        event.setStimulus(StimulusDigit9);
                        break;
                        case EventDigitStar:
                        event.setStimulus(StimulusDigitStar);
                        break;
                        case EventDigitHash:
                        event.setStimulus(StimulusDigitHash);
                        break;
                        case EventAudioFailed:
                        cerr << "Event Audio Failed\n";
                        break;
                        case EventAudioClosed:
                        cerr << "Event Audio Closed\n";
                        break;
                        default:
                        break;
                    };
                    state.currentState(&event, &state);
                }
            }

            while (state.callAgent->checkMsgReady(&read_fds))
            {
                //		cerr << "handle agent command\n";

                // incoming control packet from call agent
                Sptr < MgCommand > msg = state.callAgent->receive();
                event.setMessage(msg);
                state.currentState(&event, &state);
            }

            if (0)
            {
                // incoming audio
            }


        } // end else

    }


    return 0;
}
