
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
/// test the function parseEventsRequested(string) , in eventCoding.cxx

// $Id: test_parse_events_requested.cxx,v 1.7 2000/12/18 23:42:38 bko Exp $

#include "global.h"
#include <iostream>
#include <deque>
#include <string>
#include "MgcpParameterEventGeneric.hxx"

/**********************************************************************
 
The following functions is extern'ed here because it is being tested.
 
*/

Sptr < MgcpParameterRequestedEvent > parseEventsRequested(const string& s);

const int NUM_TESTCASES = 5;

int main(void)
{
    bool pass[NUM_TESTCASES];

    int i;

    for (i = 0; i < NUM_TESTCASES; i++)
    {
        pass[i] = false;
    }

    string test[10];

    MgcpRequestedActionType answer[10];
    Sptr < MgcpParameterRequestedEvent > test_answer[10];

    test[0] = "l/hd(n)";
    answer[0] = MgcpRequestedActionTypeNotify;
    test[1] = "l/hd";
    answer[1] = MgcpRequestedActionTypeNotify;
    test[2] = "l/hu(e())";
    answer[2] = MgcpRequestedActionTypeEmbedNotificationRequest;
    test[3] = "l/hu(d)";
    answer[3] = MgcpRequestedActionTypeUseDigitMap;
    test[4] = "l/hu(e(r([0-9#t](d),hu(n)),s(dl),d([0-9].[#t])))";
    answer[4] = MgcpRequestedActionTypeEmbedNotificationRequest;



    for (int i = 0; i < NUM_TESTCASES; i++)
    {
        test_answer[i] = parseEventsRequested(test[i]);
        Sptr < MgcpParameterRequestedEventGeneric > foo;
        foo.dynamicCast(test_answer[i]);

        if (foo->getRequestedAction().getActionType() != answer[i])
        {
            cout << i << " failed:" << endl;
            cout << foo->getRequestedAction().encode() << endl;
            pass[i] = false;
        }
        else
        {
            // passed
            pass[i] = true;
        }
    }

    bool pass_all = true;
    for (i = 0; i < NUM_TESTCASES; i++)
    {
        if (pass[i])
        {
            cout << "passed " << i << "\n";
        }
        else
        {
            cout << "failed " << i << "\n";
            pass_all = false;
        }
    }

    if (pass_all)
        exit(0);
    else
        exit( -1);
}
