
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const test_subsplit_cxx_Version =
    "$Id: test_subsplit.cxx,v 1.8 2000/12/18 23:42:38 bko Exp $";
#include "global.h"
#include <iostream>
#include <deque>
#include <string>
#include "substring.h"
#include "support.hxx"

const int NUM_TESTCASES = 3;

bool test2()
{
    string test_1("this   is   a test\n\n\n\nthat may work\n");
    sub_split_t test_1_vector;
    test_1_vector = sub_split(test_1, " \n");

    printf("!%p, %p\n", (&test_1_vector[0])->c_str(), (&test_1_vector[1])->c_str());
    printf("%p, ", test_1_vector[0].c_str());
    printf("%p\n", test_1_vector[1].c_str());
    printf("%s, %s\n", test_1_vector[0].str().c_str(), test_1_vector[1].str().c_str());
    printf("%p, %p\n", test_1_vector[0].str().c_str(), test_1_vector[1].str().c_str());
}

int main(void)
{
    test2();

    bool pass[NUM_TESTCASES];

    unsigned int i;

    for (i = 0; i < NUM_TESTCASES; i++)
    {
        pass[i] = false;
    }

    string test_1("this   is   a test\n\n\n\nthat may work\n");

    string test_2("this");

    deque < substring > test_1_vector;

    deque < substring > test_2_vector;

    test_1_vector = sub_split(test_1, " \n");

    test_2_vector = sub_split(test_2, " \n");

    if (test_1_vector.size() != 7)
    {
        for (i = 0; i < test_1_vector.size(); i++)
        {
            cerr << ":" << test_1_vector[i].str() << ":" << endl;
        }
    }
    else
    {
        if (
            !(test_1_vector[0] == "this")
            || !(test_1_vector[1] == "is")
            || !(test_1_vector[2] == "a")
            || !(test_1_vector[3] == "test")
            || !(test_1_vector[4] == "that")
            || !(test_1_vector[5] == "may")
            || !(test_1_vector[6] == "work")
        )
        {
            for (i = 0; i < test_1_vector.size(); i++)
            {
                cerr << ":" << test_1_vector[i].str() << ":" << endl;
            }
        }
        else
        {
            pass[0] = true;
        }
    }

    if ((test_2_vector.size() == 1) && (test_2_vector[0] == "this"))
    {

        pass[1] = true;
    }


    string test_3("rsip 20 ep50@lab4.private.vovida.com mgcp 0.1");
    sub_split_t vector_3 = sub_split(test_3, " ");

    if (
        !(vector_3[0] == "rsip") ||
        !(vector_3[1] == "20") ||
        !(vector_3[2] == "ep50@lab4.private.vovida.com") ||
        !(vector_3[3] == "mgcp") ||
        !(vector_3[4] == "0.1")
    )
    {
        for (i = 0; i < test_1_vector.size(); i++)
        {
            cerr << ":" << test_1_vector[i].str() << ":" << endl;
        }
    }
    else
    {
        pass[2] = true;
    }

    bool pass_all = true;
    for (i = 0; i < NUM_TESTCASES; i++)
    {
        if (pass[i])
        {
            cout << "passed " << i << "\n";
        }
        else
        {
            cout << "failed " << i << "\n";
            pass_all = false;
        }
    }

    if (pass_all)
        exit(0);
    else
        exit( -1);
}
