
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const timeoutTest_cxx_Version =
    "$Id: timeoutTest.cxx,v 1.10 2001/01/17 01:52:25 liuhong Exp $";
//#include <pthread.h>
#include "VThread.h"
#include <queue>
#include <typeinfo>
#include <map>
#include "MgAgent.hxx"

void*
process(void* gateway)
{
    MgAgent* gw;

    cout << "entering process loop\n";

    gw = static_cast < MgAgent* > (gateway);

    gw->threadProcess();
    cout << "exiting process loop\n";
    return NULL;
}

/*********************************************************************
 
main
 
*********************************************************************/
int main(int argc, char* argv[])
{
    int port = 5050;
    int callAgentPort = 5010;
    char* agent_host = "localhost";

    MgAgent callAgent(agent_host, callAgentPort, port);



    callAgent.debugFlag = true;
    VThread processThread;

    int errNum = processThread.spawn (process, &callAgent);

    if (errNum > 0)
    {
        cerr << "Cannot create thread: " << strerror (errNum) << endl;
    }


#if 0
    // create and send a Notify msg regarding the offHook event
    MgNotify ntfy("foo", "200",
                  (MgLinePackage::notifyOffHookTransition()));

    callAgent.send(ntfy);
#endif
    while (1)
    {

        Sptr < MgCommand > cmd = callAgent.receive();

        cout << "got msg\n";

        switch (cmd->getType())
        {
            case MgCreateConnectionType:
            {
                cout << "create connection\n";
                Sptr < MgCreateConnection > p;
                p.dynamicCast(cmd);

                MgcpConnectionOptions y;
                y = p->getLocalConnectionOptions();

                if (y.getEchoCancellation())
                {
                    cout << "echo cancellation on\n";
                }
                else
                {
                    cout << "echo cancellation off\n";
                }
            }
            break;

            case MgNotificationRequestType:
            {
                cout << "MgcNotificationRequestType\n";
                cout << "number of requestedEvents: ";
                Sptr < MgNotificationRequest > p;
                p.dynamicCast(cmd);
                cout << p->getNumRequestedEvents() << "\n";

                for (int i = 0; i < p->getNumRequestedEvents() ; i++)
                {
                    Sptr < MgRequestedEvent > event = p->getItemRequestedEvents(i);
                    if (event != 0)
                    {
                        cout << "event " << i << "\n";
                        //		    cout << event << "\n";
                        //cout << "xxx: " << event->getPackageType() << "\n";
                        switch (event->getPackageType())
                        {
                            case MgcpGenericMediaPackageType:
                            {
                                cout << "Generic : ";
                                Sptr < MgGenericPackageRequestedEvent > e;
                                e.dynamicCast(event);
                                switch (e->getType())
                                {
                                    case MgcGenericPackageReportFailure:
                                    cout << "ReportFailure\n";
                                    break;
				default:
				  break;

                                }
                            }
                            break;
			default:
			  break;
                        }
                        // try to get the actioin

                        cout << "action count: " << event->getNumAction() << "\n";

                        for (int j = 0; j < event->getNumAction(); j++)
                        {
                            cout << "action " << j << "\n";
                            //cout << "x: " << event->getItemAction(j).getActionType() << "\n";

                        }

                    }
                }


                break;
            }
            default:
            {
	      // cerr << "Unexpected command " << cmd->getType() << endl;
            }
        }

        // this server never replies
        //    callAgent.reply(*cmd);

    }

    return 0;
}
