
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgAgent_cxx_Version =
"$Id: MgAgent.cxx,v 1.48 2001/01/26 02:45:44 liuhong Exp $";



#include "MgAgent.hxx"
// #include <sys/time.h>
#include <sys/types.h>
#include <time.h>
//#include <pthread.h>
#include "MgcpDebugManager.hxx"
#include "MgcpResponseCode.hxx"
#include "MgcpParse.hxx"
#include "MgParse.hxx"

const long MinMgTimeout = 500000;
const long MgTimeout = 1000;  // 1 sec
const long MgTimeoutCap = 4000;  // 4 secs


MgAgent::MgAgent(int portLocal)
    : timeoutMsgs(MinMgTimeout, UniqueId("", -1, -1) ),
      timeoutResp(MinMgTimeout, UniqueId("", -1, -1)),
      useAsync(false)
{
    server = 0;
    server = new MgcpReceiver( portLocal );

    msgWaiting = false;
    respWaiting = false;
    queueMutex = new VMutex;
    networkMutex = new VMutex;

    msgPending = new VCondition;
    respPending = new VCondition;
}

MgAgent::MgAgent(const char* address, int portRemote, int portLocal)
    : timeoutMsgs(MinMgTimeout, UniqueId("", -1, -1)),
      timeoutResp(MinMgTimeout, UniqueId("", -1, -1)),
      useAsync(false)

{
    server =  0;
    server = new MgcpReceiver( portLocal );

    Sptr<MgcpTransmitter> ptr = new MgcpTransmitter(address, portRemote, server);
    // this defaults to 0
    callAgentList.push_back(ptr);

    msgWaiting = false;
    respWaiting = false;
    queueMutex = new VMutex;
    networkMutex = new VMutex;

    msgPending = new VCondition;
    respPending = new VCondition;
}


MgAgentHandle MgAgent::addAgent(const char* address, int portRemote)
{
    Sptr<MgcpTransmitter> ptr = new MgcpTransmitter(address, portRemote);

    callAgentList.push_back(ptr);
    return (callAgentList.size() - 1);
}


void MgAgent::flushMessages()
{
    // xxx -- run this in your own lock!!!
    while(pendingMessages.begin() != pendingMessages.end() 
	  && (pendingMessages.front() == 0))
    {
	pendingMessages.pop_front();
    }
}


void MgAgent::processPending()
{
//    cout << "got message\n";
    MgcpPacket* data = 0;
    NetworkAddress sourceAddr;
	
    data = server->receive();
    
    if (!data)
      return;

     sourceAddr = data->getAddress();
     char* p = data->getMessageLocation();
     vector<string> parameters;
     string s, tempstr,st_temp;
     const char* st;
     int initpos = 0;
     string myData = string(p);
     string::size_type itemPlace = myData.find("\r\n.\r\n");
     if( itemPlace != string::npos)
	 {
       while( itemPlace != string::npos )
	   {
         st_temp = string(myData,initpos,myData.size()-(myData.size()-itemPlace));
         parameters.push_back( st_temp );
         tempstr = string(myData,itemPlace+5,myData.size()-itemPlace-5);
         myData = tempstr;
         itemPlace = myData.find("\r\n.\r\n");
       }
       parameters.push_back(myData);
	   //  split_t parameters(split(myData, "\r\n.\r\n"));
	   //  split_t::const_iterator iter = parameters.begin();
       //  parameters = split(myData, "\r\n.\r\n");
         vector<string>::const_iterator iter = parameters.begin(); 
         while( iter != parameters.end() )
		 {
          s = (*iter);
          st = s.c_str();
          MgcpCommand* message = parseMessage(st, s.size());
          SingleProcess( sourceAddr, message );
          iter++;
		}
     } 
     else 
    {
      MgcpCommand* message = parseMessage(data->getMessageLocation(), data->getMessageSize());
      SingleProcess( sourceAddr, message );
    }
    delete data;
    data = 0;
    return;
}

void MgAgent::SingleProcess(NetworkAddress sourceAddr, MgcpCommand* message)
{

    if(dynamic_cast<MgcpResponse*>(message))
    {
        int transactionID = message->getTransactionId();

	// get an unique id
	UniqueId uID( sourceAddr.getHostName(), sourceAddr.getPort(), transactionID);
	
        
	// the responses need to be matched (via a map) to
	// their proper originators
	MgMessageStuff* original = 0;
	original = sentMessages[ uID ];
	if(!original)
	{
	    // this is not a useful message -- warn and throw away
	    cerr << "unexpected response received: tid:\n";
	    cerr << message->getTransactionId() << "\n";
	    cerr << "-------------------- message --------------------\n";
	    // should get the message printed here
	    cerr << "--------------------   end   --------------------\n";
	
	    // free the unmatched response message
	    delete message;

	    return;
	}
	else
	{
	    queueMutex->lock();
	    original->command->setResponsePtr
		(dynamic_cast<MgcpResponse*>(message));

	    DEBUG(cout << "deleting TID (found): " << message->getTransactionId() << endl;);

	    sentMessages.erase( uID );
	    original->ready = true;
	    queueMutex->unlock();

//#ifdef NCS_PROFILE
	    // check to see if this is acknowledgement to a final response
	    // if true, forward it to the user
	    if ( (dynamic_cast<MgcpResponse*>(message))->getResponseCode() == ReasonResponseAcknowledgement )
	    {
		queueMutex->lock();
		//	sentMessages.erase( uID );
		//	timeoutResp.remove( original->nodeInTimer );
		pendingMessages.push_back(original->command);
		msgWaiting = true;
		queueMutex->unlock();

		if(original->heapAlloc == true)
	        {
		   delete original;
	        }
	    }
	    else
	    {
//#endif
	        if(useAsync == false)
	        {
		    original->responsePending->signal();
	        }
	        else
	        {
		    queueMutex->lock();
		    pendingMessages.push_back(original->command);
		    msgWaiting = true;
		    queueMutex->unlock();
			    
		    if(original->heapAlloc == true)
		    {
		        delete original;
		    }
	        }
//#ifdef NCS_PROFILE	 
	    }  
//#endif
	}
    }
    else if(message != 0)  // received a command
    {
        int transactionID = message->getTransactionId();

	// convert to MgCommand
	Sptr<MgCommand> msg = mgcpToMg(message);

	DEBUG(cout << "received:  tid: " << msg->getDataPtr()->getTransactionId() << endl;);

	// look for response acknowledgement attribute in the command
	string respAck = MgGetRespAckString(msg);

	// parse the response ack and set flags in sentResponse for confirmed responses
	if (respAck.length() !=0 )
	{
	  processRangeInRespAck(respAck,sourceAddr );
	}

       
	/// get an unique id for this command
	UniqueId uID( sourceAddr.getHostName(), sourceAddr.getPort(), transactionID);
	
	// check to see if the command is a duplicate of a previous command for which
	// we have already sent a response, if true and the response is not yet confirmed, re-send the same
	// response, if the response has been confirmed, simply ignore the command
	Sptr<MgMessageStuff> original = 0;
	original = sentResponses[uID];
	if ( original != 0 )
	{
	  if (!original->getconfirmed())
	  {
	      string host = sourceAddr.getHostName();

	      // re-send the same response
	      Sptr<MgcpTransmitter> transmitter = new MgcpTransmitter( host.c_str(), sourceAddr.getPort(), server);
	      original->command->getResponsePtr()->send(*transmitter);  
	      
	      return;
	  }
	  else  // the response has already been confirmed, ignore the command
	  {
	      return;
	  }
	}

	/// check to see if the command is a duplicate of an existing
	/// command waiting for processing, if true, simple ignore this one
	map<UniqueId, int>::iterator p = TransWaiting.find( uID );

	if(p != TransWaiting.end() )
	{   
	    MgCommandType cmdType = msg->getType();
	    if (( cmdType != MgCreateConnectionType ) &&
		( cmdType != MgModifyConnectionType ))
	    {   /// this is a duplicate command,ignore it (and it is not CRCX nor MDCX )
                /// For duplicate CRCX and MDCX that user hasn't sent responses for (checked earlier),
                /// the stack should pass the dup to the user (processed next)
	        return;
	    }
	}


	/// now it is really a new command, pass it on to the user for processing
	/// or it may be a dup CRCX or MDCX that user hasn't yet sent responses for

	if(!(msg == 0))
	{
	    DEBUG(cout << "Got good msg\n";);
	    
	    // now, enqueue the message
	    queueMutex->lock();
	    
	    pendingMessages.push_back(msg);

	    /// insert it into the list of currently waiting commands
     	    TransWaiting[uID] = transactionID;
	   
	    msgWaiting = true;
	    msgPending->signal();
	    queueMutex->unlock();
	}
    }

    return;
}


void 
MgAgent::handleTimeouts(struct timeval* tv)
{
    MgMessageStuff* original = 0;

    // handle timeout for sentMessages
    while(!timeoutMsgs.sleepFor(tv))
    {

        UniqueId uID( timeoutMsgs.top() );

	DEBUG(cout << "handleTimeouts got TID: " << timeoutMsgs.top() << endl;);

	map< UniqueId, MgMessageStuff* >::iterator p = sentMessages.find( uID );
	
	if(p != sentMessages.end() && p->second)
	{
	    original = p->second;

	    string host = uID.getHost();	   
	   
	    // re-send the same command
	    if ( original->timeoutCount < LONG_TIMER )
	    {
	      long next = 0;
	      // calculate next timeout value
	      if (original->timeoutCount == 0 )
	      {
		next =static_cast<long>( MgTimeout * ( 1+ (rand()/(RAND_MAX+1.0))));		
	      }
	      else
	      {
		next = static_cast<long>(original->lastTimeout * (1+ (rand()/(RAND_MAX+1.0))));		
       	      }

	      if ( (next+original->timeoutCount) > LONG_TIMER )
		  next = LONG_TIMER - original->timeoutCount;
	      
	      if ( next > MgTimeoutCap )
		next = MgTimeoutCap;

	      original->timeoutCount += next;
	      original->lastTimeout = next;

	      Sptr<MgcpTransmitter> transmitter = new MgcpTransmitter( host.c_str(), uID.getPort(), server);
	      original->command->getDataPtr()->send(*transmitter);  
	      
	      // set up timeout for this newly sent command again
	      queueMutex->lock();
	      timeoutMsgs.pop();
	      timeoutMsgs.insert( uID, original->lastTimeout);
	      original->nodeInTimer = TimerObj<UniqueId>(timeoutMsgs.getlastInserted());
	      queueMutex->unlock();
	    }
	    else  // reached maximum re-send within LONG_TIMER, send a timeout response to user
	    {
	        queueMutex->lock();
	    	
		MgcpResponse* xyz = new MgcpResponse;
		xyz->setResponseCode(TimeoutNoResponse);
	
		original->command->setResponsePtr
		  (dynamic_cast<MgcpResponse*>(xyz));
	    
		DEBUG(cout << "deleting TID (timeout): " << timeoutMsgs.top() << endl;);
		sentMessages.erase( p );
		original->ready = true;
		queueMutex->unlock();
	
		if(useAsync == false)
		{
		    original->responsePending->signal();
		}
		else
		{
		    // this is a message for delivery
		    queueMutex->lock();
		    pendingMessages.push_back(original->command);
		    msgWaiting = true;
		    queueMutex->unlock();

		    // free the original
		    if(original->heapAlloc == true)
		    {
			original->command = 0;
			delete original;
		    }
		}

		timeoutMsgs.pop();
	    }
    	}
	else
	  timeoutMsgs.pop();

    }
    // wake this puppy up

    struct timeval tv2;

    tv2.tv_sec = 0;
    tv2.tv_usec = 500000;

    // handle time out for sentResponses, do we accept 500000 usec wait ???
    while(!timeoutResp.sleepFor(&tv2))
    {	
        queueMutex->lock();
	map< UniqueId, Sptr<MgMessageStuff> >::iterator p = 
	    sentResponses.find(timeoutResp.top());
	queueMutex->unlock();

	if( (p != sentResponses.end() ) && (p->second != 0) )
	{
	    queueMutex->lock();	   
	    sentResponses.erase(timeoutResp.top());
	    queueMutex->unlock();
	}
	timeoutResp.pop();
    }

    return;
}

void MgAgent::threadProcess()
{
    networkMutex->lock();

    fd_set read_fds;
    fd_set real_read_fds;
    struct timeval tv;

    FD_ZERO(&real_read_fds);

    server->addToFdSet(&real_read_fds);
    int maxFd = 128;

    for(;;)
    {
	memcpy(&read_fds, &real_read_fds, sizeof(fd_set));

	tv.tv_sec = 0;
	tv.tv_usec = 500000;

	handleTimeouts(&tv);

	int ret_val = select(maxFd, &read_fds, 0, 0, &tv);

	if(ret_val < 0)
	{
	    cerr << "problem in select\n";
	}

	if(server->checkIfSet(&read_fds))
	{
	    ODEBUG(DEBUG_NTWK, 3, cout << "got network data\n";);
	    processPending();
	}

    }

    networkMutex->unlock();
}


Sptr<MgCommand> MgAgent::getNextMessage()
{

    XDEBUG(1, cerr << "Begin *** MgAgent::getNextMessage() ***" << endl;);
//    fetchNewData();
    queueMutex->lock();

//    flushMessages();

    Sptr<MgCommand> returnVal = 0;

    while(pendingMessages.begin() == pendingMessages.end())
    {
	msgPending->wait(queueMutex);
    }

    // at this point, there has to be something in the queue
    returnVal = pendingMessages.front();
    pendingMessages.pop_front();
    if(pendingMessages.begin() == pendingMessages.end())
    {
	msgWaiting = false;
    }

    queueMutex->unlock();

    DEBUG(cout << "returning:  tid: " << returnVal->getDataPtr()->getTransactionId() << endl;);

    XDEBUG(1, cerr << "End *** MgAgent::getNextMessage() ***" << endl;);

    return returnVal;

}

bool MgAgent::send(MgCommand& command, MgAgentHandle handle)
{
    assert(useAsync == false);
    XDEBUG(1, cerr << "Begin ***  MgAgent::send *** " << endl;);

    assert(command.getDataPtr());
    command.getDataPtr()->readyToSend();

    MgMessageStuff original;
    original.command = command.copy();

    queueMutex->lock();

    XDEBUG(2, cerr << "adding tid: \n");
    XDEBUG(2, cerr << command.getDataPtr()->getTransactionId() << "\n");

    DEBUG(cout << "Adding TID: " << original.command->getDataPtr()->getTransactionId() << endl;);
    
    NetworkAddress targetAddr = (callAgentList[handle])->getremoteAddr();
    UniqueId uID( targetAddr.getHostName(), targetAddr.getPort(),original.command->getDataPtr()->getTransactionId() );

    
    sentMessages[ uID ] = &original;
   
    original.command->getDataPtr()->send(*callAgentList[handle]);

    // set up the timeout
   
    timeoutMsgs.insert( uID, MgTimeout);
    original.nodeInTimer = timeoutMsgs.getlastInserted();
   

    while(original.ready == false) {
	original.responsePending->wait(queueMutex);
    }
    
    queueMutex->unlock();

    Sptr<MgcpResponse> x = new MgcpResponse(*(dynamic_cast<MgcpResponse*> (original.command->getResponsePtr())));
;
    //  x.dynamicCast(original.command->getResponseSptr());
    command.setResponseSptr(x);
    

    XDEBUG(1, cerr << "End *** MgAgent::send *** " << endl;);

    return true;
}



bool MgAgent::reply(const Sptr<MgCommand>& command, MgAgentHandle handle)
{
    assert(useAsync == false);
    command->getResponsePtr()->
	setTransactionId(command->getDataPtr()->getTransactionId());

    DEBUG(cout << "reply:  tid: " << command->getDataPtr()->getTransactionId() << endl;);

    command->getResponsePtr()->readyToSend();
    command->getResponsePtr()->send(*callAgentList[handle]);

    // save the copy of the sent response in sentResponses
    NetworkAddress remoteAddr = callAgentList[handle]->getremoteAddr();

    int transID = command->getDataPtr()->getTransactionId();
    UniqueId uID( remoteAddr.getHostName(), remoteAddr.getPort(), transID);

    Sptr<MgMessageStuff> original = new MgMessageStuff;
    //original->heapAlloc = true;
    //original->command = command->copy();
    original->command = command;
    original->command->setResponseSptr( command->getResponseSptr() );
    //original->setconfirmed(false);

    
    queueMutex->lock();
    sentResponses[uID] = original;

    // set up the time out
    timeoutResp.insert( uID, LONG_TIMER);


    //   MgCommandType cmdType = command->getType();
    MgcpResponseCode respCode = (dynamic_cast<MgcpResponse*>(command->getResponsePtr()))->getResponseCode();

    /// make sure it is NOT a provisional response before we take it out of TransWaiting
    /// because if it is a provisional response, it should remain in TransWaiting until the final
    /// response comes in
    if ( respCode != ReasonTransactionInProgress )
    {
      // take the transaction out of the list of outstanding commands in TransWaiting
      map<UniqueId, int>::iterator p = TransWaiting.find( uID );

      if(p != TransWaiting.end() )
      {   
	  TransWaiting.erase( uID );	    
      }  

/* This chunk of code needs to re-worked at in sentResponses and timeoutResp
   keep this here for future reference, faking a sentMessage may interfere with
   a real sentMessage with same transaction ID
//#ifdef NCS_PROFILE
      // if it is a final response for CRCX or MDCX with empty K: line
      // it is asking the call agent of acknowledgement of receiving this final response
      // and it is subject to retransmission if the call agent fails to acknowledge
      // therefore we need to insert it into sentMessages for timeout and retransmission
      if (( respCode == ReasonTransactionExecuted ) &&
	   (( cmdType == MgCreateConnectionType ) ||
	    ( cmdType == MgModifyConnectionType )))
      {

	Sptr<MgCreateConnection> crcxPtr;
	Sptr<MgModifyConnection> mdcxPtr;
	bool containEmptyK = false;

	if ( cmdType == MgCreateConnectionType )
	{
	    crcxPtr.dynamicCast( command );
	    if ( crcxPtr != 0 )
	      containEmptyK = crcxPtr->containEmptyResponseAck();
	}
	else 
	{
	    mdcxPtr.dynamicCast( command );
	    if ( mdcxPtr != 0 )
	      containEmptyK = mdcxPtr->containEmptyResponseAck();
	}

	if ( containEmptyK == true )
	{
	// fake a command for retransmission in sentMessages
	    MgMessageStuff* original = new MgMessageStuff;
	    original->heapAlloc = true;
	    original->command = command->copy();
	    // save the response message for later retransmission without composing
            // because composing a response message from request is funny
	    (original->command->getDataPtr())->setMessage((command->getResponsePtr())->getMessage());
	    sentMessages[ uID ] = original;
	    //    timeoutMsgs.insert( uID,MgTimeout );
	    // test only
	    timeoutMsgs.insert( uID, 4000 );
	    // end of test only
	    original->nodeInTimer = timeoutMsgs.getlastInserted();
	}
      }
//#endif // NCS_PROFILE
end of code reference */
    }

    queueMutex->unlock();

    return true;
}


Sptr<MgCommand> 
MgAgent::receive()
{
    assert(useAsync == false);
    DEBUG(cerr<< "Begin *** MgAgent::receive *** " << endl;);
    return getNextMessage();
}

///
bool MgAgent::asyncSend(MgCommand& command, MgAgentHandle handle)
{
    assert(useAsync == true);

    assert(command.getDataPtr());
    command.getDataPtr()->readyToSend();

    MgMessageStuff* original = new MgMessageStuff;
    original->heapAlloc = true;
    original->command = command.copy();

    queueMutex->lock();

    XDEBUG(2, cerr << "adding tid: \n");
    XDEBUG(2, cerr << command.getDataPtr()->getTransactionId() << "\n");

    DEBUG(cout << "Adding TID: " << original->command->getDataPtr()->getTransactionId() << endl;);
    
    NetworkAddress targetAddr = (callAgentList[handle])->getremoteAddr();
    UniqueId uID( targetAddr.getHostName(), targetAddr.getPort(),original->command->getDataPtr()->getTransactionId() );

    
    sentMessages[ uID ] = original;
    

    original->command->getDataPtr()->send(*callAgentList[handle]);

    
    timeoutMsgs.insert( uID,MgTimeout );
    original->nodeInTimer = timeoutMsgs.getlastInserted();
  

    queueMutex->unlock();

    return true;
}





///
bool MgAgent::asyncReply(const Sptr<MgCommand>& command, MgAgentHandle handle)
{
    assert(useAsync == true);
    command->getResponsePtr()->
	setTransactionId(command->getDataPtr()->getTransactionId());
    
    command->getResponsePtr()->readyToSend();
    command->getResponsePtr()->send(*callAgentList[handle]); 

    // save the copy of the sent response in sentResponses
    NetworkAddress remoteAddr = callAgentList[handle]->getremoteAddr();

    int transID = command->getDataPtr()->getTransactionId();
    UniqueId uID( remoteAddr.getHostName(), remoteAddr.getPort(), transID);
  
    Sptr<MgMessageStuff> original = new MgMessageStuff;
    //original->heapAlloc = true;
    //original->command = command->copy();
    original->command = command;
    original->command->setResponseSptr( command->getResponseSptr() );
    //original->setconfirmed(false);

    queueMutex->lock();
    sentResponses[uID] = original;

    // set up the time out
    timeoutResp.insert( uID, LONG_TIMER);

    MgcpResponseCode respCode = (dynamic_cast<MgcpResponse*>(command->getResponsePtr()))->getResponseCode();

    /// make sure it is NOT a provisional response before we take it out of TransWaiting
    /// because if it is a provisional response, it should remain in TransWaiting until the final
    /// response comes in
    if ( respCode != ReasonTransactionInProgress )
    {
    // take the transaction out of the list of outstanding commands in TransWaiting
      map<UniqueId, int>::iterator p = TransWaiting.find( uID );

      if(p != TransWaiting.end() )
      {   
	  TransWaiting.erase( uID );	    
      } 
    }

    queueMutex->unlock();

    return true;
}

///
Sptr<MgCommand> 
MgAgent::asyncReceive()
{
    assert(useAsync == true);
    return getNextMessage();
}

// this is the part that handles messages that have timed out and the like
void MgAgent::asyncProcess()
{
    setAsync();
    threadProcess();
}

    
void MgAgent::addToFdSet( fd_set* set )
{
    server->addToFdSet(set);
}

bool MgAgent::checkIfSet( fd_set* set )
{
    return server->checkIfSet(set);
}

bool MgAgent::checkMsgReady( fd_set* set )
{
    // do something useful here, like process all of the things that you can

    if(msgWaiting == false)
    {
	return false;
    }
    else
    {
	queueMutex->lock();
	msgWaiting = !(pendingMessages.begin() == pendingMessages.end());
	queueMutex->unlock();
    }
    return msgWaiting;
}

bool MgAgent::commandPending()
{
    return msgWaiting;
}

int MgAgent::getFd()
{
    return 10; // xxx
}


void MgAgent::processRangeInRespAck( const string& s, const NetworkAddress& srcAddr )
{
  //  split_t  test_K_vector;
  // split_t  test_num_vector;
    split_t  test_K_vector;
    split_t  test_num_vector;

    int start =0;
    int end = 0;
    bool isRange = false;

    /// get "K:" and the rest into test_K_vector
    // test_K_vector = split(s, ":");
    test_K_vector = split(s, ":");

    ///get the list of ranges if any
    if (test_K_vector.size() >= 1)  
      // test_num_vector = split(test_K_vector[1], ",");
      test_num_vector = split(test_K_vector[1], ",");

    if(test_num_vector.size() != 0)
    {
	for(unsigned int i = 0; i < test_num_vector.size(); i++)
	{
	  if (getRange(test_num_vector[i], &start, &end, &isRange))
	  {
	    if (isRange)
	    {	      
              for (int i=start; i<=end; i++)
	      {
                 Sptr<MgMessageStuff> original = 0;
		 UniqueId uID(srcAddr.getHostName(), srcAddr.getPort(), i );

		 queueMutex->lock();
		 map<UniqueId, Sptr<MgMessageStuff> >::iterator p = sentResponses.find( uID );

		 if(p != sentResponses.end() )
		 {
		     original = p->second;
		     if(original !=0 )
		     {
			 original->setconfirmed(true);
		     }
		 }
		 queueMutex->unlock();
	      }	      
	    }
	    else
	    {
	         Sptr<MgMessageStuff> original = 0;
		 UniqueId uID(srcAddr.getHostName(), srcAddr.getPort(), start );

		 queueMutex->lock();
		 map<UniqueId, Sptr<MgMessageStuff> >::iterator p = sentResponses.find( uID );

		 if(p != sentResponses.end() )
		 {
		     original = p->second;
		     if(original !=0 )
		     {
			 original->setconfirmed(true);
		     }
		 }
		 queueMutex->unlock();
	    }
	  }
	  
	  /// reset before use again
	  start = 0;
	  end = 0;
	  isRange=false;
	}
    }

    return;
}
   
