#ifndef MG_AGENT_HXX
#define MG_AGENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgAgent_hxx_Version = 
"$Id: MgAgent.hxx,v 1.4 2001/02/01 01:22:56 icahoon Exp $";



#include "Sptr.hxx"
#include "global.h"
#include "MgCommand.hxx"
#include <deque>
#include "VMutex.h"
#include "VCondition.h"
#include "Timer.h"
#include "MgcpCommand.hxx"
#include "support.hxx"

typedef int MgAgentHandle;
const MgAgentHandle MgAgentInvalid = -1;


class MgMessageStuff
{
public:
    MgMessageStuff() {
	ready = false;
	heapAlloc = false;
	responsePending = new VCondition;
	gettimeofday(&sentTime, 0);
	timeoutCount = 0;  
	lastTimeout = 0;
        confirmed = false;
    }
    ~MgMessageStuff()  { delete responsePending; }

    bool getconfirmed() {return confirmed;}
    void setconfirmed(bool b) { confirmed = b;}

    VCondition* responsePending;
    Sptr<MgCommand> command;    
    bool ready; 
    bool heapAlloc;
    struct timeval sentTime;
    long timeoutCount;
    long lastTimeout;
    bool confirmed;

    TimerObj<UniqueId> nodeInTimer;
};

/** This class represents a transport connection between the gateway
    and the agent from the gateway's perspective */

class MgAgent
{
public:
    /** this constructor does not take any outgoing agents initially --
	all of them must be constructed on the fly.  
        @param portLocal the local port to listen for the agent on.
    */
    MgAgent( int portLocal=DEFAULT_MGCP_PORT_FOR_GATEWAY );

    /** constructor.  
        @param address the address/hostname of the agent
        @param portRemote The port of the agent
        @param portLocal the local port to listen for the agent on.
    */
    MgAgent( const char* address, 
	     int portRemote=DEFAULT_MGCP_PORT_FOR_CALLAGENT,
	     int portLocal=DEFAULT_MGCP_PORT_FOR_GATEWAY );


    ///
    MgAgentHandle addAgent(const char* address, int portRemote=DEFAULT_MGCP_PORT_FOR_CALLAGENT);

    /// blocking send of a message
    bool send(MgCommand& command,  MgAgentHandle handle=0);
    /// blocking reply of a message
//    bool reply(MgCommand& command,  MgAgentHandle handle=0);
    /// blocking reply of a message
    bool reply(const Sptr<MgCommand>& command, MgAgentHandle handle=0);

    /// blocking receive of a message
    Sptr<MgCommand> receive();

    ///
    void setAsync() { useAsync = true; }

    ///
    void setSync() { useAsync = false; }

    /// non-blocking send of a message
    bool asyncSend(MgCommand& command, MgAgentHandle handle=0);

    /// non-blocking reply to a message
    bool asyncReply(const Sptr<MgCommand>& command, 
		    MgAgentHandle handle=0);
    /// non-blocking receive of a message
    Sptr<MgCommand> asyncReceive();

    /** process timeouts and resends if the non-blocking APIs are
	called */

    void asyncProcess(); 

    /** add the appropriate FDs to the fd_set for the stack (for
	select) */

    void addToFdSet( fd_set* set );

    /** check if any of the appropriate FDs are set for the stack
	(post-select) */

    bool checkIfSet( fd_set* set );
    /// check if any messages are waiting to be receive()'d.
    bool checkMsgReady( fd_set* set );

    ///
    bool debugFlag;

    ///

    bool commandPending();

    ///

    int getFd();

    /// private function gets data from the network (cleanly?)
    Sptr<MgCommand> getNextMessage();

    ///
    void flushMessages();

    ///
    void threadProcess();

    ///
    void processPending();

    ///
//    void dispatchMessage(MgMessageStuff* original);

    // API:ignore begin
    ///
    void handleTimeouts(struct timeval* tv);
    // API:ignore end

    

private:

	/* added this function */
    void SingleProcess(NetworkAddress sourceAddr, MgcpCommand* message);

    /** process response acknowledgement attribute and 
	set flags for confirmed responses in sentResponses
    */
    void processRangeInRespAck( const string& s, 
				const NetworkAddress& srcAddr );

   
    /// list of agents to transmit to
    vector<Sptr<MgcpTransmitter> > callAgentList;

    /// data for receiving from the call agent
    Sptr<MgcpReceiver> server;

    /// enqueueing thing

    deque< Sptr<MgCommand> > pendingMessages;

    /// a map of requests currently waiting for processing
    map< UniqueId, int > TransWaiting;

    /// true if a message is waiting
    bool msgWaiting;

    /// a response is waiting
    bool respWaiting;

    /// mutex for the queue
    VMutex* queueMutex;
    /// mutex for network
    VMutex* networkMutex;
    /// condition variable for a pending message
    VCondition* msgPending;
    /// condition variable for a response pending
    VCondition* respPending;

    /// sent message information
    /// map<int, MgMessageStuff*> sentMessages;
    map<UniqueId, MgMessageStuff* > sentMessages;

     /// time out system for sentMessages
    Timer<UniqueId> timeoutMsgs;

    /// sent responses
    map<UniqueId, Sptr<MgMessageStuff> > sentResponses;

    /// time out system for sentResponses
    Timer<UniqueId> timeoutResp;

    /// true if using async
    bool useAsync;

};

#endif // MG_AGENT_HXX
