
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgCommand_cxx_version  =
"$Id: MgCommand.cxx,v 1.51 2001/02/01 01:22:56 icahoon Exp $";

#include "global.h"
#include "MgCommand.hxx"
#include "MgcpCommand.hxx"
#include "MgcpParameter.hxx"
#include "SdpSession.hxx"


///
Sptr<MgRequestedEvent> getOneMgRequestedEvent(
    int index,
    MgcpParameterRequestedEvents* ptr
    );

///
Sptr<MgEvent> getOneMgEvent( MgcpGenericEvent& ge );

///
Sptr<MgRequestedEvent> getOneMgRequestedEvent(
    int index,
    MgcpParameterRequestedEvents* ptr
    )
{
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterRequestedEvent* event = ptr->getDecodedItem(index);

	MgcpParameterRequestedEventGeneric* p = dynamic_cast<MgcpParameterRequestedEventGeneric*>(event);

	MgRequestedEvent* rPtr = 0;
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    switch(ge.getPackageType())
	    {
#ifndef NCS_PROFILE
	    case    MgcpNoPackageType:
	    case    MgcpGenericMediaPackageType:
//		cout << "got Generic package\n";
		rPtr = new MgGenericPackageRequestedEvent(ge);
//		cout << "foo: " << rPtr->getPackageType() << "\n";
		break;
	    case    MgcpDTMFPackageType:
		rPtr = new MgDTMFPackageRequestedEvent(ge);
		break;
	    case    MgcpMFPackageType:
		rPtr = new MgMFPackageRequestedEvent(ge);
		break;
	    case    MgcpTrunkPackageType:
		rPtr = new MgTrunkPackageRequestedEvent(ge);
		break;
	    case    MgcpLinePackageType:
		rPtr = new MgLinePackageRequestedEvent(ge);
		break;
	    case    MgcpHandsetPackageType:
		rPtr = new MgHandsetPackageRequestedEvent(ge);
		break;
	    case    MgcpRTPPackageType:
		rPtr = new MgRTPPackageRequestedEvent(ge);
		break;
	    case    MgcpNetworkAccessServerPackageType:
		rPtr = new MgNetworkAccessServerPackageRequestedEvent(ge);
		break;
	    case    MgcpAnnouncementPackageType:
		rPtr = new MgAnnouncementPackageRequestedEvent(ge);
		break;
	    case    MgcpScriptPackageType:
		rPtr = new MgScriptPackageRequestedEvent(ge);
		break;
	    case    MgcpXVoiceMailPackageType:
		rPtr = new MgXVoiceMailPackageRequestedEvent(ge);
		break;
	    case    MgcpAllPackagesType:
	        rPtr = new MgAllPackagesRequestedEvent(ge);
		break;
#else
		/// NCS Profile types
	    case    MgcpNCSLinePackageType:
	        rPtr = new MgNCSLinePackageRequestedEvent(ge);
		break;
	    case    MgcpNCSAdsiPackageType:
	        rPtr = new MgNCSAdsiPackageRequestedEvent(ge);
		break;
	    case    MgcpNCSDefaultPackageType:
	        rPtr = new MgNCSLinePackageRequestedEvent(ge);
		break;
	    case    MgcpNCSAllPackagesType:
	        rPtr = new MgNCSAllPackagesRequestedEvent(ge);
		break;
	    default:
	        rPtr = 0;
		break;
#endif
	    }
	    if(rPtr)
	    {
		rPtr->addAction(p->getRequestedAction());
	    }
	    return rPtr;
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgRequestedEvent;
}

///
Sptr<MgEvent> getOneMgEvent( MgcpGenericEvent& ge )
{
    switch( ge.getPackageType() )
    {
#ifndef NCS_PROFILE
      case  MgcpNoPackageType:
      case  MgcpGenericMediaPackageType:
	    return new MgGenericPackageEvent(ge);
	    break;
      case  MgcpDTMFPackageType:
	    return new MgDTMFPackageEvent(ge);
	    break;
      case  MgcpMFPackageType:
	    return new MgMFPackageEvent(ge);
	    break;
      case  MgcpTrunkPackageType:
	    return new MgTrunkPackageEvent(ge);
	    break;
      case  MgcpLinePackageType:
	    return new MgLinePackageEvent(ge);
	    break;
      case  MgcpHandsetPackageType:
	    return new MgHandsetPackageEvent(ge);
	    break;
      case  MgcpRTPPackageType:
	    return new MgRTPPackageEvent(ge);
	    break;
      case  MgcpNetworkAccessServerPackageType:
	    return new MgNetworkAccessServerPackageEvent(ge);
	    break;
      case  MgcpAnnouncementPackageType:
	    return new MgAnnouncementPackageEvent(ge);
	    break;
      case  MgcpScriptPackageType:
	    return new MgScriptPackageEvent(ge);
	    break;
      case  MgcpXVoiceMailPackageType:
	    return new MgXVoiceMailPackageEvent(ge);
	    break;
      case  MgcpAllPackagesType:
	    return new MgAllPackagesEvent(ge);
	    break;
#else
      /// NCS Profile
      case  MgcpNCSLinePackageType:
        return new MgNCSLinePackageEvent(ge);
        break;
      case MgcpNCSAdsiPackageType:
        return new MgNCSAdsiPackageEvent(ge);
        break;
      case MgcpNCSDefaultPackageType:
        return new MgNCSLinePackageEvent(ge); // default package is Line package for NCS
        break;
      case MgcpNCSAllPackagesType:
        return new MgNCSAllPackagesEvent(ge);
        break;
#endif
      default:
        return 0;
        break;
   }
}


///
MgCommand::~MgCommand()
{

}

MgCommandType MgCommand::getType()
{
    return MgInvalidCommandType;
}

///
string MgEndpointConfiguration::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgEndpointConfiguration::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
/////
MgcpBearerInformation MgEndpointConfiguration::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
	(data->retrieve(MgcpParameterTypeBearerInformation));
    if(!ptr) { return MgcpBearerInformationEnumInvalid; }
    return ptr->getDecoded();
}

/////
string MgCreateConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgCreateConnection::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
void MgCreateConnection::setEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = new MgcpParameterResponseAcknowledgement(string(""));

    responseMsg->insert(ptr);
    return;
}

bool MgCreateConnection::containEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeResponseAcknowledgement));
    
    if ( ptr != 0 )
    {
        string s = ptr->getDecoded();
	if (s == "")
	  return( true );
    }

    return( false );
}

///
MgcpBearerInformation MgCreateConnection::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
	(data->retrieve(MgcpParameterTypeBearerInformation));
    if(!ptr) { return MgcpBearerInformationEnumInvalid; }
    return ptr->getDecoded();
}
/////
string MgCreateConnection::getCallId()
{
    MgcpParameterCallId* ptr;
    ptr = dynamic_cast<MgcpParameterCallId*>
	(data->retrieve(MgcpParameterTypeCallId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///
string MgCreateConnection::getRequestIdentifier() const
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///////
MgcpConnectionOptions MgCreateConnection::getLocalConnectionOptions()
{
    MgcpParameterLocalConnectionOptions* ptr;
    ptr = dynamic_cast<MgcpParameterLocalConnectionOptions*>
	(data->retrieve(MgcpParameterTypeLocalConnectionOptions));
    if(ptr)
    {
        return ptr->getDecoded();
    }
    else
    {
        return MgcpConnectionOptions();
    }
}

///
string MgCreateConnection::getConnectionMode()
{
    MgcpParameterConnectionMode* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionMode*>
	(data->retrieve(MgcpParameterTypeConnectionMode));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

//////
int MgCreateConnection::getNumRequestedEvents()
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


Sptr<MgRequestedEvent> MgCreateConnection::getItemRequestedEvents(int index)
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    return getOneMgRequestedEvent(index, ptr);
}
//
//////
int MgCreateConnection::getNumSignalRequests()
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgCreateConnection::getItemSignalRequests(int index)
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
///////
//string MgCreateConnection::getNotifiedEntity()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
string MgCreateConnection::getDigitMap()
{
    MgcpParameterDigitMap* ptr;
    ptr = dynamic_cast<MgcpParameterDigitMap*> 
	(data->retrieve(MgcpParameterTypeDigitMap));
    if(ptr)
    {
	return ptr->getDecoded();
    }
    else
    {
	return string("");
    }
}
///////
//string MgCreateConnection::getSecondEndpointId()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
//string MgCreateConnection::getQuarantineHandling()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
//////
int MgCreateConnection::getNumDetectEvents()
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgCreateConnection::getItemDetectEvents(int index)
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//

///
SdpSession MgCreateConnection::getRemoteConnectionDescriptor()
{
    MgcpParameterRemoteConnectionDescriptor* ptr;
    ptr = dynamic_cast<MgcpParameterRemoteConnectionDescriptor*>
        (data->retrieve(MgcpParameterTypeRemoteConnectionDescriptor));
    //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}

///
bool MgCreateConnection::setConnectionId(const string& parameter)
{
    Sptr<MgcpParameterConnectionId> ptr;

    ptr = new MgcpParameterConnectionId(parameter);

    responseMsg->insert(ptr);
    return true;
}

///
string MgCreateConnection::getConnectionId()
{
    Sptr<MgcpParameterConnectionId> ptr;

    ptr.dynamicCast(responseMsg->retrieveParam(MgcpParameterTypeConnectionId));

    if(ptr == static_cast<MgcpParameterConnectionId*>(0))
    {
	cerr << "empty connectionid\n";
	return string();
    }
    else 
    {
	return ptr->getDecoded();
    }
}

////
//MgCreateConnection::setSpecificEndpointId(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//



///
bool MgCreateConnection::setLocalConnectionDescriptor(const SdpSession& parameter)
{
    Sptr<MgcpParameterLocalConnectionDescriptor> ptr;

    ptr = new MgcpParameterLocalConnectionDescriptor(parameter);

    responseMsg->insert(ptr);
    return true;
}


SdpSession MgCreateConnection::getLocalConnectionDescriptor()
{
    MgcpParameterLocalConnectionDescriptor* ptr = 
	responseMsg->getLocalConnect();
    //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}


///
bool MgCreateConnection::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool MgCreateConnection::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}

///
string MgModifyConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgModifyConnection::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr= 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

void MgModifyConnection::setEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = new MgcpParameterResponseAcknowledgement(string(""));

    responseMsg->insert(ptr);
    return;
}


bool MgModifyConnection::containEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeResponseAcknowledgement));
    
    if ( ptr != 0 )
    {
        string s = ptr->getDecoded();
	if (s == "")
	  return( true );
    }

    return( false );
}


MgcpBearerInformation MgModifyConnection::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
	(data->retrieve(MgcpParameterTypeBearerInformation));
    if(!ptr) { return MgcpBearerInformationEnumInvalid; }
    return ptr->getDecoded();
}

string MgModifyConnection::getCallId()
{
    MgcpParameterCallId* ptr;
    ptr = dynamic_cast<MgcpParameterCallId*>
	(data->retrieve(MgcpParameterTypeCallId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgModifyConnection::getConnectionId()
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
	(data->retrieve(MgcpParameterTypeConnectionId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///
string MgModifyConnection::getRequestIdentifier() const
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///////
MgcpConnectionOptions MgModifyConnection::getLocalConnectionOptions()
{
    MgcpParameterLocalConnectionOptions* ptr;
    ptr = dynamic_cast<MgcpParameterLocalConnectionOptions*>
	(data->retrieve(MgcpParameterTypeLocalConnectionOptions));
    if(ptr)
    {
        return ptr->getDecoded();
    }
    else
    {
        return MgcpConnectionOptions();
    }
}

///////
string MgModifyConnection::getConnectionMode()
{
    MgcpParameterConnectionMode* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionMode*>
	(data->retrieve(MgcpParameterTypeConnectionMode));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

//////
int MgModifyConnection::getNumRequestedEvents()
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


Sptr<MgRequestedEvent> MgModifyConnection::getItemRequestedEvents(int index)
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    return getOneMgRequestedEvent(index, ptr);
}


//
//////
int MgModifyConnection::getNumSignalRequests()
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgModifyConnection::getItemSignalRequests(int index)
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//
///////
//string MgModifyConnection::getNotifiedEntity()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));//
//    return ptr->getDecoded();
//}
///////
string MgModifyConnection::getDigitMap()
{
    MgcpParameterDigitMap* ptr;
    ptr = dynamic_cast<MgcpParameterDigitMap*> 
	(data->retrieve(MgcpParameterTypeDigitMap));
    if(ptr)
    {
	return ptr->getDecoded();
    }
    else
    {
	return string("");
    }
}
///////
//string MgModifyConnection::getQuarantineHandling()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
//////
int MgModifyConnection::getNumDetectEvents()
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgModifyConnection::getItemDetectEvents(int index)
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//
///
SdpSession MgModifyConnection::getRemoteConnectionDescriptor()
{
    MgcpParameterRemoteConnectionDescriptor* ptr;
    ptr = dynamic_cast<MgcpParameterRemoteConnectionDescriptor*>
        (data->retrieve(MgcpParameterTypeRemoteConnectionDescriptor));
    //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}


///
bool MgModifyConnection::setLocalConnectionDescriptor(const SdpSession& parameter)
{
    Sptr<MgcpParameterLocalConnectionDescriptor> ptr;

    ptr = new MgcpParameterLocalConnectionDescriptor(parameter);

    data->insert(ptr);
    responseMsg->insert(ptr);
    return true;
}



///
bool MgModifyConnection::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool
MgModifyConnection::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}

//
//
/////
string MgDeleteConnectionAgent::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgDeleteConnectionAgent::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr= 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
/////
MgcpBearerInformation MgDeleteConnectionAgent::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
	(data->retrieve(MgcpParameterTypeBearerInformation));
    if(!ptr) { return MgcpBearerInformationEnumInvalid; }
    return ptr->getDecoded();
}
/////
string MgDeleteConnectionAgent::getCallId()
{
    MgcpParameterCallId* ptr;
    ptr = dynamic_cast<MgcpParameterCallId*>
	(data->retrieve(MgcpParameterTypeCallId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgDeleteConnectionAgent::getConnectionId()
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
	(data->retrieve(MgcpParameterTypeConnectionId));   
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
/////
string MgDeleteConnectionAgent::getRequestIdentifier() const
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
//////
int MgDeleteConnectionAgent::getNumRequestedEvents()
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


Sptr<MgRequestedEvent> MgDeleteConnectionAgent::getItemRequestedEvents(int index)
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    return getOneMgRequestedEvent(index, ptr);
}


//
//////
int MgDeleteConnectionAgent::getNumSignalRequests()
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgDeleteConnectionAgent::getItemSignalRequests(int index)
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//
///////
//string MgDeleteConnectionAgent::getNotifiedEntity()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
//string MgDeleteConnectionAgent::getReasonCode()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
string MgDeleteConnectionAgent::getDigitMap()
{
    MgcpParameterDigitMap* ptr;
    ptr = dynamic_cast<MgcpParameterDigitMap*> 
	(data->retrieve(MgcpParameterTypeDigitMap));
    if(ptr)
    {
	return ptr->getDecoded();
    }
    else
    {
	return string("");
    }
}
///////
//string MgDeleteConnectionAgent::getConnectionParameters()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
//string MgDeleteConnectionAgent::getQuarantineHandling()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
//////
int MgDeleteConnectionAgent::getNumDetectEvents()
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgDeleteConnectionAgent::getItemDetectEvents(int index)
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//////
//MgDeleteConnectionAgent::setConnectionParameters(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//

///
bool MgDeleteConnectionAgent::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool MgDeleteConnectionAgent::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}



/////
string MgNotificationRequest::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgNotificationRequest::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr= 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///
MgcpBearerInformation MgNotificationRequest::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
	(data->retrieve(MgcpParameterTypeBearerInformation));
    if(!ptr) { return MgcpBearerInformationEnumInvalid; }
    return ptr->getDecoded();
}

///
string MgNotificationRequest::getRequestIdentifier() const
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}


///
bool
MgNotificationRequest::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool
MgNotificationRequest::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}





//////
int MgNotificationRequest::getNumRequestedEvents()
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


Sptr<MgRequestedEvent> MgNotificationRequest::getItemRequestedEvents(int index)
{
    MgcpParameterRequestedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (data->retrieve(MgcpParameterTypeRequestedEvents));
    return getOneMgRequestedEvent(index, ptr);
}


//////
int MgNotificationRequest::getNumSignalRequests()
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgNotificationRequest::getItemSignalRequests(int index)
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (data->retrieve(MgcpParameterTypeSignalRequests));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
//
///////
//string MgNotificationRequest::getNotifiedEntity()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
///////
string MgNotificationRequest::getDigitMap()
{
    MgcpParameterDigitMap* ptr;
    ptr = dynamic_cast<MgcpParameterDigitMap*> 
	(data->retrieve(MgcpParameterTypeDigitMap));
    if(ptr)
    {
	return ptr->getDecoded();
    }
    else
    {
	return string("");
    }
}
///////
//string MgNotificationRequest::getQuarantineHandling()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
//////
int MgNotificationRequest::getNumDetectEvents()
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}
//
////
Sptr<MgEvent> MgNotificationRequest::getItemDetectEvents(int index)
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (data->retrieve(MgcpParameterTypeDetectEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgEvent;
}
////
MgNotify::MgNotify(const string& EndpointId, const string& RequestIdentifier, const MgNotificationEvent& ObservedEvents) : data(new MgcpCommandNotify)
{
    {
        Sptr<MgcpParameterEndpointId> ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
    {
	Sptr<MgcpParameterRequestId> ptr;

	ptr = new MgcpParameterRequestId(RequestIdentifier);

	data->insert(ptr);
    }
    {
	/// presume it's empty
	Sptr<MgcpParameterObservedEvents> ptr;

	ptr = new MgcpParameterObservedEvents;

	// now do something with the ObservedEvents

	MgcpParameterEventGeneric* x = new MgcpParameterEventGeneric(ObservedEvents.getEvent());
	    
	ptr->insert(Sptr<MgcpParameterEventGeneric>(x));

	data->insert(ptr);
    }
}

//
bool
MgNotify::setResponseAck(const string& parameter)
{
    Sptr<MgcpParameterResponseAcknowledgement> ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
    return true;
}


////
//MgNotify::setNotifiedEntity(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//

///
bool MgNotify::addObservedEvents(const MgNotificationEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterObservedEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeObservedEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterObservedEvents;
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = new MgcpParameterEventGeneric(parameter.getEvent());

    ptr->insert(x);

    data->insert(ptr);
    return true;
}

///
void MgNotify::clearObservedEvents()
{
    data->clear(MgcpParameterTypeObservedEvents);
}





int MgNotify::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}

string MgNotify::getResponseString()
{
    return responseMsg->getComment();
}


/**********************************************************************
 MgAuditEndpoint
**********************************************************************/
/////
string MgAuditEndpoint::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

string MgAuditEndpoint::getConnectionId()
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
	(data->retrieve(MgcpParameterTypeConnectionId)); 
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}


///
string MgAuditEndpoint::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr= 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///
int MgAuditEndpoint::getNumRequestedInfo()
{
    MgcpParameterRequestedInfo*  ptr = 0;
    //   ptr = dynamic_cast<MgcpParameterEndpointIdList*> (data->retrieve(MgcpParameterTypeEndpointIdList));
    
    ptr = dynamic_cast<MgcpParameterRequestedInfo*> 
              (data->retrieve(MgcpParameterTypeRequestedInfo));

    if(!ptr)
    {
	// no such parameter in the map
	return 0;
    }
    else
    {
	// get the number of EndpointIds in the EndpointIdList
	return ptr->getDecodedNum();
    } 
}

///
MgcpParameterType MgAuditEndpoint::getItemRequestedInfo( int index )
{
    MgcpParameterRequestedInfo* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedInfo*>
        (data->retrieve(MgcpParameterTypeRequestedInfo));

    if(!ptr)
    {
	// no such parameter at all, should first check the size !!!
	assert(0);
	return MgcpParameterTypeUnknownParameter;
    }
    else
    {
	return( ptr->getDecodedItem( index ) );	
    }
}

///
bool MgAuditEndpoint::setEndpointId(const string& parameter)
{
    MgcpParameterEndpointId* ptr;

    ptr = new MgcpParameterEndpointId(parameter);

    data->insert(ptr);
    return true;
}

bool MgAuditEndpoint::setConnectionId(const string& parameter)
{
    MgcpParameterConnectionId* ptr;

    ptr = new MgcpParameterConnectionId(parameter);

    responseMsg->insert(ptr);
    return true;
}



///
bool MgAuditEndpoint::setBearerInformation(const MgcpBearerInformation& parameter)
{
    Sptr<MgcpParameterBearerInformation> ptr;

    ptr = new MgcpParameterBearerInformation(parameter);
    
    responseMsg->insert(ptr);
    return true;
}
 

bool MgAuditEndpoint::setNumEndPoints(int parameter)
{
    MgcpParameterNumEndPoints* ptr;
    
    ptr = new MgcpParameterNumEndPoints(parameter);
    
    responseMsg->insert(ptr);
    return true;
}
      

int MgAuditEndpoint::getMaxEndPointIds()
{
    MgcpParameterMaxEndPointIds* ptr= 0;
    ptr = dynamic_cast<MgcpParameterMaxEndPointIds*>
	(data->retrieve(MgcpParameterTypeMaxEndPointIds));
    if(!ptr) { return 0; }
    return ptr->getDecoded();
}


bool 
MgAuditEndpoint::setCapabilities(MgcpParameterCapabilities& cap)
{
    MgcpParameterCapabilities *ptr;
    
    ptr = new MgcpParameterCapabilities(cap);

    responseMsg->insert(ptr);
    return true;
}


bool 
MgAuditEndpoint::setConnectionParameters(MgcpParameterConnectionParameters& cp)
{
    MgcpParameterConnectionParameters *ptr;
    
    ptr = new MgcpParameterConnectionParameters(cp);
    
    responseMsg->insert(ptr);
    return true;
}



/// add EndpointIds to response message sent to call agent
bool MgAuditEndpoint::addToEndpointIdList( const string& s )
{
     /// presume it's empty
    Sptr<MgcpParameterEndpointIdList> ptr;

    //   ptr.dynamicCast(data->retrieveParam( MgcpParameterTypeEndpointIdList ));
    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam( MgcpParameterTypeEndpointIdList ));

    if(!ptr)
    {
	ptr = new MgcpParameterEndpointIdList;
	
	(this->getResponseSptr())->insert(ptr);
    }
    
    // now insert the EndpointId to the list
     
    Sptr<MgcpParameterEndpointId>  x = new MgcpParameterEndpointId( s );

    return ( ptr->insert( x ) );
}


bool MgAuditEndpoint::addtoConnectionIdList( const string& s )
{
    Sptr<MgcpParameterConnectionId> ptr;

    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam( MgcpParameterTypeConnectionId));

    if( !ptr )
    {
        ptr = new MgcpParameterConnectionId;
      
        (this->getResponseSptr())->insert(ptr);

    }

    string* temp = new string(s);
    return( ptr->insert(*temp) );

	//   return( ptr->insert(s) );    
}    


///
bool MgAuditEndpoint::addRequestedEvents(const MgRequestedEvent& parameter)
{
    
    Sptr<MgcpParameterRequestedEvents> ptr;

    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam(MgcpParameterTypeRequestedEvents));
    
	// ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeRequestedEvents));
   
   if(!ptr)
   {
       ptr = new MgcpParameterRequestedEvents;

       (this->getResponseSptr())->insert(ptr);
   }
  
   MgcpGenericEvent event = parameter.getEvent();
   Sptr<MgcpParameterRequestedEventGeneric> r = new MgcpParameterRequestedEventGeneric(event);
   
   return ( ptr->insert(r) );    
} 

///
//MgAuditEndpoint::clearRequestedEvents()
//{
//}
//


///
bool MgAuditEndpoint::addSignalRequests(const MgSignalEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterSignalRequests> ptr;
   
    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam( MgcpParameterTypeSignalRequests ));

    if(!ptr)
    {
	ptr = new MgcpParameterSignalRequests;

	this->getResponseSptr()->insert(ptr);
    }
    
    // now do something with the Signal Requests
    
    MgcpParameterSignalGeneric* x = 
	new MgcpParameterSignalGeneric(parameter.getEvent());
	    
    ptr->insert(x);

    return true;
}



///
bool MgAuditEndpoint::addEventStates(const MgSignalEvent& parameter)
{
    Sptr<MgcpParameterEventStates> ptr;
    
    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam(MgcpParameterTypeEventStates));

   if(!ptr)
   {
       ptr = new MgcpParameterEventStates;

       this->getResponseSptr()->insert(ptr);
   }
  
   MgcpParameterSignalGeneric* x = new MgcpParameterSignalGeneric(parameter.getEvent());

   ptr->insert(x);
   
   return true;
 }
///
void MgAuditEndpoint::clearSignalRequests()
{
    this->getResponseSptr()->clear( MgcpParameterTypeSignalRequests );
}


bool 
MgAuditEndpoint::addObservedEvents(const MgNotificationEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterObservedEvents> ptr;

    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam(MgcpParameterTypeObservedEvents));	

// ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeObservedEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterObservedEvents;

        (this->getResponseSptr())->insert(ptr);
    }
    
    // now do something with the Observed Events... insert them.
    
   Sptr<MgcpParameterEventGeneric> x = new MgcpParameterEventGeneric(parameter.getEvent());
     
   return ( ptr->insert(x) ); 
	  
}


///
void 
MgAuditEndpoint::clearObservedEvents()
{
    data->clear(MgcpParameterTypeObservedEvents);
}

///
bool MgAuditEndpoint::setQuarantineHandling(
    MgcpNotificationHandling handling, 
    MgcpQuarantineTreatment treatment
    )
{
    Sptr<MgcpParameterQuarantineHandling> ptr = 0;
    ptr = new MgcpParameterQuarantineHandling(handling, treatment);
	
    this->getResponseSptr()->insert(ptr);
    return( true );
}


///
bool MgAuditEndpoint::addDetectEvents(const MgRequestedEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterDetectEvents> ptr;
   
    ptr.dynamicCast(((MgcpResponse*)(this->getResponsePtr()))->retrieveParam( MgcpParameterTypeDetectEvents ));

    if(!ptr)
    {
	ptr = new MgcpParameterDetectEvents;

	this->getResponseSptr()->insert(ptr);
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = 
	new MgcpParameterEventGeneric(parameter.getEvent());
	    
    ptr->insert(x);
    return true;
}

///
void MgAuditEndpoint::clearDetectEvents()
{
    this->getResponseSptr()->clear( MgcpParameterTypeDetectEvents );
}


//
bool
MgAuditEndpoint::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool
MgAuditEndpoint::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}

///
bool MgAuditEndpoint::setDigitMap(const string& parameter)
{
    Sptr<MgcpParameterDigitMap> ptr;
    
    ptr = new MgcpParameterDigitMap(parameter);

    responseMsg->insert(ptr);
    return true;
}


bool MgAuditEndpoint::setNotifiedEntity(const string& parameter)
{
    Sptr<MgcpParameterNotifiedEntity> ptr;
    
    ptr = new MgcpParameterNotifiedEntity(parameter);

    responseMsg->insert(ptr);
    return true;
}


bool MgAuditEndpoint::setRequestId(const string& parameter)
{
    Sptr<MgcpParameterRequestId> ptr;
    
    ptr = new MgcpParameterRequestId(parameter);

    responseMsg->insert(ptr);
    return true;
}



///
//MgAuditEndpoint::setRequestedInfo(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//
//////
//MgAuditEndpoint::setQuarantineHandling(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//
//////
//MgAuditEndpoint::addDetectEvents(const MgcRequestedEvent& parameter)
//{
//}
//
////
//MgAuditEndpoint::clearDetectEvents()
//{
//}
//
/////

/********************************************************
MgAuditConnection
**********************************************************/
string MgAuditConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgAuditConnection::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}
///
string MgAuditConnection::getConnectionId()
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
	(data->retrieve(MgcpParameterTypeConnectionId)); 
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

int MgAuditConnection::getNumConnectionId()
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
        (data->retrieve(MgcpParameterTypeConnectionId));

    if(!ptr)
    {
	   // no such parameter at all, should first check the size !!!
	   assert(0);
       //  return MgcpParameterTypeUnknownParameter;
    }
    else
    {
	      return( ptr->getDecodedNum() );	
    }
}  


string MgAuditConnection::getConnectionIdItem(int i)
{
    MgcpParameterConnectionId* ptr;
    ptr = dynamic_cast<MgcpParameterConnectionId*>
        (data->retrieve(MgcpParameterTypeConnectionId));

    if(!ptr)
    {
	   // no such parameter at all, should first check the size !!!
	   assert(0);
       return string(""); 
	   //  return MgcpParameterTypeUnknownParameter;
    }
    else
    {
	      return( ptr->getDecodedItem(i) );	
    }
}  

///
int MgAuditConnection::getNumRequestedInfo()
{
    MgcpParameterRequestedInfo*  ptr = 0;
    
    ptr = dynamic_cast<MgcpParameterRequestedInfo*> 
              (data->retrieve(MgcpParameterTypeRequestedInfo));

    if(!ptr)
    {
	// no such parameter in the map
	return 0;
    }
    else
    {
	// get the number of EndpointIds in the EndpointIdList
	return ptr->getDecodedNum();
    } 
}

///
MgcpParameterType MgAuditConnection::getItemRequestedInfo( int index )
{
    MgcpParameterRequestedInfo* ptr;
    ptr = dynamic_cast<MgcpParameterRequestedInfo*>
        (data->retrieve(MgcpParameterTypeRequestedInfo));

    if(!ptr)
    {
	// no such parameter at all, should first check the size !!!
	assert(0);
	return MgcpParameterTypeUnknownParameter;
    }
    else
    {
	return( ptr->getDecodedItem( index ) );	
    }
}
////
bool MgAuditConnection::setCallId(const string& parameter)
{
    Sptr<MgcpParameterCallId> ptr;

    ptr = new MgcpParameterCallId(parameter);

    responseMsg->insert(ptr);
    return true;
}

///
bool MgAuditConnection::setNotifiedEntity(const string& parameter)
{
    Sptr<MgcpParameterNotifiedEntity> ptr;
    
    ptr = new MgcpParameterNotifiedEntity(parameter);

    responseMsg->insert(ptr);
    return true;
}

////
bool MgAuditConnection::setLocalConnectionOptions(const MgcpConnectionOptions& parameter)
{
    MgcpParameterLocalConnectionOptions* ptr;

    ptr = new MgcpParameterLocalConnectionOptions(parameter);

    responseMsg->insert(ptr);
    return true;
}

//////
bool MgAuditConnection::setConnectionMode(const string& parameter)
{
    MgcpParameterConnectionMode* ptr;

    ptr = new MgcpParameterConnectionMode(parameter);

    responseMsg->insert(ptr);
    return true;
}

//
//
//////
bool MgAuditConnection::setConnectionParameters(const MgcpParameterConnectionParameters& parameter)
{
    MgcpParameterConnectionParameters* ptr;

    ptr = new MgcpParameterConnectionParameters(parameter);

    responseMsg->insert(ptr);
    return true;
}


///
bool MgAuditConnection::setRemoteConnectionDescriptor(const SdpSession& parameter)
{
    MgcpParameterRemoteConnectionDescriptor* ptr;

    ptr = new MgcpParameterRemoteConnectionDescriptor(parameter);

    responseMsg->insert(ptr);
    return true;
}

//
//
//////
bool MgAuditConnection::setLocalConnectionDescriptor(const SdpSession& parameter)
{
    MgcpParameterLocalConnectionDescriptor* ptr;

    ptr = new MgcpParameterLocalConnectionDescriptor(parameter);

    responseMsg->insert(ptr);
    return true;
}


///
bool
MgAuditConnection::setResponseCode(const MgcpResponseCode& code)
{
    responseMsg->setResponseCode(code);
    return true;
}


///
bool
MgAuditConnection::setResponseString(string response)
{
    responseMsg->setComment(response);
    return true;
}
///




int MgDeleteConnectionGw::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}

string MgDeleteConnectionGw::getResponseString()
{
    return responseMsg->getComment();
}


///
bool
MgDeleteConnectionGw::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
    return true;
}

 
MgRestartInProgress::MgRestartInProgress(const string& EndpointId, const string& RestartMethod, const string& RestartDelay) : data(new MgcpCommandRestartInProgress)
{
    {
        Sptr<MgcpParameterEndpointId> ptr;
	
	ptr = new MgcpParameterEndpointId(EndpointId);

	data->insert(ptr);
    }

    // xxx -- need to add stuff for restartMethod and restartDelay

}

int MgRestartInProgress::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}

string MgRestartInProgress::getResponseString()
{
    return responseMsg->getComment();
}
