#ifndef MG_COMMAND_HXX
#define MG_COMMAND_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgCommand_hxx_Version = 
"$Id: MgCommand.hxx,v 1.5 2001/01/26 02:45:48 liuhong Exp $";


#include "global.h"
#include "MgTypes.hxx"
#include "MgPackage.hxx"
#include "MgcEvent.hxx"
#include "MgcpCommand.hxx"
#include "Sptr.hxx"
#include "MgcpConnectionOptions.hxx"

/** This class is the base class for all commands sent and received on
   the Gateway side.
*/
   
class MgCommand
{
public:
    /// Create an empty message
    MgCommand() : responseMsg(new MgcpResponse) {}

    /// Create a command in response to a previous message 
    MgCommand(MgcpCommand* x) { 
        MgcpResponse* p = dynamic_cast<MgcpResponse*>(x);
        if(p) {responseMsg = p;}
    };

    /// Delete the command and the response it contains 
    virtual ~MgCommand();

    virtual MgCommand* copy() {
	return new MgCommand(*this);
    };

    /// get the type of the command
    virtual MgCommandType getType();

    /// set the response error code 
//    bool setResponseCode(MgcpResponseCode code) {assert(0/*need to implement*/);};

    /// Set the string describing the error that occcured
//    bool setResponseString(string response);

    /// Get a pointer to the raw text of the message
    virtual MgcpCommand* getDataPtr() { return 0; }

    /// Get a point to the response message
        MgcpCommand* getResponsePtr() { return responseMsg.getPtr(); }

    /// Get a pointer to the response message
	// Sptr<MgcpCommand> getResponseSptr() { return responseMsg; }
	Sptr<MgcpResponse> getResponseSptr() { return responseMsg; }

    /// Set the response message 
        void setResponsePtr(MgcpResponse* p) { responseMsg = p; }

    /// Set the response message 
    void setResponseSptr(const Sptr<MgcpResponse>& p) { responseMsg = p; }

protected:
    /// Copy of the response message 
    Sptr<MgcpResponse> responseMsg;

};

/// Endpoint configuration message received by the gateway sideclass MgEndpointConfiguration : public MgCommand

class MgEndpointConfiguration: public MgCommand
{
public: 
    /// Create a message to an specific endpoint
//    MgEndpointConfiguration(const string& EndpointId);

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgEndpointConfiguration(const MgEndpointConfiguration& x) : MgCommand(x), data(x.data)
	{}
    MgEndpointConfiguration(MgcpCommand* x) { 
        MgcpCommandEndpointConfiguration* p = dynamic_cast<MgcpCommandEndpointConfiguration*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgEndpointConfiguration(*this);
    };

    /// get the type of the command
    MgCommandType getType() { return MgEndpointConfigurationType; } ;

    /// get the endpoint id send by the gateway
    string getEndpointId();

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    string getResponseAck();

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }

    ///
    MgcpBearerInformation getBearerInformation();

private:
    /// data for the endpoint configuration
    Sptr<MgcpCommandEndpointConfiguration> data;

};


/// this object represents a create connection message received by the gateway 
class MgCreateConnection : public MgCommand
{
public: 
    /// create a connection on the given endpoint. Set the callID and mode
//    MgCreateConnection(const string& EndpointId, const string& CallId);

    MgCreateConnection(const MgCreateConnection& x) : MgCommand(x), data(x.data) { }

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgCreateConnection(MgcpCommand* x) { 
        MgcpCommandCreateConnection* p = dynamic_cast<MgcpCommandCreateConnection*>(x);
        if(p) {data = p;}
    };

    ///
    MgCreateConnection() : MgCommand() { 
       MgcpCommandCreateConnection* p = new MgcpCommandCreateConnection;
        if(p) {data = p;}     
    }

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgCreateConnection(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgCreateConnectionType; } ;

    /// Get endpoint that created the notification 
    string getEndpointId();

    ///
    int getTransactionId() { return( data->getTransactionId() ); }

    ///
    void setTransactionId(int x) { 
      data->setTransactionId(x);
      responseMsg->setTransactionId(x);
    }

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    /// set a empty response ack parameter for final response if 
    /// a provisional response was sent out earlier
    void setEmptyResponseAck();

    bool containEmptyResponseAck();

    ///
    MgcpBearerInformation getBearerInformation();

    /// Find out the call id 
    string getCallId();

    /// get the request identifier given in a previous message 
    string getRequestIdentifier() const;

    ///
    MgcpConnectionOptions getLocalConnectionOptions();

    /// get the connection mode of the connection
    string getConnectionMode();

    /// get the number of requested events in this message
    int getNumRequestedEvents();

    /// get the i'th event in this message
    Sptr<MgRequestedEvent> getItemRequestedEvents(int index);

    /// get the number of signalled events in this message
    int getNumSignalRequests();

    /// get the i'th signal in this message
    Sptr<MgEvent> getItemSignalRequests(int index);

    /// get who the notification is being sent to
//    string getNotifiedEntity();

    ///
    string getDigitMap();

    ///
//    string getSecondEndpointId();

    ///
//    string getQuarantineHandling();

    /// get the number of DetectEvents in this message
    int getNumDetectEvents();

    /// get the i'th DetectEvent in this message
    Sptr<MgEvent> getItemDetectEvents(int index);

    /// Get protocol informaiton for remote side of connection
    SdpSession getRemoteConnectionDescriptor();

    /// specify the connection id 
    bool setConnectionId(const string& parameter);

    string getConnectionId();

    ///
//    setSpecificEndpointId(const string& parameter);

    /// set the Local connection descriptor
    bool setLocalConnectionDescriptor(const SdpSession& parameter);

    SdpSession getLocalConnectionDescriptor();

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// get the response code
    MgcpResponseCode getResponseCode() {
          return responseMsg->getResponseCode();
    }

    /// set the description of any response error
    bool setResponseString(string response);

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:
    /// Holds the data for this command
    Sptr<MgcpCommandCreateConnection> data;

};



/// message received by the gateway requesting that a connection be modified  
class MgModifyConnection : public MgCommand
{
public: 
//    MgModifyConnection(const string& EndpointId, const string& CallId, const string& ConnectionId);

    ///
    MgModifyConnection(const MgModifyConnection& x) : MgCommand(x), data(x.data) {};

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgModifyConnection(MgcpCommand* x) { 
        MgcpCommandModifyConnection* p = dynamic_cast<MgcpCommandModifyConnection*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgModifyConnection(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgModifyConnectionType; } ;

    /// Get endpoint that created the notification 
    string getEndpointId();

    ///
    int getTransactionId() { return( data->getTransactionId() ); }

    ///
    void setTransactionId(int x) { 
      data->setTransactionId(x);
      responseMsg->setTransactionId(x);
    }

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    /// set a empty response ack parameter for final response if 
    /// a provisional response was sent out earlier
    void setEmptyResponseAck();

    bool containEmptyResponseAck();

    ///
    MgcpBearerInformation getBearerInformation();

    /// Find out the call id 
    string getCallId();

    /// get the connection id
    string getConnectionId();

    /// get the request identifier given in a previous message 
    string getRequestIdentifier() const;

    ///
    MgcpConnectionOptions getLocalConnectionOptions();

    /// get the connection mode of the connection
    string getConnectionMode();

    /// get the number of requested events in this message
    int getNumRequestedEvents();

    /// get the i'th event in this message
    Sptr<MgRequestedEvent> getItemRequestedEvents(int index);

    /// get the number of signalled events in this message
    int getNumSignalRequests();

    /// get the i'th signal in this message
    Sptr<MgEvent> getItemSignalRequests(int index);

    /// get who the notification is being sent to
//    string getNotifiedEntity();

    ///
    string getDigitMap();

    ///
//    string getQuarantineHandling();

    /// get the number of DetectEvents in this message
    int getNumDetectEvents();

    /// get the i'th DetectEvent in this message
    Sptr<MgEvent> getItemDetectEvents(int index);

    /// Get protocol informaiton for remote side of connection
    SdpSession getRemoteConnectionDescriptor();

    /// set the Local connection descriptor
    bool setLocalConnectionDescriptor(const SdpSession& parameter);

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// get the response code
    MgcpResponseCode getResponseCode() {
          return responseMsg->getResponseCode();
    }

    /// set the description of any response error
    bool setResponseString(string response);

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:

    /// Holds the data for this command
    Sptr<MgcpCommandModifyConnection> data;

};

/// message received by the gateway requesting that a connection be closed  
class MgDeleteConnectionAgent : public MgCommand
{
public: 
    ///
//    MgDeleteConnectionAgent(const string& EndpointId);

    ///
    MgDeleteConnectionAgent(const MgDeleteConnectionAgent& x) : MgCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgDeleteConnectionAgent(MgcpCommand* x) { 
        MgcpCommandDeleteConnectionAgent* p = dynamic_cast<MgcpCommandDeleteConnectionAgent*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgDeleteConnectionAgent(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgDeleteConnectionAgentType; } ;

    /// Get endpoint that created the notification 
    string getEndpointId();

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    ///
    MgcpBearerInformation getBearerInformation();

    /// Find out the call id 
    string getCallId();

    /// find out the connection ID
    string getConnectionId();

    /// get the request identifier given in a previous message 
    string getRequestIdentifier() const;

    /// get the number of requested events in this message
    int getNumRequestedEvents();

    /// get the i'th event in this message
    Sptr<MgRequestedEvent> getItemRequestedEvents(int index);

    /// get the number of signalled events in this message
    int getNumSignalRequests();

    /// get the i'th signal in this message
    Sptr<MgEvent> getItemSignalRequests(int index);

    /// get who the notification is being sent to
//    string getNotifiedEntity();

    ///
//    string getReasonCode();

    ///
    string getDigitMap();

    ///
//    string getConnectionParameters();

    ///
//    string getQuarantineHandling();

    /// get the number of DetectEvents in this message
    int getNumDetectEvents();

    /// get the i'th DetectEvent in this message
    Sptr<MgEvent> getItemDetectEvents(int index);

    ///
//    bool setConnectionParameters(const string& parameter);

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// set the description of any response error
    bool setResponseString(string response);

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:

    /// Holds the data for this command
    Sptr<MgcpCommandDeleteConnectionAgent> data;

};


/// message received by the gateway that indicates which events to look for and which signals to send to the hardware
class MgNotificationRequest : public MgCommand
{
public: 
    /// construct this message
//    MgNotificationRequest() : data(new MgcpCommandNotificationRequest)  {};

    ///
    MgNotificationRequest(const MgNotificationRequest& x ) : MgCommand(x), data(x.data)  {};

    ///
//    MgNotificationRequest(const string& EndpointId);

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgNotificationRequest(MgcpCommand* x) { 
        MgcpCommandNotificationRequest* p = dynamic_cast<MgcpCommandNotificationRequest*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgNotificationRequest(*this);
    };

    ///
    /// Get the type of the command
    MgCommandType getType() { return MgNotificationRequestType; } ;

    ///
    /// Get endpoint that created the notification 
    string getEndpointId();

    ///
    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    ///
    MgcpBearerInformation getBearerInformation();

    /// get the request identifier given in a previous message 
    string getRequestIdentifier() const;

    /// get the number of requested events in this message
    int getNumRequestedEvents();

    /// get the i'th event in this message
    Sptr<MgRequestedEvent> getItemRequestedEvents(int index);

    /// get the number of signalled events in this message
    int getNumSignalRequests();

    /// get the i'th signal in this message
    Sptr<MgEvent> getItemSignalRequests(int index);

    /// get who the notification is being sent to
//    string getNotifiedEntity();

    ///
    string getDigitMap();

    ///
//    string getQuarantineHandling();

    /// get the number of DetectEvents in this message
    int getNumDetectEvents();

    /// get the i'th DetectEvent in this message
    Sptr<MgEvent> getItemDetectEvents(int index);

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// set the description of any response error
    bool setResponseString(string response);

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:
    /// Holds the data for this command
    Sptr<MgcpCommandNotificationRequest> data;


};


/// message sent to the agent indicating that a waited-for event has occured
class MgNotify : public MgCommand
{
public: 
    /// construct a notification message to be sent to the agent
    MgNotify(const string& EndpointId, const string& RequestIdentifier, const MgNotificationEvent& ObservedEvents);


    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgNotify(MgcpCommand* x) { 
        MgcpCommandNotify* p = dynamic_cast<MgcpCommandNotify*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgNotify(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgNotifyType; };

    ///
    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    bool setResponseAck(const string& parameter);

    ///
//    bool setNotifiedEntity(const string& parameter);

    ///
    bool addObservedEvents(const MgNotificationEvent& parameter);

    /// Clear the list of observed events that have been added to this message
    void clearObservedEvents();

    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }
private:
    /// Holds the data for this command
    Sptr<MgcpCommandNotify> data;

};


/// message receive by the gateway requesting all info about an endpoint
class MgAuditEndpoint : public MgCommand
{
public: 
    ///
//    MgAuditEndpoint(const string& EndpointId);

    MgAuditEndpoint(const MgAuditEndpoint& x) : MgCommand(x), data(x.data) {};

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgAuditEndpoint(MgcpCommand* x) { 
        MgcpCommandAuditEndpoint* p = dynamic_cast<MgcpCommandAuditEndpoint*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgAuditEndpoint(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgAuditEndpointType; } ;

    /// Get endpoint that created the notification 
    string getEndpointId();

    /// get the connection id of this message
    string getConnectionId();

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    /// get the number of items in RequestedInfo
    int getNumRequestedInfo();

    /// get the i'th item in RequestedInfo
    MgcpParameterType getItemRequestedInfo( int index );    

    /// set the endpoint id for this message
    bool setEndpointId(const string& parameter);

     /// specify the connection id 
    bool setConnectionId(const string& parameter);
    
    ///
    bool setBearerInformation(const MgcpBearerInformation& parameter);

    /// don't need and it's wrong, should remove, dup of setRequestId
    //    bool setRequestIdentifier(const string& parameter);

    /// don't need, should remove
    //  bool setLocalConnectionOptions(const MgcpConnectionOptions& parameter); 

    ///
    bool addToEndpointIdList( const string& parameter );
  
    ///
    bool addRequestedEvents(const MgRequestedEvent& parameter);

    ///add to connection Id List
    bool addtoConnectionIdList(const string& parameter);

    ///
	//  void clearRequestedEvents();

    ///
    bool addSignalRequests(const MgSignalEvent& parameter);
    ///
    bool addEventStates(const MgSignalEvent& parameter);

    ///
    void clearSignalRequests();

    /// add an observed event to this message
    bool addObservedEvents(const MgNotificationEvent& parameter);
    /// clear the list of observed events in this message
    void clearObservedEvents();

    ///
    bool setDigitMap(const string& parameter);

    ///
    bool setNotifiedEntity(const string& parameter);

    ///
    bool setNumEndPoints(int parameter);
   
    /// 
    int getMaxEndPointIds();

    ///
    bool setCapabilities(MgcpParameterCapabilities& cap);

    ///
    bool setConnectionParameters(MgcpParameterConnectionParameters& cp);
	 
    ///
    bool setRequestId(const string& parameter);

   
    /// 
    bool setQuarantineHandling(MgcpNotificationHandling handling=MgcpNotificationHandlingStep, 
			       MgcpQuarantineTreatment treatment=MgcpQuarantineTreatmentProcess);

    ///
    bool addDetectEvents(const MgRequestedEvent& parameter);

    ///
    void clearDetectEvents();

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// set the description of any response error
    bool setResponseString(string response);

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:
    /// Holds the data for this command
    Sptr<MgcpCommandAuditEndpoint> data;

};


/// message receive by the gateway requesting all info about a connection
class MgAuditConnection : public MgCommand
{
public: 
    ///
//    MgAuditConnection(const string& EndpointId, const string& ConnectionId);
    ///
    MgAuditConnection(const MgAuditConnection& x) : MgCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgAuditConnection(MgcpCommand* x) { 
        MgcpCommandAuditConnection* p = dynamic_cast<MgcpCommandAuditConnection*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgAuditConnection(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgAuditConnectionType; } ;

    /// Get endpoint that created the notification 
    string getEndpointId();

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();
    
    ///get the connection id
    string getConnectionId();

    /// get the num of connection ids of this message
    int getNumConnectionId();
    
    ///get the i'th item in connection id
    string getConnectionIdItem(int i);

    /// get the number of items in RequestedInfo
    int getNumRequestedInfo();

    /// get the i'th item in RequestedInfo
    MgcpParameterType getItemRequestedInfo( int index ); 
    
    /// Specify the call id 
    bool setCallId(const string& parameter);

    /// set the notified entity
    bool setNotifiedEntity(const string& parameter);

    ///
    bool setLocalConnectionOptions(const MgcpConnectionOptions& parameter);

    /// set the connection mode of this message
    bool setConnectionMode(const string& parameter);
    
   ///
   bool setConnectionParameters(const MgcpParameterConnectionParameters& parameter);

    /// set the Remote connection descriptor
    bool setRemoteConnectionDescriptor(const SdpSession& parameter);

    /// set the Local connection descriptor
    bool setLocalConnectionDescriptor(const SdpSession& parameter);

    /// set the response error code 
    bool setResponseCode(const MgcpResponseCode& code);

    /// set the description of any response error
    bool setResponseString(string response);

	/// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:
    /// Holds the data for this command
    Sptr<MgcpCommandAuditConnection> data;

};


/// message sent to the agent indicating a restart has occured
class MgRestartInProgress : public MgCommand
{
public: 
    ///
    MgRestartInProgress() : data(new MgcpCommandRestartInProgress)  {};

    ///
    MgRestartInProgress(const MgRestartInProgress& x ) : MgCommand(x), data(x.data)  {};

    ///
    MgRestartInProgress(MgcpCommand* x) { 
        MgcpCommandRestartInProgress* p = dynamic_cast<MgcpCommandRestartInProgress*>(x);
        if(p) {data = p;}
    };

    /// constructor for this class
    MgRestartInProgress(const string& EndpointId, const string& RestartMethod, const string& RestartDelay);

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgRestartInProgress(*this);
    };

    /// Get the type of the command
    MgCommandType getType() { return MgRestartInProgressType; } ;


    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:
    /// Holds the data for this command
    Sptr<MgcpCommandRestartInProgress> data;
};


/// message sent to agent indicating that a connection has been closed
class MgDeleteConnectionGw : public MgCommand
{
public: 

    /// constructor for this class
    MgDeleteConnectionGw() : data (new MgcpCommandDeleteConnectionGw) {} ;


    /// constructor for this class
    MgDeleteConnectionGw(const MgDeleteConnectionGw& x ) : MgCommand(x), data(x.data) {} ;

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgDeleteConnectionGw(MgcpCommand* x) { 
        MgcpCommandDeleteConnectionGw* p = dynamic_cast<MgcpCommandDeleteConnectionGw*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgCommand* copy() {
	return new MgDeleteConnectionGw(*this);
    };

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    bool setResponseAck(const string& parameter);


    /// Get the type of the command
    MgCommandType getType() { return MgDeleteConnectionGwType; };

    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

private:

    /// Holds the data for this command
    Sptr<MgcpCommandDeleteConnectionGw> data;

};


#endif
