
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgEvent_cxx_version =
"$Id: MgEvent.cxx,v 1.12 2001/02/01 01:22:56 icahoon Exp $";


#include "global.h"
#include "MgEvent.hxx"
#include "MgcpParameterEventGeneric.hxx"
#include "MgcpResponseCode.hxx"

ostream& operator<<(ostream& s, const MgEventAction& x) 
{
    x.print(s);
    return s;
}

ostream& operator<<(ostream& s, MgRequestedEvent& x) 
{
    x.print(s);
    return s;
}

///
void MgRequestedEvent::addAction(const MgEventAction& action)
{
    myAction = action;
}


///
void MgRequestedEvent::addAction(const MgcpRequestedAction& action)
{
    myAction = MgEventAction(action);
}


///
void MgRequestedEvent::clearAction()
{
    myAction = MgEventAction(MgcpRequestedActionTypeNotify);
}


/** this function returns the number of actions in this event */
int MgRequestedEvent::getNumAction()
{
    return 1;
}


///
MgEventAction MgRequestedEvent::getItemAction(int i) const
{
    return myAction;
}

ostream& MgRequestedEvent::print(ostream& s)
{
    s << "MgRequestedEvent *** start ***" << endl;
    s << "MgRequestedEvent Event: " << myEvent << endl;
    s << "MgRequestedEvent Action: " << myAction << endl;
    s << "MgRequestedEvent *** end   ***" << endl;
    return s;
}


/**********************************************************************

			    MgEventAction

 **********************************************************************/


MgEventAction&     
MgEventAction::operator= (const MgEventAction& x)
{
    myActionType = x.myActionType;
    return *this;
}

///
MgcpRequestedActionType MgEventAction::getActionType() const
{
    return myActionType.getActionType();
}

MgEventAction::MgEventAction()
    : 
    myActionType(MgcpRequestedActionTypeNotify)
{
}



MgEventAction::MgEventAction(const MgEventAction& x)
    : 
    myActionType(x.myActionType)
{
}


MgEventAction::MgEventAction(const MgcpRequestedAction& action)
    : 
    myActionType(action)
{
}


MgEventAction::MgEventAction(const MgEmbeddedNotificationRequest& embedded)
{
    Sptr<MgcpEmbeddedRequest> requestCopy 
	= new MgcpEmbeddedRequest(embedded.getImplementation());

    myActionType.insertEmbedded(requestCopy);
}


ostream&
MgEventAction::print(ostream& s) const
{
    s << "MgEventAction *** start ***" << endl;
    s << myActionType;
    s << "MgEventAction *** end   ***" << endl;
    return s;
}


/*********************************************************************

		    MgEmbeddedNotificationRequest

 *********************************************************************/

///
MgEmbeddedNotificationRequest::MgEmbeddedNotificationRequest()
{
}


///
int 
MgEmbeddedNotificationRequest::getNumRequestedEvents()
{
    return request.getDecodedNumEvents();
}


///
Sptr<MgRequestedEvent> 
MgEmbeddedNotificationRequest::getItemRequestedEvents(int i)
{
    Sptr<MgcpParameterRequestedEvent> event(request.getDecodedEvent(i));

    Sptr<MgcpParameterRequestedEventGeneric> eventGeneric;

    eventGeneric.dynamicCast(event);

    if(eventGeneric != 0)
    {
	MgRequestedEvent* mgEvent;
	mgEvent = new MgRequestedEvent(eventGeneric->getGenericEvent());
	Sptr<MgRequestedEvent> sEvent(mgEvent);
	return sEvent;
    }
    else
    {
	return Sptr<MgRequestedEvent>(0);
    }
}


///
int 
MgEmbeddedNotificationRequest::getNumSignalRequests()
{
    return request.getDecodedNumSignals();
}


///
Sptr<MgEvent> 
MgEmbeddedNotificationRequest::getItemSignalRequests(int i)
{
    Sptr<MgcpParameterSignal> signal(request.getDecodedSignal(i));

    Sptr<MgcpParameterSignalGeneric> genericSignal;

    genericSignal.dynamicCast(signal);

    if(genericSignal != 0)
    {
	MgEvent* event;
	event = new MgEvent(genericSignal->getGenericEvent());
	Sptr<MgEvent> sEvent(event);
	return sEvent;
    }
    else
    {
	return Sptr<MgEvent>(0);
    }
}


///
string 
MgEmbeddedNotificationRequest::getDigitMap()
{
    return request.getDecodedMap().getDecoded();
}


///
MgcpEmbeddedRequest 
MgEmbeddedNotificationRequest::getImplementation() const
{
    return request;
}
