#ifndef MG_EVENT_HXX
#define MG_EVENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgEvent_hxx_Version = 
"$Id: MgEvent.hxx,v 1.5 2001/02/01 01:22:56 icahoon Exp $";


#include "global.h"
#include "MgcpPackageTypes.hxx"
#include "MgcpGenericEvent.hxx"
#include <string>
#include "Sptr.hxx"
#include "MgcpParameterEventGeneric.hxx"


class MgEvent;
class MgEmbeddedNotificationRequest;

class MgEventAction;
class MgRequestedEvent;
extern ostream& operator<<(ostream& s, const MgEventAction& x);
extern ostream& operator<<(ostream& s, const MgRequestedEvent& x);

///
class MgEventAction
{
public:
    ///
    MgEventAction();
    ///
    MgEventAction(const MgEventAction& x);
    ///
    MgEventAction(const MgcpRequestedAction& action);
    ///
    MgEventAction(const MgEmbeddedNotificationRequest& embedded);
    ///
    MgEventAction& operator= (const MgEventAction& x);
    ///
    MgcpRequestedActionType getActionType() const;

    ///
    ostream& print(ostream& s) const;

    ///
    friend ostream& operator<<(ostream& s, const MgEventAction& x);

private:
    ///
    MgcpRequestedAction myActionType;
};


///
class MgRequestedEvent
{
public:
    ///
    MgRequestedEvent() {}
    ///
    MgRequestedEvent(const MgcpGenericEvent& x) : myEvent(x) {}

    ///
    virtual ~MgRequestedEvent() {}
    ///
    MgcpPackageType getPackageType() { return myEvent.getPackageType(); }

    MgcpGenericEvent getEvent() const { return myEvent; }

    /// Get the i'th paramter 
    string getParameter(int index) { return myEvent.getParameter(index); }

    /// Get the number of paramters - usually 0 - depends on type
    int getNumParameters() { return myEvent.getParameterSize(); }

    /** return the number of actions in this event */
    int getNumAction();
    /// return the actual action in this event
    MgEventAction getItemAction(int i) const;

    ///
    void addAction(const MgEventAction& action);

    ///
    void addAction(const MgcpRequestedAction& action);

    ///
    void clearAction();

    ///
    ostream& print(ostream& s);

    ///
    friend ostream& operator<<(ostream& s, MgRequestedEvent& x);

protected:
    MgcpGenericEvent myEvent;
    MgEventAction myAction;
};


// TODO: fix the implementation for this class as well

///
class MgEmbeddedNotificationRequest
{
public:
    ///
    MgEmbeddedNotificationRequest();

    ///
    int getNumRequestedEvents();
    ///
    Sptr<MgRequestedEvent> getItemRequestedEvents(int i);
    ///
    int getNumSignalRequests();
    ///
    Sptr<MgEvent> getItemSignalRequests(int i);
    ///
    string getDigitMap();

    MgcpEmbeddedRequest getImplementation() const;

private:
    ///
    MgcpEmbeddedRequest request;
};

///
class MgEvent
{
public:
    MgEvent() {}
    MgEvent(const MgcpGenericEvent& x) : myEvent(x) {};
    virtual ~MgEvent() {}
    MgcpPackageType getPackageType() { return myEvent.getPackageType(); }

    MgcpGenericEvent getEvent() const { return myEvent; }

    /// Get the i'th paramter 
    string getParameter(int index) { return myEvent.getParameter(index); }

    /// Get the number of paramters - usually 0 - depends on type
    int getNumParameters() { return myEvent.getParameterSize(); }

    /** this function returns the number of actions in this event
	(which is 0 in the case of events which are not RequestedEvents) */
    int getNumAction();
    ///
    MgEventAction getItemAction() const;

protected:
    MgcpGenericEvent myEvent;
};


///
class MgNotificationEvent 
{
public:
    MgNotificationEvent();
    MgNotificationEvent(const MgcpGenericEvent& x) : myEvent(x) {}

    MgcpGenericEvent getEvent() const { return myEvent; }

protected:
    MgcpGenericEvent myEvent;
};

///
class MgSignalEvent 
{
public:
    MgSignalEvent() {}
    MgSignalEvent(const MgcpGenericEvent& x) : myEvent(x) {}
    ~MgSignalEvent() {}

    MgcpGenericEvent getEvent() const { return myEvent; }

protected:
    MgcpGenericEvent myEvent;
};



#endif
