#ifndef MG_PACKAGE_HXX
#define MG_PACKAGE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgPackage_hxx_Version = 
"$Id: MgPackage.hxx,v 1.4.4.1 2001/02/13 21:32:48 liuhong Exp $";



#include "global.h"
#include "MgcpEventTypes.hxx"
#include "MgEvent.hxx"
#include <string>



#ifndef NCS_PROFILE
///
class MgGenericPackage
{
public:
    ///
    static MgNotificationEvent notifyModemDetect(string s="");
    ///
    static MgNotificationEvent notifyFaxToneDetect(string s="");
    ///
    static MgNotificationEvent notifyLongDurationConnection(string s="");
    ///
    static MgNotificationEvent notifyPatternDetected(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgGenericPackageEvent : public MgEvent
{
public:
    ///
    MgGenericPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpGenericPackageItemType getType() { return myEvent.getGenericPackageType(); }

private:
/// only the Mg stack can construct these 
};


class MgDTMFPackage
{
public:
    ///
    static MgNotificationEvent notifyDTMF_0(string s="");
    ///
    static MgNotificationEvent notifyDTMF_1(string s="");
    ///
    static MgNotificationEvent notifyDTMF_2(string s="");
    ///
    static MgNotificationEvent notifyDTMF_3(string s="");
    ///
    static MgNotificationEvent notifyDTMF_4(string s="");
    ///
    static MgNotificationEvent notifyDTMF_5(string s="");
    ///
    static MgNotificationEvent notifyDTMF_6(string s="");
    ///
    static MgNotificationEvent notifyDTMF_7(string s="");
    ///
    static MgNotificationEvent notifyDTMF_8(string s="");
    ///
    static MgNotificationEvent notifyDTMF_9(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Hash(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Star(string s="");
    ///
    static MgNotificationEvent notifyDTMF_A(string s="");
    ///
    static MgNotificationEvent notifyDTMF_B(string s="");
    ///
    static MgNotificationEvent notifyDTMF_C(string s="");
    ///
    static MgNotificationEvent notifyDTMF_D(string s="");
    ///
    static MgNotificationEvent notifyLongDurationIndicator(string s="");
    ///
    static MgNotificationEvent notifyWildcardMatch(string s="");
    ///
    static MgNotificationEvent notifyInterdigitTimer(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgDTMFPackageEvent : public MgEvent
{
public:
    ///
    MgDTMFPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpDTMFPackageItemType getType() { return myEvent.getDTMFPackageType(); }


    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgMFPackage
{
public:
    static MgNotificationEvent notifyMF_0(string s="");
    ///
    static MgNotificationEvent notifyMF_1(string s="");
    ///
    static MgNotificationEvent notifyMF_2(string s="");
    ///
    static MgNotificationEvent notifyMF_3(string s="");
    ///
    static MgNotificationEvent notifyMF_4(string s="");
    ///
    static MgNotificationEvent notifyMF_5(string s="");
    ///
    static MgNotificationEvent notifyMF_6(string s="");
    ///
    static MgNotificationEvent notifyMF_7(string s="");
    ///
    static MgNotificationEvent notifyMF_8(string s="");
    ///
    static MgNotificationEvent notifyMF_9(string s="");
    ///
    static MgNotificationEvent notifyWildcardMatch(string s="");
    ///
    static MgNotificationEvent notifyInterdigitTimer(string s="");
    ///
    static MgNotificationEvent notifyMFK0(string s="");
    ///
    static MgNotificationEvent notifyMFK1(string s="");
    ///
    static MgNotificationEvent notifyMFK2(string s="");
    ///
    static MgNotificationEvent notifyMFS0(string s="");
    ///
    static MgNotificationEvent notifyMFS1(string s="");
    ///
    static MgNotificationEvent notifyMFS2(string s="");
    ///
    static MgNotificationEvent notifyMFS3(string s="");
    ///
    static MgNotificationEvent notifyWink(string s="");
    ///
    static MgNotificationEvent notifyWinkOff(string s="");
    ///
    static MgNotificationEvent notifyIncomingSeizure(string s="");
    ///
    static MgNotificationEvent notifyReturnSeizure(string s="");
    ///
    static MgNotificationEvent notifyUnseizeCircuit(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgMFPackageEvent : public MgEvent
{
public:
    ///
    MgMFPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpMFPackageItemType getType() { return myEvent.getMFPackageType(); }


    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgTrunkPackage
{
public:
    static MgNotificationEvent notifyContinuityTone(string s="");
    ///
    static MgNotificationEvent notifyContinuityTest(string s="");
    ///
    static MgNotificationEvent notifyOldMilliwattTone(string s="");
    ///
    static MgNotificationEvent notifyNewMilliwattTone(string s="");
    ///
    static MgNotificationEvent notifyTestLine(string s="");
    ///
    static MgNotificationEvent notifyNoCircuit(string s="");
    ///
    static MgNotificationEvent notifyAnswerSupervision(string s="");
    ///
    static MgNotificationEvent notifyReorderTone(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgTrunkPackageEvent : public MgEvent
{
public:
    ///
    MgTrunkPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpTrunkPackageItemType getType() { return myEvent.getTrunkPackageType(); }


    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgLinePackage
{
public:
    static MgNotificationEvent notifyOffHookTransition(string s="");
    ///
    static MgNotificationEvent notifyOnHookTransition(string s="");
    ///
    static MgNotificationEvent notifyFlashHook(string s="");
    ///
    static MgNotificationEvent notifyAnswerTone(string s="");
    ///
    static MgNotificationEvent notifyNetworkBusy(string s="");
    ///
    static MgNotificationEvent notifyPromptTone(string s="");
    ///
    static MgNotificationEvent notifyErrorTone(string s="");
    ///
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgLinePackageEvent : public MgEvent
{
public:
    ///
    MgLinePackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpLinePackageItemType getType() { return myEvent.getLinePackageType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgHandsetPackage
{
public:
    static MgNotificationEvent notifyAdsiDisplay(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyOffHookTransition(string s="");
    ///
    static MgNotificationEvent notifyOnHookTransition(string s="");
    ///
    static MgNotificationEvent notifyFlashHook(string s="");
    ///
    static MgNotificationEvent notifyAnswerTone(string s="");
    ///
    static MgNotificationEvent notifyBusyTone(string s="");
    ///
    static MgNotificationEvent notifyCallWaitingTone(string s="");
    ///
    static MgNotificationEvent notifyDialTone(string s="");
    ///
    static MgNotificationEvent notifyNetworkBusy(string s="");
    ///
    static MgNotificationEvent notifyRinging(string s="");
    ///
    static MgNotificationEvent notifyDistinctiveRinging(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyPromptTone(string s="");
    ///
    static MgNotificationEvent notifyErrorTone(string s="");
    ///
    static MgNotificationEvent notifyStutterDialtone(string s="");
    ///
    static MgNotificationEvent notifyAlertingTone(string s="");
    ///
    static MgNotificationEvent notifyRecorderWarningTone(string s="");
    ///
    static MgNotificationEvent notifySITTone(string s="");
    ///
    static MgNotificationEvent notifyCallingCardServiceTone(string s="");
    ///
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyOffHookWarningTone(string s="");
    ///
    static MgNotificationEvent notifyDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgHandsetPackageEvent : public MgEvent
{
public:
    ///
    MgHandsetPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    ///
    MgHandsetPackageEvent() {};

    /// get the type of event that the Gateway is sending to us
//    MgcpHandsetPackageItemType getType() { return objectType; };
    MgcpHandsetPackageItemType getType() { return myEvent.getHandsetType(); }

    ///
//    setType(MgcpHandsetPackageItemType x) { myEvent.setHandsetType(x); };
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//   int getNumParameters();
private:
    MgcpHandsetPackageItemType objectType;
/// only the Mg stack can construct these 
};


class MgRTPPackage
{
public:
    ///
    static MgNotificationEvent notifyUsedCodecChanged(string s="");
    ///
    static MgNotificationEvent notifySamplingRateChanged(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyJitterBufferSizeChanged(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyPacketLossExceeded(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyQualityAlert(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgRTPPackageEvent : public MgEvent
{
public:
    ///
    MgRTPPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpRTPPackageItemType getType() { return myEvent.getRTPType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgNetworkAccessServerPackage
{
public:
    static MgNotificationEvent notifyPacketArrival(string s="");
    ///
    static MgNotificationEvent notifyCallBackRequest(string s="");
    ///
    static MgNotificationEvent notifyCarrierLost(string s="");
    ///
    static MgNotificationEvent notifyAuthorizationSucceeded(string s="");
    ///
    static MgNotificationEvent notifyAuthorizationDenied(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgNetworkAccessServerPackageEvent : public MgEvent
{
public:
    ///
    MgNetworkAccessServerPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpNetworkAccessServerPackageItemType getType() { return myEvent.getNASType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgAnnouncementPackage
{
public:
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgAnnouncementPackageEvent : public MgEvent
{
public:
    ///
    MgAnnouncementPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpAnnouncementPackageItemType getType() { return myEvent.getAnnounceType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


class MgScriptPackage
{
public:
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgScriptPackageEvent : public MgEvent
{
public:
    ///
    MgScriptPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpScriptPackageItemType getType() { return myEvent.getScriptType(); }

    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
// only the Mg stack can construct these 
};


class MgXVoiceMailPackage
{
public:
    ///
    static MgNotificationEvent notifyMailboxNumber(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyMessageWaitingInd(const string& arg, string s="");
};


/// This class has things the Gateway can send to the agent 
class MgXVoiceMailPackageEvent : public MgEvent
{
public:
    ///
    MgXVoiceMailPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpXVoiceMailPackageItemType getType() { return myEvent.getXVoiceMailType();}
private:
// only the Mg stack can construct these 
};


class MgAllPackages
{
public:
    /// 
    static MgNotificationEvent notifyModemDetect(string s="");
    ///
    static MgNotificationEvent notifyFaxToneDetect(string s="");
    ///
    static MgNotificationEvent notifyLongDurationConnection(string s="");
    ///
    static MgNotificationEvent notifyPatternDetected(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
    ///
    static MgNotificationEvent notifyDTMF_0(string s="");
    ///
    static MgNotificationEvent notifyDTMF_1(string s="");
    ///
    static MgNotificationEvent notifyDTMF_2(string s="");
    ///
    static MgNotificationEvent notifyDTMF_3(string s="");
    ///
    static MgNotificationEvent notifyDTMF_4(string s="");
    ///
    static MgNotificationEvent notifyDTMF_5(string s="");
    ///
    static MgNotificationEvent notifyDTMF_6(string s="");
    ///
    static MgNotificationEvent notifyDTMF_7(string s="");
    ///
    static MgNotificationEvent notifyDTMF_8(string s="");
    ///
    static MgNotificationEvent notifyDTMF_9(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Hash(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Star(string s="");
    ///
    static MgNotificationEvent notifyDTMF_A(string s="");
    ///
    static MgNotificationEvent notifyDTMF_B(string s="");
    ///
    static MgNotificationEvent notifyDTMF_C(string s="");
    ///
    static MgNotificationEvent notifyDTMF_D(string s="");
    ///
    static MgNotificationEvent notifyLongDurationIndicator(string s="");
    ///
    static MgNotificationEvent notifyWildcardMatch(string s="");
    ///
    static MgNotificationEvent notifyInterdigitTimer(string s="");
    ///
    static MgNotificationEvent notifyMFK0(string s="");
    ///
    static MgNotificationEvent notifyMFK1(string s="");
    ///
    static MgNotificationEvent notifyMFK2(string s="");
    ///
    static MgNotificationEvent notifyMFS0(string s="");
    ///
    static MgNotificationEvent notifyMFS1(string s="");
    ///
    static MgNotificationEvent notifyMFS2(string s="");
    ///
    static MgNotificationEvent notifyMFS3(string s="");
    ///
    static MgNotificationEvent notifyWink(string s="");
    ///
    static MgNotificationEvent notifyWinkOff(string s="");
    ///
    static MgNotificationEvent notifyIncomingSeizure(string s="");
    ///
    static MgNotificationEvent notifyReturnSeizure(string s="");
    ///
    static MgNotificationEvent notifyUnseizeCircuit(string s="");
    ///
    static MgNotificationEvent notifyContinuityTone(string s="");
    ///
    static MgNotificationEvent notifyContinuityTest(string s="");
    ///
    static MgNotificationEvent notifyOldMilliwattTone(string s="");
    ///
    static MgNotificationEvent notifyNewMilliwattTone(string s="");
    ///
    static MgNotificationEvent notifyTestLine(string s="");
    ///
    static MgNotificationEvent notifyNoCircuit(string s="");
    ///
    static MgNotificationEvent notifyAnswerSupervision(string s="");
    ///
    static MgNotificationEvent notifyReorderTone(string s="");
    ///
    static MgNotificationEvent notifyOffHookTransition(string s="");
    ///
    static MgNotificationEvent notifyOnHookTransition(string s="");
    ///
    static MgNotificationEvent notifyFlashHook(string s="");
    ///
    static MgNotificationEvent notifyAnswerTone(string s="");
    ///
    static MgNotificationEvent notifyNetworkBusy(string s="");
    ///
    static MgNotificationEvent notifyPromptTone(string s="");
    ///
    static MgNotificationEvent notifyErrorTone(string s="");
    ///
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyAdsiDisplay(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyBusyTone(string s="");
    ///
    static MgNotificationEvent notifyCallWaitingTone(string s="");
    ///
    static MgNotificationEvent notifyDialTone(string s="");
    ///
    static MgNotificationEvent notifyRinging(string s="");
    ///
    static MgNotificationEvent notifyDistinctiveRinging(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyStutterDialtone(string s="");
    ///
    static MgNotificationEvent notifyAlertingTone(string s="");
    ///
    static MgNotificationEvent notifyRecorderWarningTone(string s="");
    ///
    static MgNotificationEvent notifySITTone(string s="");
    ///
    static MgNotificationEvent notifyCallingCardServiceTone(string s="");
    ///
    static MgNotificationEvent notifyOffHookWarningTone(string s="");
    ///
    static MgNotificationEvent notifyUsedCodecChanged(string s="");
    ///
    static MgNotificationEvent notifySamplingRateChanged(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyJitterBufferSizeChanged(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyPacketLossExceeded(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyQualityAlert(string s="");
    ///
    static MgNotificationEvent notifyPacketArrival(string s="");
    ///
    static MgNotificationEvent notifyCallBackRequest(string s="");
    ///
    static MgNotificationEvent notifyCarrierLost(string s="");
    ///
    static MgNotificationEvent notifyAuthorizationSucceeded(string s="");
    ///
    static MgNotificationEvent notifyAuthorizationDenied(string s="");
    ///
    static MgNotificationEvent notifyMailboxNumber(const string& arg, string s="");
    ///
    static MgNotificationEvent notifyMessageWaitingInd(const string& arg, string s="");
};


class MgAllPackagesEvent : public MgEvent
{
 public:
    ///
    MgAllPackagesEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpAllPackagesItemType getType() { return( myEvent.getAllPackagesType() ); }

};


/**********************************************************************/

/***********************************************************************************/

/// This class has things the Gateway can send to the agent 
class MgGenericPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgGenericPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpGenericPackageItemType getType() { return myEvent.getGenericPackageType(); }

private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgDTMFPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgDTMFPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpDTMFPackageItemType getType() { return myEvent.getDTMFPackageType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgMFPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgMFPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpMFPackageItemType getType() { return myEvent.getMFPackageType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgTrunkPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgTrunkPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpTrunkPackageItemType getType() { return myEvent.getTrunkPackageType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgLinePackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgLinePackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpLinePackageItemType getType() { return myEvent.getLinePackageType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgHandsetPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgHandsetPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    ///
    MgHandsetPackageRequestedEvent() {};

    /// get the type of event that the Gateway is sending to us
//    MgcpHandsetPackageItemType getType() { return objectType; };
    MgcpHandsetPackageItemType getType() { return myEvent.getHandsetType(); }

    ///
//    setType(MgcpHandsetPackageItemType x) { myEvent.setHandsetType(x); };
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//   int getNumParameters();
private:
    MgcpHandsetPackageItemType objectType;
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgRTPPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///

    MgRTPPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpRTPPackageItemType getType() { return myEvent.getRTPType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgNetworkAccessServerPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgNetworkAccessServerPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpNetworkAccessServerPackageItemType getType() { return myEvent.getNASType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgAnnouncementPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgAnnouncementPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpAnnouncementPackageItemType getType() { return myEvent.getAnnounceType(); }

    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgScriptPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgScriptPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpScriptPackageItemType getType() { return myEvent.getScriptType(); }

    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
// only the Mg stack can construct these 
};


/// This class has things the Gateway can send to the agent 
class MgXVoiceMailPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgXVoiceMailPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpXVoiceMailPackageItemType getType() { return myEvent.getXVoiceMailType(); }
private:
/// only the Mg stack can construct these 
};


class MgAllPackagesRequestedEvent : public MgRequestedEvent
{
public:

  ///
  MgAllPackagesRequestedEvent( const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
  ///
  MgcpAllPackagesItemType getType() { return myEvent.getAllPackagesType(); }

};
#endif  // end of #ifndef NCS_PROFILE  


/*================================================================================
   NCS Profile packages and events
================================================================================*/

#ifdef NCS_PROFILE
class MgNCSLinePackage
{
public:
    /// 
    static MgNotificationEvent notifyModemDetect(string s="");
    ///
    static MgNotificationEvent notifyFaxToneDetect(string s="");
    ///
    static MgNotificationEvent notifyLongDurationConnection(string s="");
    ///
    static MgNotificationEvent notifyMediaStart(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
    ///
    static MgNotificationEvent notifyDTMF_0(string s="");
    ///
    static MgNotificationEvent notifyDTMF_1(string s="");
    ///
    static MgNotificationEvent notifyDTMF_2(string s="");
    ///
    static MgNotificationEvent notifyDTMF_3(string s="");
    ///
    static MgNotificationEvent notifyDTMF_4(string s="");
    ///
    static MgNotificationEvent notifyDTMF_5(string s="");
    ///
    static MgNotificationEvent notifyDTMF_6(string s="");
    ///
    static MgNotificationEvent notifyDTMF_7(string s="");
    ///
    static MgNotificationEvent notifyDTMF_8(string s="");
    ///
    static MgNotificationEvent notifyDTMF_9(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Hash(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Star(string s="");
    ///
    static MgNotificationEvent notifyDTMF_A(string s="");
    ///
    static MgNotificationEvent notifyDTMF_B(string s="");
    ///
    static MgNotificationEvent notifyDTMF_C(string s="");
    ///
    static MgNotificationEvent notifyDTMF_D(string s="");
    ///
    static MgNotificationEvent notifyWildcardMatch(string s="");
    ///
    static MgNotificationEvent notifyInterdigitTimer(string s="");
    ///
    static MgNotificationEvent notifyOffHookTransition(string s="");
    ///
    static MgNotificationEvent notifyOnHookTransition(string s="");
    ///
    static MgNotificationEvent notifyFlashHook(string s="");
    ///
    static MgNotificationEvent notifyAnswerTone(string s="");
    ///
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyOffHookWarningTone(string s="");
    ///
    static MgNotificationEvent notifyTeletypeDeviceForDeaf(string s="");
    ///
    static MgNotificationEvent notifyAllEvents(string s="");
};


class MgNCSLinePackageEvent : public MgEvent
{
public:
    ///
    MgNCSLinePackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// 
    MgcpNCSLinePackageItemType getType() { return myEvent.getNCSLinePackageType(); }

private:

};


class MgNCSAdsiPackage
{
public:
  /// no event to notify call agent of

};


class MgNCSAdsiPackageEvent : public MgEvent
{
public:
    ///
    MgNCSAdsiPackageEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// 
    MgcpNCSAdsiPackageItemType getType() { return myEvent.getNCSAdsiPackageType(); }

private:

};


class MgNCSAllPackages
{
public:
    /// events that can be requested by call agent to be notified
    /// 
    static MgNotificationEvent notifyModemDetect(string s="");
    ///
    static MgNotificationEvent notifyFaxToneDetect(string s="");
    ///
    static MgNotificationEvent notifyLongDurationConnection(string s="");
    ///
    static MgNotificationEvent notifyMediaStart(string s="");
    ///
    static MgNotificationEvent notifyReportFailure(string s="");
    ///
    static MgNotificationEvent notifyDTMF_0(string s="");
    ///
    static MgNotificationEvent notifyDTMF_1(string s="");
    ///
    static MgNotificationEvent notifyDTMF_2(string s="");
    ///
    static MgNotificationEvent notifyDTMF_3(string s="");
    ///
    static MgNotificationEvent notifyDTMF_4(string s="");
    ///
    static MgNotificationEvent notifyDTMF_5(string s="");
    ///
    static MgNotificationEvent notifyDTMF_6(string s="");
    ///
    static MgNotificationEvent notifyDTMF_7(string s="");
    ///
    static MgNotificationEvent notifyDTMF_8(string s="");
    ///
    static MgNotificationEvent notifyDTMF_9(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Hash(string s="");
    ///
    static MgNotificationEvent notifyDTMF_Star(string s="");
    ///
    static MgNotificationEvent notifyDTMF_A(string s="");
    ///
    static MgNotificationEvent notifyDTMF_B(string s="");
    ///
    static MgNotificationEvent notifyDTMF_C(string s="");
    ///
    static MgNotificationEvent notifyDTMF_D(string s="");
    ///
    static MgNotificationEvent notifyLongDurationIndicator(string s="");
    ///
    static MgNotificationEvent notifyWildcardMatch(string s="");
    ///
    static MgNotificationEvent notifyInterdigitTimer(string s="");
    ///
    static MgNotificationEvent notifyOffHookTransition(string s="");
    ///
    static MgNotificationEvent notifyOnHookTransition(string s="");
    ///
    static MgNotificationEvent notifyFlashHook(string s="");
    ///
    static MgNotificationEvent notifyAnswerTone(string s="");
    ///
    static MgNotificationEvent notifyReportOnCompletion(string s="");
    ///
    static MgNotificationEvent notifyOffHookWarningTone(string s="");
    ///
    static MgNotificationEvent notifyTeletypeDeviceForDeaf(string s="");
    ///
    static MgNotificationEvent notifyAllEvents(string s="");   

};


class MgNCSAllPackagesEvent : public MgEvent
{
public:
    ///
    MgNCSAllPackagesEvent(const MgcpGenericEvent& e) : MgEvent(e) {}
    /// 
    MgcpNCSAllPackagesItemType getType() { return myEvent.getNCSAllPackagesType(); }

private:

};

class MgNCSLinePackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgNCSLinePackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// 
    MgcpNCSLinePackageItemType getType() { return myEvent.getNCSLinePackageType(); }

private:

};

class MgNCSAdsiPackageRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgNCSAdsiPackageRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// 
    MgcpNCSAdsiPackageItemType getType() { return myEvent.getNCSAdsiPackageType(); }

private:

};

class MgNCSAllPackagesRequestedEvent : public MgRequestedEvent
{
public:
    ///
    MgNCSAllPackagesRequestedEvent(const MgcpGenericEvent& e) : MgRequestedEvent(e) {}
    /// 
    MgcpNCSAllPackagesItemType getType() { return myEvent.getNCSAllPackagesType(); }

private:

};
#endif // end of #ifdef NCS_PROFILE


#endif
