
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* MgParse_cxx_Version =
"$Id: MgParse.cxx,v 1.8 2001/01/26 02:45:57 liuhong Exp $";


#include "global.h"
#include "MgParse.hxx"
#include "MgcpParse.hxx"


Sptr<MgCommand>
mgcpToMg(MgcpCommand* test_item)
{
    XDEBUG(1, cerr << "Begin *** mgcpToMg() *** " << endl;);

    MgCommand* msg = 0;

    assert(test_item);

    switch(test_item->getType())
    {
    case MgcpCommandTypeResponse:
         msg = new MgCommand(test_item);
         break;

    case MgcpCommandTypeEndpointConfiguration:
         msg = new MgEndpointConfiguration(test_item);
         break;

    case MgcpCommandTypeNotificationRequest:
         msg = new MgNotificationRequest(test_item);
         break;

    case MgcpCommandTypeNotify:
         msg = new MgNotify(test_item);
         break;

    case MgcpCommandTypeCreateConnection:
         msg = new MgCreateConnection(test_item);
         break;

    case MgcpCommandTypeModifyConnection:
         msg = new MgModifyConnection(test_item);
         break;

    case MgcpCommandTypeDeleteConnection:
         msg = new MgDeleteConnectionAgent(test_item);
         break;

    case MgcpCommandTypeAuditEndpoint:
         msg = new MgAuditEndpoint(test_item);
         break;

    case MgcpCommandTypeAuditConnection:
         msg = new MgAuditConnection(test_item);
         break;

    case MgcpCommandTypeRestartInProgress:
         msg = new MgRestartInProgress(test_item);
         break;

    default:
         cpLog( LOG_ERR, "Unknown MgcpCommandType !");
	 assert( 0 );
         break;
    }

    assert(msg);

    XDEBUG(1, cerr << "End *** mgcpToMg() ***" << endl;);

    DEBUG(cout << "received:  tid: " << msg->getDataPtr()->getTransactionId() << endl;);

    return Sptr<MgCommand>(msg);
}


string encodeToSend(MgCommand& msg)
{
    assert(msg.getDataPtr());
    return msg.getDataPtr()->encodeToSend();
}


Sptr<MgCommand> stringToMg(string& str)
{
    MgcpCommand* cmd = parseMessage(str.c_str(), str.length());

    return mgcpToMg(cmd);
}

ostream& printMgCommand(Sptr<MgCommand> cmd, ostream& s)
{
    s << "disassembling MgCommand" << endl;

    switch(cmd->getType())
    {
	case MgInvalidCommandType:
	    s << "Type: Invalid Command" << endl;
	    break;
	case MgEndpointConfigurationType:
	    s << "Type: Endpoint Configuration" << endl;
	    break;
	case MgCreateConnectionType:
	    s << "Type: Create Connection" << endl;
	    break;
	case MgModifyConnectionType:
	    s << "Type: Modify Connection" << endl;
	    break;
	case MgDeleteConnectionAgentType:
	    s << "Type: Delete Connection (from Agent)" << endl;
	    break;
	case MgNotificationRequestType:
	{
	    s << "Type: Notification Request" << endl;
	    Sptr<MgNotificationRequest> nr;
	    nr.dynamicCast(cmd);
	    s << "Endpoint ID: " << nr->getEndpointId() << endl;
	    for(int i = 0; i < nr->getNumRequestedEvents() ; i++)
	    {
		Sptr<MgRequestedEvent> myRequestedEvent 
		    = nr->getItemRequestedEvents(i);
		myRequestedEvent->print(s);
	    }
	}
	break;
	case MgNotifyType:
	    s << "Type: Notify" << endl;
	    break;
	case MgAuditEndpointType:
	    s << "Type: Audit Endpoint" << endl;
	    break;
	case MgAuditConnectionType:
	    s << "Type: Audit Connection" << endl;
	    break;
	case MgRestartInProgressType:
	    s << "Type: Restart In Progress" << endl;
	    break;
	case MgDeleteConnectionGwType:
	    s << "Type: Delete Connection (from GW)" << endl;
	    break;
    }

    return s;
}


// get the response Acknowledgement string from a MgCommand
string MgGetRespAckString(Sptr<MgCommand> cmd)
{
    switch(cmd->getType())
    {
	case MgInvalidCommandType:
	   break;
	   
	case MgEndpointConfigurationType:
	{ 
	  Sptr<MgEndpointConfiguration> epcfPtr;

	  epcfPtr.dynamicCast(cmd);
	  if (epcfPtr != 0)
	     return (epcfPtr->getResponseAck());
	  else
	    break;
	}

	case MgCreateConnectionType:
	{
	  Sptr<MgCreateConnection> crcxPtr;

	  crcxPtr.dynamicCast(cmd);
	  if (crcxPtr != 0)
	     return (crcxPtr->getResponseAck());
	  else
	    break;
	}
	  
	case MgModifyConnectionType:
	{
	  Sptr<MgModifyConnection> mdcxPtr;

	  mdcxPtr.dynamicCast(cmd);
	  if (mdcxPtr != 0)
	     return (mdcxPtr->getResponseAck());
	  else
	    break;
	}
       
	case MgDeleteConnectionAgentType:
	{
	  Sptr<MgDeleteConnectionAgent> dlcxPtr;

	  dlcxPtr.dynamicCast(cmd);
	  if (dlcxPtr != 0)
	     return (dlcxPtr->getResponseAck());
	  else
	    break;
	}
       
	case MgNotificationRequestType:
	{
	  Sptr<MgNotificationRequest> rqntPtr;

	  rqntPtr.dynamicCast(cmd);
	  if (rqntPtr != 0 )
	     return (rqntPtr->getResponseAck());
	  else
	    break;
	} 
	
	case MgNotifyType:
	  break;
	  
	case MgAuditEndpointType:
	{
	  Sptr<MgAuditEndpoint> auepPtr;

	  auepPtr.dynamicCast(cmd);
	  if (auepPtr != 0)
	     return (auepPtr->getResponseAck());
	  else
	    break;
	}  
	   
	case MgAuditConnectionType:
	{
	  Sptr<MgAuditConnection> aucxPtr;

	  aucxPtr.dynamicCast(cmd);
	  if (aucxPtr != 0)
	     return (aucxPtr->getResponseAck());
	  else
	    break;
	}

	case MgRestartInProgressType:
	  break;
	  
	case MgDeleteConnectionGwType:
	  break;
	   
        default:
	   break;
    }

  return "";
}
