
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgcCommand_cxx_version =
"$Id: MgcCommand.cxx,v 1.54 2001/02/01 01:22:56 icahoon Exp $";



#include "MgcCommand.hxx"
#include "MgcpCommand.hxx"
#include "MgcpParameter.hxx"





/**********************************************************************

	     These functions implement multiple commands

 **********************************************************************/
Sptr<MgcEvent> getOneMgcEvent( MgcpGenericEvent& ge )
{
    switch(ge.getPackageType())
    {
#ifndef NCS_PROFILE
      case    MgcpNoPackageType:
      case    MgcpGenericMediaPackageType:
	      return new MgcGenericPackageEvent(ge);
              break;
      case    MgcpDTMFPackageType:
	      return new MgcDTMFPackageEvent(ge);
	      break;
      case    MgcpMFPackageType:
	      return new MgcMFPackageEvent(ge);
	      break;
      case    MgcpTrunkPackageType:
	      return new MgcTrunkPackageEvent(ge);
	      break;
      case    MgcpLinePackageType:
	      return new MgcLinePackageEvent(ge);
	      break;
      case    MgcpHandsetPackageType:
	      return new MgcHandsetPackageEvent(ge);
	      break;
      case    MgcpRTPPackageType:
	      return new MgcRTPPackageEvent(ge);
	      break;
      case    MgcpNetworkAccessServerPackageType:
	      return new MgcNetworkAccessServerPackageEvent(ge);
	      break;
      case    MgcpAnnouncementPackageType:
	      return new MgcAnnouncementPackageEvent(ge);
	      break;
      case    MgcpScriptPackageType:
	      return new MgcScriptPackageEvent(ge);
	      break;
      case    MgcpXVoiceMailPackageType:
	      return new MgcXVoiceMailPackageEvent(ge);
	      break;
      case    MgcpAllPackagesType:
	      return new MgcAllPackagesEvent(ge);
	      break;
#else 
	      /// NCS Profile types
    case      MgcpNCSLinePackageType:
              return new MgcNCSLinePackageEvent(ge);
	      break;
    case      MgcpNCSAdsiPackageType:
              return new MgcNCSAdsiPackageEvent(ge);
	      break;
    case      MgcpNCSDefaultPackageType:
              return new MgcNCSLinePackageEvent(ge);
	      break;
    case      MgcpNCSAllPackagesType:
              return new MgcNCSAllPackagesEvent(ge);
	      break;
#endif 
    default:
              return 0;
	      break;
  }
}


//
bool 
addOneRequestedEvent(const MgcRequestedEvent& parameter, 
		     Sptr<MgcpRequest> data)
{
    /// presume it's empty
    Sptr<MgcpParameterRequestedEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeRequestedEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterRequestedEvents;
    }
    
    // now do something with the Requested Events... insert them.

    MgcpParameterRequestedEventGeneric* x = 
	new MgcpParameterRequestedEventGeneric(parameter.getEvent(), parameter.getItemAction(0).getInternal());
	    
    ptr->insert(x);

    data->insert(ptr);
    return true;
}


//
//
bool 
addOneSignalRequest(const MgcSignalEvent& parameter,
		    Sptr<MgcpRequest> data)
{
    /// presume it's empty
    Sptr<MgcpParameterSignalRequests> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeSignalRequests));

    if(!ptr)
    {
	ptr = new MgcpParameterSignalRequests;
    }
    
    // now do something with the Signal Requests
    
    MgcpParameterSignalGeneric* x = 
	new MgcpParameterSignalGeneric(parameter.getEvent());
	    
    ptr->insert(x);

    data->insert(ptr);
    return true;
}


/**********************************************************************

	       functions to implement specific commands

 **********************************************************************/



MgcCommand::~MgcCommand(void)
{
}

// constructor
MgcEndpointConfiguration::MgcEndpointConfiguration(const string& EndpointId)
    : data(new MgcpCommandEndpointConfiguration)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
}


//
void MgcEndpointConfiguration::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

   data->insert(ptr);
}


//
string 
MgcEndpointConfiguration::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

/********************************************************************/

// constructor
MgcCreateConnection::MgcCreateConnection(const string& EndpointId, const string& CallId, const string& ConnectionMode)
    : data(new MgcpCommandCreateConnection)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
    {
        MgcpParameterCallId* ptr;

        ptr = new MgcpParameterCallId(CallId);

        data->insert(ptr);
    }
    {
        MgcpParameterConnectionMode* ptr;

        ptr = new MgcpParameterConnectionMode(ConnectionMode);

        data->insert(ptr);
    }
}


int MgcCreateConnection::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}

//
void MgcCreateConnection::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

   data->insert(ptr);
}

///
bool MgcCreateConnection::containEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeResponseAcknowledgement));
    
    if ( ptr != 0 )
    {
        string s = ptr->getDecoded();
	if (s == "")
	  return( true );
    }

    return( false );
}

////
bool MgcCreateConnection::setBearerInformation(const MgcpBearerInformation& parameter)
{
    MgcpParameterBearerInformation* ptr;

    ptr = new MgcpParameterBearerInformation(parameter);

    data->insert(ptr);
    return true;
}

///
bool MgcCreateConnection::setRequestIdentifier(const string& parameter)
{
    MgcpParameterRequestId* ptr;

    ptr = new MgcpParameterRequestId(parameter);

    data->insert(ptr);
    return true;
}


//
bool MgcCreateConnection::setLocalConnectionOptions(const MgcpConnectionOptions& parameter)
{
    MgcpParameterLocalConnectionOptions* ptr;

    ptr = new MgcpParameterLocalConnectionOptions(parameter);

    data->insert(ptr);
    return true;
}

//
bool 
MgcCreateConnection::addRequestedEvents(const MgcRequestedEvent& parameter)
{
    return addOneRequestedEvent(parameter, data);
}


//
void MgcCreateConnection::clearRequestedEvents()
{
    data->clear(MgcpParameterTypeRequestedEvents);
}
//

//
bool 
MgcCreateConnection::addSignalRequests(const MgcSignalEvent& parameter)
{
    return addOneSignalRequest(parameter, data);
}


//
void MgcCreateConnection::clearSignalRequests()
{
    data->clear(MgcpParameterTypeSignalRequests);
}


//
bool MgcCreateConnection::setNotifiedEntity(const string& parameter)
{
    MgcpParameterNotifiedEntity* ptr;

    ptr = new MgcpParameterNotifiedEntity(parameter);

    data->insert(ptr);
    return true;
}
//
//

//////
bool MgcCreateConnection::setDigitMap(const string& parameter)
{
    MgcpParameterDigitMap* ptr;

    ptr = new MgcpParameterDigitMap(parameter);

    data->insert(ptr);
    return true;
}

//////
bool MgcCreateConnection::setSecondEndpointId(const string& parameter)
{
    MgcpParameterSecondEndpointId* ptr;
    
    ptr = new MgcpParameterSecondEndpointId(parameter);
    
    data->insert(ptr);
    return true;
}
//
//

///
bool MgcCreateConnection::setQuarantineHandling(
    MgcpNotificationHandling handling, 
    MgcpQuarantineTreatment treatment
    )
{
    MgcpParameterQuarantineHandling* ptr;
    
    ptr = new MgcpParameterQuarantineHandling(handling, treatment);
    
    data->insert(ptr);
    return true;
}

//
bool 
MgcCreateConnection::addDetectEvents(const MgcRequestedEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterDetectEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeDetectEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterDetectEvents;
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = 
	new MgcpParameterEventGeneric(parameter.getEvent());
	    
    ptr->insert(x);

    data->insert(ptr);
    return true;
}

//
void MgcCreateConnection::clearDetectEvents()
{
    data->clear(MgcpParameterTypeDetectEvents);
}

///
void MgcCreateConnection::setRemoteConnectionDescriptor(const SdpSession& parameter)
{
    MgcpParameterRemoteConnectionDescriptor* ptr;

    ptr = new MgcpParameterRemoteConnectionDescriptor(parameter);

    data->insert(ptr);
}

//
string MgcCreateConnection::getConnectionId()
{
    Sptr<MgcpParameterConnectionId> ptr;

    ptr.dynamicCast(responseMsg->retrieveParam(MgcpParameterTypeConnectionId));

    if(ptr == static_cast<MgcpParameterConnectionId*>(0))
    {
	cerr << "empty connectionid\n";
	return string();
    }
    else 
    {
	return ptr->getDecoded();
    }
}

/////
//string MgcCreateConnection::getSpecificEndpointId()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}

///
SdpSession MgcCreateConnection::getLocalConnectionDescriptor()
{
    MgcpParameterLocalConnectionDescriptor* ptr = 
	responseMsg->getLocalConnect();
    //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}

//
string 
MgcCreateConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}


/**********************************************************************/

/// constructor

MgcModifyConnection::MgcModifyConnection(const string& EndpointId, const string& CallId, const string& ConnectionId, const string& ConnectionMode)
    : data(new MgcpCommandModifyConnection)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
    {
        MgcpParameterCallId* ptr;

        ptr = new MgcpParameterCallId(CallId);

        data->insert(ptr);
    }
    {
        MgcpParameterConnectionId* ptr;

        ptr = new MgcpParameterConnectionId(ConnectionId);

        data->insert(ptr);
    }
    {
        MgcpParameterConnectionMode* ptr;

        ptr = new MgcpParameterConnectionMode(ConnectionMode);

        data->insert(ptr);
    }
}



int MgcModifyConnection::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}

void MgcModifyConnection::setRemoteConnectionDescriptor(SdpSession const &parameter)
{
    MgcpParameterRemoteConnectionDescriptor* ptr;

    ptr = new MgcpParameterRemoteConnectionDescriptor(parameter);

    data->insert(ptr);
}

//
void MgcModifyConnection::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
}

///
bool MgcModifyConnection::containEmptyResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;

    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeResponseAcknowledgement));
    
    if ( ptr != 0 )
    {
        string s = ptr->getDecoded();
	if (s == "")
	  return( true );
    }

    return( false );
}


////
bool MgcModifyConnection::setBearerInformation(const MgcpBearerInformation& parameter)
{
    MgcpParameterBearerInformation* ptr;

    ptr = new MgcpParameterBearerInformation(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
bool MgcModifyConnection::setRequestIdentifier(const string& parameter)
{
    MgcpParameterRequestId* ptr;

    ptr = new MgcpParameterRequestId(parameter);

    data->insert(ptr);
    return true;
}

//////
bool MgcModifyConnection::setLocalConnectionOptions(const MgcpConnectionOptions& parameter)
{
    MgcpParameterLocalConnectionOptions* ptr;

    ptr = new MgcpParameterLocalConnectionOptions(parameter);

    data->insert(ptr);
    return true;
}

//
bool 
MgcModifyConnection::addRequestedEvents(const MgcRequestedEvent& parameter)
{
    return addOneRequestedEvent(parameter, data);
}

//
void MgcModifyConnection::clearRequestedEvents()
{
    data->clear(MgcpParameterTypeRequestedEvents);
}


//
bool 
MgcModifyConnection::addSignalRequests(const MgcSignalEvent& parameter)
{
    return addOneSignalRequest(parameter, data);
}

////
void MgcModifyConnection::clearSignalRequests()
{
    data->clear(MgcpParameterTypeSignalRequests);

}
//
//////
bool MgcModifyConnection::setNotifiedEntity(const string& parameter)
{
    MgcpParameterNotifiedEntity* ptr;

    ptr = new MgcpParameterNotifiedEntity(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
bool MgcModifyConnection::setDigitMap(const string& parameter)
{
    MgcpParameterDigitMap* ptr;

    ptr = new MgcpParameterDigitMap(parameter);

    data->insert(ptr);
    return true;
}

///
bool MgcModifyConnection::setQuarantineHandling(
    MgcpNotificationHandling handling, 
    MgcpQuarantineTreatment treatment
    )
{
    MgcpParameterQuarantineHandling* ptr;
    
    ptr = new MgcpParameterQuarantineHandling(handling, treatment);
    
    data->insert(ptr);
    return true;
}

//
bool 
MgcModifyConnection::addDetectEvents(const MgcRequestedEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterDetectEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeDetectEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterDetectEvents;
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = 
	new MgcpParameterEventGeneric(parameter.getEvent());
	    
    ptr->insert(x);

//    cout << *ptr << "\n";
    data->insert(ptr);
    return true;
}


////
void MgcModifyConnection::clearDetectEvents()
{
    data->clear(MgcpParameterTypeDetectEvents);
}
//

///
SdpSession MgcModifyConnection::getLocalConnectionDescriptor()
{
    MgcpParameterLocalConnectionDescriptor* ptr = 
	responseMsg->getLocalConnect();
    //assert(ptr);

    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}

//
string 
MgcModifyConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

/**********************************************************************/

// constructor
MgcDeleteConnectionAgent::MgcDeleteConnectionAgent(const string& EndpointId)
    : data(new MgcpCommandDeleteConnectionAgent)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
}

//
void MgcDeleteConnectionAgent::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
}


////
bool MgcDeleteConnectionAgent::setBearerInformation(const MgcpBearerInformation& parameter)
{
    MgcpParameterBearerInformation* ptr;

    ptr = new MgcpParameterBearerInformation(parameter);

    data->insert(ptr);
    return true;
}

////
void MgcDeleteConnectionAgent::setCallId(const string& parameter)
{
    MgcpParameterCallId* ptr;

    ptr = new MgcpParameterCallId(parameter);

    data->insert(ptr);
}


//
void MgcDeleteConnectionAgent::setConnectionId(const string& parameter)
{
    MgcpParameterConnectionId* ptr;

    ptr = new MgcpParameterConnectionId(parameter);

    data->insert(ptr);
}

int MgcDeleteConnectionAgent::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}


////
bool MgcDeleteConnectionAgent::setRequestIdentifier(const string& parameter)
{
    MgcpParameterRequestId* ptr;

    ptr = new MgcpParameterRequestId(parameter);

    data->insert(ptr);
    return true;
}

//
bool 
MgcDeleteConnectionAgent::addRequestedEvents(const MgcRequestedEvent& parameter)
{
    return addOneRequestedEvent(parameter, data);
}


//
void MgcDeleteConnectionAgent::clearRequestedEvents()
{
    data->clear(MgcpParameterTypeRequestedEvents);
}


//
bool 
MgcDeleteConnectionAgent::addSignalRequests(const MgcSignalEvent& parameter)
{
    return addOneSignalRequest(parameter, data);
}

////
void MgcDeleteConnectionAgent::clearSignalRequests()
{
    data->clear(MgcpParameterTypeSignalRequests);
}
//
//////
bool MgcDeleteConnectionAgent::setNotifiedEntity(const string& parameter)
{
    MgcpParameterNotifiedEntity* ptr;

    ptr = new MgcpParameterNotifiedEntity(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
//MgcDeleteConnectionAgent::setReasonCode(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//
//////
bool MgcDeleteConnectionAgent::setDigitMap(const string& parameter)
{
    MgcpParameterDigitMap* ptr;

    ptr = new MgcpParameterDigitMap(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
//MgcDeleteConnectionAgent::setConnectionParameters(const string& parameter)
//{
//    inactive* ptr;
//
//    ptr = new inactive(parameter);
//
//   data->insert(ptr);
//}
//
//
//////
bool MgcDeleteConnectionAgent::setQuarantineHandling(
    MgcpNotificationHandling handling, 
    MgcpQuarantineTreatment treatment
    )
{
    MgcpParameterQuarantineHandling* ptr;
    
    ptr = new MgcpParameterQuarantineHandling(handling, treatment);
    
    data->insert(ptr);
    return true;
}

//
bool 
MgcDeleteConnectionAgent::addDetectEvents(const MgcRequestedEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterDetectEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeDetectEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterDetectEvents;
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = 
	new MgcpParameterEventGeneric(parameter.getEvent());
	    
    ptr->insert(x);

//    cout << *ptr << "\n";
    data->insert(ptr);
    return true;
}
////
void MgcDeleteConnectionAgent::clearDetectEvents()
{
    data->clear(MgcpParameterTypeDetectEvents);
}

//
string 
MgcDeleteConnectionAgent::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

///////
//string MgcDeleteConnectionAgent::getConnectionParameters()
//{
//    inactive* ptr;
//    ptr = dynamic_cast<inactive*>
//	(data->retrieve(inactive));
//    return ptr->getDecoded();
//}
////

/**********************************************************************/

// constructor

MgcNotificationRequest::MgcNotificationRequest(const string& EndpointId, const string& RequestIdentifier)
    : data(new MgcpCommandNotificationRequest)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
    {
	MgcpParameterRequestId* ptr;

	ptr = new MgcpParameterRequestId(RequestIdentifier);

	data->insert(ptr);
    }
}

//
void MgcNotificationRequest::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
}

///
int MgcNotificationRequest::getResponseCode(void)
{
    return responseMsg->getResponseCode();
}



////
bool MgcNotificationRequest::setBearerInformation(const MgcpBearerInformation& parameter)
{
    MgcpParameterBearerInformation* ptr;

    ptr = new MgcpParameterBearerInformation(parameter);

    data->insert(ptr);
    return true;
}

//
bool 
MgcNotificationRequest::addRequestedEvents(const MgcRequestedEvent& parameter)
{
    return addOneRequestedEvent(parameter, data);
}


//
void MgcNotificationRequest::clearRequestedEvents()
{
    data->clear(MgcpParameterTypeRequestedEvents);
}


//
bool 
MgcNotificationRequest::addSignalRequests(const MgcSignalEvent& parameter)
{
    return addOneSignalRequest(parameter, data);
}

///
void MgcNotificationRequest::clearSignalRequests()
{
    data->clear(MgcpParameterTypeSignalRequests);
}
//
//////
bool MgcNotificationRequest::setNotifiedEntity(const string& parameter)
{
    MgcpParameterNotifiedEntity* ptr;

    ptr = new MgcpParameterNotifiedEntity(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
bool MgcNotificationRequest::setDigitMap(const string& parameter)
{
    MgcpParameterDigitMap* ptr;

    ptr = new MgcpParameterDigitMap(parameter);

    data->insert(ptr);
    return true;
}
//
//
//////
bool MgcNotificationRequest::setQuarantineHandling(
    MgcpNotificationHandling handling, 
    MgcpQuarantineTreatment treatment
    )
{
    MgcpParameterQuarantineHandling* ptr;
    
    ptr = new MgcpParameterQuarantineHandling(handling, treatment);
    
    data->insert(ptr);
    return true;
}

//
bool 
MgcNotificationRequest::addDetectEvents(const MgcRequestedEvent& parameter)
{
    /// presume it's empty
    Sptr<MgcpParameterDetectEvents> ptr;

    ptr.dynamicCast(data->retrieveParam(MgcpParameterTypeDetectEvents));

    if(!ptr)
    {
	ptr = new MgcpParameterDetectEvents;
    }
    
    // now do something with the Requested Events... insert them.
    
    MgcpParameterEventGeneric* x = 
	new MgcpParameterEventGeneric(parameter.getEvent());
	    
    ptr->insert(x);

//    cout << *ptr << "\n";
    data->insert(ptr);
    return true;
}


////
void MgcNotificationRequest::clearDetectEvents()
{
    data->clear(MgcpParameterTypeDetectEvents);
}

//
string 
MgcNotificationRequest::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

/**********************************************************************/

/////
Sptr<MgcEvent> MgcNotify::getItemObservedEvents(int index)
{
    MgcpParameterObservedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterObservedEvents*>
        (data->retrieve(MgcpParameterTypeObservedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);

	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgcEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgcEvent;
}

//
int 
MgcNotify::getNumObservedEvents(void)
{
    MgcpParameterObservedEvents* ptr;
    ptr = dynamic_cast<MgcpParameterObservedEvents*>
        (data->retrieve(MgcpParameterTypeObservedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


//
void 
MgcNotify::setResponseCode(MgcpResponseCode code)
{
    responseMsg->setResponseCode(code);
}

//
void 
MgcNotify::setResponseString(string response)
{
    responseMsg->setComment(response);
}

//
string 
MgcNotify::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

//
string 
MgcNotify::getResponseAck()
{
    MgcpParameterResponseAcknowledgement* ptr = 0;
    ptr = dynamic_cast<MgcpParameterResponseAcknowledgement*>
	(data->retrieve(MgcpParameterTypeResponseAcknowledgement));
    if ( !ptr ) { return string(""); }  // there is no resp ack attribute
    return ptr->getDecoded();
}

//
string 
MgcNotify::getRequestIdentifier()
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr)
    {
	// error -- not found...
	return string("");
    }
    return ptr->getDecoded();
}

//
string 
MgcNotify::getNotifiedEntity()
{
    MgcpParameterNotifiedEntity* ptr;
    ptr = dynamic_cast<MgcpParameterNotifiedEntity*>
	(data->retrieve(MgcpParameterTypeNotifiedEntity));
    if(!ptr)
    {
	// error -- not found...
	return string("");
    }
    return ptr->getDecoded();
}


/********************************************************************
 class MgcAuditEndpoint methods
 **************************************************************/
///
MgcAuditEndpoint::MgcAuditEndpoint(const string& EndpointId)
    : data(new MgcpCommandAuditEndpoint)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
}

///
void MgcAuditEndpoint::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

    data->insert(ptr);
}


///
bool MgcAuditEndpoint::addToRequestedInfo( MgcpParameterType type )
{
    /// presume it's empty
    Sptr<MgcpParameterRequestedInfo> ptr;

    ptr.dynamicCast(data->retrieveParam( MgcpParameterTypeRequestedInfo ));

    if(!ptr)
    {
	ptr = new MgcpParameterRequestedInfo;
	
	data->insert(ptr);
    }
    
    // now insert the parameter type into RequestedInfo
    if ( (type != MgcpParameterTypeUnknownParameter ) &&
	 (type != MgcpParameterTypeMaxValue ))
        ptr->insert( type );

    return( true );
}


///
string MgcAuditEndpoint::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}


int MgcAuditEndpoint::getNumEndpointIds()

{
    MgcpParameterEndpointIdList*  ptr;
    //   ptr = dynamic_cast<MgcpParameterEndpointIdList*> (data->retrieve(MgcpParameterTypeEndpointIdList));
    
    ptr = dynamic_cast<MgcpParameterEndpointIdList*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeEndpointIdList));

    if(!ptr)
    {
	// no such parameter in the map
	return 0;
    }
    else
    {
	// get the number of EndpointIds in the EndpointIdList
	return ptr->getDecodedNum();
    }
}


string MgcAuditEndpoint::getItemEndpointIdList( int index )
{
    MgcpParameterEndpointIdList* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointIdList*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeEndpointIdList));

    if(!ptr)
    {
	// no such parameter at all, should first check the size !!!
	assert(0);
	return( 0 );
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEndpointId* id_ptr = ptr->getDecodedItem(index);
	if ( id_ptr != 0 )
	  return ( id_ptr->getDecoded() );
	else
	  return( 0 );
    }
}

//
string MgcAuditEndpoint::getConnectionId()
{
    Sptr<MgcpParameterConnectionId> ptr;
    ptr.dynamicCast(responseMsg->retrieveParam(MgcpParameterTypeConnectionId));

    if(0 == ptr)
    {
	cerr << "empty connection id\n";
	return string("");
    }

    return ptr->getDecoded();
}


int MgcAuditEndpoint::getNumEndPoints()
{
    Sptr<MgcpParameterNumEndPoints> ptr;
    ptr.dynamicCast(responseMsg->retrieveParam(MgcpParameterTypeNumEndPoints));

    if(0 == ptr)
    {
	cerr << "empty connection id\n";
	return 0;
    }

    return ptr->getDecoded();
}


bool MgcAuditEndpoint::setMaxEndPointIds(int parameter)
{
    MgcpParameterMaxEndPointIds* ptr;

    ptr = new MgcpParameterMaxEndPointIds(parameter);
    
    data->insert(ptr);
	return true;

    
}






///Get the Num of ConnectionIds in the ConnectionIdList
int MgcAuditEndpoint::getNumConnectionIds()

{
    MgcpParameterConnectionId*  ptr;
    //   ptr = dynamic_cast<MgcpParameterEndpointIdList*> (data->retrieve(MgcpParameterTypeEndpointIdList));
    
    ptr = dynamic_cast<MgcpParameterConnectionId*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionId));

    if(!ptr)
    {
	// no such parameter in the map
	return 0;
    }
    else
    {
	// get the number of EndpointIds in the EndpointIdList
	return ptr->getDecodedNum();
    }
}


string
MgcAuditEndpoint::getItemConnectionId( int index )
{
  MgcpParameterConnectionId* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionId*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionId));

    if(!ptr)
	{
      assert(0);
      return( 0 );
    }
    else
    {
	  // string id = ptr->getDecodedItem( index );
      return( ptr->getDecodedItem( index ) );
	  /*   if( id != 0 )
        return( id_ptr->getDe );
      else
	  return( 0 ); */
	}
}

/********************************************************
       APIs FOR CAPABILITIES
**************************************************************/
bool
MgcAuditEndpoint::getIsPacketRange()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
        return( ptr->isPacketizationRange() );
    }
}

int
MgcAuditEndpoint::getPacketization()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
        return( ptr->getPacketization() );
    }
}

void
MgcAuditEndpoint::getPacketRange(int* p1, int* p2)
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	  return( ptr->getPacketizationRange(p1, p2) );
	}
}

bool
MgcAuditEndpoint::getIsBandwidthRange()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	  return( ptr->isBandwidthRange() );
    }
}

int
MgcAuditEndpoint::getBandwidth()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
        return( ptr->getBandwidth() );
    }
}

void
MgcAuditEndpoint::getBandwidthRange(int* p1, int* p2)
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
        return( ptr->getBandwidthRange(p1, p2) );
    }
}

int
MgcAuditEndpoint::getTypeOfService()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
        return( ptr->getTypeofService() );
    }
}

bool
MgcAuditEndpoint::getEchoCancellation()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getEchoCancellation() );
    }
}

bool
MgcAuditEndpoint::getSilenceSuppression()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getSilenceSuppression() );
    }
}


int
MgcAuditEndpoint::getNumComp()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getCompressionAlgoNum() );
    }
}

string
MgcAuditEndpoint::getCompAlgoItem(int k)
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getCompressionAlgoItem(k) );
    }
}

int
MgcAuditEndpoint::getNumModes()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getConnectionModeNum() );
    }
}


string
MgcAuditEndpoint::getModeItem(int k)
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getConnectionModeItem(k) );
    }
}

int
MgcAuditEndpoint::getNumEvents()
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getEventPackageNum() );
    }
}


MgcpPackageType
MgcAuditEndpoint::getEventItem(int k)
{
  MgcpParameterCapabilities* ptr;
  ptr = dynamic_cast<MgcpParameterCapabilities*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCapabilities));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getEventPackageItem(k) );
    }
}



/**************************************************************
      END CAPABILITIES APIs
***************************************************************/


/// 
MgcpBearerInformation MgcAuditEndpoint::getBearerInformation()
{
    MgcpParameterBearerInformation* ptr;
    ptr = dynamic_cast<MgcpParameterBearerInformation*>
   (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeBearerInformation));
    return ptr->getDecoded();
}

///
string MgcAuditEndpoint::getRequestIdentifier()
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(data->retrieve(MgcpParameterTypeRequestId));
    if(!ptr) { return string(""); }
    return ptr->getDecoded();
}

///
MgcpConnectionOptions MgcAuditEndpoint::getLocalConnectionOptions()
{
    MgcpParameterLocalConnectionOptions* ptr;
    ptr = dynamic_cast<MgcpParameterLocalConnectionOptions*>
	(data->retrieve(MgcpParameterTypeLocalConnectionOptions));
    if(!ptr)
       return MgcpConnectionOptions();
    return ptr->getDecoded();
}

/// 
int MgcAuditEndpoint::getNumRequestedEvents()
{
    MgcpParameterRequestedEvents* ptr;
	//  ptr = dynamic_cast<MgcpParameterRequestedEvents*>
	//     (data->retrieve(MgcpParameterTypeRequestedEvents));
    
    ptr = dynamic_cast<MgcpParameterRequestedEvents*>(dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeRequestedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


////
Sptr<MgcEvent> MgcAuditEndpoint::getItemRequestedEvents(int index)
  //getOneMgRequestedEvent(int index)
{ 

   MgcpParameterRequestedEvents* ptr;

   ptr = dynamic_cast<MgcpParameterRequestedEvents*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeRequestedEvents)); 
   //  ptr = dynamic_cast<MgcpParameterRequestedEvents*>
   //     (data->retrikieve(MgcpParameterTypeRequestedEvents));
    
   if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterRequestedEvent* event = ptr->getDecodedItem(index);

	MgcpParameterRequestedEventGeneric* p = dynamic_cast<MgcpParameterRequestedEventGeneric*>(event);

	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgcEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgcEvent;
}
//


///
int MgcAuditEndpoint::getNumSignalRequests()
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeSignalRequests ));


    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}


///
Sptr<MgcEvent>  MgcAuditEndpoint::getItemSignalRequests(int index)
{
    MgcpParameterSignalRequests* ptr;
    ptr = dynamic_cast<MgcpParameterSignalRequests*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeSignalRequests ));

    if( !ptr )
    {
	// no such parameter at all, should first check the size
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgcEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    
    return 0;
}

///
int MgcAuditEndpoint::getNumEventStates()
{
    MgcpParameterEventStates* ptr;
    ptr = dynamic_cast<MgcpParameterEventStates*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeEventStates ));


    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}

///
Sptr<MgcEvent>  MgcAuditEndpoint::getItemEventStates(int index)
{
    MgcpParameterEventStates* ptr;
    ptr = dynamic_cast<MgcpParameterEventStates*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeEventStates ));

    if( !ptr )
    {
	// no such parameter at all, should first check the size
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterSignal* event = ptr->getDecodedItem(index);

	MgcpParameterSignalGeneric* p = dynamic_cast<MgcpParameterSignalGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    // use the generic convert function
            return( getOneMgcEvent( ge ) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    
    return 0;
}



///
int MgcAuditEndpoint::getNumObservedEvents()
{
    MgcpParameterObservedEvents* ptr;
	//   ptr = dynamic_cast<MgcpParameterObservedEvents*>
    //    (data->retrieve(MgcpParameterTypeObservedEvents));
    
     ptr = dynamic_cast<MgcpParameterObservedEvents*>(dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeObservedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}

/// 
Sptr<MgcEvent> MgcAuditEndpoint::getItemObservedEvents(int index)
{
    MgcpParameterObservedEvents* ptr;
	//   ptr = dynamic_cast<MgcpParameterObservedEvents*>
    //    (data->retrieve(MgcpParameterTypeObservedEvents));
    
    ptr = dynamic_cast<MgcpParameterObservedEvents*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeObservedEvents));
    if(!ptr)
    {
	// this is nothing... is that an error?
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);

	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgcEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return new MgcEvent;
}

///
string MgcAuditEndpoint::getDigitMap()
{
    MgcpParameterDigitMap* ptr;
    ptr = dynamic_cast<MgcpParameterDigitMap*>
	(dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeDigitMap));
    return ptr->getDecoded();

}

///
string MgcAuditEndpoint::getNotifiedEntity()
{
    MgcpParameterNotifiedEntity* ptr;
    ptr = dynamic_cast<MgcpParameterNotifiedEntity*>
	(dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeNotifiedEntity));
    return ptr->getDecoded();
}


///
string MgcAuditEndpoint::getRequestId()
{
    MgcpParameterRequestId* ptr;
    ptr = dynamic_cast<MgcpParameterRequestId*>
	(dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeRequestId));
    return ptr->getDecoded();
}


///
MgcpQuarantineTreatment MgcAuditEndpoint::getTreatmentQuarantineHandling()
{

    MgcpParameterQuarantineHandling* ptr;
    ptr = dynamic_cast<MgcpParameterQuarantineHandling*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeQuarantineHandling ));

    if(!ptr)
    {
	// no such parameter at all, should first check the size !!!
	assert(0);
	return( MgcpQuarantineTreatmentProcess );  // return default
    }
    else
    {
	return( ptr->getTreatmentQuarantineHandling() );
    }
}

///
MgcpNotificationHandling MgcAuditEndpoint::getNotificationQuarantineHandling()
{
    MgcpParameterQuarantineHandling* ptr;
    ptr = dynamic_cast<MgcpParameterQuarantineHandling*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeQuarantineHandling ));

    if(!ptr)
    {
	// no such parameter at all, should first check the size !!!
	assert(0);
	return( MgcpNotificationHandlingStep );  // return default
    }
    else
    {
	return( ptr->getNotificationQuarantineHandling() );
    } 
}


///
int MgcAuditEndpoint::getNumDetectEvents()
{
    MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeDetectEvents ));

    if( !ptr )
    {
	// there is nothing
	return 0;
    }
    else
    {
	// need to decode an individual item
	return ptr->getDecodedNum();
    }
}

///
Sptr<MgcEvent> MgcAuditEndpoint::getItemDetectEvents(int index)
{
   MgcpParameterDetectEvents* ptr;
    ptr = dynamic_cast<MgcpParameterDetectEvents*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve( MgcpParameterTypeDetectEvents ));

    if( !ptr )
    {
	// no such parameter, should first check the size !!
	assert(0);
	return 0;
    }
    else
    {
	// need to decode an individual item
	MgcpParameterEvent* event = ptr->getDecodedItem(index);

	MgcpParameterEventGeneric* p = dynamic_cast<MgcpParameterEventGeneric*>(event);
	if(p) 
	{
//	    cout << "got generic event\n";
	    MgcpGenericEvent ge(p->getGenericEvent());

	    return( getOneMgcEvent(ge) );
	}
	else
	{
//	    cout << "did not get generic event\n";
	}
    }
    cout << "got UnknownEvent\n";
    return 0;
}



/**********************************************************************
 MgcAuditConnection methods
 **********************************************************************/
/// 
MgcAuditConnection::MgcAuditConnection(const string& EndpointId, const string& ConnectionId)
    : data(new MgcpCommandAuditConnection)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
    {
        MgcpParameterConnectionId* ptr;

        ptr = new MgcpParameterConnectionId(ConnectionId);

        data->insert(ptr);
    }
}

MgcAuditConnection::MgcAuditConnection(const string& EndpointId)
    : data(new MgcpCommandAuditConnection)
{
    {
        MgcpParameterEndpointId* ptr;

        ptr = new MgcpParameterEndpointId(EndpointId);

        data->insert(ptr);
    }
}

//
void MgcAuditConnection::setResponseAck(const string& parameter)
{
    MgcpParameterResponseAcknowledgement* ptr;

    ptr = new MgcpParameterResponseAcknowledgement(parameter);

   data->insert(ptr);
}

bool MgcAuditConnection::addToRequestedInfo( MgcpParameterType type )
{
    /// presume it's empty
    Sptr<MgcpParameterRequestedInfo> ptr;

    ptr.dynamicCast(data->retrieveParam( MgcpParameterTypeRequestedInfo ));

    if(!ptr)
    {
	ptr = new MgcpParameterRequestedInfo;
	
	data->insert(ptr);
    }
    
    // now insert the parameter type into RequestedInfo
    if ( (type != MgcpParameterTypeUnknownParameter ) &&
	 (type != MgcpParameterTypeMaxValue ))
        ptr->insert( type );

    return( true );
}

///
string MgcAuditConnection::getCallId()
{
    MgcpParameterCallId* ptr;
     ptr = dynamic_cast<MgcpParameterCallId*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeCallId));
	if( !ptr )
	{
        assert(0);
        return string("");
	}
    return ptr->getDecoded();
}

///
string 
MgcAuditConnection::getNotifiedEntity()
{
    MgcpParameterNotifiedEntity* ptr;
    ptr = dynamic_cast<MgcpParameterNotifiedEntity*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeNotifiedEntity));
	if( !ptr )
	{
        assert(0);
        return string("");
	}
    
    return ptr->getDecoded();
}

//
string 
MgcAuditConnection::getConnectionId()
{
    Sptr<MgcpParameterConnectionId> ptr;
    ptr.dynamicCast(responseMsg->retrieveParam(MgcpParameterTypeConnectionId));

    if(0 == ptr)
    {
	cerr << "empty connection id\n";
	return string("");
    }

    return ptr->getDecoded();
}


//
MgcpConnectionOptions 
MgcAuditConnection::getLocalConnectionOptions()
{
    MgcpParameterLocalConnectionOptions* ptr;
   ptr = dynamic_cast<MgcpParameterLocalConnectionOptions*> 
             (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeLocalConnectionOptions));
   
    if(!ptr) return MgcpConnectionOptions();
    return ptr->getDecoded();
}

///////
string MgcAuditConnection::getConnectionMode()
{
    MgcpParameterConnectionMode* ptr;
	ptr = dynamic_cast<MgcpParameterConnectionMode*> 
              (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionMode));
  
    if(!ptr) return string("");
    return ptr->getDecoded();

}
///////
int
MgcAuditConnection::getPacketsSent()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getPacketsSent() );
    }
}


int
MgcAuditConnection::getOctetsSent()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getOctetsSent() );
    }
}


int
MgcAuditConnection::getPacketsRcvd()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getPacketsRcvd() );
    }
}


int
MgcAuditConnection::getOctetsRcvd()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getOctetsRcvd() );
    }
}



int
MgcAuditConnection::getPacketsLost()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getPacketsLost() );
    }
}

int
MgcAuditConnection::getJitter()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getJitter() );
    }
}


int
MgcAuditConnection::getLatency()
{
  MgcpParameterConnectionParameters* ptr;
  ptr = dynamic_cast<MgcpParameterConnectionParameters*>
        (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeConnectionParameters));

    if(!ptr)
    {
        assert(0);
    }
    else
    {
	return( ptr->getLatency() );
    }
}


//
SdpSession MgcAuditConnection::getRemoteConnectionDescriptor()
{
    MgcpParameterRemoteConnectionDescriptor* ptr;
     ptr = dynamic_cast<MgcpParameterRemoteConnectionDescriptor*> 
         (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeRemoteConnectionDescriptor));    
	 //   ptr = dynamic_cast<MgcpParameterRemoteConnectionDescriptor*>
     //   (data->retrieve(MgcpParameterTypeRemoteConnectionDescriptor));
    //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}

//
SdpSession MgcAuditConnection::getLocalConnectionDescriptor()
{
    MgcpParameterLocalConnectionDescriptor* ptr;
     ptr = dynamic_cast<MgcpParameterLocalConnectionDescriptor*> 
         (dynamic_cast<MgcpResponse*>(this->getResponsePtr())->retrieve(MgcpParameterTypeLocalConnectionDescriptor)); 
	//	responseMsg->getLocalConnect(); 
     //assert(ptr);
    if(!ptr)
    {
	// do something
	SdpSession junk;
	return junk;
    }
    else
    {
	return ptr->getDecoded();
    }
}

//
string 
MgcAuditConnection::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

/**********************************************************************/


string MgcRestartInProgress::getRestartDelay()
{
    return string("");
}


string MgcRestartInProgress::getRestartMethod()
{
    return string("");
}


string MgcRestartInProgress::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}

void MgcRestartInProgress::setResponseCode(MgcpResponseCode code)
{
    responseMsg->setResponseCode(code);
}

void MgcRestartInProgress::setResponseString(string response)
{
    responseMsg->setComment(response);
}


/**********************************************************************/

void MgcDeleteConnectionGw::setResponseCode(MgcpResponseCode code)
{
    responseMsg->setResponseCode(code);
}

void MgcDeleteConnectionGw::setResponseString(string response)
{
    responseMsg->setComment(response);
}

//
string 
MgcDeleteConnectionGw::getEndpointId()
{
    MgcpParameterEndpointId* ptr;
    ptr = dynamic_cast<MgcpParameterEndpointId*>
	(data->retrieve(MgcpParameterTypeEndpointId));
    return ptr->getDecoded();
}
