#ifndef MGC_COMMAND_HXX
#define MGC_COMMAND_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcCommand_hxx_Version = 
"$Id: MgcCommand.hxx,v 1.5 2001/01/26 02:46:03 liuhong Exp $";



#include "global.h"
#include "MgcTypes.hxx"
#include "MgcPackage.hxx"
#include "MgcpCommand.hxx"
#include "MgcEvent.hxx"
#include "Sptr.hxx"
#include "MgcpConnectionOptions.hxx"

/// holds a command message that is sent or received from the gateway
class MgcCommand
{
public:
    /// Create an empty message
    MgcCommand(const MgcCommand& x) : responseMsg(x.responseMsg) {};

    /// Create an empty message
    MgcCommand() : responseMsg(new MgcpResponse) {};
   
    /// Create a command in response to a previous message 
    MgcCommand(MgcpCommand* message)
    { 
	 MgcpResponse* p = dynamic_cast<MgcpResponse*>(message);
	 if(p) {responseMsg = p;}
    };

    /// Delete the command and the response it contains 
    virtual ~MgcCommand();

    /// make a copy of the MgCommand of the appropriate type
    virtual MgcCommand* copy() {
	return new MgcCommand(*this);
    };

    /// Return the type of the command 
    virtual MgcCommandType getType() { return MgcInvalidCommandType; };

    /// Set the code indicating an error occurent in processing this command
//    void setResponseCode(MgcpResponseCode code);

    /// Set the string describing the error that occcured
//    void setResponseString(string response);

    /// Get a pointer to the raw text of the message
    virtual MgcpCommand* getDataPtr() { return 0; }

    /// Get a point to the response message
        MgcpCommand* getResponsePtr() { return responseMsg.getPtr(); }

    /// Get a point to the response message
	// Sptr<MgcpCommand> getResponseSptr() { return responseMsg; }
	 Sptr<MgcpResponse> getResponseSptr() { return responseMsg; }

    /// Set the response message 
        void setResponsePtr(MgcpResponse* p) { responseMsg = p; }

    /// Set the response message 
    void setResponseSptr(const Sptr<MgcpResponse>& p) { responseMsg = p; }

    /// Return the type of the command 
    virtual string getEndpointId() { return string(); };

    /// 
    virtual bool addRequestedEvents(const MgcRequestedEvent& parameter) { return true; }

    virtual bool addSignalRequests(const MgcSignalEvent& parameter) { return true; }

protected:
    /// Copy of the response message 
    Sptr<MgcpResponse> responseMsg;
};



/// holds an Endpoing Configure message sent to the gateway
class MgcEndpointConfiguration : public MgcCommand
{
public: 
    /// Create a message to an specific endpoint
    MgcEndpointConfiguration(const string& endpointID);

    /// Create a message to an specific endpoint
    MgcEndpointConfiguration(const MgcEndpointConfiguration& x) : MgcCommand(x), data(x.data) {};

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcEndpointConfiguration(MgcpCommand* x)
    { 
       MgcpCommandEndpointConfiguration* p
	  = dynamic_cast<MgcpCommandEndpointConfiguration*>(x);
       if (p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcEndpointConfiguration(*this);
    };
    
    /// Get the type of the command
    MgcCommandType getType() { return MgcEndpointConfigurationType; };

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    /// Get the response error code 
    int getResponseCode()
    { return responseMsg->getResponseCode(); }
    
    /// Get the description of any response error 
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandEndpointConfiguration> data;

};



/// message to create a new connection. Is sent to the gateway
class MgcCreateConnection : public MgcCommand
{
public: 
    /// create a connection on the given endpoint. Set the callID and mode
    MgcCreateConnection(const string& endpointID,
			const string& callId,
			const string& connectionMode);

    ///
    MgcCreateConnection(const MgcCreateConnection& x) : MgcCommand(x), data(x.data) {}
    
    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    /// Create by coping another message 
    MgcCreateConnection(MgcpCommand* x)
    { 
        MgcpCommandCreateConnection* p
	   = dynamic_cast<MgcpCommandCreateConnection*>(x);
        if (p) {data = p;}
    };

    ///
    MgcCreateConnection() : MgcCommand() {
        MgcpCommandCreateConnection* p
	   = new MgcpCommandCreateConnection;
        if (p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcCreateConnection(*this);
    };
    
    ///  Get the type of the command
    MgcCommandType getType() { return MgcCreateConnectionType; };

    ///
    int getTransactionId() { return( data->getTransactionId() ); }

    ///
    void setTransactionId( int x ) { 
      data->setTransactionId(x);
      responseMsg->setTransactionId(x); 
    }

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    /// check to see if there is an empty response ack parameter
    /// if there is one, call agent has to send an ack for receiving 
    /// this final response for createconnection
    bool containEmptyResponseAck();

    /// set response code to ack the final response with empty k: line
    void setResponseCode( MgcpResponseCode code ) {
      responseMsg->setResponseCode( code );
    }

    /// Set the optional bearer information 
    bool setBearerInformation(const MgcpBearerInformation& parameter);

    /// Set the request identifier that will be passed back  
    bool setRequestIdentifier(const string& parameter);

    /// Set locacal connetion paramters 
    bool setLocalConnectionOptions(const MgcpConnectionOptions& parameter);

    /// Add an event that we wish to be notified about 
    bool addRequestedEvents(const MgcRequestedEvent& parameter);
    /// Clear list of evnets that have been added to this message
    void clearRequestedEvents();

    /// Add a signal that we wish the gateway to produce 
    bool addSignalRequests(const MgcSignalEvent& parameter);
    /// Clear the list of signals that have added to this message 
    void clearSignalRequests();

    /// Set who should be notified about events 
    bool setNotifiedEntity(const string& parameter);

    /// Specifiy a digimap for the gateway to collect 
    bool setDigitMap(const string& parameter);

    /// Specify the other enpoint of this call 
    bool setSecondEndpointId(const string& parameter);

    /// Specify how to deal with quarantined events 
    bool setQuarantineHandling(MgcpNotificationHandling handling, 
			       MgcpQuarantineTreatment treatment);

    /// Add list of ents that have been detected 
    bool addDetectEvents(const MgcRequestedEvent& parameter);
    /// Clear the list of detected events that have been added to this message
    void clearDetectEvents();

    /// Set the information descriping the connection data of the remote side
    void setRemoteConnectionDescriptor(const SdpSession& parameter);

    /// Get the ID of this connection 
    string getConnectionId();

    ///
//    string getSpecificEndpointId();

    /// Get the connection protocol information for this endpoint 
    SdpSession getLocalConnectionDescriptor();
    
    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();
    
private:
    /// Holds the data for this command 
    Sptr<MgcpCommandCreateConnection> data;
};



/// Modify the paramter of an connection 
class MgcModifyConnection : public MgcCommand
{
public: 
    /// Create a modification for a given endpoint 
    MgcModifyConnection(const string& EndpointId,
			const string& CallId,
			const string& ConnectionId,
			const string& ConnectionMode);

    ///
    MgcModifyConnection(const  MgcModifyConnection& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcModifyConnection(MgcpCommand* x)
    { 
        MgcpCommandModifyConnection* p
	   = dynamic_cast<MgcpCommandModifyConnection*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcModifyConnection(*this);
    };

    ///  Get the type of the command
    MgcCommandType getType() { return MgcModifyConnectionType; };

    ///
    int getTransactionId() { return( data->getTransactionId() ); }

    ///
    void setTransactionId( int x ) { 
      data->setTransactionId(x);
      responseMsg->setTransactionId(x); 
    }

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    /// check to see if there is an empty response ack parameter
    /// if there is one, call agent has to send an ack for receiving 
    /// this final response for createconnection
    bool containEmptyResponseAck();

    /// set response code to ack the final response with empty k: line
    void setResponseCode( MgcpResponseCode code ) {
      responseMsg->setResponseCode( code );
    }

    /// Set the optional bearer information 
    bool setBearerInformation(const MgcpBearerInformation& parameter);

    /// Set the optional request identifier 
    bool setRequestIdentifier(const string& parameter);

    /// Set locacal connetion paramters 
    bool setLocalConnectionOptions(const MgcpConnectionOptions& parameter);

    /// Add an event that we wish to be notified about 
    bool addRequestedEvents(const MgcRequestedEvent& parameter);
    ///Clear list of evnets that have been added to this message
    void clearRequestedEvents();

    ///Add a signal that we wish the gateway to produce 
    bool addSignalRequests(const MgcSignalEvent& parameter);
    ///Clear the list of signals that have added to this message
    void clearSignalRequests();

    ///Set who should be notified about events
    bool setNotifiedEntity(const string& parameter);

    ///Specifiy a digimap for the gateway to collect
    bool setDigitMap(const string& parameter);
    
    /// Specify how to deal with quarantined events 
    bool setQuarantineHandling(MgcpNotificationHandling handling, 
			       MgcpQuarantineTreatment treatment);

    ///Add list of ents that have been detected 
    bool addDetectEvents(const MgcRequestedEvent& parameter);
    ///Clear the list of detected events that have been added to this message
    void clearDetectEvents();

    ///Set the information descriping the connection data of the remote side
    void setRemoteConnectionDescriptor(const SdpSession& parameter);

    /// Get the connection protocol information for this endpoint 
    SdpSession getLocalConnectionDescriptor();

    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();
private:
    ///Holds the data for this command 
    Sptr<MgcpCommandModifyConnection> data;

};



/// This message is sent from the agen to the gateway to end a call
class MgcDeleteConnectionAgent : public MgcCommand
{
public: 
    /// message to delete the call at the specified endpoint 
    MgcDeleteConnectionAgent(const string& EndpointId);

    ///
    MgcDeleteConnectionAgent(const  MgcDeleteConnectionAgent& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcDeleteConnectionAgent(MgcpCommand* x)
    { 
        MgcpCommandDeleteConnectionAgent* p
	   = dynamic_cast<MgcpCommandDeleteConnectionAgent*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcDeleteConnectionAgent(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcDeleteConnectionAgentType; };


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    /// Set the optional bearer information 
    bool setBearerInformation(const MgcpBearerInformation& parameter);

    /// Specify the call id 
    void setCallId(const string& parameter);

    /// specify the connection id 
    void setConnectionId(const string& parameter);

    /// Set the optional request identifier 
    bool setRequestIdentifier(const string& parameter);

    /// Add an event that we wish to be notified about 
    bool addRequestedEvents(const MgcRequestedEvent& parameter);
    /// Clear the list of signals that have added to this message 
    void clearRequestedEvents();

    /// Add a signal that we wish the gateway to produce 
    bool addSignalRequests(const MgcSignalEvent& parameter);
    ///Clear the list of signals that have added to this message 
    void clearSignalRequests();

    /// Set who should be notified about events 
    bool setNotifiedEntity(const string& parameter);

    /// Specify why the call is being ended 
//    setReasonCode(const string& parameter);

    /// Specifiy a digimap for the gateway to collect
    bool setDigitMap(const string& parameter);

    ///
//    setConnectionParameters(const string& parameter);

    /// Specify how to deal with quarantined events
    bool setQuarantineHandling(MgcpNotificationHandling handling, 
			       MgcpQuarantineTreatment treatment);

    /// Add list of ents that have been detected 
    bool addDetectEvents(const MgcRequestedEvent& parameter);
    /// Clear the list of detected events that have been added to this message
    void clearDetectEvents();

    /// Get parameters describing actual connection 
//    string getConnectionParameters();

    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error 
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();
    
private:
    /// Holds the data for this command 
    Sptr<MgcpCommandDeleteConnectionAgent> data;

};


/// Message to tell the gateway to notify you of certian events
class MgcNotificationRequest : public MgcCommand
{
public: 
    /// Create message to enpoint. RequestId will be retunred in notificaion
    MgcNotificationRequest(const string& endpointID,
			   const string& requestIdentifier);

    ///
    MgcNotificationRequest(const  MgcNotificationRequest& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcNotificationRequest(MgcpCommand* x)
    { 
        MgcpCommandNotificationRequest* p
	   = dynamic_cast<MgcpCommandNotificationRequest*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcNotificationRequest(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcNotificationRequestType; };


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    ///  Set the optional bearer information 
    bool setBearerInformation(const MgcpBearerInformation& parameter);

    /// Add an event that we wish to be notified about 
    bool addRequestedEvents(const MgcRequestedEvent& parameter);
    /// Clear list of evnets that have been added to this message
    void clearRequestedEvents();

    /// Add a signal that we wish the gateway to produce 
    bool addSignalRequests(const MgcSignalEvent& parameter);
    /// Clear the list of signals that have added to this message 
    void clearSignalRequests();

    /// Set who should be notified about events 
    bool setNotifiedEntity(const string& parameter);

    /// Specifiy a digimap for the gateway to collect 
    bool setDigitMap(const string& parameter);

    /// Specify how to deal with quarantined events 
    bool setQuarantineHandling(MgcpNotificationHandling handling, 
			       MgcpQuarantineTreatment treatment);

    /// Add list of ents that have been detected 
    bool addDetectEvents(const MgcRequestedEvent& parameter);
    /// Clear the list of detected events that have been added to this message
    void clearDetectEvents();

    /// Get the response error code 
    int getResponseCode();

    /// Get the description of any response error
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandNotificationRequest> data;
};


/// This Notify command is received by the agent from the gateway 
class MgcNotify : public MgcCommand
{
public: 
    /// Create a notification from a specific endpoint 
//    MgcNotify(const string& EndpointId);
   
    ///
    MgcNotify(const  MgcNotify& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcNotify(MgcpCommand* x)
    { 
        MgcpCommandNotify* p = dynamic_cast<MgcpCommandNotify*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcNotify(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcNotifyType; } ;


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /// Get endpoint that created the notification 
    string getEndpointId();

    /// this function will be handled automatically in future versionsby the stack.  users of the stack should never call it.
    string getResponseAck();

    /// get the request identifier given in a previous message 
    string getRequestIdentifier();

    /// get who the notification is being sent to
    string getNotifiedEntity();

    /// Get the number of events in this message 
    int getNumObservedEvents();
    /// Get the i'th event 
    Sptr<MgcEvent> getItemObservedEvents(int index);

    /// set the response error code 
    void setResponseCode(MgcpResponseCode code);
    /// set the description of any response error
    void setResponseString(string response);

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandNotify> data;
};


/// message to the gateway requesting all info about an endpoint
class MgcAuditEndpoint: public MgcCommand
{
public: 
    /// ask about specifed end point
    MgcAuditEndpoint(const string& endpointID);

    ///
    MgcAuditEndpoint(const  MgcAuditEndpoint& x) : MgcCommand(x), data(x.data) {}
    
    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcAuditEndpoint(MgcpCommand* x)
    { 
        MgcpCommandAuditEndpoint* p
	   = dynamic_cast<MgcpCommandAuditEndpoint*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcAuditEndpoint(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcAuditEndpointType; };


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

    /// add RequestedInfo
    bool addToRequestedInfo( MgcpParameterType type );
    

    /// Get endpoint beeing audited
    string getEndpointId();

    /// get the number of EndpointIds in the EndpointIdList parameter
    int getNumEndpointIds();

    /// get the i'th EndpointId
    string getItemEndpointIdList( int index );

    /// Find out the connection id
	 string getConnectionId();

     int getNumEndPoints();

     bool setMaxEndPointIds(int parameter);
     ///
	 /***************************************
            APIs for Capabilities
	 *********************************************/
     bool getIsPacketRange();
     int getPacketization();
     void getPacketRange(int* p1, int* p2);
     bool getIsBandwidthRange();
     int getBandwidth();
     void getBandwidthRange(int* p1, int* p2);
     int getTypeOfService();
     bool getEchoCancellation();
     bool getSilenceSuppression();
     int getNumComp();
     string  getCompAlgoItem(int k);
     int getNumModes();
     string getModeItem(int k);
     int getNumEvents();
     MgcpPackageType getEventItem(int k);

	 /****************************************************
           end Capabilities APIs
	 ***********************************************************/
 
    /// get the number of ConnectionIds in the ConnectionIdList parameter
    int getNumConnectionIds();

    ///Get the i'th  connection Id from the Connection Id list
    string getItemConnectionId( int index );

    
    ///
    MgcpBearerInformation getBearerInformation();

    ///
    string getRequestIdentifier();

    ///
    MgcpConnectionOptions getLocalConnectionOptions();

    /// get the number of requested events in this message
    int getNumRequestedEvents();

    /// get the i'th event in this message
    Sptr<MgcEvent> getItemRequestedEvents(int index);
    
    /// get the number of signalled events in this message
    int getNumSignalRequests();

    /// get the i'th signal in this message
    Sptr<MgcEvent> getItemSignalRequests(int index);

    ///get the Num of signal events 
    int getNumEventStates();

    ///get the i'th signal 
    Sptr<MgcEvent> getItemEventStates(int index);

    /// Get the number of events in this message
    int getNumObservedEvents();

    /// Get the i'th event 
    Sptr<MgcEvent> getItemObservedEvents(int index);

    ///
    string getDigitMap();

    ///
    string getNotifiedEntity();
   
    ///
    string getRequestId(); 

    /// 
    MgcpQuarantineTreatment getTreatmentQuarantineHandling();

    ///
    MgcpNotificationHandling getNotificationQuarantineHandling();

    ///
    int getNumDetectEvents();

    ///
    Sptr<MgcEvent> getItemDetectEvents(int index);

    /// Get the response error code 
    int getResponseCode()
      { return responseMsg->getResponseCode(); }

    /// Get the description of any response error 
    string getResponseString();

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandAuditEndpoint> data;
};



/// message to the gateway requesting all info about an connection
class MgcAuditConnection: public MgcCommand
{
public: 
    /// ask about a specifed endpoint and/or connection 
    MgcAuditConnection(const string& EndpointId, const string& ConnectionId);
    MgcAuditConnection(const string& EndpointId);
    ///
    MgcAuditConnection(const  MgcAuditConnection& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcAuditConnection(MgcpCommand* x) { 
        MgcpCommandAuditConnection* p
	   = dynamic_cast<MgcpCommandAuditConnection*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcAuditConnection(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcAuditConnectionType; }


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /** this function will be handled automatically in future versions
	by the stack.  users of the stack should never call it.  */
    void setResponseAck(const string& parameter);

     ///add item to requestedinfo 
     bool addToRequestedInfo( MgcpParameterType type );

    /// Find out the call id 
    string getCallId();

    ///Get the notified Entity
   string getNotifiedEntity();

   ///
   string getConnectionId();

    ///get the local connection options
    MgcpConnectionOptions getLocalConnectionOptions();

    ///
   string getConnectionMode();

 ///  Get parameters describing actual connection 
      int getPacketsSent();
     int getOctetsSent();
     int getPacketsRcvd();
     int getOctetsRcvd();
     int getPacketsLost();
     int getJitter();
     int getLatency();
   
 /// Get protocol information for remote side of connection
    SdpSession getRemoteConnectionDescriptor();

    /// Get protocol information for this gateyay's side
    SdpSession getLocalConnectionDescriptor();

    /// Get the response error code 
    int getResponseCode()
      { return responseMsg->getResponseCode(); }

    /// Get the description of any response error
    string getResponseString();

    /// Get endpoint that created the notification 
    string getEndpointId();

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandAuditConnection> data;
};



/// message received by the agent from the gateway to indicate the gatewya is restarting
class MgcRestartInProgress : public MgcCommand
{
public: 
    ///
//    MgcRestartInProgress();
    ///
    MgcRestartInProgress(const  MgcRestartInProgress& x) : MgcCommand(x),  data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcRestartInProgress(MgcpCommand* x) { 
        MgcpCommandRestartInProgress* p
	   = dynamic_cast<MgcpCommandRestartInProgress*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcRestartInProgress(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcRestartInProgressType; };


    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    // TODO - Shouldn't this class have a getReason

    /// get the delay for the restart
    string getRestartDelay();

    /// get the method of restart
    string getRestartMethod();

    /// Get endpoint beeing audited
    string getEndpointId();
    
    /// set the response error code 
    void setResponseCode(MgcpResponseCode x);

    /// set the description of any response error
    void setResponseString(string s);

private:
    /// Holds the data for this command
    Sptr<MgcpCommandRestartInProgress> data;
};



/// This delete connection received by the agent from the gateway 
class MgcDeleteConnectionGw : public MgcCommand
{
public: 
    ///
//    MgcDeleteConnectionGw();
    ///
    MgcDeleteConnectionGw(const  MgcDeleteConnectionGw& x) : MgcCommand(x), data(x.data) {}

    /** this function is used by the stack.  Stack users should never call 
	it, as it will probably disappear by the next version.
    */
    MgcDeleteConnectionGw(MgcpCommand* x) { 
        MgcpCommandDeleteConnectionGw* p
	   = dynamic_cast<MgcpCommandDeleteConnectionGw*>(x);
        if(p) {data = p;}
    };

    /// make a copy of the MgCommand of the appropriate type
    MgcCommand* copy() {
	return new MgcDeleteConnectionGw(*this);
    };

    /// Get the type of the command
    MgcCommandType getType() { return MgcDeleteConnectionGwType; } ;

    /// Get pointer to object holding data.  This function is used internally by the stack.  users of the stack should never call it.
    MgcpCommand* getDataPtr() { return data.getPtr(); }
//    MgcpCommand* getDataPtr() { return &data; }

    /// set the response error code 
    void setResponseCode(MgcpResponseCode code);
    ///  set the description of any response error
    void setResponseString(string response);

    /// Get endpoint that created the notification 
    string getEndpointId();

private:
    /// Holds the data for this command 
    Sptr<MgcpCommandDeleteConnectionGw> data;
};


#endif
