
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgcEvent_cxx_version =
"$Id: MgcEvent.cxx,v 1.25 2001/02/01 01:22:56 icahoon Exp $";



#include "MgcEvent.hxx"
#include "MgcpParameterEventGeneric.hxx"
#include "MgcpResponseCode.hxx"

///
const char OKResponse[] = "ok";


/**********************************************************************

			    MgcEventAction

 **********************************************************************/

MgcEventAction::MgcEventAction() 
    : myActionType(MgcpRequestedActionTypeNotify)
{
}
 

MgcEventAction::MgcEventAction(const MgcEventAction& action) 
    : myActionType(action.myActionType)
{ 
}
 

MgcEventAction::MgcEventAction(const MgcpRequestedAction& action) 
    : myActionType(action)
{ 
}
 

MgcEventAction::MgcEventAction(const MgcEmbeddedNotificationRequest& embedded)
{
    Sptr<MgcpEmbeddedRequest> requestCopy 
	= new MgcpEmbeddedRequest(embedded.getImplementation());

    myActionType.insertEmbedded(requestCopy);
}
 

MgcEventAction& 
MgcEventAction::operator= (const MgcEventAction& x) 
{ 
    this->myActionType = x.myActionType; return *this; 
}
 

MgcpRequestedActionType 
MgcEventAction::getActionType() const 
{
    return myActionType.getActionType();
}


MgcpRequestedAction
MgcEventAction::getInternal() const
{
    return myActionType;
}



/**********************************************************************

			  MgcRequestedEvent

 **********************************************************************/


///
void MgcRequestedEvent::addAction(const MgcEventAction& action)
{
    myAction = action;
}


///
void MgcRequestedEvent::addAction(MgcpRequestedActionType action)
{
    myAction = MgcEventAction(action);
}


/** this function returns the number of actions in this event */
int MgcRequestedEvent::getNumAction()
{
    return 1;
}


///
MgcEventAction MgcRequestedEvent::getItemAction(int i) const
{
    return myAction;
}

///
void MgcRequestedEvent::clearAction()
{
    myAction = MgcEventAction(MgcpRequestedActionTypeNotify);
}

ostream& MgcRequestedEvent::print(ostream& s) const
{
    s << "Requested Event" << endl;
    s << endl;
    return s;
}



/*********************************************************************

		    MgcEmbeddedNotificationRequest

 *********************************************************************/

MgcEmbeddedNotificationRequest::MgcEmbeddedNotificationRequest()
{

}

void
MgcEmbeddedNotificationRequest::addRequestedEvents(const MgcRequestedEvent& parameter)
{
    Sptr<MgcpParameterRequestedEvent> event
	= new MgcpParameterRequestedEventGeneric(parameter.getEvent());
    myRequest.insertEvent(event);
}


void
MgcEmbeddedNotificationRequest::clearRequestedEvents()
{
    myRequest.clearEvents();
}


void
MgcEmbeddedNotificationRequest::addSignalRequests(const MgcSignalEvent& parameter)
{
    Sptr<MgcpParameterSignalGeneric> mySignal 
	= new MgcpParameterSignalGeneric(parameter.getEvent());

    myRequest.insertSignal(mySignal);
}


void MgcEmbeddedNotificationRequest::clearSignalRequests()
{
    myRequest.clearSignals();
}

void
MgcEmbeddedNotificationRequest::setDigitMap(const string& parameter)
{
    myRequest.insertMap(MgcpParameterDigitMap(parameter));
}



MgcpEmbeddedRequest 
MgcEmbeddedNotificationRequest::getImplementation() const
{
    return myRequest;
}
