#ifndef MGC_EVENT_HXX
#define MGC_EVENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcEvent_h_Version = 
"$Id: MgcEvent.hxx,v 1.4 2001/01/26 02:46:07 liuhong Exp $";



#include "global.h"
#include "MgcpPackageTypes.hxx"
#include "MgcpGenericEvent.hxx"
#include <string>
#include "Sptr.hxx"
#include "MgcpParameterEventGeneric.hxx"

class MgcEmbeddedNotificationRequest;
class MgcSignalEvent;
class MgcRequestedEventList;
class MgcEvent;


///
class MgcEventAction
{
public:
    ///
    MgcEventAction();
    ///
    MgcEventAction(const MgcEventAction& action);
    ///
    MgcEventAction(const MgcpRequestedAction& action);
    ///
    MgcEventAction(const MgcEmbeddedNotificationRequest& embedded);
    ///
    MgcEventAction& operator= (const MgcEventAction& x);
    ///
    MgcpRequestedActionType getActionType() const;
    ///
    MgcpRequestedAction getInternal() const;

private:
    ///
    MgcpRequestedAction myActionType;
};


///
class MgcRequestedEvent 
{
public:
    /// not implemented
    MgcRequestedEvent();

    ///
    MgcRequestedEvent(MgcpGenericEvent event) 
	: myEvent(event)
	{};

    ///
    MgcRequestedEvent(const MgcRequestedEvent& e) 
	: myEvent(e.myEvent),
	myAction(e.myAction) 
	{};

    ///
    void addAction(const MgcEventAction& action);

    ///
    void addAction(MgcpRequestedActionType action);

    ///
    void clearAction();

    ///
    int getNumAction();

    ///
    MgcEventAction getItemAction(int i) const;

    ///
    MgcpGenericEvent getEvent() const { return myEvent; }

    ///
    ostream& print(ostream& s) const;

protected:
    MgcpGenericEvent myEvent;
    MgcEventAction myAction;
};


/// TODO: fix the implementation for this class
class MgcEmbeddedNotificationRequest
{
public:
    ///
    MgcEmbeddedNotificationRequest();

    ///
    void addRequestedEvents(const MgcRequestedEvent& parameter);
    ///
    void clearRequestedEvents();
    ///
    void addSignalRequests(const MgcSignalEvent& parameter);
    ///
    void clearSignalRequests();
    ///
    void setDigitMap(const string& parameter);
    ///
    MgcpEmbeddedRequest getImplementation() const;

private:
    ///
    MgcpEmbeddedRequest myRequest;
};


///
class MgcSignalEvent 
{
public:
    MgcSignalEvent();
    MgcSignalEvent(const MgcpGenericEvent& x) : myEvent(x) {}
 
    ~MgcSignalEvent() {}
    MgcpGenericEvent getEvent() const { return myEvent; }

protected:
    MgcpGenericEvent myEvent;
};

///
class MgcEvent
{
public:
    MgcEvent() {}
    MgcEvent(const MgcpGenericEvent& x) : myEvent(x) {};
    virtual ~MgcEvent() {}
    MgcpPackageType getPackageType() { return myEvent.getPackageType(); };

    MgcpGenericEvent getEvent() const { return myEvent; }

    /// Get the i'th paramter 
    string getParameter(int index) { return myEvent.getParameter(index); }

    /// Get the number of paramters - usually 0 - depends on type
    int getNumParameters() { return myEvent.getParameterSize(); }

protected:
    MgcpGenericEvent myEvent;
};

#endif
