
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgcGateway_cxx_version =
"$Id: MgcGateway.cxx,v 1.54 2001/02/01 01:22:56 icahoon Exp $";



#include "MgcGateway.hxx"
#include <sys/types.h>
#include <time.h>
#include "VMutex.h"
#include "VCondition.h"
#include "MgcpDebugManager.hxx"
#include "MgcpResponseCode.hxx"
#include "MgcpParse.hxx"
#include "MgcParse.hxx"

#include <iostream>

const long MinMgcTimeout = 500000;
const long MgcTimeout    = 1000;  // 1 sec
const long MgcTimeoutCap = 4000;  // 4 secs
const long MgcLongTran   = 5000;  // 5 secs 


MgcGateway::MgcGateway(int portLocal)
    : timeoutMsgs(MinMgcTimeout, UniqueId("", -1, -1) ),
      timeoutResp(MinMgcTimeout, UniqueId("", -1, -1)),
      useAsync(false) ,
      useOne(true)
{
    server = 0;
    server = new MgcpReceiver( portLocal );

    msgWaiting = false;
    respWaiting = false;
    queueMutex = new VMutex;
    networkMutex = new VMutex;

    msgPending = new VCondition;
    respPending = new VCondition;
    networkPending = new VCondition;
}


MgcGateway::MgcGateway(const char* address, int portRemote, int portLocal)
    : timeoutMsgs(MinMgcTimeout, UniqueId("", -1, -1)),
      timeoutResp(MinMgcTimeout, UniqueId("", -1, -1)),
      useAsync(false) ,
      useOne(true)
{
    server = 0;
    server = new MgcpReceiver( portLocal );
    
    Sptr<MgcpTransmitter>  ptr = new MgcpTransmitter( address, portRemote, server );

    gatewayList.push_back(ptr); 

    msgWaiting = false;
    respWaiting = false;
    queueMutex = new VMutex;
    networkMutex = new VMutex;

    msgPending = new VCondition;
    respPending = new VCondition;
    networkPending = new VCondition;
}


MgcGatewayHandle 
MgcGateway::addGateway(const char* address, int portRemote)
{
    Sptr<MgcpTransmitter>  ptr = new MgcpTransmitter(address, portRemote, server);

    gatewayList.push_back(ptr);
    return (gatewayList.size() - 1);
}


void 
MgcGateway::flushMessages()
{
    // xxx -- run this in your own lock!!!
    while(pendingMessages.begin() != pendingMessages.end() 
	  && (pendingMessages.front() == 0))
    {
	pendingMessages.pop_front();
    }
}


void
MgcGateway::processPendingBlock()
{
    useOne = false;
    for(;;)
    {
        networkMutex->lock();
	while (pendingPackets.begin() == pendingPackets.end() )
	{
	    networkPending->wait(networkMutex);
	}
        networkMutex->unlock();
	processPending();
    }
}


bool 
MgcGateway::processPending()
{
    NetworkAddress sourceAddr;

    networkMutex->lock();
    if (pendingPackets.begin() == pendingPackets.end() )
    {
        networkMutex->unlock();
//	ODEBUG(DEBUG_NTWK, 1, cout << "NTWK: exiting processPending\n";);
	return false;
    }

    ODEBUG(DEBUG_NTWK, 1, cout << "NTWK: starting process packet\n";);
    
    MgcpPacket* data = 0;
    data = pendingPackets.front();
    pendingPackets.pop_front();

    networkMutex->unlock();

    if (!data)
      return false;  // should we return false on failure ??

    sourceAddr = data->getAddress();
     char* p = data->getMessageLocation();

    string s, st_temp, tempstr;
    vector<string> parameters;
    int initpos = 0;
    const char* st;
    bool flag;
    string myData = string(p);
    string::size_type itemPlace = myData.find("\r\n.\r\n");
     if( itemPlace != string::npos )
	 {
        while( itemPlace != string::npos )
	   {
         st_temp = string(myData,initpos,myData.size()-(myData.size()-itemPlace));
         parameters.push_back( st_temp );
         tempstr = string(myData,itemPlace+5,myData.size()-itemPlace-5);
         myData = tempstr;
         itemPlace = myData.find("\r\n.\r\n");
       }
       parameters.push_back(myData);
	   vector<string>::const_iterator iter = parameters.begin(); 
        while( iter != parameters.end() )
		{
          s = (*iter);
          st = s.c_str();
          MgcpCommand* message = parseMessage(st, s.size());
          flag = SingleProcess( message, sourceAddr );
          iter++;
		}
     } 
    else
	{
	  int size = data->getMessageSize();
	  char * loc = data->getMessageLocation();;
	  
	  MgcpCommand* message 
	       = parseMessage( loc, size );
	  //   MgcpCommand* message 
	  //   = parseMessage(data->getMessageLocation(), data->getMessageSize());
       flag = SingleProcess( message, sourceAddr );
    }

    delete data; 
    data = 0;
    return flag;
}    


 bool 
MgcGateway::SingleProcess(MgcpCommand* message, NetworkAddress sourceAddr)
 {
    if(dynamic_cast<MgcpResponse*>(message))
    {
        int transactionID = message->getTransactionId();

	/// get an unique id
	UniqueId uID( sourceAddr.getHostName(), sourceAddr.getPort(), transactionID);
	
	ODEBUG(DEBUG_NTWK, 3, cout << "NTWK: got response: " << message->getTransactionId() << endl;);

	// this is a response, and needs to be matched (via a
	// map) to its originator

	MgcMessageStuff* original;

	MgcMsgStuffMap::iterator p = 
	    sentMessages.find( uID );

	if(p == sentMessages.end())
	{
	    // this response did not match any sent message

	    // warn and discard

	    cerr << "unexpected response received from host: ";
	    cerr << sourceAddr.getHostName() << "port: ";
            cerr << sourceAddr.getPort() << "transaction ID: ";
	    cerr << message->getTransactionId() << "\n";
	    cerr << "-------------------- message --------------------\n";
	    cerr << "--------------------   end   --------------------\n";
	    
	    // free the unmatched response message
	    delete message;

	    return false;
	}
	else
	{
	    // this response matched a sent message

	    original = p->second;

	    if(!original)
	    {
		cerr << "map was incorrect!\n";
		return false;
	    }
	    else
	    {
		queueMutex->lock();
		original->command->setResponsePtr
		    (dynamic_cast<MgcpResponse*>(message));

		ODEBUG(DEBUG_NTWK, 2, cout << "NTWK: deleting TID (found): " << message->getTransactionId() << endl;);
		
		// this is a provisional response, we should increase retransmission timer for it
		// since a final response with execution result will come in later
		if ((dynamic_cast<MgcpResponse*>(message))->getResponseCode() == ReasonTransactionInProgress )
		{
		    TimerObj<UniqueId> node = original->nodeInTimer;
		    timeoutMsgs.remove( node );
		    timeoutMsgs.insert( uID, MgcLongTran );
		    original->nodeInTimer = timeoutMsgs.getlastInserted();		      
		}
		else  // it is a final response, delete the original request in sentMessages
		{
		    sentMessages.erase( uID );
		}

		original->ready = true;
		queueMutex->unlock();

		if(useAsync == false)
		{
		    // we would only forward the final response to the user in sync mode
		    if ((dynamic_cast<MgcpResponse*>(message))->getResponseCode() != ReasonTransactionInProgress )
		        original->responsePending->signal();
		}
		else
		{
		    queueMutex->lock();
		    pendingMessages.push_back(original->command);
		    msgPending->signal();
		    queueMutex->unlock();
		    msgWaiting = true;

		    if(original->heapAlloc == true)
		    {
		       original->command = 0;
		       delete original;
		    }
		}
	    }
        }
    }
    else if(message != 0) // received a command
    {
        int transactionID = message->getTransactionId();

	// this is a received message, so convert to MgcCommand

	Sptr<MgcCommand> msg = mgcpToMgc(message);

	// look for response acknowledgement attribute in the command

	string respAck = MgcGetRespAckString(msg);

	// parse the response ack and set flags in sentResponse for confirmed responses
	if (respAck.length() !=0 )
	{
	   processRangeInRespAck(respAck, sourceAddr);
	}

       
	/// get an unique id for this command
	UniqueId uID( sourceAddr.getHostName(), sourceAddr.getPort(), transactionID);
	
	/** check to see if the command is a duplicate of 
	    a previous command for which we have already sent 
	    a response, if true and the response is not yet 
	    confirmed, re-send the same response, if the response 
	    has been confirmed, simply ignore the command.
	*/
	Sptr<MgcMessageStuff> original = 0;

	map< UniqueId, Sptr<MgcMessageStuff> >::iterator mapiterP = 
	    sentResponses.find( uID );

	if( (mapiterP != sentResponses.end()) && (mapiterP->second != 0) )
	{
	  original = mapiterP->second;

	  if (!original->getconfirmed()) // the sent response is not confirmed yet, so just re-send the response without passing the command to the usr
	  {
	      string host = sourceAddr.getHostName();
	      /* char targethost[255];

	      for (int i=0; i<255; i++)
		{ targethost[i] = host[i];}
	      targethost[254] = '\0';
	      */

	      // re-send the same response
	      Sptr<MgcpTransmitter> transmitter = new MgcpTransmitter( host.c_str(), sourceAddr.getPort(), server);
	      original->command->getResponsePtr()->send(*transmitter);  
	     
	      return false;	      
	  }
	  else  // the response has already been confirmed, ignore the command
	  {
	      return false;
	  }
	}

	/// check to see if the command is a duplicate of an existing
	/// command waiting for processing, if true, simple ignore this one
     
	map<UniqueId, int>::iterator p = TransWaiting.find( uID );

	if(p != TransWaiting.end() )
	{   /// this is a duplicate command,ignore it
	    return false;
	}

	if(!(msg == 0))   ///  a real valid command, pass it to the user
	{
	    ODEBUG(DEBUG_NTWK, 3, cout << "NTWK: Got good msg\n";);
	    
	    // now, enqueue the message
	    queueMutex->lock();
	    
	    pendingMessages.push_back(msg);
	 

	    /// insert it into the list of currently waiting commands
	    TransWaiting[uID] = transactionID;

	    msgPending->signal();
	    msgWaiting = true;
	    queueMutex->unlock();
	}
    }
    else
    {
	cerr << "empty message from parse\n";
    }

    ODEBUG(DEBUG_NTWK, 1, cout << "NTWK: finish process packet\n";);

    return true;
}
	


bool 
MgcGateway::handleTimeouts(struct timeval* tv)
{
    MgcMessageStuff* original;
    bool retval = false;

  
    // handle time out sentMessages, i.e. sent commands
    while (!timeoutMsgs.sleepFor(tv))
    {
	retval = true;
	ODEBUG(DEBUG_NTWK, 3, cout << "handleTimeouts got TID: " << timeoutMsgs.top() << endl;);
	
	UniqueId uID( timeoutMsgs.top() );

	MgcMsgStuffMap::iterator p = sentMessages.find( uID );
	
	if(p != sentMessages.end() && p->second)
	{
	    original = p->second;
	    
	    string host = uID.getHost();	   
    
	    // re-send the same command
	    if (original->timeoutCount < LONG_TIMER )
	    {
	      long next = 0;
	      // calculate next timeout value
	      if (original->timeoutCount == 0 )
	      {
		next = static_cast<long>( MgcTimeout * ( 1+ (rand()/(RAND_MAX+1.0))));		
	      }
	      else
	      {
		next = static_cast<long>(original->lastTimeout*(1+ (rand()/(RAND_MAX+1.0))));		
       	      }

	      if ( (next+original->timeoutCount) > LONG_TIMER )
		  next = LONG_TIMER - original->timeoutCount;
	      
	      if ( next > MgcTimeoutCap )
		next = MgcTimeoutCap;

	      original->timeoutCount += next;
	      original->lastTimeout = next;

	      Sptr<MgcpTransmitter> transmitter = new MgcpTransmitter( host.c_str(), uID.getPort(), server);
	      original->command->getDataPtr()->send(*transmitter);  
	      
	      // set up timeout for this newly sent command again
	      queueMutex->lock();
	      timeoutMsgs.pop();
	      timeoutMsgs.insert( uID, original->lastTimeout );
	      original->nodeInTimer = timeoutMsgs.getlastInserted();
	      queueMutex->unlock();
	  }
	  else  // reached maximum re-send, send a timeout response to user
	  {
	        queueMutex->lock();
	    	
		MgcpResponse* xyz = new MgcpResponse;
		xyz->setResponseCode(TimeoutNoResponse);
	
		original->command->setResponsePtr
		  (dynamic_cast<MgcpResponse*>(xyz));
	    
		ODEBUG(DEBUG_NTWK, 3, cout << "deleting TID (timeout): " << timeoutMsgs.top() << endl;);

		sentMessages.erase(p);
		original->ready = true;
		queueMutex->unlock();
	
		if(useAsync == false)
		{
		    original->responsePending->signal();
		}
		else
		{
		    // this is a message for delivery
		    queueMutex->lock();
		    pendingMessages.push_back(original->command);
		    msgWaiting = true;
		    queueMutex->unlock();

		    // free the original
		    if(original->heapAlloc == true)
		    {
			original->command = 0;
			delete original;
		    }
		}

		timeoutMsgs.pop();
	  }
	}
	else
	  timeoutMsgs.pop();

    }
 
    struct timeval tv2;

    tv2.tv_sec = 0;
    tv2.tv_usec = 500000;

    // handle time out for sentResponses
    while(!timeoutResp.sleepFor(&tv2))
    {
        queueMutex->lock();
	map< UniqueId, Sptr<MgcMessageStuff> >::iterator p = 
	    sentResponses.find(timeoutResp.top());
	queueMutex->unlock();

	if( (p != sentResponses.end() ) && (p->second != 0) )
	{	   
	    queueMutex->lock();
	    sentResponses.erase(p);
	    queueMutex->unlock();
	}
      
	timeoutResp.pop();
    }
 
    return retval;
}


void 
MgcGateway::handlePacket()
{
    MgcpPacket* data = 0;

    networkMutex->lock();
    data = server->receive();
    pendingPackets.push_back(data);
    networkMutex->unlock();
    networkPending->signal();
    ODEBUG(DEBUG_NTWK, 4, cout << "NTWK: pushed packet onto stack\n";);
}


bool 
microcheck(int fd)
{
    struct timeval tv = {0, 0};
    fd_set fds;

    FD_ZERO(&fds);
    FD_SET(fd, &fds);

    int retval = select(fd+1, &fds, 0, 0, &tv);
    if(retval > 0)
    {
	return FD_ISSET(fd, &fds);
    }
    else
    {
	return false;
    }
}


void 
MgcGateway::threadProcess()
{
    ODEBUG(DEBUG_NTWK, 2, cout << "NTWK: entering threadProcess (" << this << ")" << endl;);

    fd_set read_fds;
    fd_set real_read_fds;
    struct timeval tv;

    FD_ZERO(&real_read_fds);

    server->addToFdSet(&real_read_fds);
//    int maxFd = 32;

    for(;;)
    {
	bool incomingMsg;

	tv.tv_sec = 0;
	tv.tv_usec = 500000;

	handleTimeouts(&tv);

	server->addToFdSet(&read_fds);
	int maxFd = 128;

        if( incomingMsg = microcheck(server->getFd()) ) 
	{
	    // get pending msgs from network
	    while(incomingMsg)
	    {
 		ODEBUG(DEBUG_NTWK, 3, cout << "NTWK: got network data\n";);
		handlePacket();
		incomingMsg = microcheck(server->getFd());
	    }
	}
	else if (handleTimeouts(&tv))
	{
	    // handle timeouts actually handles timeous, but
	    // if it has handled a timeout it should skip the rest of
	    // these checks by doing nothing in the body code
	    // here
	}
	else if (useOne && processPending() )
	{
	    // processPending actually handles up to one pending
	    // message.  if it has handled a msg it should skip the
	    // rest of these checks by doing nothing in the body code
	    // here
	}
	else
	{
	    // there is nothing to do -- wait quietly for a network
	    // message or a timeout

	    memcpy(&read_fds, &real_read_fds, sizeof(fd_set));

	    // tv is set by the handleTimouts, above

	    int ret_val = select(maxFd, &read_fds, 0, 0, &tv);

	    if(ret_val < 0)
	    {
		cerr << "problem in select\n";
	    }
	}
    }

}


Sptr<MgcCommand> 
MgcGateway::getNextMessage()
{

    ODEBUG(DEBUG_NTWK, 3, cerr << "Begin *** MgcGateway::getNextMessage() ***" << endl;);
//    fetchNewData();
    queueMutex->lock();

//    flushMessages();

    Sptr<MgcCommand> returnVal = 0;

    while(pendingMessages.begin() == pendingMessages.end())
    {
	msgPending->wait(queueMutex);
    }

    // at this point, there has to be something in the queue
    returnVal = pendingMessages.front();
    pendingMessages.pop_front();
    if(pendingMessages.begin() == pendingMessages.end())
    {
	msgWaiting = false;
    }

    queueMutex->unlock();
    return returnVal;

    ODEBUG(DEBUG_NTWK, 3, cerr << "End *** MgcGateway::getNextMessage() ***" << endl;);

}


bool 
MgcGateway::send(MgcCommand& command, MgcGatewayHandle handle)
{
    assert(useAsync == false);
    ODEBUG(DEBUG_NTWK, 3, cerr << "Begin ***  MgcGateway::send *** " << endl;);

    assert(command.getDataPtr());
    command.getDataPtr()->readyToSend();

    MgcMessageStuff original;
    original.command = command.copy();

    queueMutex->lock();

    ODEBUG(DEBUG_NTWK, 2, cerr<< "adding tid: \n";) ;
    ODEBUG(DEBUG_NTWK, 2, cerr << command.getDataPtr()->getTransactionId() << "\n";);

    ODEBUG(DEBUG_NTWK, 3, cout << "Adding TID: " << original.command->getDataPtr()->getTransactionId() << endl;);
    
    NetworkAddress targetAddr = (gatewayList[handle])->getremoteAddr();
    UniqueId uID( targetAddr.getHostName(), targetAddr.getPort(),original.command->getDataPtr()->getTransactionId() );
	

    sentMessages[ uID ] = &original;

    original.command->getDataPtr()->send(*gatewayList[handle]);

    // set up the timeout

    timeoutMsgs.insert( uID , MgcTimeout);
    original.nodeInTimer = timeoutMsgs.getlastInserted();

    while(original.ready == false) {
	original.responsePending->wait(queueMutex);
    }
    
    queueMutex->unlock();

    // xxx bad hack to fix no response problem in agent!
    Sptr<MgcpResponse> x;
    x.dynamicCast(original.command->getResponseSptr());
    command.setResponseSptr(x);

    ODEBUG(DEBUG_NTWK, 3, cerr << "End *** MgcGateway::send *** " << endl;);

    return true;
}





bool 
MgcGateway::reply(const Sptr<MgcCommand>& command, MgcGatewayHandle handle)
{
    assert(useAsync == false);
    command->getResponsePtr()->
	setTransactionId(command->getDataPtr()->getTransactionId());

    ODEBUG(DEBUG_NTWK, 3, cout << "reply:  tid: " << command->getDataPtr()->getTransactionId() << endl;);

    command->getResponsePtr()->send(*gatewayList[handle]);

    // save the copy of the sent response in sentResponses
    NetworkAddress remoteAddr = (gatewayList[handle])->getremoteAddr();

    int transID = command->getDataPtr()->getTransactionId();
    UniqueId uID( remoteAddr.getHostName(), remoteAddr.getPort(), transID);

    MgcpResponseCode respCode = (dynamic_cast<MgcpResponse*>(command->getResponsePtr()))->getResponseCode();
    if ( respCode != ReasonResponseAcknowledgement ) // make sure it's not a response ack 000 to a final response 200 
    {
      Sptr<MgcMessageStuff> original = new MgcMessageStuff;
    // original->heapAlloc = true;
    // original->command = command->copy();
      original->command = command;
      original->command->setResponseSptr( command->getResponseSptr() );
    // original->setconfirmed(false);

      queueMutex->lock();
      sentResponses[uID] = original;

    // set up the time out
      timeoutResp.insert( uID, LONG_TIMER);

    // take the transaction out of the list of outstanding commands in TransWaiting
      map<UniqueId, int>::iterator p = TransWaiting.find( uID );

      if(p != TransWaiting.end() )
      {   
	  TransWaiting.erase( uID );	    
      }   

      queueMutex->unlock();
    }

    return true;
}


Sptr<MgcCommand> 
MgcGateway::receive()
{
    assert(useAsync == false);
    ODEBUG(DEBUG_NTWK, 3, cerr<< "Begin *** MgcGateway::receive *** " << endl;);
    return getNextMessage();
}


///
bool 
MgcGateway::asyncSend(MgcCommand& command, MgcGatewayHandle handle)
{
    assert(useAsync == true);

    assert(command.getDataPtr());
    command.getDataPtr()->readyToSend();

    MgcMessageStuff* original = new MgcMessageStuff;
    original->heapAlloc = true;
    original->command = command.copy();

    queueMutex->lock();

    ODEBUG(DEBUG_NTWK, 2, cerr << "adding tid: \n");
    ODEBUG(DEBUG_NTWK, 2, cerr << command.getDataPtr()->getTransactionId() << "\n");

    ODEBUG(DEBUG_NTWK, 3, cout << "Adding TID: " << original->command->getDataPtr()->getTransactionId() << endl;);
    
    NetworkAddress targetAddr = (gatewayList[handle])->getremoteAddr();
    UniqueId uID( targetAddr.getHostName(), targetAddr.getPort(),original->command->getDataPtr()->getTransactionId() );

    sentMessages[ uID ] = original;

    original->command->getDataPtr()->send(*gatewayList[handle]);

    // set up the timeout

    timeoutMsgs.insert( uID , MgcTimeout);
    original->nodeInTimer = timeoutMsgs.getlastInserted();

    queueMutex->unlock();

    return true;
}





bool 
MgcGateway::asyncReply(const Sptr<MgcCommand>& command, MgcGatewayHandle handle)
{
    assert(useAsync == true);
    command->getResponsePtr()->
	setTransactionId(command->getDataPtr()->getTransactionId());
    command->getResponsePtr()->send(*gatewayList[handle]);

    // save the copy of the sent response in sentResponses
    NetworkAddress remoteAddr = gatewayList[handle]->getremoteAddr();

    int transID = command->getDataPtr()->getTransactionId();
    UniqueId uID( remoteAddr.getHostName(), remoteAddr.getPort(), transID);

    MgcpResponseCode respCode = (dynamic_cast<MgcpResponse*>(command->getResponsePtr()))->getResponseCode();
    if ( respCode != ReasonResponseAcknowledgement ) // make sure it's not a response ack 000 to a final response 200 
    {
      Sptr<MgcMessageStuff> original = new MgcMessageStuff;
    //original->heapAlloc = true;
    //original->command = command->copy();
      original->command = command;
      original->command->setResponseSptr( command->getResponseSptr() );
    //original->setconfirmed(false);

      queueMutex->lock();
      sentResponses[uID] = original;

    // set up the time out
      timeoutResp.insert( uID, LONG_TIMER);

    // take the transaction out of the list of outstanding commands in TransWaiting
      map<UniqueId, int>::iterator p = TransWaiting.find( uID );

      if(p != TransWaiting.end() )
      {   
	  TransWaiting.erase( uID );	    
      }      
    }

    queueMutex->unlock();

    return true;
}


///
Sptr<MgcCommand> 
MgcGateway::asyncReceive()
{
    assert(useAsync == true);
    return getNextMessage();
}


/// this is the part that handles messages that have timed out and the like
void 
MgcGateway::asyncProcess()
{
    setAsync();
    threadProcess();
}

    
void 
MgcGateway::addToFdSet( fd_set* set )
{
    server->addToFdSet(set);
}


bool 
MgcGateway::checkIfSet( fd_set* set )
{
    return server->checkIfSet(set);
}


bool 
MgcGateway::checkMsgReady( fd_set* set )
{
    // do something useful here, like process all of the things that you can

    if(msgWaiting == false)
    {
	return false;
    }
    else
    {
	queueMutex->lock();
	msgWaiting = !(pendingMessages.begin() == pendingMessages.end());
	queueMutex->unlock();
    }
    return msgWaiting;
}


bool 
MgcGateway::commandPending()
{
    return msgWaiting;
}


int 
MgcGateway::getFd()
{
    return 10; // xxx
}


void MgcGateway::processRangeInRespAck( const string& s, const NetworkAddress& srcAddr )
{
  //  split_t  test_K_vector;
  //  split_t  test_num_vector;
    split_t  test_K_vector;
    split_t  test_num_vector;

    int start =0;
    int end = 0;
    bool isRange = false;

    /// get "K:" and the rest into test_K_vector
    //  test_K_vector = split(s, ":");
    test_K_vector = split(s, ":");

    ///get the list of ranges if any
    if (test_K_vector.size() >= 1)  
      // test_num_vector = split(test_K_vector[1], ",");
      test_num_vector = split(test_K_vector[1], ",");

    if(test_num_vector.size() != 0)
    {
	for(unsigned int i = 0; i < test_num_vector.size(); i++)
	{
	  if (getRange(test_num_vector[i], &start, &end, &isRange))
	  {
	    if (isRange)
	    {
	      
              for (int i=start; i<=end; i++)
	      {
                 Sptr<MgcMessageStuff> original = 0;
		 UniqueId uID(srcAddr.getHostName(), srcAddr.getPort(), i );

		 queueMutex->lock();
		 map<UniqueId, Sptr<MgcMessageStuff> >::iterator p = sentResponses.find( uID );

		 if(p != sentResponses.end() )
		 {
		     original = p->second;
	             if(original !=0 )
		     {
			 original->setconfirmed(true);
		     }
		 }
		 queueMutex->unlock();
	      }
	      
	    }
	    else
	    {
	         Sptr<MgcMessageStuff> original = 0;
		 UniqueId uID(srcAddr.getHostName(), srcAddr.getPort(), start );

		 queueMutex->lock();
		 map<UniqueId, Sptr<MgcMessageStuff> >::iterator p = sentResponses.find( uID );

		 if(p != sentResponses.end() )
		 {
		     original = p->second;
		     if( original !=0 )
		     {
			 original->setconfirmed(true);
		     }
		 }
		 queueMutex->unlock();
	    }
	  }
	  
	  /// reset before use again
	  start = 0;
	  end = 0;
	  isRange=false;
	}
    }

    return;
}
   
