#ifndef MGC_PACKAGE_HXX
#define MGC_PACKAGE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcPackage_hxx_Version = 
"$Id: MgcPackage.hxx,v 1.5.4.1 2001/02/13 21:32:50 liuhong Exp $";


#include "global.h"
#include "MgcpEventTypes.hxx"
#include "MgcEvent.hxx"
#include <string>


#ifndef NCS_PROFILE
///
class MgcGenericPackage
{
public:
    ///
    static MgcRequestedEvent requestModemDetect(string s="");
    ///
    static MgcRequestedEvent requestFaxToneDetect(string s="");
    ///
    static MgcRequestedEvent requestLongDurationConnection(string s="");
    ///
    static MgcRequestedEvent requestPatternDetected(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalPatternDetectedOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalPatternDetectedOff(string s="");
    ///
    static MgcSignalEvent signalRingBackToneOn(string s="");
    ///
    static MgcSignalEvent signalRingBackToneOff(string s="");
    ///
    static MgcSignalEvent signalRingBackOnConnectionOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalRingBackOnConnectionOff(string s="");
    ///
    static MgcSignalEvent signalConfirmTone(string s="");
    ///
    static MgcSignalEvent signalNetworkCongestionToneOn(string s="");
    ///
    static MgcSignalEvent signalNetworkCongestionToneOff(string s="");
    ///
    static MgcSignalEvent signalInterceptToneOn(string s="");
    ///
    static MgcSignalEvent signalInterceptToneOff(string s="");
    ///
    static MgcSignalEvent signalPreemtionToneOn(string s="");
    ///
    static MgcSignalEvent signalPreemtionToneOff(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcGenericPackageEvent : public MgcEvent
{
public:
    /// constructor
    MgcGenericPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// destructor
//    ~MgcGenericPackageEvent() {};
    /// get the type of event that the Gateway is sending to us
    MgcpGenericPackageItemType getType() { return myEvent.getGenericPackageType(); }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcDTMFPackage
{
public:
    static MgcRequestedEvent requestDTMF_0(string s="");
    ///
    static MgcRequestedEvent requestDTMF_1(string s="");
    ///
    static MgcRequestedEvent requestDTMF_2(string s="");
    ///
    static MgcRequestedEvent requestDTMF_3(string s="");
    ///
    static MgcRequestedEvent requestDTMF_4(string s="");
    ///
    static MgcRequestedEvent requestDTMF_5(string s="");
    ///
    static MgcRequestedEvent requestDTMF_6(string s="");
    ///
    static MgcRequestedEvent requestDTMF_7(string s="");
    ///
    static MgcRequestedEvent requestDTMF_8(string s="");
    ///
    static MgcRequestedEvent requestDTMF_9(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Hash(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Star(string s="");
    ///
    static MgcRequestedEvent requestDTMF_A(string s="");
    ///
    static MgcRequestedEvent requestDTMF_B(string s="");
    ///
    static MgcRequestedEvent requestDTMF_C(string s="");
    ///
    static MgcRequestedEvent requestDTMF_D(string s="");
    ///
    static MgcRequestedEvent requestLongDurationIndicator(string s="");
    ///
    static MgcRequestedEvent requestWildcardMatch(string s="");
    ///
    static MgcRequestedEvent requestInterdigitTimer(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalDTMF_0(string s="");
    ///
    static MgcSignalEvent signalDTMF_1(string s="");
    ///
    static MgcSignalEvent signalDTMF_2(string s="");
    ///
    static MgcSignalEvent signalDTMF_3(string s="");
    ///
    static MgcSignalEvent signalDTMF_4(string s="");
    ///
    static MgcSignalEvent signalDTMF_5(string s="");
    ///
    static MgcSignalEvent signalDTMF_6(string s="");
    ///
    static MgcSignalEvent signalDTMF_7(string s="");
    ///
    static MgcSignalEvent signalDTMF_8(string s="");
    ///
    static MgcSignalEvent signalDTMF_9(string s="");
    ///
    static MgcSignalEvent signalDTMF_Hash(string s="");
    ///
    static MgcSignalEvent signalDTMF_Star(string s="");
    ///
    static MgcSignalEvent signalDTMF_A(string s="");
    ///
    static MgcSignalEvent signalDTMF_B(string s="");
    ///
    static MgcSignalEvent signalDTMF_C(string s="");
    ///
    static MgcSignalEvent signalDTMF_D(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcDTMFPackageEvent : public MgcEvent
{
public:
    ///
    MgcDTMFPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    ///
    MgcDTMFPackageEvent() : myType(MgcDTMFPackageUndefined) {};
    ///
//    ~MgcDTMFPackageEvent() {};
    ///
    MgcDTMFPackageEvent(MgcpDTMFPackageItemType x) : myType(x) {};
    /// get the type of event that the Gateway is sending to us
    MgcpDTMFPackageItemType getType() { return myEvent.getDTMFPackageType(); }
    ///
    MgcpPackageType getPackageType() { return MgcpDTMFPackageType; };
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
    MgcpDTMFPackageItemType myType;
/// only the Mgc stack can construct these 
};

class MgcMFPackage
{
public:
    ///
    static MgcRequestedEvent requestMF_0(string s="");
    ///
    static MgcRequestedEvent requestMF_1(string s="");
    ///
    static MgcRequestedEvent requestMF_2(string s="");
    ///
    static MgcRequestedEvent requestMF_3(string s="");
    ///
    static MgcRequestedEvent requestMF_4(string s="");
    ///
    static MgcRequestedEvent requestMF_5(string s="");
    ///
    static MgcRequestedEvent requestMF_6(string s="");
    ///
    static MgcRequestedEvent requestMF_7(string s="");
    ///
    static MgcRequestedEvent requestMF_8(string s="");
    ///
    static MgcRequestedEvent requestMF_9(string s="");
    ///
    static MgcRequestedEvent requestWildcardMatch(string s="");
    ///
    static MgcRequestedEvent requestInterdigitTimer(string s="");
    ///
    static MgcRequestedEvent requestMFK0(string s="");
    ///
    static MgcRequestedEvent requestMFK1(string s="");
    ///
    static MgcRequestedEvent requestMFK2(string s="");
    ///
    static MgcRequestedEvent requestMFS0(string s="");
    ///
    static MgcRequestedEvent requestMFS1(string s="");
    ///
    static MgcRequestedEvent requestMFS2(string s="");
    ///
    static MgcRequestedEvent requestMFS3(string s="");
    ///
    static MgcRequestedEvent requestWink(string s="");
    ///
    static MgcRequestedEvent requestWinkOff(string s="");
    ///
    static MgcRequestedEvent requestIncomingSeizure(string s="");
    ///
    static MgcRequestedEvent requestReturnSeizure(string s="");
    ///
    static MgcRequestedEvent requestUnseizeCircuit(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalMF_0(string s="");
    ///
    static MgcSignalEvent signalMF_1(string s="");
    ///
    static MgcSignalEvent signalMF_2(string s="");
    ///
    static MgcSignalEvent signalMF_3(string s="");
    ///
    static MgcSignalEvent signalMF_4(string s="");
    ///
    static MgcSignalEvent signalMF_5(string s="");
    ///
    static MgcSignalEvent signalMF_6(string s="");
    ///
    static MgcSignalEvent signalMF_7(string s="");
    ///
    static MgcSignalEvent signalMF_8(string s="");
    ///
    static MgcSignalEvent signalMF_9(string s="");
    ///
    static MgcSignalEvent signalMFK0(string s="");
    ///
    static MgcSignalEvent signalMFK1(string s="");
    ///
    static MgcSignalEvent signalMFK2(string s="");
    ///string s=""
    static MgcSignalEvent signalMFS0(string s="");
    ///
    static MgcSignalEvent signalMFS1(string s="");
    ///
    static MgcSignalEvent signalMFS2(string s="");
    ///
    static MgcSignalEvent signalMFS3(string s="");
    ///
    static MgcSignalEvent signalWink(string s="");
    ///
    static MgcSignalEvent signalWinkOff(string s="");
    ///
    static MgcSignalEvent signalIncomingSeizureOn(string s="");
    ///
    static MgcSignalEvent signalIncomingSeizureOff(string s="");
    ///
    static MgcSignalEvent signalReturnSeizureOn(string s="");
    ///
    static MgcSignalEvent signalReturnSeizureOff(string s="");
    ///
    static MgcSignalEvent signalUnseizeCircuitOn(string s="");
    ///
    static MgcSignalEvent signalUnseizeCircuitOff(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcMFPackageEvent : public MgcEvent
{
public:
    ///
    MgcMFPackageEvent(const MgcpGenericEvent& e)  : MgcEvent(e) {}
    ///
//    ~MgcMFPackageEvent() {};
    /// get the type of event that the Gateway is sending to us
    MgcpMFPackageItemType getType() { return myEvent.getMFPackageType() ; }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcTrunkPackage
{
public:
    static MgcRequestedEvent requestContinuityTone(string s="");
    ///
    static MgcRequestedEvent requestContinuityTest(string s="");
    ///
    static MgcRequestedEvent requestOldMilliwattTone(string s="");
    ///
    static MgcRequestedEvent requestNewMilliwattTone(string s="");
    ///
    static MgcRequestedEvent requestTestLine(string s="");
    ///
    static MgcRequestedEvent requestNoCircuit(string s="");
    ///
    static MgcRequestedEvent requestAnswerSupervision(string s="");
    ///
    static MgcRequestedEvent requestReorderTone(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

    ///
    static MgcSignalEvent signalContinuityToneOn(string s="");
    ///
    static MgcSignalEvent signalContinuityToneOff(string s="");
    ///
    static MgcSignalEvent signalContinuityTestOn(string s="");
    ///
    static MgcSignalEvent signalContinuityTestOff(string s="");
    ///
    static MgcSignalEvent signalLoopbackOn(string s="");
    ///
    static MgcSignalEvent signalLoopbackOff(string s="");
    ///
    static MgcSignalEvent signalOldMilliwattToneOn(string s="");
    ///
    static MgcSignalEvent signalOldMilliwattToneOff(string s="");
    ///
    static MgcSignalEvent signalNewMilliwattToneOn(string s="");
    ///
    static MgcSignalEvent signalNewMilliwattToneOff(string s="");
    ///
    static MgcSignalEvent signalTestLineOn(string s="");
    ///
    static MgcSignalEvent signalTestLineOff(string s="");
    ///
    static MgcSignalEvent signalNoCircuitOn(string s="");
    ///
    static MgcSignalEvent signalNoCircuitOff(string s="");
    ///
    static MgcSignalEvent signalAnswerSupervisionOn(string s="");
    ///
    static MgcSignalEvent signalAnswerSupervisionOff(string s="");
    ///
    static MgcSignalEvent signalReorderToneOn(string s="");
    ///
    static MgcSignalEvent signalReorderToneOff(string s="");
    //no signal for blocking
};


/// This class has things the Gateway can send to the agent 
class MgcTrunkPackageEvent : public MgcEvent
{
public:
    ///
    MgcTrunkPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    ///
//    ~MgcTrunkPackageEvent();
    /// get the type of event that the Gateway is sending to us
    MgcpTrunkPackageItemType getType() { return myEvent.getTrunkPackageType() ; }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcLinePackage
{
public:
    static MgcRequestedEvent requestOffHookTransition(string s="");
    ///
    static MgcRequestedEvent requestOnHookTransition(string s="");
    ///
    static MgcRequestedEvent requestFlashHook(string s="");
    ///
    static MgcRequestedEvent requestAnswerTone(string s="");
    ///
    static MgcRequestedEvent requestNetworkBusy(string s="");
    ///
    static MgcRequestedEvent requestPromptTone(string s="");
    ///
    static MgcRequestedEvent requestErrorTone(string s="");
    ///
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

    ///
    static MgcSignalEvent signalAdsiDisplay(const string& arg, string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOn(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOff(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOn(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOff(string s="");
    ///
    static MgcSignalEvent signalBusyToneOn(string s="");
    ///
    static MgcSignalEvent signalBusyToneOff(string s="");
    ///
    static MgcSignalEvent signalCallerId(const string& arg, string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone1(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone2(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone3(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone4(string s="");
    ///
    static MgcSignalEvent signalDialToneOn(string s="");
    ///
    static MgcSignalEvent signalDialToneOff(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingIndOn(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingIndOff(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOn(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOff(string s="");
    ///
    static MgcSignalEvent signalRingingOn(string s="");
    ///
    static MgcSignalEvent signalRingingOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOff(string s="");
    ///
    static MgcSignalEvent signalRingsplash(string s="");
    ///
    static MgcSignalEvent signalPromptTone(string s="");
    ///
    static MgcSignalEvent signalErrorTone(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOn(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOff(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOn(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOff(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOn(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOff(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveTonePattern(const string& arg, string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcLinePackageEvent : public MgcEvent
{
public:
    ///
    MgcLinePackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    ///
    MgcLinePackageEvent() : myType(MgcLinePackageUndefined) {}
    ///
    MgcLinePackageEvent(MgcpLinePackageItemType x) : myType(x) {}
    /// get the type of event that the Gateway is sending to us

    MgcpLinePackageItemType getType() { return myEvent.getLinePackageType(); }

    ///
    MgcpPackageType getPackageType() { return MgcpLinePackageType; };


    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
// only the Mgc stack can construct these 
    MgcpLinePackageItemType myType;

};
///
class MgcHandsetPackage
{
public:
    ///
    static MgcRequestedEvent requestAdsiDisplay(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestOffHookTransition(string s="");
    ///
    static MgcRequestedEvent requestOnHookTransition(string s="");
    ///
    static MgcRequestedEvent requestFlashHook(string s="");
    ///
    static MgcRequestedEvent requestAnswerTone(string s="");
    ///
    static MgcRequestedEvent requestBusyTone(string s="");
    ///
    static MgcRequestedEvent requestCallWaitingTone(string s="");
    ///
    static MgcRequestedEvent requestDialTone(string s="");
    ///
    static MgcRequestedEvent requestNetworkBusy(string s="");
    ///
    static MgcRequestedEvent requestRinging(string s="");
    ///
    static MgcRequestedEvent requestDistinctiveRinging(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestPromptTone(string s="");
    ///
    static MgcRequestedEvent requestErrorTone(string s="");
    ///
    static MgcRequestedEvent requestStutterDialtone(string s="");
    ///
    static MgcRequestedEvent requestAlertingTone(string s="");
    ///
    static MgcRequestedEvent requestRecorderWarningTone(string s="");
    ///
    static MgcRequestedEvent requestSITTone(string s="");
    ///
    static MgcRequestedEvent requestCallingCardServiceTone(string s="");
    ///
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestOffHookWarningTone(string s="");
    ///
    static MgcRequestedEvent requestDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

    ///
    static MgcSignalEvent signalAdsiDisplay(const string& arg, string s="");
    ///
    static MgcSignalEvent signalOffHookTransitionOn(string s="");
    ///
    static MgcSignalEvent signalOffHookTransitionOff(string s="");
    ///
    static MgcSignalEvent signalOnHookTransitionOn(string s="");
    ///
    static MgcSignalEvent signalOnHookTransitionOff(string s="");
    ///
    static MgcSignalEvent signalFlashHook(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOn(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOff(string s="");
    ///
    static MgcSignalEvent signalBusyToneOn(string s="");
    ///
    static MgcSignalEvent signalBusyToneOff(string s="");
    ///
    static MgcSignalEvent signalCallWaitingToneOn(string s="");
    ///
    static MgcSignalEvent signalCallWaitingToneOff(string s="");
    ///
    static MgcSignalEvent signalDialToneOn(string s="");
    ///
    static MgcSignalEvent signalDialToneOff(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOn(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOff(string s="");
    ///
    static MgcSignalEvent signalRingingOn(string s="");
    ///
    static MgcSignalEvent signalRingingOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOff(string s="");
    ///
    static MgcSignalEvent signalPromptTone(string s="");
    ///
    static MgcSignalEvent signalErrorTone(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOn(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOff(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOn(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOff(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOn(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOff(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveTonePattern(const string& arg, string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcHandsetPackageEvent : public MgcEvent
{
public:
    ///
    MgcHandsetPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// Allows the stack to generate an ObservedEvent
    MgcHandsetPackageEvent() : myType(MgcHandsetPackageUndefined) {};

    ///
    MgcHandsetPackageEvent(MgcpHandsetPackageItemType type) 
	: myType(type) { };

    ///
    MgcpPackageType getPackageType() { return MgcpHandsetPackageType; };
    /// get the type of event that the Gateway is sending to us
    MgcpHandsetPackageItemType getType() { return myEvent.getHandsetType(); }
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();

private:
// only the Mgc stack can construct these 
//    MgcpGenericEvent myEvent;
    MgcpHandsetPackageItemType myType;
};

///
class MgcRTPPackage
{
public:
    ///
    static MgcRequestedEvent requestUsedCodecChanged(string s="");
    ///
    static MgcRequestedEvent requestSamplingRateChanged(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestJitterBufferSizeChanged(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestPacketLossExceeded(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestQualityAlert(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcRTPPackageEvent : public MgcEvent
{
public:
    ///
    MgcRTPPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpRTPPackageItemType getType() {return  myEvent.getRTPType(); }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcNetworkAccessServerPackage
{
public:
    static MgcRequestedEvent requestPacketArrival(string s="");
    ///
    static MgcRequestedEvent requestCallBackRequest(string s="");
    ///
    static MgcRequestedEvent requestCarrierLost(string s="");
    ///
    static MgcRequestedEvent requestAuthorizationSucceeded(string s="");
    ///
    static MgcRequestedEvent requestAuthorizationDenied(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

};


/// This class has things the Gateway can send to the agent 
class MgcNetworkAccessServerPackageEvent : public MgcEvent
{
public:
    ///
    MgcNetworkAccessServerPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpNetworkAccessServerPackageItemType getType() { return myEvent.getNASType(); }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcAnnouncementPackage
{
public:
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalPlayAnAnnouncementOn(const string& arg0, const string& arg1, string s="");
    ///
    static MgcSignalEvent signalPlayAnAnnouncementOff(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcAnnouncementPackageEvent : public MgcEvent
{
public:
    ///
    MgcAnnouncementPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpAnnouncementPackageItemType getType() { return myEvent.getAnnounceType(); }
    /// Get the i'th paramter 
///    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
///    int getNumParameters();
private:
/// only the Mgc stack can construct these 
};

class MgcScriptPackage
{
public:
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalLoadAJavaScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAJavaScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadAPerlScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAPerlScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadATCLScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadATCLScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadAnXMLScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAnXMLScriptOff(string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcScriptPackageEvent : public MgcEvent
{
public:
    ///
    MgcScriptPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpScriptPackageItemType getType() { return myEvent.getScriptType(); }
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
// only the Mgc stack can construct these 
};

class MgcXVoiceMailPackage
{
public:
    ///
    static MgcRequestedEvent requestMailboxNumber(const string& arg, string s="" );
    ///
    static MgcRequestedEvent requestMessageWaitingInd(const string& arg, string s="");
    ///
    ///  static MgcRequestedEvent requestAllEvents();


    ///
    static MgcSignalEvent signalCallingName(const string& arg, string s="");
    ///
    static MgcSignalEvent signalCallerId(const string& arg, string s="");
    ///
    static MgcSignalEvent signalReasonCallForward(string s="");
    ///
    static MgcSignalEvent signalReasonForwardNoAnswer(string s="");
    ///
    static MgcSignalEvent signalReasonForwardBusy(string s="");
    ///
    static MgcSignalEvent signalReasonForwardOnTime(const string& arg, string s="");
    ///
    static MgcSignalEvent signalMailboxNumber(const string& arg, string s="");
    ///
    static MgcSignalEvent signalMessageWaitingInd(const string& arg, string s="");
};


/// This class has things the Gateway can send to the agent 
class MgcXVoiceMailPackageEvent : public MgcEvent
{
public:
    ///
    MgcXVoiceMailPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    /// get the type of event that the Gateway is sending to us
    MgcpXVoiceMailPackageItemType getType() { return myEvent.getXVoiceMailType(); };
    /// Get the i'th paramter 
//    string getParameter(int index);
    /// Get the number of paramters - usually 0 - depends on type
//    int getNumParameters();
private:
// only the Mgc stack can construct these 
};


class MgcAllPackages
{
 public:
    ///
    static MgcRequestedEvent requestModemDetect(string s="");
    ///
    static MgcRequestedEvent requestFaxToneDetect(string s="");
    ///
    static MgcRequestedEvent requestLongDurationConnection(string s="");
    ///
    static MgcRequestedEvent requestPatternDetected(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestDTMF_0(string s="");
    ///
    static MgcRequestedEvent requestDTMF_1(string s="");
    ///
    static MgcRequestedEvent requestDTMF_2(string s="");
    ///
    static MgcRequestedEvent requestDTMF_3(string s="");
    ///
    static MgcRequestedEvent requestDTMF_4(string s="");
    ///
    static MgcRequestedEvent requestDTMF_5(string s="");
    ///
    static MgcRequestedEvent requestDTMF_6(string s="");
    ///
    static MgcRequestedEvent requestDTMF_7(string s="");
    ///
    static MgcRequestedEvent requestDTMF_8(string s="");
    ///
    static MgcRequestedEvent requestDTMF_9(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Hash(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Star(string s="");
    ///
    static MgcRequestedEvent requestDTMF_A(string s="");
    ///
    static MgcRequestedEvent requestDTMF_B(string s="");
    ///
    static MgcRequestedEvent requestDTMF_C(string s="");
    ///
    static MgcRequestedEvent requestDTMF_D(string s="");
    ///
    static MgcRequestedEvent requestLongDurationIndicator(string s="");
    ///
    static MgcRequestedEvent requestWildcardMatch(string s="");
    ///
    static MgcRequestedEvent requestInterdigitTimer(string s="");
    ///
    static MgcRequestedEvent requestMFK0(string s="");
    ///
    static MgcRequestedEvent requestMFK1(string s="");
    ///
    static MgcRequestedEvent requestMFK2(string s="");
    ///
    static MgcRequestedEvent requestMFS0(string s="");
    ///
    static MgcRequestedEvent requestMFS1(string s="");
    ///
    static MgcRequestedEvent requestMFS2(string s="");
    ///
    static MgcRequestedEvent requestMFS3(string s="");
    ///
    static MgcRequestedEvent requestWink(string s="");
    ///
    static MgcRequestedEvent requestWinkOff(string s="");
    ///
    static MgcRequestedEvent requestIncomingSeizure(string s="");
    ///
    static MgcRequestedEvent requestReturnSeizure(string s="");
    ///
    static MgcRequestedEvent requestUnseizeCircuit(string s="");
    ///
    static MgcRequestedEvent requestContinuityTone(string s="");
    ///
    static MgcRequestedEvent requestContinuityTest(string s="");
    ///
    static MgcRequestedEvent requestOldMilliwattTone(string s="");
    ///
    static MgcRequestedEvent requestNewMilliwattTone(string s="");
    ///
    static MgcRequestedEvent requestTestLine(string s="");
    ///
    static MgcRequestedEvent requestNoCircuit(string s="");
    ///
    static MgcRequestedEvent requestAnswerSupervision(string s="");
    ///
    static MgcRequestedEvent requestReorderTone(string s="");
    ///
    static MgcRequestedEvent requestOffHookTransition(string s="");
    ///
    static MgcRequestedEvent requestOnHookTransition(string s="");
    ///
    static MgcRequestedEvent requestFlashHook(string s="");
    ///
    static MgcRequestedEvent requestAnswerTone(string s="");
    ///
    static MgcRequestedEvent requestNetworkBusy(string s="");
    ///
    static MgcRequestedEvent requestPromptTone(string s="");
    ///
    static MgcRequestedEvent requestErrorTone(string s="");
    ///
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestAdsiDisplay(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestBusyTone(string s="");
    ///
    static MgcRequestedEvent requestCallWaitingTone(string s="");
    ///
    static MgcRequestedEvent requestDialTone(string s="");
    ///
    static MgcRequestedEvent requestRinging(string s="");
    ///
    static MgcRequestedEvent requestDistinctiveRinging(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestStutterDialtone(string s="");
    ///
    static MgcRequestedEvent requestAlertingTone(string s="");
    ///
    static MgcRequestedEvent requestRecorderWarningTone(string s="");
    ///
    static MgcRequestedEvent requestSITTone(string s="");
    ///
    static MgcRequestedEvent requestCallingCardServiceTone(string s="");
    ///
    static MgcRequestedEvent requestOffHookWarningTone(string s="");
    ///
    static MgcRequestedEvent requestUsedCodecChanged(string s="");
    ///
    static MgcRequestedEvent requestSamplingRateChanged(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestJitterBufferSizeChanged(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestPacketLossExceeded(const string& arg, string s="");
    ///
    static MgcRequestedEvent requestQualityAlert(string s="");
    ///
    static MgcRequestedEvent requestPacketArrival(string s="");
    ///
    static MgcRequestedEvent requestCallBackRequest(string s="");
    ///
    static MgcRequestedEvent requestCarrierLost(string s="");
    ///
    static MgcRequestedEvent requestAuthorizationSucceeded(string s="");
    ///
    static MgcRequestedEvent requestAuthorizationDenied(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");


    ///
    static MgcSignalEvent signalPlayAnAnnouncementOn(const string& arg0, const string& arg1, string s="");
    ///
    static MgcSignalEvent signalPlayAnAnnouncementOff(string s="");
    ///
    static MgcSignalEvent signalPatternDetectedOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalPatternDetectedOff(string s="");
    ///
    static MgcSignalEvent signalRingBackToneOn(string s="");
    ///
    static MgcSignalEvent signalRingBackToneOff(string s="");
    ///
    static MgcSignalEvent signalRingBackOnConnectionOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalRingBackOnConnectionOff(string s="");
    ///
    static MgcSignalEvent signalConfirmTone(string s="");
    ///
    static MgcSignalEvent signalNetworkCongestionToneOn(string s="");
    ///
    static MgcSignalEvent signalNetworkCongestionToneOff(string s="");
    ///
    static MgcSignalEvent signalInterceptToneOn(string s="");
    ///
    static MgcSignalEvent signalInterceptToneOff(string s="");
    ///
    static MgcSignalEvent signalPreemtionToneOn(string s="");
    ///
    static MgcSignalEvent signalPreemtionToneOff(string s="");
    ///
    static MgcSignalEvent signalDTMF_0(string s="");
    ///
    static MgcSignalEvent signalDTMF_1(string s="");
    ///
    static MgcSignalEvent signalDTMF_2(string s="");
    ///
    static MgcSignalEvent signalDTMF_3(string s="");
    ///
    static MgcSignalEvent signalDTMF_4(string s="");
    ///
    static MgcSignalEvent signalDTMF_5(string s="");
    ///
    static MgcSignalEvent signalDTMF_6(string s="");
    ///
    static MgcSignalEvent signalDTMF_7(string s="");
    ///
    static MgcSignalEvent signalDTMF_8(string s="");
    ///
    static MgcSignalEvent signalDTMF_9(string s="");
    ///
    static MgcSignalEvent signalDTMF_Hash(string s="");
    ///
    static MgcSignalEvent signalDTMF_Star(string s="");
    ///
    static MgcSignalEvent signalDTMF_A(string s="");
    ///
    static MgcSignalEvent signalDTMF_B(string s="");
    ///
    static MgcSignalEvent signalDTMF_C(string s="");
    ///
    static MgcSignalEvent signalDTMF_D(string s="");
    ///
    static MgcSignalEvent signalMFK0(string s="");
    ///
    static MgcSignalEvent signalMFK1(string s="");
    ///
    static MgcSignalEvent signalMFK2(string s="");
    ///
    static MgcSignalEvent signalMFS0(string s="");
    ///
    static MgcSignalEvent signalMFS1(string s="");
    ///
    static MgcSignalEvent signalMFS2(string s="");
    ///
    static MgcSignalEvent signalMFS3(string s="");
    ///
    static MgcSignalEvent signalWink(string s="");
    ///
    static MgcSignalEvent signalWinkOff(string s="");
    ///
    static MgcSignalEvent signalIncomingSeizureOn(string s="");
    ///
    static MgcSignalEvent signalIncomingSeizureOff(string s="");
    ///
    static MgcSignalEvent signalReturnSeizureOn(string s="");
    ///
    static MgcSignalEvent signalReturnSeizureOff(string s="");
    ///
    static MgcSignalEvent signalUnseizeCircuitOn(string s="");
    ///
    static MgcSignalEvent signalUnseizeCircuitOff(string s="");
    ///
    static MgcSignalEvent signalContinuityToneOn(string s="");
    ///
    static MgcSignalEvent signalContinuityToneOff(string s="");
    ///
    static MgcSignalEvent signalContinuityTestOn(string s="");
    ///
    static MgcSignalEvent signalContinuityTestOff(string s="");
    ///
    static MgcSignalEvent signalLoopbackOn(string s="");
    ///
    static MgcSignalEvent signalLoopbackOff(string s="");
    ///
    static MgcSignalEvent signalOldMilliwattToneOn(string s="");
    ///
    static MgcSignalEvent signalOldMilliwattToneOff(string s="");
    ///
    static MgcSignalEvent signalNewMilliwattToneOn(string s="");
    ///
    static MgcSignalEvent signalNewMilliwattToneOff(string s="");
    ///
    static MgcSignalEvent signalTestLineOn(string s="");
    ///
    static MgcSignalEvent signalTestLineOff(string s="");
    ///
    static MgcSignalEvent signalNoCircuitOn(string s="");
    ///
    static MgcSignalEvent signalNoCircuitOff(string s="");
    ///
    static MgcSignalEvent signalAnswerSupervisionOn(string s="");
    ///
    static MgcSignalEvent signalAnswerSupervisionOff(string s="");
    ///
    static MgcSignalEvent signalReorderToneOn(string s="");
    ///
    static MgcSignalEvent signalReorderToneOff(string s="");
    ///
    static MgcSignalEvent signalAdsiDisplay(const string& arg, string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOn(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOff(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOn(string s="");
    ///
    static MgcSignalEvent signalAnswerToneOff(string s="");
    ///
    static MgcSignalEvent signalBusyToneOn(string s="");
    ///
    static MgcSignalEvent signalBusyToneOff(string s="");
    ///
    static MgcSignalEvent signalCallerId(const string& arg, string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone1(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone2(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone3(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone4(string s="");
    ///
    static MgcSignalEvent signalDialToneOn(string s="");
    ///
    static MgcSignalEvent signalDialToneOff(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingIndOn(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingIndOff(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOn(string s="");
    ///
    static MgcSignalEvent signalNetworkBusyOff(string s="");
    ///
    static MgcSignalEvent signalRingingOn(string s="");
    ///
    static MgcSignalEvent signalRingingOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalDistinctiveRingingOff(string s="");
    ///
    static MgcSignalEvent signalRingsplash(string s="");
    ///
    static MgcSignalEvent signalPromptTone(string s="");
    ///
    static MgcSignalEvent signalErrorTone(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOn(string s="");
    ///
    static MgcSignalEvent signalStutterDialtoneOff(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOn(string s="");
    ///
    static MgcSignalEvent signalAlertingToneOff(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalRecorderWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOn(string s="");
    ///
    static MgcSignalEvent signalCallingCardServiceToneOff(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOn(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningToneOff(string s="");
    ///
    static MgcSignalEvent signalDistinctiveTonePattern(const string& arg, string s="");
    ///
    static MgcSignalEvent signalOffHookTransitionOn(string s="");
    ///
    static MgcSignalEvent signalOffHookTransitionOff(string s="");
    ///
    static MgcSignalEvent signalOnHookTransitionOn(string s="");
    ///
    static MgcSignalEvent signalOnHookTransitionOff(string s="");
    ///
    static MgcSignalEvent signalFlashHook(string s="");
    ///
    static MgcSignalEvent signalLoadAJavaScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAJavaScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadAPerlScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAPerlScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadATCLScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadATCLScriptOff(string s="");
    ///
    static MgcSignalEvent signalLoadAnXMLScriptOn(const string& arg, string s="");
    ///
    static MgcSignalEvent signalLoadAnXMLScriptOff(string s="");

};

class MgcAllPackagesEvent: public MgcEvent
{
public:
  MgcAllPackagesEvent( const MgcpGenericEvent& e ) : MgcEvent( e ) {}
  MgcpAllPackagesItemType getType() { return myEvent.getAllPackagesType(); }

};
#endif  // end of #ifndef NCS_PROFILE


/************************************************************/
/* NCS PROFILE part */
/************************************************************/

#ifdef NCS_PROFILE
/// NCS Profile types

class MgcNCSLinePackage
{
public:
  /// events that call agent can request to be notified of 
    ///
    static MgcRequestedEvent requestModemDetect(string s="");
    ///
    static MgcRequestedEvent requestFaxToneDetect(string s="");
    ///
    static MgcRequestedEvent requestLongDurationConnection(string s="");
    ///
    static MgcRequestedEvent requestMediaStart(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestDTMF_0(string s="");
    ///
    static MgcRequestedEvent requestDTMF_1(string s="");
    ///
    static MgcRequestedEvent requestDTMF_2(string s="");
    ///
    static MgcRequestedEvent requestDTMF_3(string s="");
    ///
    static MgcRequestedEvent requestDTMF_4(string s="");
    ///
    static MgcRequestedEvent requestDTMF_5(string s="");
    ///
    static MgcRequestedEvent requestDTMF_6(string s="");
    ///
    static MgcRequestedEvent requestDTMF_7(string s="");
    ///
    static MgcRequestedEvent requestDTMF_8(string s="");
    ///
    static MgcRequestedEvent requestDTMF_9(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Hash(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Star(string s="");
    ///
    static MgcRequestedEvent requestDTMF_A(string s="");
    ///
    static MgcRequestedEvent requestDTMF_B(string s="");
    ///
    static MgcRequestedEvent requestDTMF_C(string s="");
    ///
    static MgcRequestedEvent requestDTMF_D(string s="");
    ///
    static MgcRequestedEvent requestLongDurationIndicator(string s="");
    ///
    static MgcRequestedEvent requestWildcardMatch(string s="");
    ///
    static MgcRequestedEvent requestInterdigitTimer(string s="");
    ///
    static MgcRequestedEvent requestOffHookTransition(string s="");
    ///
    static MgcRequestedEvent requestOnHookTransition(string s="");
    ///
    static MgcRequestedEvent requestFlashHook(string s="");
    ///
    static MgcRequestedEvent requestAnswerTone(string s="");
    ///
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestOffHookWarningTone(string s="");
    ///
    static MgcRequestedEvent requestTeletypeDeviceForDeaf(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

    /// signals that call agent can request to be applied
    ///
    static MgcSignalEvent signalRingBackTone(string s="");
    ///
    static MgcSignalEvent signalConfirmTone(string s="");
    ///
    static MgcSignalEvent signalDTMF_0(string s="");
    ///
    static MgcSignalEvent signalDTMF_1(string s="");
    ///
    static MgcSignalEvent signalDTMF_2(string s="");
    ///
    static MgcSignalEvent signalDTMF_3(string s="");
    ///
    static MgcSignalEvent signalDTMF_4(string s="");
    ///
    static MgcSignalEvent signalDTMF_5(string s="");
    ///
    static MgcSignalEvent signalDTMF_6(string s="");
    ///
    static MgcSignalEvent signalDTMF_7(string s="");
    ///
    static MgcSignalEvent signalDTMF_8(string s="");
    ///
    static MgcSignalEvent signalDTMF_9(string s="");
    ///
    static MgcSignalEvent signalDTMF_Hash(string s="");
    ///
    static MgcSignalEvent signalDTMF_Star(string s="");
    ///
    static MgcSignalEvent signalDTMF_A(string s="");
    ///
    static MgcSignalEvent signalDTMF_B(string s="");
    ///
    static MgcSignalEvent signalDTMF_C(string s="");
    ///
    static MgcSignalEvent signalDTMF_D(string s="");
    ///
    static MgcSignalEvent signalReorderTone(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOn(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOff(string s="");
    ///
    static MgcSignalEvent signalBusyTone(string s="");
    ///
    static MgcSignalEvent signalCallerId(const string& arg, string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone1(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone2(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone3(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone4(string s="");
    ///
    static MgcSignalEvent signalDialTone(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingInd(string s="");
    ///
    static MgcSignalEvent signalRinging(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging0(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging1(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging2(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging3(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging4(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging5(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging6(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging7(string s="");
    ///
    static MgcSignalEvent signalRingsplash(string s="");
    ///
    static MgcSignalEvent signalStutterDialtone(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningTone(string s="");

private:

};

class MgcNCSLinePackageEvent : public MgcEvent
{
public:
    MgcNCSLinePackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    
    MgcpNCSLinePackageItemType getType() 
	  { 
		return myEvent.getNCSLinePackageType(); 
	  }
   
private:

};

class MgcNCSAdsiPackage
{
public:
    ///
    static MgcSignalEvent signalAdsiDisplay(const string& arg, string s="");
private:

};

class MgcNCSAdsiPackageEvent : public MgcEvent
{
public:
    MgcNCSAdsiPackageEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    
    MgcpNCSAdsiPackageItemType getType() { return myEvent.getNCSAdsiPackageType(); }
   
private:

};

class MgcNCSAllPackages
{
public:
    ///
    static MgcRequestedEvent requestModemDetect(string s="");
    ///
    static MgcRequestedEvent requestFaxToneDetect(string s="");
    ///
    static MgcRequestedEvent requestLongDurationConnection(string s="");
    ///
    static MgcRequestedEvent requestMediaStart(string s="");
    ///
    static MgcRequestedEvent requestReportFailure(string s="");
    ///
    static MgcRequestedEvent requestDTMF_0(string s="");
    ///
    static MgcRequestedEvent requestDTMF_1(string s="");
    ///
    static MgcRequestedEvent requestDTMF_2(string s="");
    ///
    static MgcRequestedEvent requestDTMF_3(string s="");
    ///
    static MgcRequestedEvent requestDTMF_4(string s="");
    ///
    static MgcRequestedEvent requestDTMF_5(string s="");
    ///
    static MgcRequestedEvent requestDTMF_6(string s="");
    ///
    static MgcRequestedEvent requestDTMF_7(string s="");
    ///
    static MgcRequestedEvent requestDTMF_8(string s="");
    ///
    static MgcRequestedEvent requestDTMF_9(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Hash(string s="");
    ///
    static MgcRequestedEvent requestDTMF_Star(string s="");
    ///
    static MgcRequestedEvent requestDTMF_A(string s="");
    ///
    static MgcRequestedEvent requestDTMF_B(string s="");
    ///
    static MgcRequestedEvent requestDTMF_C(string s="");
    ///
    static MgcRequestedEvent requestDTMF_D(string s="");
    ///
    static MgcRequestedEvent requestLongDurationIndicator(string s="");
    ///
    static MgcRequestedEvent requestWildcardMatch(string s="");
    ///
    static MgcRequestedEvent requestInterdigitTimer(string s="");
    ///
    static MgcRequestedEvent requestOffHookTransition(string s="");
    ///
    static MgcRequestedEvent requestOnHookTransition(string s="");
    ///
    static MgcRequestedEvent requestFlashHook(string s="");
    ///
    static MgcRequestedEvent requestAnswerTone(string s="");
    ///
    static MgcRequestedEvent requestReportOnCompletion(string s="");
    ///
    static MgcRequestedEvent requestOffHookWarningTone(string s="");
    ///
    static MgcRequestedEvent requestTeletypeDeviceForDeaf(string s="");
    ///
    static MgcRequestedEvent requestAllEvents(string s="");

    /// signals that call agent can request to be applied
    ///
    static MgcSignalEvent signalRingBackTone(string s="");
    ///
    static MgcSignalEvent signalConfirmTone(string s="");
    ///
    static MgcSignalEvent signalDTMF_0(string s="");
    ///
    static MgcSignalEvent signalDTMF_1(string s="");
    ///
    static MgcSignalEvent signalDTMF_2(string s="");
    ///
    static MgcSignalEvent signalDTMF_3(string s="");
    ///
    static MgcSignalEvent signalDTMF_4(string s="");
    ///
    static MgcSignalEvent signalDTMF_5(string s="");
    ///
    static MgcSignalEvent signalDTMF_6(string s="");
    ///
    static MgcSignalEvent signalDTMF_7(string s="");
    ///
    static MgcSignalEvent signalDTMF_8(string s="");
    ///
    static MgcSignalEvent signalDTMF_9(string s="");
    ///
    static MgcSignalEvent signalDTMF_Hash(string s="");
    ///
    static MgcSignalEvent signalDTMF_Star(string s="");
    ///
    static MgcSignalEvent signalDTMF_A(string s="");
    ///
    static MgcSignalEvent signalDTMF_B(string s="");
    ///
    static MgcSignalEvent signalDTMF_C(string s="");
    ///
    static MgcSignalEvent signalDTMF_D(string s="");
    ///
    static MgcSignalEvent signalReorderTone(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOn(string s="");
    ///
    static MgcSignalEvent signalVisualMessageWaitingIndicatorOff(string s="");
    ///
    static MgcSignalEvent signalBusyTone(string s="");
    ///
    static MgcSignalEvent signalCallerId(const string& arg,string s="" );
    ///
    static MgcSignalEvent signalCallWaitingTone1(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone2(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone3(string s="");
    ///
    static MgcSignalEvent signalCallWaitingTone4(string s="");
    ///
    static MgcSignalEvent signalDialTone(string s="");
    ///
    static MgcSignalEvent signalMessageWaitingInd(string s="");
    ///
    static MgcSignalEvent signalRinging(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging0(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging1(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging2(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging3(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging4(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging5(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging6(string s="");
    ///
    static MgcSignalEvent signalDistinctiveRinging7(string s="");
    ///
    static MgcSignalEvent signalRingsplash(string s="");
    ///
    static MgcSignalEvent signalStutterDialtone(string s="");
    ///
    static MgcSignalEvent signalOffHookWarningTone(string s="");
    ///
    static MgcSignalEvent signalAdsiDisplay(const string& arg, string s="");
private:

};

class MgcNCSAllPackagesEvent : public MgcEvent
{
public:
    MgcNCSAllPackagesEvent(const MgcpGenericEvent& e) : MgcEvent(e) {}
    
    MgcpNCSAllPackagesItemType getType() { return myEvent.getNCSAllPackagesType(); }
   
private:

};

#endif // end of #ifdef NCS_PROFILE



#endif
