
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcParse_cxx_version =
"$Id: MgcParse.cxx,v 1.8 2001/02/01 01:22:56 icahoon Exp $";


#include "global.h"
#include "MgcParse.hxx"
#include "MgcpParse.hxx"


Sptr<MgcCommand>
mgcpToMgc(MgcpCommand* test_item)
{
    ODEBUG(DEBUG_NTWK, 3, cerr << "Begin *** mgcpToMgc() *** " << endl;);

    MgcCommand* msg = 0;

    assert(test_item);

    switch(test_item->getType())
    {
    case MgcpCommandTypeResponse:
	msg = new MgcCommand(test_item);
	break;
	    
    case MgcpCommandTypeEndpointConfiguration:
	msg = new MgcEndpointConfiguration(test_item);
	break;

    case MgcpCommandTypeNotificationRequest:
	msg = new MgcNotificationRequest(test_item);
	break;
	    
    case MgcpCommandTypeNotify:
	msg = new MgcNotify(test_item);
	break;

    case MgcpCommandTypeCreateConnection:
	msg = new MgcCreateConnection(test_item);
	break;

    case MgcpCommandTypeModifyConnection:
	msg = new MgcModifyConnection(test_item);
         break;

    case MgcpCommandTypeDeleteConnection:
         msg = new MgcDeleteConnectionGw(test_item);
         break;

    case MgcpCommandTypeAuditEndpoint:
         msg = new MgcAuditEndpoint(test_item);
         break;

    case MgcpCommandTypeAuditConnection:
         msg = new MgcAuditConnection(test_item);
         break;

    case MgcpCommandTypeRestartInProgress:
         msg = new MgcRestartInProgress(test_item);
         break;

    default:
         cpLog( LOG_ERR, "Unknown MgcpCommandType !");
	 assert( 0 );
         break;
    }

    assert(msg);

    ODEBUG(DEBUG_NTWK, 3, cerr << "End *** mgcpToMgc() *** " << endl;);

    return Sptr<MgcCommand>(msg);
}


string encodeToSend(MgcCommand& msg)
{
    assert(msg.getDataPtr());
    return msg.getDataPtr()->encodeToSend();
}


Sptr<MgcCommand> stringToMgc(string& str)
{
    MgcpCommand* cmd = parseMessage(str.c_str(), str.length());

    return mgcpToMgc(cmd);
}


string MgcGetRespAckString(Sptr<MgcCommand> cmd)
{
  switch(cmd->getType())
    {
	case MgcInvalidCommandType:
	   break;
	   
	case MgcEndpointConfigurationType:
	   break;
	  
	case MgcCreateConnectionType:
	   break;
	  
	case MgcModifyConnectionType:
	   break;

	case MgcDeleteConnectionAgentType:
	   break;

	case MgcNotificationRequestType:
	   break;

	case MgcNotifyType:
	{
	  Sptr<MgcNotify> ntfyPtr;

	  ntfyPtr.dynamicCast(cmd);
	  if (ntfyPtr != 0)
	     return (ntfyPtr->getResponseAck());
	  else  
	    break;
	}
	    
	case MgcAuditEndpointType:
	   break;

	case MgcAuditConnectionType:
	   break;

	case MgcRestartInProgressType:
	   break;

	case MgcDeleteConnectionGwType:
	   break;

        default:
	   break;
    }
  
  return "";
}
