
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* MgcpConnectionOptions_cxx_version =
"$Id: MgcpConnectionOptions.cxx,v 1.13 2001/01/26 02:46:24 liuhong Exp $";



#include "MgcpConnectionOptions.hxx"
#include <string>
#include "support.hxx"


///
void 
MgcpConnectionOptions::setPacketization(int value,int val) {
      packetization[0] = value;
      packetization[1] = val;
}

///
bool
MgcpConnectionOptions::isPacketizationRange()
{
    if( packetization[1] )
       return true;
    else
       return false;
}
///
int
MgcpConnectionOptions:: getPacketization() {
  return packetization[0];
    }
///
void
MgcpConnectionOptions::getPacketizationRange(int* p1, int* p2)
{
    *p1 = packetization[0];
    *p2 = packetization[1];
}
///
string 
MgcpConnectionOptions::PacketizationEncode() {
        string y;

       y = "p:";
	   y += itos( packetization[0] );
       y += "-";
       y += itos( packetization[1] );
       return y; 
}

bool
MgcpConnectionOptions::PacketizationParse(const ParameterSplitStr& params)
{
       ParameterSplitStr::const_iterator iter = params.begin();
       bool prevFlag = true;
       string s;
       bool bol;
     
	   while( iter != params.end() )
	   { 
         s = (*iter);
         if( prevFlag )
		   bol =  stringToInt( s, &packetization[0] );
         else
          bol =  stringToInt( s, &packetization[1]);
         prevFlag = false; 
         iter++;
       }     
       return true;
}

void 
MgcpConnectionOptions::setBandwidth(int value,int val) {
      bandwidth[0] = value;
      bandwidth[1] = val;
}

///
bool
MgcpConnectionOptions::isBandwidthRange()
{
    if( bandwidth[1] )
       return true;
    else
       return false;
}
///
int 
MgcpConnectionOptions:: getBandwidth() {
     
       return bandwidth[0];
}
///
void
MgcpConnectionOptions::getBandwidthRange(int* p1, int* p2)
{
    *p1 = bandwidth[0];
    *p2 = bandwidth[1];
}

///
string 
MgcpConnectionOptions::BandwidthEncode()
{
   string y;
    
   y = "b:";
   y += itos( bandwidth[0] );
   y += "-";
   y +=itos( bandwidth[1] );
   return y;
}   

bool
MgcpConnectionOptions::BandwidthParse(const ParameterSplitStr& params)
{
       ParameterSplitStr::const_iterator iter = params.begin();
       bool prevFlag = true;
       bool bol;
	  
       while( iter != params.end() )
	   { 
         string s = (*iter);
         if( prevFlag )
            bol= stringToInt( s, &bandwidth[0] );
         else
            bol = stringToInt( s, &bandwidth[1] );
         prevFlag = false; 
         iter++;
       }     
       return true;
}


 ///
void 
MgcpConnectionOptions::setCompressionAlgo( string value)
{  
    list.push_back( value ); 
}

void
MgcpConnectionOptions::setCompressionAlgoList( vector<string> algoList )
{
    list = algoList;
}
///
string 
MgcpConnectionOptions::getCompressionAlgo() {
        if( list.empty() )
           return (" ");
        else
           return list.front();
}

 ///
string 
MgcpConnectionOptions:: CompressionEncode() {
       string y;
       y = "a:";
       bool prevFlag = false;
       vector<string>::const_iterator iter = list.begin();
      
       while( iter != list.end() ) {
		 if(prevFlag) {
            y += ";";
         }
         y += (*iter);
         prevFlag = true;
         ++iter;  
	   }
       
      
       return y;
}

bool
MgcpConnectionOptions::CompressionAlgoParse(const ParameterSplitStr& params )
{
  ParameterSplitStr::const_iterator iter = params.begin();
  
  while( iter!= params.end() )
  {
     list.push_back((*iter));
     iter++;
  }
  return true;
}
    
///
void 
MgcpConnectionOptions::setSilenceSuppression(bool value) { 
	   silenceSuppression = value; 
 }

///
 bool 
MgcpConnectionOptions::getSilenceSuppression() { 
	return silenceSuppression; 
 }

///
string 
MgcpConnectionOptions::SilenceSuppressionEncode() {
    string y;	
    if(silenceSuppression) {
	       y = "s:on";
    	}
        else {
	      y = "s:off";
    	}
	    return y;
}


bool 
MgcpConnectionOptions::SilenceSuppressionParse(const string& parameter)
{
     if(parameter == "s:on")
    {
	  silenceSuppression = true;
    }
    else if(parameter == "s:off")
    {
	  silenceSuppression = false;
    }
    return true;
}

    
///
void
MgcpConnectionOptions:: setEchoCancellation(bool value) { 
	echoCancellation = value; 
}

///
bool 
MgcpConnectionOptions::getEchoCancellation() { 
	return echoCancellation; 
    }

///
string 
MgcpConnectionOptions::EchoCancellationEncode() {
	string y;

    if(echoCancellation) {
	    y = "e:on";
	} else {
	    y = "e:off";
	}
	return y;
}

bool
MgcpConnectionOptions::EchoCancellationParse(const string& parameter)
{
     if(parameter == "e:on")
    {
	  echoCancellation = true;
    }
    else if(parameter == "e:off")
    {
	  echoCancellation = false;
    }
    return true;
}

void 
MgcpConnectionOptions:: setTypeofService(int value) { 
	typeofService = value; 
}

///
int 
MgcpConnectionOptions::getTypeofService() { 
	return typeofService;
}

///
string
MgcpConnectionOptions::TypeofServiceEncode() {
	string y;
      
      y ="t:";
	  y += itos( typeofService );
      return y;
}

bool
MgcpConnectionOptions::TypeofServiceParse(const string& params)
{
     bool bol;
	 bol=  stringToInt( params, &typeofService );
	 return bol;
	  
}


string 
MgcpConnectionOptions:: itos(int value )
{
  // replaced by calling itos in util/support.cxx

   unsigned int temp = abs( value );

   if ( value < 0 )
   {
       return( "-" + itos( temp ) );
   }
   else
   {
       return( itos( temp ) );
   }
   

  /*
   int const bufsize = 20;
   assert(sizeof(int)<=8);
   char buffer[bufsize];
   bool negative = value<0;
   if( negative )
     value = -value;
   int endpos = bufsize;
   do {
        buffer[--endpos]=char('0'+value%10);
   }while( value /= 10 );
   if(negative)
   buffer[--endpos] = '-';
   string s(&buffer[endpos], bufsize-endpos);
   return s;
  */

}









   
