#ifndef MGCP_CONNECTION_OPTIONS_HXX
#define MGCP_CONNECTION_OPTIONS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpConnectionOptions_hxx_Version = 
"$Id: MgcpConnectionOptions.hxx,v 1.3 2001/01/26 02:46:26 liuhong Exp $";



#include "global.h"
#include <string>
#include <vector>
#include <deque>
#include "SdpSession.hxx"
#include "support.hxx"


typedef split_t ParameterSplitStr;


class MgcpConnectionOptions
{
public:
    ///
    MgcpConnectionOptions()
	{
	  echoCancellation =true;
      list.clear();
      for(int i=0;i<2;++i) {
            packetization[i] = 0;
            bandwidth[i] = 0;
      };
      typeofService = 0; 
      silenceSuppression = false;
    }
    ///
   MgcpConnectionOptions(bool echo, bool silence, vector<string> algolist, int packet1, int bandwidth1, int tos, int packet2=0, int bandwidth2=0 )
	 {
         setEchoCancellation( echo );
         setSilenceSuppression( silence );
         setCompressionAlgoList( algolist );
         setPacketization( packet1,packet2);
         setBandwidth( bandwidth1, bandwidth2 );
         setTypeofService( tos );
     }

   ///
    MgcpConnectionOptions(const MgcpConnectionOptions& x)
	{
	   echoCancellation = x.echoCancellation;
       for(int k=0;k<2;k++) {
         packetization[k] = x.packetization[k];
         bandwidth[k] = x.bandwidth[k];
       };
       typeofService = x.typeofService;
       list = x.list;
       silenceSuppression = x.silenceSuppression;
   };
   ///
	~MgcpConnectionOptions() {};
    ///
    void setPacketization(int value, int val=0);
    ///
    bool isPacketizationRange();
    ///
    int getPacketization();
    ///
    void getPacketizationRange(int* p1, int* p2); 
    ///
    string PacketizationEncode(); 
    ///
    bool PacketizationParse(const ParameterSplitStr& parameter);
    ///
    void setBandwidth(int value, int val=0);
    ///
    bool isBandwidthRange();
    ///
    int getBandwidth();
    ///
    void getBandwidthRange(int* p1, int* p2);
	/// 
    string BandwidthEncode();
    ///
    bool BandwidthParse(const ParameterSplitStr& parameter);
    ///
    void setCompressionAlgo( string value); 
    ///
    void setCompressionAlgoList( vector<string> algoList );
    ///
    string getCompressionAlgo();
    ///
    vector<string> getCompressionAlgoList()
    {
         return list;
    }
    ///
    string CompressionEncode();
    ///
    bool CompressionAlgoParse(const ParameterSplitStr& parameter);
    ///
    void setSilenceSuppression(bool value);
    ///
    bool getSilenceSuppression(); 
    ///
    string SilenceSuppressionEncode();
    ///
    bool SilenceSuppressionParse(const string& parameter);
    ///
    void setEchoCancellation(bool value);
    ///
    bool getEchoCancellation();
    ///
    string EchoCancellationEncode();
    ///
    bool EchoCancellationParse(const string& parameter);
	///
    void setTypeofService(int value);
    ///
    int getTypeofService();
    ///
    string TypeofServiceEncode(); 
    ///
    bool TypeofServiceParse(const string& parameter);
	///
    string itos( int value); 
    ///
    int getCompressionAlgoNum() { return list.size(); }
    /// 
    string getCompressionAlgoItem( int i) 
    {
           assert(static_cast<unsigned int>(i) < list.size());
           return list[i];
    }

private:
    bool echoCancellation;
    bool silenceSuppression;
    vector<string> list;
    int packetization[2], bandwidth[2];
    int typeofService;
    
};


#endif
