
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpGenericEvent_cxx_Version = 
"$Id: MgcpGenericEvent.cxx,v 1.26 2001/02/01 01:22:56 icahoon Exp $";


#include "MgcpGenericEvent.hxx"
#include "debug.h"
#include <string>


ostream& MgcpGenericEvent::printData(ostream& s)
{
    s << "Generic Event: " << getEncoding() << endl;
    return s;
}

ostream& operator<<(ostream& s, MgcpGenericEvent& x) 
{
    x.printData(s);
    return s;
}


#ifndef NCS_PROFILE
string MgcpGenericEvent::getGenericItemTypeEncoding()
{
  string encoding = "";

  switch(myGenericType)
  {
    case MgcGenericPackageModemDetect:
    {
    encoding = "mt";
    }
    break;
    case MgcGenericPackageFaxToneDetect:
    {
    encoding = "ft";
    }
    break;
    case MgcGenericPackageLongDurationConnection:
    {
    encoding = "ld";
    }
    break;
    case MgcGenericPackagePatternDetected:
    {
    encoding = "pat(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcGenericPackageRingBackTone:
    {
    encoding = "rt";
    }
    break;
    case MgcGenericPackageRingBackOnConnection:
    {
    encoding = "rbk(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcGenericPackageConfirmTone:
    {
    encoding = "cf";
    }
    break;
    case MgcGenericPackageNetworkCongestionTone:
    {
    encoding = "cg";
    }
    break;
    case MgcGenericPackageInterceptTone:
    {
    encoding = "it";
    }
    break;
    case MgcGenericPackagePreemtionTone:
    {
    encoding = "pt";
    }
    break;
    case MgcGenericPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcGenericPackageAllEvents:
    {
    encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return encoding;
}


string MgcpGenericEvent::getDTMFItemTypeEncoding()
{
  string encoding = "";

  switch(myDTMFType)
  {
    case MgcDTMFPackageDTMF_0:
    {
    encoding = "0";
    }
    break;
    case MgcDTMFPackageDTMF_1:
    {
    encoding = "1";
    }
    break;
    case MgcDTMFPackageDTMF_2:
    {
    encoding = "2";
    }
    break;
    case MgcDTMFPackageDTMF_3:
    {
    encoding = "3";
    }
    break;
    case MgcDTMFPackageDTMF_4:
    {
    encoding = "4";
    }
    break;
    case MgcDTMFPackageDTMF_5:
    {
    encoding = "5";
    }
    break;
    case MgcDTMFPackageDTMF_6:
    {
    encoding = "6";
    }
    break;
    case MgcDTMFPackageDTMF_7:
    {
    encoding = "7";
    }
    break;
    case MgcDTMFPackageDTMF_8:
    {
    encoding = "8";
    }
    break;
    case MgcDTMFPackageDTMF_9:
    {
    encoding = "9";
    }
    break;
    case MgcDTMFPackageDTMF_Hash:
    {
    encoding = "#";
    }
    break;
    case MgcDTMFPackageDTMF_Star:
    {
    encoding = "*";
    }
    break;
    case MgcDTMFPackageDTMF_A:
    {
    encoding = "a";
    }
    break;
    case MgcDTMFPackageDTMF_B:
    {
    encoding = "b";
    }
    break;
    case MgcDTMFPackageDTMF_C:
    {
    encoding = "c";
    }
    break;
    case MgcDTMFPackageDTMF_D:
    {
    encoding = "d";
    }
    break;
    case MgcDTMFPackageLongDurationIndicator:
    {
    encoding = "l";
    }
    break;
    case MgcDTMFPackageWildcardMatchAnyDigit_0_9:
    {
#ifdef CISCO_PROFILE
    encoding = "[0-9#*]";
#else
    encoding = "x";
#endif
    }
    break;
    case MgcDTMFPackageInterdigitTimer:
    {
    encoding = "t";
    }
    break;
    case MgcDTMFPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcDTMFPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getLineItemTypeEncoding()
{
  string encoding = "";

  switch(myLineType)
  {
    case MgcLinePackageAdsiDisplay:
    {
    encoding = "adsi(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcLinePackageVisualMessageWaitingIndicator:
    {
    encoding = "vmwi";
    }
    break;
    case MgcLinePackageOffHookTransition:
    {
    encoding = "hd";
    }
    break;
    case MgcLinePackageOnHookTransition:
    {
    encoding = "hu";
    }
    break;
    case MgcLinePackageFlashHook:
    {
    encoding = "hf";
    }
    break;
    case MgcLinePackageAnswerTone:
    {
    encoding = "aw";
    }
    break;
    case MgcLinePackageBusyTone:
    {
    encoding = "bz";
    }
    break;
    case MgcLinePackageCallerId:
    {
    encoding = "ci(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcLinePackageCallWaitingTone:
    {
    encoding = "wt";
    }
    break;
    case MgcLinePackageCallWaitingTone1:
    {
    encoding = "wt1";
    }
    break;
    case MgcLinePackageCallWaitingTone2:
    {
    encoding = "wt2";
    }
    break;
    case MgcLinePackageCallWaitingTone3:
    {
    encoding = "wt3";
    }
    break;
    case MgcLinePackageCallWaitingTone4:
    {
    encoding = "wt4";
    }
    break;
    case MgcLinePackageDialTone:
    {
    encoding = "dl";
    }
    break;
    case MgcLinePackageMessageWaitingInd:
    {
    encoding = "mwi";
    }
    break;
    case MgcLinePackageNetworkBusy:
    {
    encoding = "nbz";
    }
    break;
    case MgcLinePackageRinging:
    {
    encoding = "rg";
    }
    break;
    case MgcLinePackageDistinctiveRinging:
    {
    encoding = "r(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcLinePackageRingsplash:
    {
    encoding = "rs";
    }
    break;
    case MgcLinePackagePromptTone:
    {
    encoding = "p";
    }
    break;
    case MgcLinePackageErrorTone:
    {
    encoding = "e";
    }
    break;
    case MgcLinePackageStutterDialtone:
    {
    encoding = "sdl";
    }
    break;
    case MgcLinePackageAlertingTone:
    {
    encoding = "v";
    }
    break;
    case MgcLinePackageRecorderWarningTone:
    {
    encoding = "y";
    }
    break;
    case MgcLinePackageSITTone:
    {
    encoding = "sit";
    }
    break;
    case MgcLinePackageCallingCardServiceTone:
    {
    encoding = "z";
    }
    break;
    case MgcLinePackageReportOnCompletion:
    {
    encoding = "oc";
    }
    break;
    case MgcLinePackageOffHookWarningTone:
    {
    encoding = "ot";
    }
    break;
    case MgcLinePackageDistinctiveTonePattern:
    {
    encoding = "s(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcLinePackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcLinePackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getMFItemTypeEncoding()
{
  string encoding = "";

  switch(myMFType)
  {
    case MgcMFPackageMF_0:
    {
    encoding = "0";
    }
    break;
    case MgcMFPackageMF_1:
    {
    encoding = "1";
    }
    break;
    case MgcMFPackageMF_2:
    {
    encoding = "2";
    }
    break;
    case MgcMFPackageMF_3:
    {
    encoding = "3";
    }
    break;
    case MgcMFPackageMF_4:
    {
    encoding = "4";
    }
    break;
    case MgcMFPackageMF_5:
    {
    encoding = "5";
    }
    break;
    case MgcMFPackageMF_6:
    {
    encoding = "6";
    }
    break;
    case MgcMFPackageMF_7:
    {
    encoding = "7";
    }
    break;
    case MgcMFPackageMF_8:
    {
    encoding = "8";
    }
    break;
    case MgcMFPackageMF_9:
    {
    encoding = "9";
    }
    break;
    case MgcMFPackageWildcardMatchAnyDigit_0_9:
    {
    encoding = "x";
    }
    break;
    case MgcMFPackageInterdigitTimer:
    {
    encoding = "t";
    }
    break;
    case MgcMFPackageMFK0:
    {
    encoding = "k0";
    }
    break;
    case MgcMFPackageMFK1:
    {
    encoding = "k1";
    }
    break;
    case MgcMFPackageMFK2:
    {
    encoding = "k2";
    }
    break;
    case MgcMFPackageMFS0:
    {
    encoding = "s0";
    }
    break;
    case MgcMFPackageMFS1:
    {
    encoding = "s1";
    }
    break;
    case MgcMFPackageMFS2:
    {
    encoding = "s2";
    }
    break;
    case MgcMFPackageMFS3:
    {
    encoding = "s3";
    }
    break;
    case MgcMFPackageWink:
    {
    encoding = "wk";
    }
    break;
    case MgcMFPackageWinkOff:
    {
    encoding = "wko";
    }
    break;
    case MgcMFPackageIncomingSeizure:
    {
    encoding = "is";
    }
    break;
    case MgcMFPackageReturnSeizure:
    {
    encoding = "rs";
    }
    break;
    case MgcMFPackageUnseizeCircuit:
    {
    encoding = "us";
    }
    break;
    case MgcMFPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcMFPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getTrunkItemTypeEncoding()
{
  string encoding = "";
 
  switch(myTrunkType)
  {
    case MgcTrunkPackageContinuityTone:
    {
    encoding = "co1";
    }
    break;
    case MgcTrunkPackageContinuityTest:
    {
    encoding = "co2";
    }
    break;
    case MgcTrunkPackageLoopback:
    {
    encoding = "lb";
    }
    break;
    case MgcTrunkPackageOldMilliwattTone:
    {
    encoding = "om";
    }
    break;
    case MgcTrunkPackageNewMilliwattTone:
    {
    encoding = "nm";
    }
    break;
    case MgcTrunkPackageTestLine:
    {
    encoding = "tl";
    }
    break;
    case MgcTrunkPackageNoCircuit:
    {
    encoding = "zz";
    }
    break;
    case MgcTrunkPackageAnswerSupervision:
    {
    encoding = "as";
    }
    break;
    case MgcTrunkPackageReorderTone:
    {
    encoding = "ro";
    }
    break;
    case MgcTrunkPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcTrunkPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getHandsetItemTypeEncoding()
{
  string encoding = "";

  switch(myHandsetType)
  {
    case MgcHandsetPackageAdsiDisplay:
    {
    encoding = "adsi(string)";
    }
    break;
    case MgcHandsetPackageTeletypeDeviceForDeaf:
    {
    encoding = "tdd";
    }
    break;
    case MgcHandsetPackageVisualMessageWaitingIndicator:
    {
    encoding = "vmwi";
    }
    break;
    case MgcHandsetPackageOffHookTransition:
    {
    encoding = "hd";
    }
    break;
    case MgcHandsetPackageOnHookTransition:
    {
    encoding = "hu";
    }
    break;
    case MgcHandsetPackageFlashHook:
    {
    encoding = "hf";
    }
    break;
    case MgcHandsetPackageAnswerTone:
    {
    encoding = "aw";
    }
    break;
    case MgcHandsetPackageBusyTone:
    {
    encoding = "bz";
    }
    break;
    case MgcHandsetPackageCallWaitingTone:
    {
    encoding = "wt";
    }
    break;
    case MgcHandsetPackageDialTone:
    {
    encoding = "dl";
    }
    break;
    case MgcHandsetPackageNetworkBusy:
    {
    encoding = "nbz";
    }
    break;
    case MgcHandsetPackageRinging:
    {
    encoding = "rg";
    }
    break;
    case MgcHandsetPackageDistinctiveRinging:
    {
    encoding = "r(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcHandsetPackagePromptTone:
    {
    encoding = "p";
    }
    break;
    case MgcHandsetPackageErrorTone:
    {
    encoding = "e";
    }
    break;
    case MgcHandsetPackageStutterDialtone:
    {
    encoding = "sdl";
    }
    break;
    case MgcHandsetPackageAlertingTone:
    {
    encoding = "v";
    }
    break;
    case MgcHandsetPackageRecorderWarningTone:
    {
    encoding = "y";
    }
    break;
    case MgcHandsetPackageSITTone:
    {
    encoding = "t";
    }
    break;
    case MgcHandsetPackageCallingCardServiceTone:
    {
    encoding = "z";
    }
    break;
    case MgcHandsetPackageReportOnCompletion:
    {
    encoding = "oc";
    }
    break;
    case MgcHandsetPackageOffHookWarningTone:
    {
    encoding = "ot";
    }
    break;
    case MgcHandsetPackageDistinctiveTonePattern:
    {
    encoding = "s(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcHandsetPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcHandsetPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getRTPItemTypeEncoding()
{
  string encoding = "";

  switch(myRTPType)
  {
    case MgcRTPPackageUsedCodecChanged:
    {
    encoding = "uc";
    }
    break;
    case MgcRTPPackageSamplingRateChanged:
    {
    encoding = "sr(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcRTPPackageJitterBufferSizeChanged:
    {
    encoding = "ji(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcRTPPackagePacketLossExceeded:
    {
    encoding = "pl(";
    encoding += parameter[0];
   encoding +=")";
    }
    break;
    case MgcRTPPackageQualityAlert:
    {
    encoding = "qa";
    }
    break;
    case MgcRTPPackageReportFailure:
    {
    encoding = "of";
    }
    break;
  case MgcRTPPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getNASItemTypeEncoding()
{
  string encoding = "";

  switch(myNASType)
  {
    case MgcNetworkAccessServerPackagePacketArrival:
    {
    encoding = "pa";
    }
    break;
    case MgcNetworkAccessServerPackageCallBackRequest:
    {
    encoding = "cbk";
    }
    break;
    case MgcNetworkAccessServerPackageCarrierLost:
    {
    encoding = "cl";
    }
    break;
    case MgcNetworkAccessServerPackageAuthorizationSucceeded:
    {
    encoding = "au";
    }
    break;
    case MgcNetworkAccessServerPackageAuthorizationDenied:
    {
    encoding = "ax";
    }
    break;
    case MgcNetworkAccessServerPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcNetworkAccessServerPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getAnnounceItemTypeEncoding()
{
  string encoding = "";

  switch(myAnnounceType)
  {
    case MgcAnnouncementPackagePlayAnAnnouncement:
    {
    encoding = "ann(";
    encoding += parameter[0];
    encoding +=",";
    encoding += parameter[1];
    encoding +=")";
    }
    break;
    case MgcAnnouncementPackageReportOnCompletion:
    {
    encoding = "oc";
    }
    break;
    case MgcAnnouncementPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcAnnouncementPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getScriptItemTypeEncoding()
{
  string encoding = "";

  switch(myScriptType)
  {
    case MgcScriptPackageLoadAJavaScript:
    {
    encoding = "java(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcScriptPackageLoadAPerlScript:
    {
    encoding = "perl(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcScriptPackageLoadATCLScript:
    {
    encoding = "tcl(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcScriptPackageLoadAnXMLScript:
    {
    encoding = "xml(";
    encoding += parameter[0];
    encoding +=")";
    }
    break;
    case MgcScriptPackageReportOnCompletion:
    {
    encoding = "oc";
    }
    break;
    case MgcScriptPackageReportFailure:
    {
    encoding = "of";
    }
    break;
    case MgcScriptPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    default:
    cerr << "no good\n";
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getAllPackagesItemTypeEncoding()
{
  string encoding = "";

  switch(myAllPackagesType)
  {
    case MgcAllPackagesModemDetect:
    {
      encoding = "mt";
    }
    break;
    case MgcAllPackagesFaxToneDetect:
    {
      encoding = "ft";
    }
    break;
    case MgcAllPackagesLongDurationConnection:
    {
      encoding = "ld";
    }
    break;

    case MgcAllPackagesPatternDetected:
    {
      encoding = "pat("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesRingBackTone:
    {
      encoding = "rt";
    }
    break;
    case MgcAllPackagesRingBackOnConnection:
    {
      encoding = "rbk("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesConfirmTone:
    {
      encoding = "cf";
    }
    break;
    case MgcAllPackagesNetworkCongestionTone:
    {
      encoding = "cg";
    }
    break;
    case MgcAllPackagesInterceptTone:
    {
      encoding = "it";
    }
    break;
    case MgcAllPackagesPreemtionTone:
    {
      encoding = "pt";
    }
    break;
    case MgcAllPackagesReportFailure:
    {
      encoding = "of";
    }
    break;
    case MgcAllPackagesDTMF_0:
    {
      encoding = "0";
    }
    break;
    case MgcAllPackagesDTMF_1:
    {
      encoding = "1";
    }
    break;
    case MgcAllPackagesDTMF_2:
    {
      encoding = "2";
    }
    break;
    case MgcAllPackagesDTMF_3:
    {
      encoding = "3";
    }
    break;
    case MgcAllPackagesDTMF_4:
    {
      encoding = "4";
    }
    break;
    case MgcAllPackagesDTMF_5:
    {
      encoding = "5";
    }
    break;
    case MgcAllPackagesDTMF_6:
    {
      encoding = "6";
    }
    break;
    case MgcAllPackagesDTMF_7:
    {
      encoding = "7";
    }
    break;
    case MgcAllPackagesDTMF_8:
    {
      encoding = "8";
    }
    break;
    case MgcAllPackagesDTMF_9:
    {
      encoding = "9";
    }
    break;
    case MgcAllPackagesDTMF_Hash:
    {
      encoding = "#";
    }
    break;
    case MgcAllPackagesDTMF_Star:
    {
      encoding = "*";
    }
    break;
    case MgcAllPackagesDTMF_A:
    {
      encoding = "a";
    }
    break;
    case MgcAllPackagesDTMF_B:
    {
      encoding = "b";
    }
    break;
    case MgcAllPackagesDTMF_C:
    {
      encoding = "c";
    }
    break;
    case MgcAllPackagesDTMF_D:
    {
      encoding = "d";
    }
    break;
    case MgcAllPackagesLongDurationIndicator:
    {
      encoding = "l";
    }
    break;
    case MgcAllPackagesWildcardMatchAnyDigit_0_9:
    {
#ifdef CISCO_PROFILE
     encoding = "[0-9#*]";
#else
     encoding = "x";
#endif
    }
    break;
    case MgcAllPackagesInterdigitTimer:
    {
      encoding = "t";
    }
    break;
    case MgcAllPackagesAdsiDisplay:
    {
      encoding = "adsi("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesVisualMessageWaitingIndicator:
    {
      encoding = "vmwi";
    }
    break;
    case MgcAllPackagesOffHookTransition:
    {
      encoding = "hd";
    }
    break;
    case MgcAllPackagesOnHookTransition:
    {
      encoding = "hu";
    }
    break;
    case MgcAllPackagesFlashHook:
    {
      encoding = "hf";
    }
    break;
    case MgcAllPackagesAnswerTone:
    {
      encoding = "aw";
    }
    break;
    case MgcAllPackagesBusyTone:
    {
      encoding = "bz";
    }
    break;
    case MgcAllPackagesCallerId:
    {
      encoding = "ci("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesCallWaitingTone:
    {
      encoding = "wt";
    }
    break;
    case MgcAllPackagesCallWaitingTone1:
    {
      encoding = "wt1";
    }
    break;
    case MgcAllPackagesCallWaitingTone2:
    {
      encoding = "wt2";
    }
    break;
    case MgcAllPackagesCallWaitingTone3:
    {
      encoding = "wt3";
    }
    break;
    case MgcAllPackagesCallWaitingTone4:
    {
      encoding = "wt4";
    }
    break;
    case MgcAllPackagesDialTone:
    {
      encoding = "dl";
    }
    break;
    case MgcAllPackagesMessageWaitingInd:
    {
      encoding = "mwi";
    }
    break;
    case MgcAllPackagesNetworkBusy:
    {
      encoding = "nbz";
    }
    break;
    case MgcAllPackagesDistinctiveRinging:
    {
      encoding = "r("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesRingsplashOrReturnSeizure:
    {
      encoding = "rs";
    }
    break;
    case MgcAllPackagesPromptTone:
    {
      encoding = "p";
    }
    break;
    case MgcAllPackagesErrorTone:
    {
      encoding = "e";
    }
    break;
    case MgcAllPackagesStutterDialtone:
    {
      encoding = "sdl";
    }
    break;
    case MgcAllPackagesAlertingTone:
    {
      encoding = "v";
    }
    break;
    case MgcAllPackagesRecorderWarningTone:
    {
      encoding = "y";
    }
    break;
    case MgcAllPackagesSITTone:
    {
      encoding = "sit";
    }
    break;
    case MgcAllPackagesCallingCardServiceTone:
    {
      encoding = "z";
    }
    break;
    case MgcAllPackagesReportOnCompletion:
    {
      encoding = "oc";
    }
    break;
    case MgcAllPackagesOffHookWarningTone:
    {
      encoding = "ot";
    }
    break;
    case MgcAllPackagesDistinctiveTonePattern:
    {
      encoding = "s("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesK0:
    {
      encoding = "k0";
    }
    break;
    case MgcAllPackagesK1:
    {
      encoding = "k1";
    }
    break;
    case MgcAllPackagesK2:
    {
      encoding = "k2";
    }
    break;
    case MgcAllPackagesS0:
    {
      encoding = "s0";
    }
    break;
    case MgcAllPackagesS1:
    {
      encoding = "s1";
    }
    break;
    case MgcAllPackagesS2:
    {
      encoding = "s2";
    }
    break;
    case MgcAllPackagesS3:
    {
      encoding = "s3"; 
    }
    break;
    case MgcAllPackagesWink:
    {
      encoding = "wk";
    }
    break;
    case MgcAllPackagesWinkOff:
    {
      encoding = "wko";
    }
    break;
    case MgcAllPackagesIncomingSeizure:
    {
      encoding = "is";
    }
    break;
    case MgcAllPackagesUnseizeCircuit:
    {
      encoding = "us";
    }
    break;
    case MgcAllPackagesContinuityTone:
    {
      encoding = "co1";
    }
    break;
    case MgcAllPackagesContinuityTest:
    {
      encoding = "co2";
    }
    break;
    case MgcAllPackagesLoopback:
    {
      encoding = "lb";
    }
    break;
    case MgcAllPackagesOldMilliwattTone:
    {
      encoding = "om";
    }
    break;
    case MgcAllPackagesNewMilliwattTone:
    {
      encoding = "nm";
    }
    break;
    case MgcAllPackagesTestLine:
    {
      encoding = "tl";
    }
    break;
    case MgcAllPackagesNoCircuit:
    {
      encoding = "zz";
    }
    break;

    case MgcAllPackagesAnswerSupervision:
    {
      encoding = "as";
    }
    break;
    case MgcAllPackagesReorderTone:
    {
      encoding = "ro";
    }
    break;
    case MgcAllPackagesTeletypeDeviceForDeaf:
    {
      encoding = "tdd";
    }
    break;
    case MgcAllPackagesUsedCodecChanged:
    {
      encoding = "uc";
    }
    break;
    case MgcAllPackagesSamplingRateChanged:
    {
      encoding = "sr("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesJitterBufferSizeChanged:
    {
      encoding = "ji("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesLossExceeded:
    {
      encoding = "pl("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesQualityAlert:
    {
      encoding = "qa";
    }
    break;
    case MgcAllPackagesPacketArrival:
    {
      encoding = "pa";
    }
    break;
    case MgcAllPackagesCallBackRequest:
    {
      encoding = "cbk";
    }
    break;
    case MgcAllPackagesCarrierLost:
    {
      encoding = "cl";
    }
    break;
    case MgcAllPackagesAuthorizationSucceeded:
    {
      encoding = "au";
    }
    break;
    case MgcAllPackagesAuthorizationDenied:
    {
      encoding = "ax";
    }
    break;
    case MgcAllPackagesPlayAnAnnouncement:
    {
      encoding = "ann("; 
      encoding += parameter[0]; 
      encoding +=","; 
      encoding += parameter[1]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesLoadAJavaScript:
    {
      encoding = "java("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesLoadAPerlScript:
    {
      encoding = "perl("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesLoadATCLScript:
    {
      encoding = "tcl("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesLoadAnXMLScript:
    {
      encoding = "xml("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcAllPackagesAllEvents:
    {
      encoding = "all";
    }
    break;
    case MgcAllPackagesUndefined:
      cerr << "no good\n";
      break;
    default:
      cerr << "no good\n";
      break;    
  }
 if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  } 
  return( encoding );
}


//string MgcpGenericEvent::getXVoiceMailItemTypeEncoding()
//{
//
//}
#endif // end of #ifndef NCS_PROFILE


/*************************************************************/
/// NCS_PROFILE Part

#ifdef NCS_PROFILE
string MgcpGenericEvent::getNCSLinePackageItemTypeEncoding()
{
  string encoding = "";

  switch(myNCSLineType)
  {
    case MgcNCSLinePackageModemDetect:
    {
      encoding = "mt";
    }
    break;
    case MgcNCSLinePackageFaxToneDetect:
    {
      encoding = "ft";
    }
    break;
    case MgcNCSLinePackageLongDurationConnection:
    {
      encoding = "ld";
    }
    break;

    case MgcNCSLinePackageMediaStart:
    {
      encoding = "ma"; 
    }
    break;
    case MgcNCSLinePackageRingBackTone:
    {
      encoding = "rt";
    }
    break;
    case MgcNCSLinePackageConfirmTone:
    {
      encoding = "cf";
    }
    break;
    case MgcNCSLinePackageReportFailure:
    {
      encoding = "of";
    }
    break;
    case MgcNCSLinePackageDTMF_0:
    {
      encoding = "0";
    }
    break;
    case MgcNCSLinePackageDTMF_1:
    {
      encoding = "1";
    }
    break;
    case MgcNCSLinePackageDTMF_2:
    {
      encoding = "2";
    }
    break;
    case MgcNCSLinePackageDTMF_3:
    {
      encoding = "3";
    }
    break;
    case MgcNCSLinePackageDTMF_4:
    {
      encoding = "4";
    }
    break;
    case MgcNCSLinePackageDTMF_5:
    {
      encoding = "5";
    }
    break;
    case MgcNCSLinePackageDTMF_6:
    {
      encoding = "6";
    }
    break;
    case MgcNCSLinePackageDTMF_7:
    {
      encoding = "7";
    }
    break;
    case MgcNCSLinePackageDTMF_8:
    {
      encoding = "8";
    }
    break;
    case MgcNCSLinePackageDTMF_9:
    {
      encoding = "9";
    }
    break;
    case MgcNCSLinePackageDTMF_Hash:
    {
      encoding = "#";
    }
    break;
    case MgcNCSLinePackageDTMF_Star:
    {
      encoding = "*";
    }
    break;
    case MgcNCSLinePackageDTMF_A:
    {
      encoding = "A";
    }
    break;
    case MgcNCSLinePackageDTMF_B:
    {
      encoding = "B";
    }
    break;
    case MgcNCSLinePackageDTMF_C:
    {
      encoding = "C";
    }
    break;
    case MgcNCSLinePackageDTMF_D:
    {
      encoding = "D";
    }
    break;
    case MgcNCSLinePackageLongDurationIndicator:
    {
      encoding = "L";
    }
    break;
    case MgcNCSLinePackageWildcardMatchAnyDigit_0_9:
    {
      encoding = "X";
    }
    break;
    case MgcNCSLinePackageInterdigitTimer:
    {
      encoding = "t";
    }
    break;
    case MgcNCSLinePackageVisualMessageWaitingIndicator:
    {
      encoding = "vmwi";
    }
    break;
    case MgcNCSLinePackageOffHookTransition:
    {
      encoding = "hd";
    }
    break;
    case MgcNCSLinePackageOnHookTransition:
    {
      encoding = "hu";
    }
    break;
    case MgcNCSLinePackageFlashHook:
    {
      encoding = "hf";
    }
    break;
    case MgcNCSLinePackageAnswerTone:
    {
      encoding = "aw";
    }
    break;
    case MgcNCSLinePackageBusyTone:
    {
      encoding = "bz";
    }
    break;
    case MgcNCSLinePackageCallerId:
    {
      encoding = "ci("; 
      encoding += parameter[0]; encoding += ",";
      encoding += parameter[1]; encoding += ",";
      encoding += parameter[2];
      encoding +=")";
    }
    break;
    case MgcNCSLinePackageCallWaitingTone1:
    {
      encoding = "wt1";
    }
    break;
    case MgcNCSLinePackageCallWaitingTone2:
    {
      encoding = "wt2";
    }
    break;
    case MgcNCSLinePackageCallWaitingTone3:
    {
      encoding = "wt3";
    }
    break;
    case MgcNCSLinePackageCallWaitingTone4:
    {
      encoding = "wt4";
    }
    break;
    case MgcNCSLinePackageDialTone:
    {
      encoding = "dl";
    }
    break;
    case MgcNCSLinePackageMessageWaitingInd:
    {
      encoding = "mwi";
    }
    break;
    case MgcNCSLinePackageRinging:
    {
      encoding = "rg";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging1:
    {
      encoding = "r1";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging2:
    {
      encoding = "r2";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging3:
    {
      encoding = "r3";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging4:
    {
      encoding = "r4";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging5:
    {
      encoding = "r5";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging6:
    {
      encoding = "r6";
    }
    break;
    case MgcNCSLinePackageDistinctiveRinging7:
    {
      encoding = "r7";
    }
    break;
    case MgcNCSLinePackageRingsplash:
    {
      encoding = "rs";
    }
    break;
    case MgcNCSLinePackageReportOnCompletion:
    {
      encoding = "oc";
    }
    break;
    case MgcNCSLinePackageOffHookWarningTone:
    {
      encoding = "ot";
    }
    break;
    case MgcNCSLinePackageReorderTone:
    {
      encoding = "ro";
    }
    break;
    case MgcNCSLinePackageTeletypeDeviceForDeaf:
    {
      encoding = "TDD";
    }
    break;
    case MgcNCSLinePackageAllEvents:
    {
      encoding = "all";
    }
    break;
    case MgcNCSLinePackageUndefined:
      cerr << "no good\n";
      break;
    default:
      cerr << "no good\n";
      break;    
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  }
  return( encoding );
}


string MgcpGenericEvent::getNCSAdsiPackageItemTypeEncoding()
{
  string encoding = "";

  switch(myNCSAdsiType)
  {
    case MgcNCSAdsiPackageAdsiDisplay:
    {
      encoding = "adsi("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcNCSAdsiPackageAllEvents:
    {
      encoding = "all";
    }
    break;
    case MgcNCSAdsiPackageUndefined:
      cerr << "no good\n";
      break;
    default:
      cerr << "no good\n";
      break;    
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  } 
  return( encoding );
}


string MgcpGenericEvent::getNCSAllPackagesItemTypeEncoding()
{
  string encoding = "";

  switch(myNCSAllPackagesType)
  {
    case MgcNCSAllPackagesModemDetect:
    {
      encoding = "mt";
    }
    break;
    case MgcNCSAllPackagesFaxToneDetect:
    {
      encoding = "ft";
    }
    break;
    case MgcNCSAllPackagesLongDurationConnection:
    {
      encoding = "ld";
    }
    break;

    case MgcNCSAllPackagesMediaStart:
    {
      encoding = "ma"; 
    }
    break;
    case MgcNCSAllPackagesRingBackTone:
    {
      encoding = "rt";
    }
    break;
    case MgcNCSAllPackagesConfirmTone:
    {
      encoding = "cf";
    }
    break;
    case MgcNCSAllPackagesReportFailure:
    {
      encoding = "of";
    }
    break;
    case MgcNCSAllPackagesDTMF_0:
    {
      encoding = "0";
    }
    break;
    case MgcNCSAllPackagesDTMF_1:
    {
      encoding = "1";
    }
    break;
    case MgcNCSAllPackagesDTMF_2:
    {
      encoding = "2";
    }
    break;
    case MgcNCSAllPackagesDTMF_3:
    {
      encoding = "3";
    }
    break;
    case MgcNCSAllPackagesDTMF_4:
    {
      encoding = "4";
    }
    break;
    case MgcNCSAllPackagesDTMF_5:
    {
      encoding = "5";
    }
    break;
    case MgcNCSAllPackagesDTMF_6:
    {
      encoding = "6";
    }
    break;
    case MgcNCSAllPackagesDTMF_7:
    {
      encoding = "7";
    }
    break;
    case MgcNCSAllPackagesDTMF_8:
    {
      encoding = "8";
    }
    break;
    case MgcNCSAllPackagesDTMF_9:
    {
      encoding = "9";
    }
    break;
    case MgcNCSAllPackagesDTMF_Hash:
    {
      encoding = "#";
    }
    break;
    case MgcNCSAllPackagesDTMF_Star:
    {
      encoding = "*";
    }
    break;
    case MgcNCSAllPackagesDTMF_A:
    {
      encoding = "A";
    }
    break;
    case MgcNCSAllPackagesDTMF_B:
    {
      encoding = "B";
    }
    break;
    case MgcNCSAllPackagesDTMF_C:
    {
      encoding = "C";
    }
    break;
    case MgcNCSAllPackagesDTMF_D:
    {
      encoding = "D";
    }
    break;
    case MgcNCSAllPackagesLongDurationIndicator:
    {
      encoding = "L";
    }
    break;
    case MgcNCSAllPackagesWildcardMatchAnyDigit_0_9:
    {
      encoding = "X";
    }
    break;
    case MgcNCSAllPackagesInterdigitTimer:
    {
      encoding = "t";
    }
    break;
    case MgcNCSAllPackagesAdsiDisplay:
    {
      encoding = "adsi("; 
      encoding += parameter[0]; 
      encoding +=")";
    }
    break;
    case MgcNCSAllPackagesVisualMessageWaitingIndicator:
    {
      encoding = "vmwi";
    }
    break;
    case MgcNCSAllPackagesOffHookTransition:
    {
      encoding = "hd";
    }
    break;
    case MgcNCSAllPackagesOnHookTransition:
    {
      encoding = "hu";
    }
    break;
    case MgcNCSAllPackagesFlashHook:
    {
      encoding = "hf";
    }
    break;
    case MgcNCSAllPackagesAnswerTone:
    {
      encoding = "aw";
    }
    break;
    case MgcNCSAllPackagesBusyTone:
    {
      encoding = "bz";
    }
    break;
    case MgcNCSAllPackagesCallerId:
    {
      encoding = "ci("; 
      encoding += parameter[0]; encoding += ",";
      encoding += parameter[1]; encoding += ",";
      encoding += parameter[2];
      encoding +=")";
    }
    break;
    case MgcNCSAllPackagesCallWaitingTone1:
    {
      encoding = "wt1";
    }
    break;
    case MgcNCSAllPackagesCallWaitingTone2:
    {
      encoding = "wt2";
    }
    break;
    case MgcNCSAllPackagesCallWaitingTone3:
    {
      encoding = "wt3";
    }
    break;
    case MgcNCSAllPackagesCallWaitingTone4:
    {
      encoding = "wt4";
    }
    break;
    case MgcNCSAllPackagesDialTone:
    {
      encoding = "dl";
    }
    break;
    case MgcNCSAllPackagesMessageWaitingInd:
    {
      encoding = "mwi";
    }
    break;
    case MgcNCSAllPackagesRinging:
    {
      encoding = "rg";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging1:
    {
      encoding = "r1";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging2:
    {
      encoding = "r2";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging3:
    {
      encoding = "r3";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging4:
    {
      encoding = "r4";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging5:
    {
      encoding = "r5";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging6:
    {
      encoding = "r6";
    }
    break;
    case MgcNCSAllPackagesDistinctiveRinging7:
    {
      encoding = "r7";
    }
    break;
    case MgcNCSAllPackagesRingsplash:
    {
      encoding = "rs";
    }
    break;
    case MgcNCSAllPackagesReportOnCompletion:
    {
      encoding = "oc";
    }
    break;
    case MgcNCSAllPackagesOffHookWarningTone:
    {
      encoding = "ot";
    }
    break;
    case MgcNCSAllPackagesReorderTone:
    {
      encoding = "ro";
    }
    break;
    case MgcNCSAllPackagesTeletypeDeviceForDeaf:
    {
      encoding = "TDD";
    }
    break;
    case MgcNCSAllPackagesAllEvents:
    {
      encoding = "all";
    }
    break;
    case MgcNCSAllPackagesUndefined:
      cerr << "no good\n";
      break;
    default:
      cerr << "no good\n";
      break;    
  }
  if( connectionid != "")
  {
     encoding += "@";
     encoding += connectionid;
  } 
  return( encoding );
}
#endif // end of #ifdef NCS_PROFILE



string MgcpGenericEvent::getEncoding()
{
    string encoding = "";
    switch(myPackage)
    {
#ifndef NCS_PROFILE
    case MgcpGenericMediaPackageType:
    {
	encoding = "g/";
	encoding += getGenericItemTypeEncoding();	  
    }
    break;
    case MgcpDTMFPackageType:
    {
	encoding = "d/";
	encoding += getDTMFItemTypeEncoding();
    }
    break;
    case MgcpLinePackageType:
    {
	encoding = "l/";
	encoding += getLineItemTypeEncoding();
    }
    break;
    case MgcpMFPackageType:
    {
	encoding = "m/";
	encoding += getMFItemTypeEncoding();
    }
    break;
    case MgcpTrunkPackageType:
    {
	encoding = "t/";
	encoding += getTrunkItemTypeEncoding();
    }
    break;
    case MgcpHandsetPackageType:
    {
	encoding = "h/";
	encoding += getHandsetItemTypeEncoding();
    }
    break;
    case MgcpRTPPackageType:
    {
	encoding = "r/";
	encoding += getRTPItemTypeEncoding();
    }
    break;
    case MgcpNetworkAccessServerPackageType:
    {
	encoding = "n/";
	encoding += getNASItemTypeEncoding();
    }
    break;
    case MgcpAnnouncementPackageType:
    {
	encoding = "a/";
	encoding += getAnnounceItemTypeEncoding();
    }
    break;
    case MgcpScriptPackageType:
    {
	encoding = "script/";
	encoding += getScriptItemTypeEncoding();
    }
    break;
    case MgcpXVoiceMailPackageType:
    {
     // encoding = " ";
     // encoding += getXVoiceMailItemTypeEncoding();
    }
    break;
    case MgcpAllPackagesType:
    {
	encoding = "*/";
	encoding += getAllPackagesItemTypeEncoding();
    }
    break;
#else
    /// NCS Profile package types
    case MgcpNCSLinePackageType:
    {
	encoding = "L/";
	encoding += getNCSLinePackageItemTypeEncoding();
    }
    break;
    case MgcpNCSAdsiPackageType:
    {
	encoding = "S/";
	encoding += getNCSAdsiPackageItemTypeEncoding();
    }
    break;
    case MgcpNCSDefaultPackageType:
    {
	encoding = "";
	// should use Line package encoding or all packages encoding ??
	//	encoding += getNCSAllPackagesItemTypeEncoding();
	encoding += getNCSLinePackageItemTypeEncoding();
    }
    break;
    case MgcpNCSAllPackagesType:
    {
	encoding = "*/";
	encoding += getNCSAllPackagesItemTypeEncoding();
    }
    break;
#endif
    default:
      {
      cerr <<  "unknown type\n";
    }
  }

    return encoding;
}


#ifndef NCS_PROFILE
MgcpLinePackageItemType MgcpGenericEvent::decodeLinePackageItemType( const string& s )
{
    MgcpLinePackageItemType itemType = MgcLinePackageUndefined;

    if(s == "vmwi")
    {
      itemType = MgcLinePackageVisualMessageWaitingIndicator;
    }
    else if(s == "hd")
    {
      itemType = MgcLinePackageOffHookTransition;
    }
    else if(s == "hu")
    {
      itemType = MgcLinePackageOnHookTransition;
    }
    else if(s == "hf")
    {
      itemType = MgcLinePackageFlashHook;
    }
    else if(s == "aw")
    {
      itemType = MgcLinePackageAnswerTone;
    }
    else if(s == "bz")
    {
      itemType = MgcLinePackageBusyTone;
    }
    else if(s == "wt")
    {
      itemType = MgcLinePackageCallWaitingTone;
    }
    else if(s == "wt1")
    {
      itemType = MgcLinePackageCallWaitingTone1;
    }
    else if(s == "wt2")
    {
      itemType = MgcLinePackageCallWaitingTone2;
    }
    else if(s == "wt3")
    {
      itemType = MgcLinePackageCallWaitingTone3;
    }
    else if(s == "wt4")
    {
      itemType = MgcLinePackageCallWaitingTone4;
    }
    else if(s== "dl")
    {
      itemType = MgcLinePackageDialTone;
    }
    else if(s == "mwi")
    {
      itemType = MgcLinePackageMessageWaitingInd;
    }
    else if(s == "nbz")
    {
      itemType = MgcLinePackageNetworkBusy;
    }
    else if(s == "rg")
    {
      itemType = MgcLinePackageRinging;
    }
    else if(s == "rs")
    {
      itemType = MgcLinePackageRingsplash;
    }
    else if(s == "p")
    {
      itemType = MgcLinePackagePromptTone;
    }
    else if(s == "e")
    {
      itemType = MgcLinePackageErrorTone;
    }
    else if(s == "sdl")
    {
      itemType = MgcLinePackageStutterDialtone;
    }
    else if(s == "v")
    {
      itemType = MgcLinePackageAlertingTone;
    }
    else if(s == "y")
    {
      itemType = MgcLinePackageRecorderWarningTone;
    }
    else if(s == "sit")
    {
      itemType = MgcLinePackageSITTone;
    }
    else if(s == "z")
    {
      itemType = MgcLinePackageCallingCardServiceTone;
    }
    else if(s == "oc")
    {
      itemType = MgcLinePackageReportOnCompletion;
    }
    else if(s == "ot")
    {
      itemType = MgcLinePackageOffHookWarningTone;
    }
    else if(s == "of")
    {
      itemType = MgcLinePackageReportFailure; 
    }
    else if(s == "all")
    {
      itemType = MgcLinePackageAllEvents;
    }
    
    return( itemType );
}


MgcpDTMFPackageItemType MgcpGenericEvent::decodeDTMFPackageItemType( const string& s )
{
    MgcpDTMFPackageItemType itemType = MgcDTMFPackageUndefined;

    if(s == "0")
    {
      itemType = MgcDTMFPackageDTMF_0;
    }
    else if(s == "1")
    {
      itemType = MgcDTMFPackageDTMF_1;
    }
    else if(s == "2")
    {
      itemType = MgcDTMFPackageDTMF_2;
    }
    else if(s == "3")
    {
      itemType = MgcDTMFPackageDTMF_3;
    }
    else if(s == "4")
    {
      itemType = MgcDTMFPackageDTMF_4;
    }
    else if(s == "5")
    {
      itemType = MgcDTMFPackageDTMF_5;
    }
    else if(s == "6")
    {
      itemType = MgcDTMFPackageDTMF_6;
    }
    else if(s == "7")
    {
      itemType = MgcDTMFPackageDTMF_7;
    }
    else if(s == "8")
    {
      itemType = MgcDTMFPackageDTMF_8;
    }
    else if(s == "9")
    {
      itemType = MgcDTMFPackageDTMF_9;
    }
    else if(s == "#")
    {
      itemType = MgcDTMFPackageDTMF_Hash;
    }
    else if(s == "*")
    {
      itemType = MgcDTMFPackageDTMF_Star;
    }
    else if(s == "a")
    {
      itemType = MgcDTMFPackageDTMF_A;
    }
    else if(s == "b")
    {
      itemType = MgcDTMFPackageDTMF_B;
    }
    else if(s == "c")
    {
      itemType = MgcDTMFPackageDTMF_C;
    }
    else if(s == "d")
    {
      itemType = MgcDTMFPackageDTMF_D;
    }
    else if(s == "l")
    {
      itemType = MgcDTMFPackageLongDurationIndicator;
    }
#ifdef CISCO_PROFILE 
    else if(s == "[0-9#*]")
    {
      itemType = MgcDTMFPackageWildcardMatchAnyDigit_0_9;
    }
#else
    else if(s == "x")
    {
      itemType = MgcDTMFPackageWildcardMatchAnyDigit_0_9;
    }
#endif
    else if(s == "t")
    {
      itemType = MgcDTMFPackageInterdigitTimer;
    }
    else if(s == "of")
    {
      itemType = MgcDTMFPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcDTMFPackageAllEvents;
    }
    
    return( itemType );
}


MgcpGenericPackageItemType MgcpGenericEvent::decodeGenericPackageItemType( const string& s)
{
    MgcpGenericPackageItemType itemType = MgcGenericPackageUndefined;

    if(s == "mt")
    {
      itemType = MgcGenericPackageModemDetect;
    }
    else if(s == "ft")
    {
      itemType = MgcGenericPackageFaxToneDetect;
    }
    else if(s == "ld")
    {
      itemType = MgcGenericPackageLongDurationConnection;
    }
    else if(s == "rt")
    {
      itemType = MgcGenericPackageRingBackTone;
    }
    else if(s == "cf")
    {
      itemType = MgcGenericPackageConfirmTone;
    }
    else if(s == "cg")
    {
      itemType = MgcGenericPackageNetworkCongestionTone;
    }
    else if(s == "it")
    {
      itemType = MgcGenericPackageInterceptTone;
    }
    else if(s == "pt")
    {
      itemType = MgcGenericPackagePreemtionTone;
    }
    else if(s == "of")
    {
      itemType = MgcGenericPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcGenericPackageAllEvents;
    }
    
    return( itemType );
}


MgcpMFPackageItemType MgcpGenericEvent::decodeMFPackageItemType( const string& s )
{
    MgcpMFPackageItemType itemType = MgcMFPackageUndefined;

    if(s == "0")
    {
      itemType = MgcMFPackageMF_0;
    }
    else if(s == "1")
    {
      itemType = MgcMFPackageMF_1;
    }
    else if(s == "2")
    {
      itemType = MgcMFPackageMF_2;
    }
    else if(s == "3")
    {
      itemType = MgcMFPackageMF_3;
    }
    else if(s == "4")
    {
      itemType = MgcMFPackageMF_4;
    }
    else if(s == "5")
    {
      itemType = MgcMFPackageMF_5;
    }
    else if(s == "6")
    {
      itemType =MgcMFPackageMF_6; 
    }
    else if(s == "7")
    {
      itemType = MgcMFPackageMF_7;
    }
    else if(s == "8")
    {
      itemType = MgcMFPackageMF_8; 
    }
    else if(s == "9")
    {
      itemType = MgcMFPackageMF_9;
    }
    else if(s == "x")
    {
      itemType = MgcMFPackageWildcardMatchAnyDigit_0_9;
    }
    else if(s == "t")
    {
      itemType = MgcMFPackageInterdigitTimer;
    }
    else if(s == "k0")
    {
      itemType = MgcMFPackageMFK0;
    }
    else if(s == "k1")
    {
      itemType = MgcMFPackageMFK1;
    }
    else if(s == "k2")
    {
      itemType = MgcMFPackageMFK2;
    }
    else if(s == "s0")
    {
      itemType = MgcMFPackageMFS0;
    }
    else if(s == "s1")
    {
      itemType = MgcMFPackageMFS1;
    }
    else if(s == "s2")
    {
      itemType = MgcMFPackageMFS2;
    }
    else if(s == "s3")
    {
      itemType = MgcMFPackageMFS3;
    }
    else if(s == "wk")
    {
      itemType = MgcMFPackageWink;
    }
    else if(s == "wko")
    {
      itemType = MgcMFPackageWinkOff;
    }
    else if(s == "is")
    {
      itemType = MgcMFPackageIncomingSeizure;
    }
    if(s == "rs")
    {
      itemType = MgcMFPackageReturnSeizure;
    }
    else if(s == "us")
    {
      itemType = MgcMFPackageUnseizeCircuit;
    }
    else if(s == "of")
    {
      itemType = MgcMFPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcMFPackageAllEvents;
    }

    return( itemType );
}


MgcpTrunkPackageItemType MgcpGenericEvent::decodeTrunkPackageItemType( const string& s )
{
    MgcpTrunkPackageItemType itemType = MgcTrunkPackageUndefined;
 
    if(s == "co1")
    {
      itemType = MgcTrunkPackageContinuityTone;
    }
    else if(s == "co2")
    {
      itemType = MgcTrunkPackageContinuityTest;
    }
    else if(s == "lb")
    {
      itemType = MgcTrunkPackageLoopback;
    }
    else if(s == "om")
    {
      itemType = MgcTrunkPackageOldMilliwattTone;
    }
    else if(s == "nm")
    {
      itemType = MgcTrunkPackageNewMilliwattTone;
    }
    else if(s == "tl")
    {
      itemType = MgcTrunkPackageTestLine;
    }
    else if(s == "zz")
    {
      itemType = MgcTrunkPackageNoCircuit;
    }
    else if(s == "as")
    {
      itemType = MgcTrunkPackageAnswerSupervision;
    }
    else if(s == "ro")
    {
      itemType = MgcTrunkPackageReorderTone;
    }
    else if(s == "of")
    {
      itemType = MgcTrunkPackageReportFailure;
    }
    else if(s == "all")
    {
      itemType = MgcTrunkPackageAllEvents;
    }

    return( itemType );
}


MgcpHandsetPackageItemType MgcpGenericEvent::decodeHandsetPackageItemType( const string& s)
{
    MgcpHandsetPackageItemType itemType = MgcHandsetPackageUndefined;

    if(s == "adsi(string)")
    {
      itemType = MgcHandsetPackageAdsiDisplay;
    }
    else if(s == "tdd")
    {
      itemType = MgcHandsetPackageTeletypeDeviceForDeaf;
    }
    else if(s == "vmwi")
    {
      itemType = MgcHandsetPackageVisualMessageWaitingIndicator;
    }
    else if(s == "hd")
    {
      itemType = MgcHandsetPackageOffHookTransition;
    }
    else if(s == "hu")
    {
      itemType = MgcHandsetPackageOnHookTransition;
    }
    else if(s == "hf")
    {
      itemType = MgcHandsetPackageFlashHook;
    }
    else if(s == "aw")
    {
      itemType = MgcHandsetPackageAnswerTone;
    }
    else if(s == "bz")
    {
      itemType = MgcHandsetPackageBusyTone;
    }
    else if(s == "wt")
    {
      itemType = MgcHandsetPackageCallWaitingTone;
    }
    else if(s == "dl")
    {
      itemType = MgcHandsetPackageDialTone;
    }
    else if(s == "nbz")
    {
      itemType = MgcHandsetPackageNetworkBusy;
    }
    else if(s == "rg")
    {
      itemType = MgcHandsetPackageRinging;
    }
    else if(s == "p")
    {
      itemType = MgcHandsetPackagePromptTone;
    }
    else if(s == "e")
    {
      itemType = MgcHandsetPackageErrorTone;
    }
    else if(s == "sdl")
    {
      itemType = MgcHandsetPackageStutterDialtone;
    }
    else if(s == "v")
    {
      itemType = MgcHandsetPackageAlertingTone;
    }
    else if(s == "y")
    {
      itemType = MgcHandsetPackageRecorderWarningTone;
    }
    else if(s == "t")
    {
      itemType = MgcHandsetPackageSITTone;
    }
    else if(s == "z")
    {
      itemType = MgcHandsetPackageCallingCardServiceTone;
    }
    else if(s == "oc")
    {
      itemType = MgcHandsetPackageReportOnCompletion;
    }
    else if(s == "ot")
    {
      itemType = MgcHandsetPackageOffHookWarningTone;
    }
    else if(s == "of")
    {
      itemType = MgcHandsetPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcHandsetPackageAllEvents;
    }

    return( itemType );
}


MgcpRTPPackageItemType MgcpGenericEvent::decodeRTPPackageItemType( const string& s)
{
    MgcpRTPPackageItemType itemType = MgcRTPPackageUndefined;

    if(s == "uc")
    {
      itemType = MgcRTPPackageUsedCodecChanged;
    }
    else if(s == "qa")
    {
      itemType = MgcRTPPackageQualityAlert;
    }
    else if(s == "of")
    {
      itemType = MgcRTPPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcRTPPackageAllEvents;
    }
    
    return( itemType );
}


MgcpNetworkAccessServerPackageItemType MgcpGenericEvent::decodeNASPackageItemType( const string& s )
{
    MgcpNetworkAccessServerPackageItemType itemType = MgcNetworkAccessServerPackageUndefined;

    if(s == "pa")
    {
      itemType = MgcNetworkAccessServerPackagePacketArrival;
    }
    else if(s == "cbk")
    {
      itemType = MgcNetworkAccessServerPackageCallBackRequest;
    }
    else if(s == "cl")
    {
      itemType = MgcNetworkAccessServerPackageCarrierLost;
    }
    else if(s == "au")
    {
      itemType = MgcNetworkAccessServerPackageAuthorizationSucceeded;
    }
    else if(s == "ax")
    {
      itemType = MgcNetworkAccessServerPackageAuthorizationDenied;
    }
    else if(s == "of")
    {
      itemType = MgcNetworkAccessServerPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcNetworkAccessServerPackageAllEvents;
    }
    
    return( itemType );
}


MgcpAnnouncementPackageItemType MgcpGenericEvent::decodeAnnouncementPackageItemType( const string& s )
{
    MgcpAnnouncementPackageItemType itemType = MgcAnnouncementPackageUndefined;

    if(s == "oc")
    {
      itemType = MgcAnnouncementPackageReportOnCompletion;
    }
    else if(s == "of")
    {
      itemType = MgcAnnouncementPackageReportFailure;
    }
    else if(s == "all")
    {
	itemType = MgcAnnouncementPackageAllEvents;
    }
    
    return( itemType );
}


MgcpScriptPackageItemType MgcpGenericEvent::decodeScriptPackageItemType( const string& s )
{
    MgcpScriptPackageItemType itemType = MgcScriptPackageUndefined;

    if(s == "oc")
    {
      itemType = MgcScriptPackageReportOnCompletion;
    }
    else if(s == "of")
    {
      itemType = MgcScriptPackageReportFailure;
    }
    else if(s == "all")
    {
      itemType = MgcScriptPackageAllEvents;
    }

    return( itemType );
}


MgcpAllPackagesItemType MgcpGenericEvent::decodeAllPackagesItemType( const string& s )
{
    MgcpAllPackagesItemType itemType = MgcAllPackagesUndefined;

    if ( s == "mt" )
    {
      itemType = MgcAllPackagesModemDetect;
    }
    else if ( s== "ft")
    {
      itemType = MgcAllPackagesFaxToneDetect;
    }
    else if ( s== "ld")
    {
      itemType = MgcAllPackagesLongDurationConnection;
    }
    else if ( s== "rt")
    {
      itemType = MgcAllPackagesRingBackTone;
    }
    else if ( s== "cf")
    {
      itemType = MgcAllPackagesConfirmTone;
    }
    else if ( s== "cg")
    {
      itemType = MgcAllPackagesNetworkCongestionTone;
    }
    else if ( s== "it")
    {
      itemType = MgcAllPackagesInterceptTone;
    }
    else if ( s== "pt")
    {
      itemType = MgcAllPackagesPreemtionTone;
    }
    else if ( s== "of")
    {
      itemType = MgcAllPackagesReportFailure;
    }
    else if ( s== "0")
    {
      itemType = MgcAllPackagesDTMF_0;
    }
    else if ( s== "1")
    {
      itemType = MgcAllPackagesDTMF_1;
    }
    else if ( s== "2")
    {
      itemType = MgcAllPackagesDTMF_2;
    }
    else if ( s== "3")
    {
      itemType = MgcAllPackagesDTMF_3;
    }
    else if ( s== "4")
    {
      itemType = MgcAllPackagesDTMF_4;
    }
    else if ( s== "5")
    {
      itemType = MgcAllPackagesDTMF_5;
    }
    else if ( s== "6")
    {
      itemType = MgcAllPackagesDTMF_6;
    }
    else if ( s== "7")
    {
      itemType = MgcAllPackagesDTMF_7;
    }
    else if ( s== "8")
    {
      itemType = MgcAllPackagesDTMF_8;
    }
    else if ( s== "9")
    {
      itemType = MgcAllPackagesDTMF_9;
    }
    else if ( s== "#")
    {
      itemType = MgcAllPackagesDTMF_Hash;
    }
    else if ( s== "*")
    {
      itemType = MgcAllPackagesDTMF_Star;
    }
    else if ( s== "a")
    {
      itemType = MgcAllPackagesDTMF_A;
    }
    else if ( s== "b")
    {
      itemType = MgcAllPackagesDTMF_B;
    }
    else if ( s== "c")
    {
      itemType = MgcAllPackagesDTMF_C;
    }
    else if ( s== "d")
    {
      itemType = MgcAllPackagesDTMF_D;
    }
    else if ( s== "l")
    {
      itemType = MgcAllPackagesLongDurationIndicator;
    }
#ifdef CISCO_PROFILE
    else if ( s== "[0-9#*]")
    {
      itemType = MgcAllPackagesWildcardMatchAnyDigit_0_9;
    }
#else
    else if ( s== "x")
    {
      itemType = MgcAllPackagesWildcardMatchAnyDigit_0_9;
    }
#endif
    else if ( s== "t")
    {
      itemType = MgcAllPackagesInterdigitTimer;
    }
    else if ( s== "vmwi")
    {
      itemType = MgcAllPackagesVisualMessageWaitingIndicator;
    }
    else if ( s== "hd")
    {
      itemType = MgcAllPackagesOffHookTransition;
    }
    else if ( s== "hu")
    {
      itemType = MgcAllPackagesOnHookTransition;
    }
    else if ( s== "hf")
    {
      itemType = MgcAllPackagesFlashHook;
    }
    else if ( s== "aw")
    {
      itemType = MgcAllPackagesAnswerTone;
    }
    else if ( s== "bz")
    {
      itemType = MgcAllPackagesBusyTone;
    }
    else if ( s== "wt")
    {
      itemType = MgcAllPackagesCallWaitingTone;
    }
    else if ( s== "wt1")
    {
      itemType = MgcAllPackagesCallWaitingTone1;
    }
    else if ( s== "wt2")
    {
      itemType = MgcAllPackagesCallWaitingTone2;
    }
    else if ( s== "wt3")
    {
      itemType = MgcAllPackagesCallWaitingTone3;
    }
    else if ( s== "wt4")
    {
      itemType = MgcAllPackagesCallWaitingTone4;
    }
    else if ( s== "dl")
    {
      itemType = MgcAllPackagesDialTone;
    }
    else if ( s== "mwi")
    {
      itemType = MgcAllPackagesMessageWaitingInd;
    }
    else if ( s== "nbz")
    {
      itemType = MgcAllPackagesNetworkBusy;
    }
    else if ( s== "rg")
    {
	itemType = MgcAllPackagesRinging;
    }
    else if ( s== "rs")
    {
      itemType = MgcAllPackagesRingsplashOrReturnSeizure;
    }
    else if ( s== "p")
    {
      itemType = MgcAllPackagesPromptTone;
    }
    else if ( s== "e")
    {
      itemType = MgcAllPackagesErrorTone;
    }
    else if ( s== "sdl")
    {
      itemType = MgcAllPackagesStutterDialtone;
    }
    else if ( s== "v")
    {
      itemType = MgcAllPackagesAlertingTone;
    }
    else if ( s== "y")
    {
      itemType = MgcAllPackagesRecorderWarningTone;
    }
    else if ( s== "sit")
    {
      itemType = MgcAllPackagesSITTone;
    }
    else if ( s== "z")
    {
      itemType = MgcAllPackagesCallingCardServiceTone;
    }
    else if ( s== "oc")
    {
      itemType = MgcAllPackagesReportOnCompletion;
    }
    else if ( s== "ot")
    {
      itemType = MgcAllPackagesOffHookWarningTone;
    }
    else if ( s== "k0")
    {
      itemType = MgcAllPackagesK0;
    }
    else if ( s== "k1")
    {
      itemType = MgcAllPackagesK1;
    }
    else if ( s== "k2")
    {
      itemType = MgcAllPackagesK2;
    }
    else if ( s== "s0")
    {
      itemType = MgcAllPackagesS0;
    }
    else if ( s== "s1")
    {
      itemType = MgcAllPackagesS1;
    }
    else if ( s== "s2")
    {
      itemType = MgcAllPackagesS2;
    }
    else if ( s== "s3")
    {
      itemType = MgcAllPackagesS3;
    }
    else if ( s== "wk")
    {
      itemType = MgcAllPackagesWink;
    }
    else if ( s== "wko")
    {
      itemType = MgcAllPackagesWinkOff;
    }
    else if ( s== "is")
    {
      itemType = MgcAllPackagesIncomingSeizure;
    }
    else if ( s== "us")
    {
      itemType = MgcAllPackagesUnseizeCircuit;
    }
    else if ( s== "co1")
    {
      itemType = MgcAllPackagesContinuityTone;
    }
    else if ( s== "co2")
    {
      itemType = MgcAllPackagesContinuityTest;
    }
    else if ( s== "lb")
    {
      itemType = MgcAllPackagesLoopback;
    }
    else if ( s== "om")
    {
      itemType = MgcAllPackagesOldMilliwattTone;
    }
    else if ( s== "nm")
    {
      itemType = MgcAllPackagesNewMilliwattTone;
    }
    else if ( s== "tl")
    {
      itemType = MgcAllPackagesTestLine;
    }
    else if ( s== "zz")
    {
      itemType = MgcAllPackagesNoCircuit;
    }
    else if ( s== "as")
    {
      itemType = MgcAllPackagesAnswerSupervision;
    }
    else if ( s== "ro")
    {
      itemType = MgcAllPackagesReorderTone;
    }
    else if ( s== "tdd")
    {
      itemType = MgcAllPackagesTeletypeDeviceForDeaf;
    }
    else if ( s== "uc")
    {
      itemType = MgcAllPackagesUsedCodecChanged;
    }
    else if ( s== "qa")
    {
      itemType = MgcAllPackagesQualityAlert;
    }
    else if ( s== "pa")
    {
      itemType = MgcAllPackagesPacketArrival;
    }
    else if ( s== "cbk")
    {
      itemType = MgcAllPackagesCallBackRequest;
    }
    else if ( s== "cl")
    {
      itemType = MgcAllPackagesCarrierLost;
    }
    else if ( s== "au")
    {
      itemType = MgcAllPackagesAuthorizationSucceeded;
    }
    else if ( s== "ax")
    {
      itemType = MgcAllPackagesAuthorizationDenied;
    }
    else if ( s== "all")
    {
	itemType = MgcAllPackagesAllEvents;
    }
    else 
    {

      string s_2( s, 0, 2);
      if ( s_2 == "r(" )  /// should also read string into parameter
	return( MgcAllPackagesDistinctiveRinging );
      else if ( s_2 == "s(" ) /// should set string into parameter
        return( MgcAllPackagesDistinctiveTonePattern );
    

      string s_3( s, 0, 3 );
      if ( s_3 == "ci(" )  /// should set value into parameter
	return( MgcAllPackagesCallerId );
      else if ( s_3 == "sr(" )  /// should set value into parameter
	return( MgcAllPackagesSamplingRateChanged );
      else if ( s_3 == "ji(" )  /// should set value into parameter
	return( MgcAllPackagesJitterBufferSizeChanged );
      else if ( s_3 == "pl(" )  /// should set value into parameter
	return( MgcAllPackagesLossExceeded );

      string s_4( s, 0, 4 );
      if ( s_4 == "pat(" )  /// should set value into parameter
	return( MgcAllPackagesPatternDetected );
      else if ( s_4 == "rbk(" )  /// should set value into parameter
	return( MgcAllPackagesRingBackOnConnection );
      else if ( s_4 == "ann(" )  /// should set value into parameter
	return( MgcAllPackagesPlayAnAnnouncement );
      else if ( s_4 == "tcl(" )  /// should set value into parameter
	return( MgcAllPackagesLoadATCLScript );
      else if ( s_4 == "xml(" )  /// should set value into parameter
	return( MgcAllPackagesLoadAnXMLScript );

      string s_5( s, 0, 5 );
      if ( s_5 == "adsi(" )  /// should set value into parameter
	return( MgcAllPackagesAdsiDisplay );
      else if ( s_5 == "java(" )  /// should set value into parameter
	return( MgcAllPackagesLoadAJavaScript );
      else if ( s_5 == "perl(" )  /// should set value into parameter
	return( MgcAllPackagesLoadAPerlScript );
    }

    return( itemType );
}
#endif // end of #ifndef NCS_PROFILE

/****************************************************************************/
/// NCS_PROFILE Part

#ifdef NCS_PROFILE
/// NCS Profile decoding
MgcpNCSLinePackageItemType MgcpGenericEvent::decodeNCSLinePackageItemType( const string& s)
{
    MgcpNCSLinePackageItemType itemType = MgcNCSLinePackageUndefined;

    if ( s == "mt" )
    {
      itemType = MgcNCSLinePackageModemDetect;
    }
    else if ( s== "ft")
    {
      itemType = MgcNCSLinePackageFaxToneDetect;
    }
    else if ( s== "ld")
    {
      itemType = MgcNCSLinePackageLongDurationConnection;
    }
    else if ( s== "ma")
    {
	itemType = MgcNCSLinePackageMediaStart;
    }
    else if ( s== "rt")
    {
      itemType = MgcNCSLinePackageRingBackTone;
    }
    else if ( s== "cf")
    {
      itemType = MgcNCSLinePackageConfirmTone;
    }
    else if ( s== "of")
    {
      itemType = MgcNCSLinePackageReportFailure;
    }
    else if ( s== "0")
    {
      itemType = MgcNCSLinePackageDTMF_0;
    }
    else if ( s== "1")
    {
      itemType = MgcNCSLinePackageDTMF_1;
    }
    else if ( s== "2")
    {
      itemType = MgcNCSLinePackageDTMF_2;
    }
    else if ( s== "3")
    {
      itemType = MgcNCSLinePackageDTMF_3;
    }
    else if ( s== "4")
    {
      itemType = MgcNCSLinePackageDTMF_4;
    }
    else if ( s== "5")
    {
      itemType = MgcNCSLinePackageDTMF_5;
    }
    else if ( s== "6")
    {
      itemType = MgcNCSLinePackageDTMF_6;
    }
    else if ( s== "7")
    {
      itemType = MgcNCSLinePackageDTMF_7;
    }
    else if ( s== "8")
    {
      itemType = MgcNCSLinePackageDTMF_8;
    }
    else if ( s== "9")
    {
      itemType = MgcNCSLinePackageDTMF_9;
    }
    else if ( s== "#")
    {
      itemType = MgcNCSLinePackageDTMF_Hash;
    }
    else if ( s== "*")
    {
      itemType = MgcNCSLinePackageDTMF_Star;
    }
    else if (( s== "A") || ( s=="a"))
    {
      itemType = MgcNCSLinePackageDTMF_A;
    }
    else if (( s== "B") || ( s=="b"))
    {
      itemType = MgcNCSLinePackageDTMF_B;
    }
    else if (( s== "C") || ( s=="c"))
    {
      itemType = MgcNCSLinePackageDTMF_C;
    }
    else if (( s== "D") || ( s=="d"))
    {
      itemType = MgcNCSLinePackageDTMF_D;
    }
    else if (( s== "L") || ( s=="l"))
    {
      itemType = MgcNCSLinePackageLongDurationIndicator;
    }
    else if (( s== "X") || ( s=="x"))
    {
      itemType = MgcNCSLinePackageWildcardMatchAnyDigit_0_9;
    }
    else if ( s== "t")
    {
      itemType = MgcNCSLinePackageInterdigitTimer;
    }
    else if ( s== "vmwi")
    {
      itemType = MgcNCSLinePackageVisualMessageWaitingIndicator;
    }
    else if ( s== "hd")
    {
      itemType = MgcNCSLinePackageOffHookTransition;
    }
    else if ( s== "hu")
    {
      itemType = MgcNCSLinePackageOnHookTransition;
    }
    else if ( s== "hf")
    {
      itemType = MgcNCSLinePackageFlashHook;
    }
    else if ( s== "aw")
    {
      itemType = MgcNCSLinePackageAnswerTone;
    }
    else if ( s== "bz")
    {
      itemType = MgcNCSLinePackageBusyTone;
    }
    else if ( s== "wt1")
    {
      itemType = MgcNCSLinePackageCallWaitingTone1;
    }
    else if ( s== "wt2")
    {
      itemType = MgcNCSLinePackageCallWaitingTone2;
    }
    else if ( s== "wt3")
    {
      itemType = MgcNCSLinePackageCallWaitingTone3;
    }
    else if ( s== "wt4")
    {
      itemType = MgcNCSLinePackageCallWaitingTone4;
    }
    else if ( s== "dl")
    {
      itemType = MgcNCSLinePackageDialTone;
    }
    else if ( s== "mwi")
    {
      itemType = MgcNCSLinePackageMessageWaitingInd;
    }
    else if ( s=="rg")
    {
	itemType = MgcNCSLinePackageRinging;
    }
    else if ( s=="r0")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging0;
    }
    else if ( s=="r1")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging1;
    }
    else if ( s=="r2")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging2;
    }
    else if ( s=="r3")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging3;
    }
    else if ( s=="r4")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging4;
    }
    else if ( s=="r5")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging5;
    }
    else if ( s=="r6")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging6;
    }
    else if ( s=="r7")
    {
	itemType = MgcNCSLinePackageDistinctiveRinging7;
    }
    else if ( s== "rs")
    {
      itemType = MgcNCSLinePackageRingsplash;
    }
    else if ( s== "sl")
    {
      itemType = MgcNCSLinePackageStutterDialtone;
    }
    else if ( s== "oc")
    {
      itemType = MgcNCSLinePackageReportOnCompletion;
    }
    else if ( s== "ot")
    {
      itemType = MgcNCSLinePackageOffHookWarningTone;
    }
    else if ( s== "ro")
    {
      itemType = MgcNCSLinePackageReorderTone;
    }
    else if (( s== "TDD") || ( s=="tdd"))
    {
      itemType = MgcNCSLinePackageTeletypeDeviceForDeaf;
    }
    else if ( s== "all")
    {
	itemType = MgcNCSLinePackageAllEvents;
    }
    else 
    {
      // could be ci(...), parse the string into keyword and value
      string::size_type startKeyword = s.find_first_not_of(" ");
      string::size_type startBrace = s.find_first_of("(");
      string::size_type endBrace = s.find_first_of(")");

      string Keyword_tmp = s.substr(startKeyword, startBrace-startKeyword);
      string::size_type endKeyword = Keyword_tmp.find_first_of(" ");
      string Keyword = Keyword_tmp.substr(0, endKeyword);

      string Value = s.substr(startBrace+1, ((endBrace-1)-(startBrace+1)+1));
      
      if ( Keyword == "ci" )
      {  
	// ci(ti,nu,na)
        split_t value_vector = split( Value, "," );
        int i = value_vector.size();
        if ((i>0) && ( i <= 3))
        {
          for (int k=0; k<i; k++)
	  {
            parameter.push_back(value_vector[k]);
	  }
        }

	itemType = MgcNCSLinePackageCallerId;
      }    	
    }

    return( itemType );
}


MgcpNCSAdsiPackageItemType MgcpGenericEvent::decodeNCSAdsiPackageItemType( const string& s)
{
    MgcpNCSAdsiPackageItemType itemType = MgcNCSAdsiPackageUndefined;

    if(s == "all")
    {
      itemType = MgcNCSAdsiPackageAllEvents;
    }
    else
    {
      MgcpNCSAdsiPackageItemType itemType = MgcNCSAdsiPackageUndefined;
 
      string::size_type startKeyword = s.find_first_not_of(" ");
      string::size_type startBrace = s.find_first_of("(");
      string::size_type endBrace = s.find_first_of(")");

      string Keyword_tmp = s.substr(startKeyword, startBrace-startKeyword);
      string::size_type endKeyword = Keyword_tmp.find_first_of(" ");
      string Keyword = Keyword_tmp.substr(0, endKeyword);

      string Value = s.substr(startBrace+1, ((endBrace-1)-(startBrace+1)+1));
      
      if ( Keyword == "adsi" )
      {
	  parameter.push_back(Value);
	  itemType = MgcNCSAdsiPackageAdsiDisplay;
      }   
    }
    
    return( itemType );
}


MgcpNCSAllPackagesItemType MgcpGenericEvent::decodeNCSAllPackagesItemType( const string& s)
{
    MgcpNCSAllPackagesItemType itemType = MgcNCSAllPackagesUndefined;

    if ( s == "mt" )
    {
      itemType = MgcNCSAllPackagesModemDetect;
    }
    else if ( s== "ft")
    {
      itemType = MgcNCSAllPackagesFaxToneDetect;
    }
    else if ( s== "ld")
    {
      itemType = MgcNCSAllPackagesLongDurationConnection;
    }
    else if ( s== "ma")
    {
	itemType = MgcNCSAllPackagesMediaStart;
    }
    else if ( s== "rt")
    {
      itemType = MgcNCSAllPackagesRingBackTone;
    }
    else if ( s== "cf")
    {
      itemType = MgcNCSAllPackagesConfirmTone;
    }
    else if ( s== "of")
    {
      itemType = MgcNCSAllPackagesReportFailure;
    }
    else if ( s== "0")
    {
      itemType = MgcNCSAllPackagesDTMF_0;
    }
    else if ( s== "1")
    {
      itemType = MgcNCSAllPackagesDTMF_1;
    }
    else if ( s== "2")
    {
      itemType = MgcNCSAllPackagesDTMF_2;
    }
    else if ( s== "3")
    {
      itemType = MgcNCSAllPackagesDTMF_3;
    }
    else if ( s== "4")
    {
      itemType = MgcNCSAllPackagesDTMF_4;
    }
    else if ( s== "5")
    {
      itemType = MgcNCSAllPackagesDTMF_5;
    }
    else if ( s== "6")
    {
      itemType = MgcNCSAllPackagesDTMF_6;
    }
    else if ( s== "7")
    {
      itemType = MgcNCSAllPackagesDTMF_7;
    }
    else if ( s== "8")
    {
      itemType = MgcNCSAllPackagesDTMF_8;
    }
    else if ( s== "9")
    {
      itemType = MgcNCSAllPackagesDTMF_9;
    }
    else if ( s== "#")
    {
      itemType = MgcNCSAllPackagesDTMF_Hash;
    }
    else if ( s== "*")
    {
      itemType = MgcNCSAllPackagesDTMF_Star;
    }
    else if (( s== "A") || ( s=="a"))
    {
      itemType = MgcNCSAllPackagesDTMF_A;
    }
    else if (( s== "B") || ( s=="b"))
    {
      itemType = MgcNCSAllPackagesDTMF_B;
    }
    else if (( s== "C") || ( s=="c"))
    {
      itemType = MgcNCSAllPackagesDTMF_C;
    }
    else if (( s== "D") || ( s=="d"))
    {
      itemType = MgcNCSAllPackagesDTMF_D;
    }
    else if (( s== "L") || ( s=="l"))
    {
      itemType = MgcNCSAllPackagesLongDurationIndicator;
    }
    else if (( s== "X") || ( s=="x"))
    {
      itemType = MgcNCSAllPackagesWildcardMatchAnyDigit_0_9;
    }
    else if ( s== "t")
    {
      itemType = MgcNCSAllPackagesInterdigitTimer;
    }
    else if ( s== "vmwi")
    {
      itemType = MgcNCSAllPackagesVisualMessageWaitingIndicator;
    }
    else if ( s== "hd")
    {
      itemType = MgcNCSAllPackagesOffHookTransition;
    }
    else if ( s== "hu")
    {
      itemType = MgcNCSAllPackagesOnHookTransition;
    }
    else if ( s== "hf")
    {
      itemType = MgcNCSAllPackagesFlashHook;
    }
    else if ( s== "aw")
    {
      itemType = MgcNCSAllPackagesAnswerTone;
    }
    else if ( s== "bz")
    {
      itemType = MgcNCSAllPackagesBusyTone;
    }
    else if ( s== "wt1")
    {
      itemType = MgcNCSAllPackagesCallWaitingTone1;
    }
    else if ( s== "wt2")
    {
      itemType = MgcNCSAllPackagesCallWaitingTone2;
    }
    else if ( s== "wt3")
    {
      itemType = MgcNCSAllPackagesCallWaitingTone3;
    }
    else if ( s== "wt4")
    {
      itemType = MgcNCSAllPackagesCallWaitingTone4;
    }
    else if ( s== "dl")
    {
      itemType = MgcNCSAllPackagesDialTone;
    }
    else if ( s== "mwi")
    {
      itemType = MgcNCSAllPackagesMessageWaitingInd;
    }
    else if ( s=="rg")
    {
	itemType = MgcNCSAllPackagesRinging;
    }
    else if ( s=="r0")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging0;
    }
    else if ( s=="r1")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging1;
    }
    else if ( s=="r2")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging2;
    }
    else if ( s=="r3")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging3;
    }
    else if ( s=="r4")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging4;
    }
    else if ( s=="r5")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging5;
    }
    else if ( s=="r6")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging6;
    }
    else if ( s=="r7")
    {
	itemType = MgcNCSAllPackagesDistinctiveRinging7;
    }
    else if ( s== "rs")
    {
      itemType = MgcNCSAllPackagesRingsplash;
    }
    else if ( s== "sl")
    {
      itemType = MgcNCSAllPackagesStutterDialtone;
    }
    else if ( s== "oc")
    {
      itemType = MgcNCSAllPackagesReportOnCompletion;
    }
    else if ( s== "ot")
    {
      itemType = MgcNCSAllPackagesOffHookWarningTone;
    }
    else if ( s== "ro")
    {
      itemType = MgcNCSAllPackagesReorderTone;
    }
    else if (( s== "TDD") || ( s=="tdd"))
    {
      itemType = MgcNCSAllPackagesTeletypeDeviceForDeaf;
    }
    else if ( s== "all")
    {
	itemType = MgcNCSAllPackagesAllEvents;
    }
    else 
    {
      // could be ci(...) or adsi(...), parse the string into keyword and value
      string::size_type startKeyword = s.find_first_not_of(" ");
      string::size_type startBrace = s.find_first_of("(");
      string::size_type endBrace = s.find_first_of(")");

      string Keyword_tmp = s.substr(startKeyword, startBrace-startKeyword);
      string::size_type endKeyword = Keyword_tmp.find_first_of(" ");
      string Keyword = Keyword_tmp.substr(0, endKeyword);

      string Value = s.substr(startBrace+1, ((endBrace-1)-(startBrace+1)+1));
      
      if ( Keyword == "ci" )
      {  
	// ci(ti,nu,na)
        split_t value_vector = split( Value, "," );
        int i = value_vector.size();
        if ((i>0) && ( i <= 3))
        {
          for (int k=0; k<i; k++)
	  {
            parameter.push_back(value_vector[k]);
	  }
        }

	itemType = MgcNCSAllPackagesCallerId;
      }    
      else if ( Keyword == "adsi" )
      {
	parameter.push_back(Value);
	itemType = MgcNCSAllPackagesAdsiDisplay;
      }
	
    }

    return( itemType );
}
#endif  // end of #ifdef NCS_PROFILE


MgcpGenericEvent::MgcpGenericEvent(const string& s)
 : 
#ifndef NCS_PROFILE
	myPackage(MgcpNoPackageType), 
	myGenericType(MgcGenericPackageUndefined)
#else
  /// NCS Profile
        myPackage(MgcpNCSLinePackageType),
        myNCSLineType(MgcNCSLinePackageUndefined)
#endif
{
    string package(s, 0, 2);
    // char *p;
    string item; 
	//  string id = "@";
	//    p = s.search( s.begin(), s.end(),id.begin(),id.end() );
	//   bool b1 = ( p != s.end() );
	//  if( b1 ) {
    string::size_type startofid = s.find_first_of("@");
    if( startofid != string::npos )
	{
       item = s.substr(2, startofid-2);
	   //  connectionid = s.substr(startofid+1, s.end() );
       connectionid = string(s, startofid+1, s.size()-startofid-1);
    }
	  else
       item = string(s, 2, s.length()-2);
	//  string item( s, 2, s.length()-2 );

#ifndef NCS_PROFILE
    if ( package == "l/" )
    {
      myPackage = MgcpLinePackageType; 
      myLineType = decodeLinePackageItemType( item );
    }
    else if ( package == "d/" )
    {
      myPackage = MgcpDTMFPackageType;
      myDTMFType = decodeDTMFPackageItemType( item );
    }
    else if ( package == "g/" )
    {
      myPackage = MgcpGenericMediaPackageType;
      myGenericType = decodeGenericPackageItemType( item );
    }
    else if ( package == "m/" )
    {
      myPackage = MgcpMFPackageType;
      myMFType = decodeMFPackageItemType( item );
    }
    else if ( package == "t/" )
    {
      myPackage = MgcpTrunkPackageType;
      myTrunkType = decodeTrunkPackageItemType( item );
    }
    else if ( package == "h/" )
    {
      myPackage = MgcpHandsetPackageType;
      myHandsetType = decodeHandsetPackageItemType( item );
    }
    else if ( package == "r/" )
    {
      myPackage = MgcpRTPPackageType;
      myRTPType = decodeRTPPackageItemType( item );
    }
    else if ( package == "n/" )
    {
      myPackage = MgcpNetworkAccessServerPackageType;
      myNASType = decodeNASPackageItemType( item );
    }
    else if ( package == "a/" )
    {
      myPackage = MgcpAnnouncementPackageType;
      myAnnounceType = decodeAnnouncementPackageItemType( item );
    }
    else if ( package == "*/" )
    {
      myPackage = MgcpAllPackagesType;
      myAllPackagesType = decodeAllPackagesItemType( item );
    }
    else  /// is it a script package item
    {
      string scriptPackage(s, 0, 7);
      string scriptItem( s, 7, s.length()-7 );

      if ( scriptPackage == "script/" )       
	{
	  myPackage = MgcpScriptPackageType;
	  myScriptType = decodeScriptPackageItemType( item );
        }

      /// could it be a MgcpXVoiceMailPackageType item ?

    }
#else
    /// NCS Profile part
    if ((package == "L/") || (package == "l/"))
    {
      myPackage = MgcpNCSLinePackageType;
      myNCSLineType = decodeNCSLinePackageItemType( item );
    }
    else if ((package == "S/") || (package == "s/"))
    {
      myPackage = MgcpNCSAdsiPackageType;
      myNCSAdsiType = decodeNCSAdsiPackageItemType( item );
    }
    else if (package == "*/")
    {
      myPackage = MgcpNCSAllPackagesType;
      myNCSAllPackagesType = decodeNCSAllPackagesItemType( item );
    }
    else  /// could be a default package with no keyword for package
    {
      string::size_type pos = s.find("/");
      if (pos == string::npos)
	{
	  /// does not contain any package keyword, treat it 
          /// as a default package, which is line package in NCS
	  myPackage = MgcpNCSLinePackageType;
	  myNCSLineType = decodeNCSLinePackageItemType( s );
	}
      
    }
#endif

    return;   
}
