#ifndef MGCP_GENERIC_EVENT_HXX
#define MGCP_GENERIC_EVENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpGenericEvent_hxx_Version = 
"$Id: MgcpGenericEvent.hxx,v 1.5 2001/02/01 01:22:56 icahoon Exp $";



#include "global.h"
#include "MgcpPackageTypes.hxx"
#include "MgcpEventTypes.hxx"
#include <vector>
#include <string>
#include "support.hxx"

class MgcpGenericEvent;
extern ostream& operator<<(ostream& s, MgcpGenericEvent& x);

const int EVENT_PARAMETER_MAX = 4;


class MgcpGenericEvent
{
public:
#ifndef NCS_PROFILE
    MgcpGenericEvent()
	: 
	myPackage(MgcpNoPackageType), 
	myGenericType(MgcGenericPackageUndefined),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpGenericPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myGenericType(type),   
  	connectionid(id),
	parameter(EVENT_PARAMETER_MAX)

	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpDTMFPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myDTMFType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpMFPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myMFType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpTrunkPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myTrunkType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpLinePackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myLineType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpHandsetPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myHandsetType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpRTPPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myRTPType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpNetworkAccessServerPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myNASType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpAnnouncementPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myAnnounceType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpScriptPackageItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myScriptType(type),
     connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}
    
    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpAllPackagesItemType type,
             string id = ""
	)
	: 
	myPackage(pkg), 
	myAllPackagesType(type),
    connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

#else // NCS_Profile 
    MgcpGenericEvent()
	: 
	myPackage(MgcpNCSLinePackageType), 
	myNCSLineType(MgcNCSLinePackageUndefined),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpNCSLinePackageItemType type,
		     string id = ""
	)
	: 
	myPackage(pkg), 
	myNCSLineType(type),
        connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

     MgcpGenericEvent(MgcpPackageType pkg, 
		      MgcpNCSAdsiPackageItemType type,
		      string id = ""
	)
	: 
	myPackage(pkg), 
	myNCSAdsiType(type),
        connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}

    MgcpGenericEvent(MgcpPackageType pkg, 
		     MgcpNCSAllPackagesItemType type,
		     string id = ""
	)
	: 
	myPackage(pkg), 
	myNCSAllPackagesType(type),
        connectionid(id),
	parameter(EVENT_PARAMETER_MAX)
	{}
#endif
    MgcpGenericEvent(const string& s);

    void setParameter(int i, const string& x) {
	parameter[i] = x;
    }

    string getParameter(int i) {
	return parameter[i];
    }

    int getParameterSize() {
	return parameter.size();
    }

    string getEncoding();
#ifndef NCS_PROFILE
    string getGenericItemTypeEncoding();
    string getDTMFItemTypeEncoding();
    string getLineItemTypeEncoding();
    string getMFItemTypeEncoding();
    string getTrunkItemTypeEncoding();
    string getHandsetItemTypeEncoding();
    string getRTPItemTypeEncoding();
    string getNASItemTypeEncoding();
    string getAnnounceItemTypeEncoding();
    string getScriptItemTypeEncoding();
    //string getXVoiceMailItemTypeEncoding();
    string getAllPackagesItemTypeEncoding();
#else
    /// NCS Profile
    string getNCSLinePackageItemTypeEncoding();
    string getNCSAdsiPackageItemTypeEncoding();
    string getNCSAllPackagesItemTypeEncoding();
#endif


#ifndef NCS_PROFILE
    MgcpLinePackageItemType decodeLinePackageItemType( const string& s );
    MgcpDTMFPackageItemType decodeDTMFPackageItemType( const string& s );
    MgcpGenericPackageItemType decodeGenericPackageItemType( const string& s);
    MgcpMFPackageItemType decodeMFPackageItemType( const string& s );
    MgcpTrunkPackageItemType decodeTrunkPackageItemType( const string& s );
    MgcpHandsetPackageItemType decodeHandsetPackageItemType( const string& s);
    MgcpRTPPackageItemType decodeRTPPackageItemType( const string& s);
    MgcpNetworkAccessServerPackageItemType decodeNASPackageItemType( const string& s );
    MgcpAnnouncementPackageItemType decodeAnnouncementPackageItemType( const string& s );
    MgcpScriptPackageItemType decodeScriptPackageItemType( const string& s );
    MgcpAllPackagesItemType decodeAllPackagesItemType( const string& s );
#else
    /// NCS Profile
    MgcpNCSLinePackageItemType decodeNCSLinePackageItemType( const string& s );
    MgcpNCSAdsiPackageItemType decodeNCSAdsiPackageItemType( const string& s );
    MgcpNCSAllPackagesItemType decodeNCSAllPackagesItemType( const string& s );
#endif

    MgcpPackageType getPackageType() const { return myPackage; }

#ifndef NCS_PROFILE
    MgcpGenericPackageItemType getGenericPackageType() const { return myGenericType;}
    MgcpDTMFPackageItemType getDTMFPackageType() const { return myDTMFType; }
    MgcpMFPackageItemType getMFPackageType() const { return myMFType; }
    MgcpTrunkPackageItemType getTrunkPackageType() const { return myTrunkType; }
    MgcpLinePackageItemType getLinePackageType() const { return myLineType; }
    MgcpHandsetPackageItemType getHandsetType() const { return myHandsetType; }
    MgcpRTPPackageItemType getRTPType() const { return myRTPType; }
    MgcpNetworkAccessServerPackageItemType getNASType() const { return myNASType; }
    MgcpAnnouncementPackageItemType getAnnounceType() const { return myAnnounceType; }
    MgcpScriptPackageItemType getScriptType() const { return myScriptType; }
    MgcpXVoiceMailPackageItemType getXVoiceMailType() const { return myXVoiceMailType;}
    MgcpAllPackagesItemType getAllPackagesType() const { return myAllPackagesType; }
#else
    /// NCS Profile
    MgcpNCSLinePackageItemType getNCSLinePackageType() const { return myNCSLineType; }
    MgcpNCSAdsiPackageItemType getNCSAdsiPackageType() const { return myNCSAdsiType; }
    MgcpNCSAllPackagesItemType getNCSAllPackagesType() const { return myNCSAllPackagesType; }
#endif

    ostream& printData(ostream& s);

    friend ostream& operator<<(ostream& s, MgcpGenericEvent& x);

private:

    MgcpPackageType myPackage;
#ifndef NCS_PROFILE
    MgcpGenericPackageItemType myGenericType;
    MgcpDTMFPackageItemType myDTMFType;
    MgcpMFPackageItemType myMFType;
    MgcpTrunkPackageItemType myTrunkType;
    MgcpLinePackageItemType myLineType;
    MgcpHandsetPackageItemType myHandsetType;
    MgcpRTPPackageItemType myRTPType;
    MgcpNetworkAccessServerPackageItemType myNASType;
    MgcpAnnouncementPackageItemType myAnnounceType;
    MgcpScriptPackageItemType myScriptType;
    MgcpXVoiceMailPackageItemType myXVoiceMailType;
    MgcpAllPackagesItemType myAllPackagesType;
#else
    /// NCS Profile
    MgcpNCSLinePackageItemType myNCSLineType;
    MgcpNCSAdsiPackageItemType myNCSAdsiType;
    MgcpNCSAllPackagesItemType myNCSAllPackagesType;
#endif

    string connectionid;
    vector<string> parameter;

};



#endif
