#ifndef MGCP_RESPONSE_CODE_HXX
#define MGCP_RESPONSE_CODE_HXX       

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const mgcpResponseCode_hxx_Version = 
"$Id: MgcpResponseCode.hxx,v 1.3 2001/01/26 02:46:50 liuhong Exp $";



#include "global.h"

extern const char OKResponse[];


typedef enum MgcpResponseCode
{

  /* Four ranges of values have been defined:

   * values 100 - 199 indicate a provisional response,
   * values 200 - 299 indicate a successful completion,
   * values 400 - 499 indicate a transient error,
   * values 500 - 599 indicate a permanent error */

  //#ifndef NCS_PROFILE  
  ReasonEndpointStateNominal           = 000, /* 000  Endpoint state is nominal, used only in 
						      response to audit requests */
  //#else // NCS_PROFILE
  ReasonResponseAcknowledgement        = 000, /* 000  Response acknowledgement */
  //#endif

  ReasonTransactionInProgress          = 100, /* 100  The transaction is currently being executed.  
                                                      An actual completion message will follow on later */


  // should remove duplicates sometime in applications
  ReasonTransactionExecuted            = 200, /* 200  The requested transaction was executed normally */
  TransactionExecuted                  = 200, /*      duplicate ?? */
  ReasonConnectionDeleted              = 250, /* 250  The connection was deleted */
  ConnectionDeleted                    = 250, /*      duplicate ?? */



  ReasonTransientError                 = 400, /* 400  The transaction could not be executed, due to 
                                                      a transient error */
  ReasonTransientAlreadyOffHook        = 401, /* 401  The phone is already off hook */
  ReasonTransientAlreadyOnHook         = 402, /* 402  The phone is already on hook */
  ReasonTransientInsufficientResources = 403, /* 403  The transaction could not be executed, because
                                                      the endpoint does not have sufficient 
						      resources at this time */
  ReasonTransientInsufficientBandwidth = 404, /* 404  Insufficient bandwidth at this time */



  ReasonEndpointUnknown                = 500, /* 500  The transaction could not be executed, because 
                                                      the endpoint is unknown */
  ReasonEndpointUnready                = 501, /* 501  The transaction could not be executed, because 
                                                      the endpoint is not ready */
  ReasonPermanentInsufficientResources = 502, /* 502  The transaction could not be executed, because 
                                                      the endpoint does not have sufficient resources */
  ReasonProtocolError                  = 510, /* 510  The transaction could not be executed, because 
						      a protocol error was detected */
 
  ReasonUnrecognizedExtension          = 511, /* 511  The transaction could not be executed, because 
						      the command contained an unrecognized extension */
  ReasonUnsupportedEvent               = 512, /* 512  The transaction could not be executed, because 
                                                      the gateway is not equipped to detect one of the 
                                                      requested events */
  ReasonUnsupportedSignal              = 513, /* 513  The transaction could not be executed, because 
                                                      the gateway is not equipped to generate one of the 
                                                      requested signals */
  ReasonUnsupportedAnnouncement        = 514, /* 514  The transaction could not be executed, because 
						      the gateway cannot send the specified announcement */
  ReasonIncorrectConnectionId          = 515, /* 515  The transaction refers to an incorrect connection-id 
						      (may have been already deleted) */
  ReasonUnknownCallId                  = 516, /* 516  The transaction refers to an unknown call-id */
  ReasonInvalidMode                    = 517, /* 517  Unsupported or invalid mode */
  ReasonInvalidPackage                 = 518, /* 518  Unsupported or unknown package */
  ReasonNoDigitMap                     = 519, /* 519  Endpoint does not have a digit map */
  ReasonRestarting                     = 520, /* 520  The transaction could not be executed, because 
						      the endpoint is "restarting" */
  ReasonEndpointRedirected             = 521, /* 521  Endpoint redirected to another Call Agent */
  ReasonNoSuchEvent                    = 522, /* 522  No such event or signal */
  ReasonUnknownAction                  = 523, /* 523  Unknown action or illegal combination of actions */
  ReasonBadConnectionOption            = 524, /* 524  Internal inconsistency in LocalConnectionOptions */
  ReasonUnknownConnectionOption        = 525, /* 525  Unknown extension in LocalConnectionOptions */
  //  UnknownLocalConnectionOptionsExtension = 525, // duplicate ??

  ReasonPermanentInsufficientBandwidth = 526, /* 526  Insufficient bandwidth */
  ReasonMissingRemoteDescriptor        = 527, /* 527  Missing RemoteConnectionDescriptor */
  ReasonIncompatibleVersion            = 528, /* 528  Incompatible protocol version */
  ReasonHardwareFailure                = 529, /* 529  Internal hardware failure */
  ReasonCASProtocolError               = 530, /* 530  CAS signaling protocol error */
  ReasonFacilityFailure                = 531, /* 531  failure of a grouping of trunks (e.g. facility failure) */



  ReasonEndpointMalfunctioning         = 900, /* 900  Endpoint malfunctioning */
  ReasonEndpointOOS                    = 901, /* 901  Endpoint taken out of service */
  ReasonLostConnectivity               = 902, /* 902  Loss of lower layer connectivity (e.g., downstream sync) */

#ifdef NCS_PROFILE
  ReasonQoSRSVLost                     = 903, /* 903  QoS resource reservation was lost */
#endif

  TimeoutNoResponse                    = 1024,/* 1024 */
  ReasonCodeUndefined                  = 2048 /* undefined value */
} MgcpResponseCode;


/*


100  The transaction is currently being executed.  An actual completion
     message will follow on later.

200  The requested transaction was executed normally.

250  The connection was deleted.

400  The transaction could not be executed, due to a transient error.

401  The phone is already off hook

402  The phone is already on hook

403  The transaction could not be executed, because the endpoint does
     not have sufficient resources at this time

404  Insufficient bandwidth at this time

500  The transaction could not be executed, because the endpoint is unk-
     nown.

501  The transaction could not be executed, because the endpoint is not
     ready.

502  The transaction could not be executed, because the endpoint does
     not have sufficient resources

510  The transaction could not be executed, because a protocol error was
     detected.

511  The transaction could not be executed, because the command con-
     tained an unrecognized extension.

512  The transaction could not be executed, because the gateway is not
     equipped to detect one of the requested events.

513  The transaction could not be executed, because the gateway is not
     equipped to generate one of the requested signals.

514  The transaction could not be executed, because the gateway cannot
     send the specified announcement.

515  The transaction refers to an incorrect connection-id (may have been
     already deleted)

516  The transaction refers to an unknown call-id.

517  Unsupported or invalid mode.

518  Unsupported or unknown package.

519  Endpoint does not have a digit map.

520  The transaction could not be executed, because the endpoint is
     "restarting".

521  Endpoint redirected to another Call Agent.

522  No such event or signal.

523  Unknown action or illegal combination of actions

524  Internal inconsistency in LocalConnectionOptions

525  Unknown extension in LocalConnectionOptions

526  Insufficient bandwidth

527  Missing RemoteConnectionDescriptor

528  Incompatible protocol version

529  Internal hardware failure

530  CAS signaling protocol error.

531  failure of a grouping of trunks (e.g. facility failure).

000  Endpoint state is nominal. (This code is used only in response to audit requests.)

900  Endpoint malfunctioning

901  Endpoint taken out of service

902  Loss of lower layer connectivity (e.g., downstream sync)

1024 Time out no response

*/

#endif  /* MGCP_RESPONSE_CODE_HXX */
