
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgcpDebugManager_cxx_Version  =
    "$Id: MgcpDebugManager.cxx,v 1.16 2001/02/01 01:22:56 icahoon Exp $";



#include "global.h"
#include "MgcpDebugManager.hxx"
#include "support.hxx"
#include <stdlib.h>
#include <string>
#include <iostream>
#include <fstream>
#include "debug.h"

class MgcpDebugManagerImpl
{
    public:
        MgcpDebugManagerImpl()
        : msgLevel(false),
        msgLogFile(0)
        {}


        bool msgLevel;
        ofstream* msgLogFile;

};

MgcpDebugManager* MgcpDebugManager::_instance = 0;

MgcpDebugManager* MgcpDebugManager::instance()
{
    if (_instance == 0)
    {
        _instance = new MgcpDebugManager;
    }
    return _instance;
}

MgcpDebugManager::MgcpDebugManager()
{
    // calculate the debugging information

    _impl = new MgcpDebugManagerImpl;

    char* debugInfo;

    debugInfo = getenv("MGCP_MSG_LOGFILE");

    if (debugInfo && !(string(debugInfo) == ""))
    {
        CERR(cerr << "opening " << debugInfo << " for logging" << endl; );
        _impl->msgLevel = true;
        _impl->msgLogFile = new ofstream(debugInfo);
    }
    else
    {
        _impl->msgLevel = false;
    }
}

bool MgcpDebugManager::logMsg()
{
    return _impl->msgLevel;
}

ofstream* MgcpDebugManager::logMsgFile()
{
    return (_impl->msgLogFile);
}
