
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpParameter_cxx_Version =
    "$Id: MgcpParameter.cxx,v 1.44.4.1 2001/03/14 19:32:10 icahoon Exp $";


#include "global.h"
#include "MgcpParameter.hxx"
#include <strstream>
#include "MgcpParameterEventGeneric.hxx"
#include "MgcpParse.hxx"

ostream& operator<<(ostream& s, const MgcpParameter& x)
{
    return x.printData(s);
}

/**********************************************************************
 
			    MgcpParameter
 
**********************************************************************/

MgcpParameter::~MgcpParameter()
{
}


MgcpParameter* MgcpParameter::duplicate()
{
    CERR(cerr << "warning -- bad duplicate() called!\n"; );
    return 0;
}


string MgcpParameter::encode()
{

    return string("");
}


/**********************************************************************
 QuarantineHandling methods
**********************************************************************/
///
string MgcpParameterQuarantineHandling::encode()
{
    string myEncoding = "Q: ";

    if (processQuarantine)
    {
        myEncoding += "process";
    }
    else
    {
        myEncoding += "discard";
    }

    if (multipleNotificationsLoop)
    {
        myEncoding += ",loop";
    }
    else
    {
        myEncoding += ",step";
    }

    myEncoding += "\r\n";
    return myEncoding;
}

///
bool MgcpParameterQuarantineHandling::parse(const ParameterSplitType& parms )
{
    if (parms.size() > 2)
    {
        if (parms[1] == "process")
            processQuarantine = true;
        else
            processQuarantine = false;

        if (parms[2] == "loop")
            multipleNotificationsLoop = true;
        else
            multipleNotificationsLoop = false;

        return true;
    }

    return false;

}

///
MgcpQuarantineTreatment MgcpParameterQuarantineHandling::getTreatmentQuarantineHandling()
{
    if ( processQuarantine )
        return ( MgcpQuarantineTreatmentProcess );
    else
        return ( MgcpQuarantineTreatmentDiscard );
}

///
MgcpNotificationHandling MgcpParameterQuarantineHandling::getNotificationQuarantineHandling()
{
    if ( multipleNotificationsLoop )
        return ( MgcpNotificationHandlingLoop );
    else
        return ( MgcpNotificationHandlingStep );
}


/**********************************************************************
 MgcpParameterLocalConnectionOptions methods
 **********************************************************************/

string MgcpParameterLocalConnectionOptions::encode()
{
    string myEncoding = "L: ";

    myEncoding += myConnectionOptions.EchoCancellationEncode();

    myEncoding += "\r\n";
    return myEncoding;
}

bool MgcpParameterLocalConnectionOptions::parse(const ParameterSplitType& parms)
{
    ParameterSplitType::const_iterator i;

    i = parms.begin();
    if (i != parms.end())
    {
        i++;
    }

    while (i != parms.end())
    {
        myConnectionOptions.EchoCancellationParse(*i);
        i++;
    }
    return true;
}


/**********************************************************************
ConnectionMode
**********************************************************************/

///
///
MgcpParameterConnectionMode::MgcpParameterConnectionMode(const string& mode)
{
    myMode = ConnectionInactive;
    if (mode == "sendonly")
        myMode = ConnectionSendOnly;
    else if (mode == "recvonly")
        myMode = ConnectionRecvOnly;
    else if (mode == "sendrecv")
        myMode = ConnectionSendRecv;
    else if (mode == "confrnce")
        myMode = ConnectionConference;
    else if (mode == "inactive")
        myMode = ConnectionInactive;
    else if (mode == "loopback")
        myMode = ConnectionLoopback;
    else if (mode == "conttest")
        myMode = ConnectionContTest;
    else if (mode == "netwloop")
        myMode = ConnectionNetwLoop;
    else if (mode == "netwtest")
        myMode = ConnectionNetwTest;
    else if (mode == "data")
        myMode = ConnectionData;
}

string MgcpParameterConnectionMode::getDecoded()
{
    string y;
    switch (myMode)
    {
        case ConnectionSendOnly:
        {
            y = "sendonly";
        }
        break;
        case ConnectionRecvOnly:
        {
            y = "recvonly";
        }
        break;
        case ConnectionSendRecv:
        {
            y = "sendrecv";
        }
        break;
        case ConnectionConference:
        {
            y = "confrnce";
        }
        break;
        case ConnectionInactive:
        {
            y = "inactive";
        }
        break;
        case ConnectionLoopback:
        {
            y = "loopback";
        }
        break;
        case ConnectionContTest:
        {
            y = "conttest";
        }
        break;
        case ConnectionNetwLoop:
        {
            y = "netwloop";
        }
        break;
        case ConnectionNetwTest:
        {
            y = "netwtest";
        }
        break;
        case ConnectionData:
        {
            y = "data";
        }
        break;
        default:
	{

	}
        break;
    }
    return y;
}

string MgcpParameterConnectionMode::encode()
{
    string y;

    y = "M: ";
    y += getDecoded();
    y += "\r\n";
    return y;
}

bool MgcpParameterConnectionMode::parse(const ParameterSplitType& parms)
{
    myMode = ConnectionInvalid;
    if (parms.size() > 1)
    {
        if (parms[1] == "sendonly")
            myMode = ConnectionSendOnly;
        else if (parms[1] == "recvonly")
            myMode = ConnectionRecvOnly;
        else if (parms[1] == "sendrecv")
            myMode = ConnectionSendRecv;
        else if (parms[1] == "confrnce")
            myMode = ConnectionConference;
        else if (parms[1] == "inactive")
            myMode = ConnectionInactive;
        else if (parms[1] == "loopback")
            myMode = ConnectionLoopback;
        else if (parms[1] == "conttest")
            myMode = ConnectionContTest;
        else if (parms[1] == "netwloop")
            myMode = ConnectionNetwLoop;
        else if (parms[1] == "netwtest")
            myMode = ConnectionNetwTest;
        else if (parms[1] == "data")
            myMode = ConnectionData;
    }
    if (myMode != ConnectionInvalid)
        return true;
    else
        return false;
}


ostream& MgcpParameterConnectionMode::printData(ostream& s) const
{
    switch (myMode)
    {
        case ConnectionSendOnly:
        {
            s << "Send Only";
        }
        break;
        case ConnectionRecvOnly:
        {
            s << "Receive Only";
        }
        break;
        case ConnectionSendRecv:
        {
            s << "Send and Receive";
        }
        break;
        case ConnectionConference:
        {
            s << "Conference";
        }
        break;
        case ConnectionInactive:
        {
            s << "Inactive";
        }
        break;
        case ConnectionLoopback:
        {
            s << "Loopback";
        }
        break;
        case ConnectionContTest:
        {
            s << "Continuity Test";
        }
        break;
        case ConnectionNetwLoop:
        {
            s << "Network Loopback";
        }
        break;
        case ConnectionNetwTest:
        {
            s << "Network Test";
        }
        break;
        case ConnectionData:
        {
            s << "Data";
        }
        break;
        default:
        {
            s << "Invalid Connection State";
        }
        break;
    }

    return s;

}


MgcpParameterBearerInformation::MgcpParameterBearerInformation(
    /// this is a test parameter
    const MgcpBearerInformation& x)
{
    switch (x)
    {
        case MgcpBearerInformationEnumMuLaw:
        {
            encodingMethod = EncodingMethoduLaw;
        }
        break;
        case MgcpBearerInformationEnumALaw:
        {
            encodingMethod = EncodingMethodALaw;
        }
        break;
        default:
        encodingMethod = EncodingMethoduLaw;
    }
}

/**********************************************************************
Entity
**********************************************************************/

string MgcpParameterNotifiedEntity::encode()
{
    string y;

    y = "N: ";
    y += myEntityName;
    y += "\r\n";

    return y;
}

/**********************************************************************
DigitMap
**********************************************************************/


string MgcpParameterDigitMap::encode()
{
    string y;

    if (!(myDigitMap == ""))
    {
        y = "D: ";
        y += myDigitMap;
    }
    else
    {
        y = "";
    }

    y += "\r\n";
    return y;
}


string MgcpParameterDigitMap::encodeEmbedded()
{
    return myDigitMap;
}


bool MgcpParameterDigitMap::parse(const ParameterSplitType& parms)
{

    if (parms.size() > 1)
    {
        myDigitMap = parms[1];
        return true;
    }
    return false;
}


bool MgcpParameterDigitMap::parseEmbedded(const string& digitMap)
{
    myDigitMap = digitMap;
    return true;
}

/**********************************************************************
RequestId
**********************************************************************/


bool
MgcpParameterRequestId::parse(const ParameterSplitType& parms)
{
    if (parms.size() > 1)
    {
        myId = parms[1];
        return true;
    }
    return false;
}


string
MgcpParameterRequestId::encode()
{
    string y;

    y = "X: ";
    y += myId;
    y += "\r\n";

    return y;
}

/**********************************************************************
CallId
**********************************************************************/


bool
MgcpParameterCallId::parse(const ParameterSplitType& parms)
{
    if (parms.size() > 1)
    {
        myId = parms[1];
        return true;
    }
    return false;
}


string
MgcpParameterCallId::encode()
{
    string y;

    y = "C: ";
    y += myId;
    y += "\r\n";

    return y;
}


/**********************************************************************
ConnectionId
 **********************************************************************/

bool
MgcpParameterConnectionId::insert(const string& type)
{
    myIdList.push_back(type);
    return true;
}


bool
MgcpParameterConnectionId::parse(const ParameterSplitType& parms)
{
    Sptr < string > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        insert( *iter );
        ++iter;
    }

    return true;
}


string
MgcpParameterConnectionId::encode()
{
    string y;
    bool prevflag = false;

    //  DEBUG(cout << "encode: " << myIdList;);

    y = "I: ";

    vector < string > :: const_iterator iter = myIdList.begin();

    while ( iter != myIdList.end() )
    {
        if ( prevflag )
        {
            y += ",";
        }
        y += (*iter);
        prevflag = true;
        iter++;
    }

    y += "\r\n";

    // DEBUG(cout << " -> " << y << endl;);
    return y;
}

ostream& MgcpParameterConnectionId::printData( ostream& s ) const
{
    vector < string > ::const_iterator iter = myIdList.begin();

    while (iter++ != myIdList.end())
    {
        s << ( *iter ) << ", ";
    }
    s << "\n";
    return s;
}


/**********************************************************************
MaxEndPointIds
 **********************************************************************/
bool
MgcpParameterMaxEndPointIds::parse(const ParameterSplitType& parms)
{
    if (parms.size() > 1)
    {
        string s = parms[1];
        strncpy( st, s.c_str(), MAX_ENDPOINT_ID_LIST );
        return true;
    }
    return false;
}


string
MgcpParameterMaxEndPointIds::encode()
{
    string y;

    y = "ZM: ";
    y += st;
    y += "\r\n";

    return y;
}


void
MgcpParameterMaxEndPointIds::itoa( int num, char s[] )
{
    int i = 0;

    do
    {
        s[i++] = num % 10 + '0';
    }
    while (((num = num / 10) > 0) && ( i < MAX_ENDPOINT_ID_LIST ));
      
    if ( i == MAX_ENDPOINT_ID_LIST )
    {
        s[i-1] = '\0';
    }
    else
    {
        s[i] = '\0';
    }

    reverse(s);
}

void
MgcpParameterMaxEndPointIds::reverse( char s[] )
{
    int i, j, c;

    for (i = 0, j = strlen(s) - 1; i < j; i++, j--)
        c = s[i], s[i] = s[j], s[j] = c;
}

int
MgcpParameterMaxEndPointIds::atoi( char s[] )
{
    int i = 0, n;
    for (n = 0; isdigit(s[i]) && (i<MAX_ENDPOINT_ID_LIST); i++)
        n = 10 * n + (s[i] - '0');

    return n;
}



/**********************************************************************
NumEndPoints
 **********************************************************************/

bool
MgcpParameterNumEndPoints::parse(const ParameterSplitType& parms)
{
    if (parms.size() > 1)
    {
        string s = parms[1];
        strncpy(st, s.c_str(), MAX_ENDPOINT_ID_LIST );
	st[MAX_ENDPOINT_ID_LIST-1] = '\0';
        return true;
    }
    return false;
}


string
MgcpParameterNumEndPoints::encode()
{
    string y;

    y = "ZN: ";
    y += st;
    y += "\r\n";

    return y;
}


void
MgcpParameterNumEndPoints::itoa( int num, char s[] )
{
    int i = 0;

    do
    {
        s[i++] = num % 10 + '0';
    }
    while (((num = num / 10) > 0) && ( i < MAX_ENDPOINT_ID_LIST) );

    if ( i == MAX_ENDPOINT_ID_LIST )
    {
	s[i-1] = '\0';
    }
    else
    {
        s[i] = '\0';
    }

    reverse(s);
}

void
MgcpParameterNumEndPoints::reverse( char s[] )
{
    int i, j, c;

    for (i = 0, j = strlen(s) - 1; i < j; i++, j--)
        c = s[i], s[i] = s[j], s[j] = c;
}


int
MgcpParameterNumEndPoints::atoi( char s[] )
{
    int i = 0, n;
    for (n = 0; isdigit(s[i]) && ( i < MAX_ENDPOINT_ID_LIST); i++)
        n = 10 * n + (s[i] - '0');

    return n;
}




/**********************************************************************
RequestedInfo
**********************************************************************/
string MgcpParameterRequestedInfo::encodeParamType(MgcpParameterType type)
{
    string s;

    switch ( type )
    {
        case MgcpParameterTypeQuarantineHandling:
        s = "Q";
        break;
        case MgcpParameterTypeBearerInformation:
        s = "B";
        break;
        case MgcpParameterTypeNotifiedEntity:
        s = "N";
        break;
        case MgcpParameterTypeObservedEvents:
        s = "O";
        break;
        case MgcpParameterTypeConnectionParameters:
        s = "P";
        break;
        case MgcpParameterTypeReasonCode:
        s = "E";
        break;
        case MgcpParameterTypeRequestedEvents:
        s = "R";
        break;
        case MgcpParameterTypeSignalRequests:
        s = "S";
        break;
        case MgcpParameterTypeRequestId:
        s = "X";
        break;
        case MgcpParameterTypeConnectionId:
        s = "I";
        break;
        case MgcpParameterTypeDigitMap:
        s = "D";
        break;
        case MgcpParameterTypeDetectEvents:
        s = "T";
        break;
        case MgcpParameterTypeLocalConnectionOptions:
        s = "L";
        break;
        case MgcpParameterTypeConnectionMode:
        s = "M";
        break;
        case MgcpParameterTypeLocalConnectionDescriptor:
        s = "LC";
        break;
        case MgcpParameterTypeRemoteConnectionDescriptor:
        s = "RC";
        break;
        case MgcpParameterTypeRestartMethod:
        s = "RM";
        break;
        case MgcpParameterTypeRestartDelay:
        s = "RD";
        break;
        case MgcpParameterTypeCapabilities:
        s = "A";
        break;
        case MgcpParameterTypeEventStates:
        s = "ES";
        break;
        default:
        s = "";
        break;
    }
    return s;
}


MgcpParameterType MgcpParameterRequestedInfo::decodeParamType( string s )
{
    MgcpParameterType e;

    if ( s == "q" )
        e = MgcpParameterTypeQuarantineHandling;
    else if ( s == "b" )
        e = MgcpParameterTypeBearerInformation;
    else if ( s == "n" )
        e = MgcpParameterTypeNotifiedEntity;
    else if ( s == "o" )
        e = MgcpParameterTypeObservedEvents;
    else if ( s == "p" )
        e = MgcpParameterTypeConnectionParameters;
    else if ( s == "e" )
        e = MgcpParameterTypeReasonCode;
    else if ( s == "r" )
        e = MgcpParameterTypeRequestedEvents;
    else if ( s == "s" )
        e = MgcpParameterTypeSignalRequests;
    else if ( s == "x" )
        e = MgcpParameterTypeRequestId;
    else if ( s == "i" )
        e = MgcpParameterTypeConnectionId;
    else if ( s == "d" )
        e = MgcpParameterTypeDigitMap;
    else if ( s == "t" )
        e = MgcpParameterTypeDetectEvents;
    else if ( s == "l" )
        e = MgcpParameterTypeLocalConnectionOptions;
    else if ( s == "m" )
        e = MgcpParameterTypeConnectionMode;
    else if ( s == "lc" )
        e = MgcpParameterTypeLocalConnectionDescriptor;
    else if ( s == "rc" )
        e = MgcpParameterTypeRemoteConnectionDescriptor;
    else if ( s == "rm" )
        e = MgcpParameterTypeRestartMethod;
    else if ( s == "rd" )
        e = MgcpParameterTypeRestartDelay;
    else if ( s == "a" )
        e = MgcpParameterTypeCapabilities;
    else if ( s == "es" )
        e = MgcpParameterTypeEventStates;
    else
        e = MgcpParameterTypeUnknownParameter;

    return ( e );
}


string MgcpParameterRequestedInfo::encode()
{
    string s;
    bool prevFlag = false;


    s = "F: ";

    vector < MgcpParameterType > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        if (prevFlag)
        {
            s += ",";
        }
        s += encodeParamType( *iter );
        prevFlag = true;
        ++iter;
    }

    s += "\r\n";

    return s;
}

bool MgcpParameterRequestedInfo::insert( MgcpParameterType type )
{
    list.push_back( type );
    return true;
}


ostream& MgcpParameterRequestedInfo::printData( ostream& s )
{
    vector < MgcpParameterType > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        //	s << *(iter->getPtr()) << "\n";
        s << encodeParamType( *iter ) << ", ";
        ++iter;
    }
    s << "\n";
    return s;
}


bool MgcpParameterRequestedInfo::parse( const ParameterSplitType& parms)
{
    MgcpParameterType e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        e = decodeParamType( (*iter) );
        if ( e != MgcpParameterTypeUnknownParameter )
        {
            insert(e);
        }
        ++iter;
    }

    return true;
}


/**********************************************************************
 class MgcpParameterEndpointIdList methods
**********************************************************************/

string MgcpParameterEndpointIdList::encode()
{
    string s = "Z: ";
    bool prevFlag = false;
    vector < Sptr < MgcpParameterEndpointId > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        if ( prevFlag )
        {
            s += "Z: ";
        }
        s += ((*iter)->encode());
        s += "\r\n";
        prevFlag = true;
        ++iter;
    }
    return s;
}


///
bool MgcpParameterEndpointIdList::insert(Sptr < MgcpParameterEndpointId > id)
{
    //    cout << "    inserting endpoint id " << id << "\n";
    list.push_back(id);
    return true;
}


///
ostream& MgcpParameterEndpointIdList::printData(ostream& s) const
{
    vector < Sptr < MgcpParameterEndpointId > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        s << *(iter->getPtr()) << "\n";
        ++iter;
    }
    s << "\n";
    return s;
}



///
bool MgcpParameterEndpointIdList::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterEndpointId > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        // ...
        e = new MgcpParameterEndpointId((*iter));

        insert(e);

        ++iter;
    }

    return true;
}


/******************************
 class EventList methods
******************************/


string
MgcpParameterEventList::encode()
{
    string s;
    bool prevFlag = false;

    vector < Sptr < MgcpParameterEvent > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        if (!*iter)
        {
            CERR(cerr << "error encoding because null...\n"; );
        }
        else
        {
            Sptr < MgcpParameterEvent > x = *iter;

            if (prevFlag)
            {
                s += ",";
            }
            s += (x->encode());
            prevFlag = true;
        }
        ++iter;
    }

    s += "\r\n";

    return s;
}


bool
MgcpParameterEventList::insert(Sptr < MgcpParameterEvent > event)
{
    //    cout << "    inserting event " << event << "\n";
    list.push_back(event);
    return true;
}



ostream&
MgcpParameterEventList::printData(ostream& s) const
{
    vector < Sptr < MgcpParameterEvent > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        s << *(iter->getPtr()) << "\n";
        ++iter;
    }
    s << "\n";
    return s;
}


/******************************
RequestedEvents
******************************/


string
MgcpParameterRequestedEvents::encode()
{
    string s;

    s = "R: ";

    vector < Sptr < MgcpParameterRequestedEvent > > ::const_iterator iter = requestList.begin();
    bool prevFlag = false;

    while (iter != requestList.end())
    {
        if (!*iter)
        {
            CERR(cerr << "error encoding because null...\n"; );
        }
        else
        {
            Sptr < MgcpParameterRequestedEvent > x = *iter;

            if (prevFlag)
            {
                s += ",";
            }
            s += (x->encode());
            prevFlag = true;
        }
        ++iter;
    }

    s += "\r\n";

    return s;
}


bool
MgcpParameterRequestedEvents::insert(Sptr < MgcpParameterRequestedEvent > event)
{
    if ( event != 0 )
    {
        requestList.push_back(event);
        return( true );
    }
    else
    {
        return( false );
    }
}


ostream&
MgcpParameterRequestedEvents::printData(ostream& s) const
{
    vector < Sptr < MgcpParameterRequestedEvent > > ::const_iterator iter = requestList.begin();

    while (iter != requestList.end())
    {
        s << *(iter->getPtr()) << "\n";
        ++iter;
    }
    s << "\n";
    return s;
}


bool
MgcpParameterRequestedEvents::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterRequestedEvent > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        // ...
        ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: event: " << *iter << "\n"; );

        //	e = parseEventsRequested(*iter);
        e = parseEventsRequested((*iter));

        if (e != 0)
        {
            insert(e);
        }
        ++iter;
    }

    return true;
}


/******************************
ObservedEvents
******************************/

bool
MgcpParameterObservedEvents::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterEvent > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }


    while (iter != parms.end())
    {
        if ((matchString((*iter), "^[0-9a-dA-D]*$") == 0) ||
                (matchString((*iter), "^[dD][/][*#0-9]*$") == 0))
        {
            HIGH_DEBUG(cout << "got a pattern" << endl; );
            // this is a digit map, so parse and generate these events

            int i = 0;

            string x = (*iter);

            int len = x.length();

            while (i < len)
            {
                e = 0;
                char digit = x[i];
#ifndef NCS_PROFILE
                if (digit == '0')
                {
                    MgcpGenericEvent genEvent("d/0");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '1')
                {
                    MgcpGenericEvent genEvent("d/1");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '2')
                {
                    MgcpGenericEvent genEvent("d/2");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '3')
                {
                    MgcpGenericEvent genEvent("d/3");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '4')
                {
                    MgcpGenericEvent genEvent("d/4");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '5')
                {
                    MgcpGenericEvent genEvent("d/5");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '6')
                {
                    MgcpGenericEvent genEvent("d/6");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '7')
                {
                    MgcpGenericEvent genEvent("d/7");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '8')
                {
                    MgcpGenericEvent genEvent("d/8");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '9')
                {
                    MgcpGenericEvent genEvent("d/9");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '*')
                {
                    MgcpGenericEvent genEvent("d/*");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '#')
                {
                    MgcpGenericEvent genEvent("d/#");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
#else // NCS_PROFILE
		if (digit == '0')
                {
                    MgcpGenericEvent genEvent("l/0");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '1')
                {
                    MgcpGenericEvent genEvent("l/1");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '2')
                {
                    MgcpGenericEvent genEvent("l/2");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '3')
                {
                    MgcpGenericEvent genEvent("l/3");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '4')
                {
                    MgcpGenericEvent genEvent("l/4");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '5')
                {
                    MgcpGenericEvent genEvent("l/5");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '6')
                {
                    MgcpGenericEvent genEvent("l/6");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '7')
                {
                    MgcpGenericEvent genEvent("l/7");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '8')
                {
                    MgcpGenericEvent genEvent("l/8");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '9')
                {
                    MgcpGenericEvent genEvent("l/9");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '*')
                {
                    MgcpGenericEvent genEvent("l/*");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
                else if (digit == '#')
                {
                    MgcpGenericEvent genEvent("l/#");
                    e = new MgcpParameterEventGeneric(genEvent);
                }
#endif
                if (e != 0)
                {
                    insert(e);
                }
                i++;
            }
        }
        else
        {
            e = parseEvents(*iter);
            if (e != 0)
            {
                insert(e);
            }
        }
        ++iter;
    }

    return true;
}


string
MgcpParameterObservedEvents::encode()
{
    string s;

    s = "O: ";
    s += (MgcpParameterEventList::encode());

    // \r\n added by EventList::encode

    return s;
}


/******************************
DetectEvents
******************************/


bool
MgcpParameterDetectEvents::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterEvent > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        // ...
        e = parseEvents(*iter);
        if (e != 0)
        {
            insert(e);
        }
        ++iter;
    }

    return true;
}


string
MgcpParameterDetectEvents::encode()
{
    string s;

    s = "T: ";
    s += (MgcpParameterEventList::encode());

    // \r\n added by EventList::encode

    return s;
}


/******************************
SignalRequests
******************************/


string
MgcpParameterSignalRequests::encode()
{
    string s;
    bool prevFlag = false;


    s = "S: ";

    vector < Sptr < MgcpParameterSignal > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        if (prevFlag)
        {
            s += ",";
        }
        s += ((*iter)->encode());
        prevFlag = true;
        ++iter;
    }

    s += "\r\n";

    return s;
}

bool
MgcpParameterSignalRequests::insert(Sptr < MgcpParameterSignal > event)
{
    //    cout << "    inserting signal " << event << "\n";
    list.push_back(event);
    return true;
}



ostream&
MgcpParameterSignalRequests::printData(ostream& s) const
{
    vector < Sptr < MgcpParameterSignal > > ::const_iterator iter = list.begin();

    while (iter != list.end())
    {
        s << *(iter->getPtr()) << "\n";
        ++iter;
    }
    s << "\n";
    return s;
}



bool
MgcpParameterSignalRequests::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterSignal > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if (iter != parms.end())
    {
        ++iter;
    }

    while (iter != parms.end())
    {
        // ...
        e = parseSignals(*iter);
        if (e != 0)
        {
            insert(e);
        }
        ++iter;
    }

    return true;
}



/*********************************************************
      Methods of MgcpParameterEventStates
**********************************************************/


string
MgcpParameterEventStates::encode()
{
    string s;
    bool prevFlag = false;

    s = "ES: ";

    vector < Sptr < MgcpParameterSignal > > ::const_iterator iter = list.begin();

    while ( iter != list.end() )
    {
        if (prevFlag)
        {
            s += ",";
        }
        s += ((*iter)->encode());
        prevFlag = true;
        ++iter;
    }

    s += "\r\n";

    return s;
}

bool
MgcpParameterEventStates::insert(Sptr < MgcpParameterSignal > event)
{
    list.push_back(event);
    return true;
}

ostream&
MgcpParameterEventStates::printData(ostream& s) const
{
    vector < Sptr < MgcpParameterSignal > > ::const_iterator iter = list.begin();

    while ( iter != list.end() )
    {
        s << *(iter->getPtr()) << "\n";
        ++iter;
    }

    s << "\n";
    return s;
}

bool
MgcpParameterEventStates::parse(const ParameterSplitType& parms)
{
    Sptr < MgcpParameterSignal > e;
    ParameterSplitType::const_iterator iter = parms.begin();

    if ( iter != parms.end())
    {
        ++iter;
    }

    while ( iter != parms.end() )
    {
        e = parseSignals(*iter);
        if ( e != 0 )
        {
            insert(e);
        }
        ++iter;
    }

    return true;
}


/**********************************************************************
         Methods for Capabilities
***********************************************************************/

string MgcpParameterCapabilities::PacketizationEncode()
{
    return myConnectionOptions.PacketizationEncode();
}


string MgcpParameterCapabilities::BandwidthEncode()
{
    return myConnectionOptions.BandwidthEncode();
}

string MgcpParameterCapabilities::CompressionEncode()
{
    return myConnectionOptions.CompressionEncode();
}

string MgcpParameterCapabilities::EventPackageEncode()
{
    string y;
    bool prevFlag = false;
    vector < MgcpPackageType > ::const_iterator iter = EventPackageList.begin();
    y = "v:";
    while ( iter != EventPackageList.end() )
    {
        if ( prevFlag )
            y += ";";
        switch ( *iter )
        {
#ifndef NCS_PROFILE
            case MgcpGenericMediaPackageType:
            y += "G";
            break;
            case MgcpDTMFPackageType:
            y += "D";
            break;
            case MgcpMFPackageType:
            y += "M";
            break;
            case MgcpTrunkPackageType:
            y += "T";
            break;
            case MgcpLinePackageType:
            y += "L";
            break;
            case MgcpHandsetPackageType:
            y += "H";
            break;
            case MgcpRTPPackageType:
            y += "R";
            break;
            case MgcpNetworkAccessServerPackageType:
            y += "N";
            break;
            case MgcpAnnouncementPackageType:
            y += "A";
            break;
            case MgcpScriptPackageType:
            y += "Script";
            break;
#else // NCS_PROFILE
            case MgcpNCSLinePackageType:
            y += "L";
            break;
            case MgcpNCSAdsiPackageType:
            y += "S";
            break;
#endif
            default:
            break;
        }
        prevFlag = true;
        ++iter;
    }
    return y;
}

///
bool MgcpParameterCapabilities::setConnectionMode(const string& mode)
{
    if (mode == "sendonly")
        ConnectionModeList.push_back(ConnectionSendOnly);
    else if (mode == "recvonly")
        ConnectionModeList.push_back(ConnectionRecvOnly);
    else if (mode == "sendrecv")
        ConnectionModeList.push_back(ConnectionSendRecv);
    else if (mode == "confrnce")
        ConnectionModeList.push_back(ConnectionConference);
    else if (mode == "inactive")
        ConnectionModeList.push_back(ConnectionInactive);
    else if (mode == "loopback")
        ConnectionModeList.push_back(ConnectionLoopback);
    else if (mode == "conttest")
        ConnectionModeList.push_back(ConnectionContTest);
    else if (mode == "netwloop")
        ConnectionModeList.push_back(ConnectionNetwLoop);
    else if (mode == "netwtest")
        ConnectionModeList.push_back(ConnectionNetwTest);
    else if (mode == "data")
        ConnectionModeList.push_back(ConnectionData);
    else
        return( false );

    return( true );
}

string MgcpParameterCapabilities::getConnectionModeItem(int i)
{
    assert( static_cast < unsigned int > (i) < ConnectionModeList.size() );
    string y;
    MgcpParameterConnectionModeEnum myMode = ConnectionModeList[i];
    switch (myMode)
    {
        case ConnectionSendOnly:
        {
            y = "sendonly";
        }
        break;
        case ConnectionRecvOnly:
        {
            y = "recvonly";
        }
        break;
        case ConnectionSendRecv:
        {
            y = "sendrecv";
        }
        break;
        case ConnectionConference:
        {
            y = "confrnce";
        }
        break;
        case ConnectionInactive:
        {
            y = "inactive";
        }
        break;
        case ConnectionLoopback:
        {
            y = "loopback";
        }
        break;
        case ConnectionContTest:
        {
            y = "conttest";
        }
        break;
        case ConnectionNetwLoop:
        {
            y = "netwloop";
        }
        break;
        case ConnectionNetwTest:
        {
            y = "netwtest";
        }
        break;
        case ConnectionData:
        {
            y = "data";
        }
        break;
        default:
        {

	}
        break;
    }
    return y;
}

string MgcpParameterCapabilities::ConnectionModeEncode()
{
    string y;
    bool prevFlag = false;
    y = "m:";

    int index = 0;

    vector < MgcpParameterConnectionModeEnum > ::const_iterator iter = ConnectionModeList.begin();

    while ( iter != ConnectionModeList.end() )
    {
        if ( prevFlag )
            y += ";";

	y += getConnectionModeItem( index );
	index++;

        prevFlag = true;
        ++iter;
    }
    return y;
}

string MgcpParameterCapabilities::EchoCancellationEncode()
{
    return myConnectionOptions.EchoCancellationEncode();
}


string MgcpParameterCapabilities::TypeofServiceEncode()
{
    return myConnectionOptions.TypeofServiceEncode();
}

string MgcpParameterCapabilities::SilenceSuppressionEncode()
{
    return myConnectionOptions.SilenceSuppressionEncode();
}

string MgcpParameterCapabilities::encode()
{
    string y;
    y = "A: ";

    y += PacketizationEncode();
    y += ",";
    y += BandwidthEncode();
    y += ",";
    y += CompressionEncode();
    y += ",";
    y += EventPackageEncode();
    y += ",";
    y += ConnectionModeEncode();
    y += ",";
    y += EchoCancellationEncode();
    y += ",";
    y += TypeofServiceEncode();
    y += ",";
    y += SilenceSuppressionEncode();
    y += "\r\n";

    return y;
}


bool
MgcpParameterCapabilities::parse(const ParameterSplitType& parms)
{
    ParameterSplitType::const_iterator i;
    string temp_str;

    i = parms.begin();
    if ( i != parms.end() )
    {
        i++;
    }
    while ( i != parms.end() )
    {
        temp_str = (*i);
        string temp(temp_str, 0, 2);
        if ( temp == "p:" )
        {
            string pack( temp_str, 2, temp_str.size() - 2 );
            split_t param(split(pack, "-"));
            myConnectionOptions.PacketizationParse( param );
        }
        else if ( temp == "a:")
        {
            string algoStr(temp_str, 2, temp_str.size() - 2);
            split_t param(split(algoStr, ";"));
            myConnectionOptions.CompressionAlgoParse( param );
        }
        else if ( temp == "e:")
        {
            myConnectionOptions.EchoCancellationParse( *i );
        }
        else if ( temp == "s:")
        {
            myConnectionOptions.SilenceSuppressionParse( *i);
        }
        else if (temp == "v:")
        {
            string events( temp_str, 2, temp_str.size() - 2 );
            split_t param(split( events, ";" ));
            EventPackageParse( param );
        }
        else if (temp == "m:")
        {
            string modes( temp_str, 2, temp_str.size() - 2 );
            split_t param(split(modes, ";"));
            ConnectionModeParse( param );
        }
        else if (temp == "b:")
        {
            string band( temp_str, 2, temp_str.size() - 2 );
            split_t param(split(band, "-"));
            myConnectionOptions.BandwidthParse( param );
        }
        else if (temp == "t:")
        {
            string tosStr(temp_str, 2, temp_str.size() - 2);
            myConnectionOptions.TypeofServiceParse(tosStr);
        }
        else
            return false;
        ++i;
    }
    return true;
}

bool
MgcpParameterCapabilities::ConnectionModeParse(const ParameterSplitType& param)
{
    ParameterSplitType::const_iterator iter = param.begin();
    string st;

    while ( iter != param.end() )
    {
        st = (*iter);
	if ( !setConnectionMode( st ) )
	{
	    return false;
	}

        iter++;
    }
    return true;
}


bool
MgcpParameterCapabilities::EventPackageParse( const ParameterSplitType& param)
{
    ParameterSplitType::const_iterator iter = param.begin();
    string st;

    while ( iter != param.end() )
    {
        st = (*iter);
#ifndef NCS_PROFILE
        if ( st == "g")
            EventPackageList.push_back( MgcpGenericMediaPackageType );
        else if ( st == "d")
            EventPackageList.push_back( MgcpDTMFPackageType );
        else if ( st == "m")
            EventPackageList.push_back( MgcpMFPackageType );
        else if ( st == "t")
            EventPackageList.push_back( MgcpTrunkPackageType );
        else if ( st == "l")
            EventPackageList.push_back( MgcpLinePackageType );
        else if ( st == "h")
            EventPackageList.push_back( MgcpHandsetPackageType );
        else if ( st == "r")
            EventPackageList.push_back( MgcpRTPPackageType );
        else if ( st == "n")
            EventPackageList.push_back( MgcpNetworkAccessServerPackageType );
        else if ( st == "a")
            EventPackageList.push_back( MgcpAnnouncementPackageType );
        else if ( st == "Script")
            EventPackageList.push_back( MgcpScriptPackageType );
        else
            return ( false );
#else // NCS_PROFILE
        if ( st == "l")
            EventPackageList.push_back( MgcpNCSLinePackageType );
        else if ( st == "s")
            EventPackageList.push_back( MgcpNCSAdsiPackageType );
        else
            return ( false );
#endif
        ++iter;
    }
    return true;
}





/********************************************************
  Connection Parameters 
************************************************************/
void
MgcpParameterConnectionParameters::CopyElements(const MgcpParameterConnectionParameters& myparameter)
{
    packetssent = myparameter.packetssent;
    octetssent = myparameter.octetssent;
    packetsrcvd = myparameter.packetsrcvd;
    octetsrcvd = myparameter.octetsrcvd;
    packetslost = myparameter.packetslost;
    jitter = myparameter.jitter;
    latency = myparameter.latency;
}

int
MgcpParameterConnectionParameters:: operator== (const MgcpParameterConnectionParameters& myparameter) const
{
    return ( packetssent == myparameter.packetssent &&
             octetssent == myparameter.octetssent &&
             packetsrcvd == myparameter.packetsrcvd &&
             octetsrcvd == myparameter.octetsrcvd &&
             packetslost == myparameter.packetslost &&
             jitter == myparameter.jitter &&
             latency == myparameter.latency );
}


string
MgcpParameterConnectionParameters:: itos(int value )
{
    int const bufsize = 20;
    assert(sizeof(int) <= 8);
    char buffer[bufsize];
    bool negative = value < 0;
    if ( negative )
        value = -value;
    int endpos = bufsize;
    do
    {
        buffer[--endpos] = char('0' + value % 10);
    }
    while ( value /= 10 );
    if (negative)
        buffer[--endpos] = '-';
    string s(&buffer[endpos], bufsize - endpos);
    return s;
}

string
MgcpParameterConnectionParameters::encode()
{
    string y;

    y = "P: ";

    y += "PS:";
    y += itos( packetssent );
    y += ",";
    y += "OS:";
    y += itos( octetssent );
    y += ",";
    y += "PR:";
    y += itos( packetsrcvd );
    y += ",";
    y += "OR:";
    y += itos( octetsrcvd );
    y += ",";
    y += "PL:";
    y += itos( packetslost );
    y += ",";
    y += "JI:";
    y += itos( jitter );
    y += ",";
    y += "LA:";
    y += itos( latency );

    y += "\r\n";
    return y;
}

void MgcpParameterConnectionParameters::setPacketsSent(int ps)
{
    packetssent = ps;
}

int MgcpParameterConnectionParameters::getPacketsSent()
{
    return packetssent;
}

void MgcpParameterConnectionParameters::setOctetsSent(int os)
{
    octetssent = os;
}
int MgcpParameterConnectionParameters::getOctetsSent()
{
    return octetssent;
}

void MgcpParameterConnectionParameters::setPacketsRcvd(int pr)
{
    packetsrcvd = pr;
}
int MgcpParameterConnectionParameters::getPacketsRcvd()
{
    return packetsrcvd;
}

void MgcpParameterConnectionParameters::setOctetsRcvd(int r)
{
    octetsrcvd = r;
}
int MgcpParameterConnectionParameters::getOctetsRcvd()
{
    return octetsrcvd;
}

void MgcpParameterConnectionParameters::setPacketsLost(int pl)
{
    packetslost = pl;
}
int MgcpParameterConnectionParameters::getPacketsLost()
{
    return packetslost;
}

void MgcpParameterConnectionParameters::setJitter(int ji)
{
    jitter = ji;
}
int MgcpParameterConnectionParameters::getJitter()
{
    return jitter;
}

void MgcpParameterConnectionParameters::setLatency(int la)
{
    latency = la;
}
int MgcpParameterConnectionParameters::getLatency()
{
    return latency;
}

bool MgcpParameterConnectionParameters::parse(const ParameterSplitType& param)
{
    ParameterSplitType::const_iterator iter = param.begin();

    if ( iter != param.end() )
    {
        iter++;
    }
    string s;
    bool bol;
    while ( iter != param.end() )
    {
        s = (*iter);
        string temp(s, 0, 3);
        string st(s, 3, s.size() - 3);
        if (temp == "ps:")
            bol = stringToInt( st, &packetssent );
        else if (temp == "os:")
            bol = stringToInt( st, &octetssent );
        else if (temp == "pr:")
            bol = stringToInt( st, &packetsrcvd);
        else if (temp == "or:")
            bol = stringToInt( st, &octetsrcvd );
        else if (temp == "pl:")
            bol = stringToInt( st, &packetslost );
        else if (temp == "ji:")
            bol = stringToInt( st, &jitter );
        else if (temp == "la:")
            bol = stringToInt( st, &latency );
        else
            return false;
        ++iter;
    }
    return bol;
}
