
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpParameterEventGeneric_cxx_Version =
    "$Id: MgcpParameterEventGeneric.cxx,v 1.35.4.1 2001/03/14 19:32:10 icahoon Exp $";


#include "global.h"
#include "debug.h"
#include "MgcpParameterEventGeneric.hxx"
#include "MgcpGenericEvent.hxx"
#include "support.hxx"
#include "MgcpParse.hxx"


ostream& operator<<(ostream& s, const MgcpRequestedAction& x)
{
    x.printData(s);
    return s;
}

/**********************************************************************
 
		  MgcpParameterRequestedEventGeneric
 
 **********************************************************************/



ostream&
MgcpParameterRequestedEventGeneric::printData(ostream& s)
{
    myGenericEvent.printData(s);
    myAction.printData(s);
    return s;
}

/**********************************************************************
 
	       MGCP generic embedded requests
 
 **********************************************************************/


/// this is the internal implementation of this class.
class MgcpEmbeddedRequest_impl_
{
    public:

        ///
        MgcpEmbeddedRequest_impl_();
        ///
        MgcpEmbeddedRequest_impl_(const MgcpEmbeddedRequest_impl_& e);

        ///
        void parse(string myString);
        ///
        string encode();
        ///
        bool insertEvent(Sptr < MgcpParameterRequestedEvent > newEvent);
        ///
        bool insertSignal(Sptr < MgcpParameterSignal > newSignal);
        ///
        bool insertMap(MgcpParameterDigitMap newMap);

        ///
        void clearEvents();
        ///
        void clearSignals();

        ///
        int getDecodedNumEvents();
        ///
        Sptr < MgcpParameterRequestedEvent > getDecodedEvent(int i);
        ///
        int getDecodedNumSignals();
        ///
        Sptr < MgcpParameterSignal > getDecodedSignal(int i);
        ///
        MgcpParameterDigitMap getDecodedMap();

        ///
        ostream& printData(ostream& s) const;

    private:
        ///
        vector < Sptr < MgcpParameterRequestedEvent > > myEvents;
        ///
        vector < Sptr < MgcpParameterSignal > > mySignals;
        ///
        MgcpParameterDigitMap myMap;
};


///
MgcpEmbeddedRequest::MgcpEmbeddedRequest()
{
    Pimpl_ = new MgcpEmbeddedRequest_impl_;
}


///
MgcpEmbeddedRequest::MgcpEmbeddedRequest(
    const MgcpEmbeddedRequest& e
)
{
    Pimpl_ = new MgcpEmbeddedRequest_impl_(*e.Pimpl_);
}


MgcpEmbeddedRequest::~MgcpEmbeddedRequest(
)
{
    delete Pimpl_;
    Pimpl_ = 0;
}


void
MgcpEmbeddedRequest::parse(string embeddedBehaviorStr)
{
    Pimpl_->parse(embeddedBehaviorStr);
}


string
MgcpEmbeddedRequest::encode()
{
    return Pimpl_->encode();
}


///
bool
MgcpEmbeddedRequest::insertEvent(
    Sptr < MgcpParameterRequestedEvent > newEvent
)
{
    return Pimpl_->insertEvent(newEvent);
}


///
bool
MgcpEmbeddedRequest::insertSignal(
    Sptr < MgcpParameterSignal > newSignal
)
{
    return Pimpl_->insertSignal(newSignal);
}


///
bool
MgcpEmbeddedRequest::insertMap(
    MgcpParameterDigitMap newMap
)
{
    return Pimpl_->insertMap(newMap);
}


///
void
MgcpEmbeddedRequest::clearEvents()
{
    Pimpl_->clearEvents();
}


///
void
MgcpEmbeddedRequest::clearSignals()
{
    Pimpl_->clearSignals();
}


///
int MgcpEmbeddedRequest::getDecodedNumEvents()
{
    return Pimpl_->getDecodedNumEvents();
}


///
Sptr<MgcpParameterRequestedEvent> 
MgcpEmbeddedRequest::getDecodedEvent(int i)
{
    return Pimpl_->getDecodedEvent(i);
}


///
int MgcpEmbeddedRequest::getDecodedNumSignals()
{
    return Pimpl_->getDecodedNumSignals();
}


///
Sptr<MgcpParameterSignal> 
MgcpEmbeddedRequest::getDecodedSignal(int i)
{
    return Pimpl_->getDecodedSignal(i);
}

///
MgcpParameterDigitMap MgcpEmbeddedRequest::getDecodedMap()
{
    return Pimpl_->getDecodedMap();
}

///
ostream& MgcpEmbeddedRequest::printData(ostream& s) const
{
    return Pimpl_->printData(s);
}



/**********************************************************************
 
		   Embedded Request Implementation
 
 **********************************************************************/

MgcpEmbeddedRequest_impl_::MgcpEmbeddedRequest_impl_()
{}



MgcpEmbeddedRequest_impl_::MgcpEmbeddedRequest_impl_(
    const MgcpEmbeddedRequest_impl_& e
)
        : myEvents(e.myEvents),
        mySignals(e.mySignals),
        myMap(e.myMap)
{}



void
MgcpEmbeddedRequest_impl_::parse(string embeddedBehaviorStr)
{
    // do something about this.
    ODEBUG(DEBUG_PARSE, 2, cerr << "EmbeddedRequest parsing: " << embeddedBehaviorStr << "\n"; );

    //    cout << "entering test" << embeddedBehaviorStr << endl;

    split_t embeddedParameters
    = split_paren_match(embeddedBehaviorStr, string(","));

    //    cout << "split parameters" << endl;

    split_t::iterator nextEmbeddedParameter
    = embeddedParameters.begin();

    if ((nextEmbeddedParameter != embeddedParameters.end())
            && ((*nextEmbeddedParameter)[0] == 'r'))
    {
        ODEBUG(DEBUG_PARSE, 2, cerr << "Got Embedded RequestedEvent: " << (*nextEmbeddedParameter) << "\n"; );
        // embedded RequestedEvent

        // now we need to parse the embedded requested event

        // something like

        // R([0-9#T](D),hu(N))

        string myNext((*nextEmbeddedParameter));

        //	cerr << "getting requested events: " << myNext << endl;

        string embeddedRequestedEvents
        = str_paren_match(myNext);

        //	cerr << "done with getting requested events" << endl;

        string eStr(embeddedRequestedEvents);

        split_t embeddedRequestedEventsList
        = split_paren_match(eStr, ",");

        split_t::iterator nextEmbeddedRequestedEvent
        = embeddedRequestedEventsList.begin();
        while (nextEmbeddedRequestedEvent !=
                embeddedRequestedEventsList.end())
        {
            // for each of these requested events, you can call another parser

            string tmp = (*nextEmbeddedRequestedEvent);

            Sptr < MgcpParameterRequestedEvent > newEvent
            = parseEventsRequested(tmp);

            if (newEvent != 0)
            {
                insertEvent(newEvent);
            }
            nextEmbeddedRequestedEvent++;
        }
        nextEmbeddedParameter++;
    }

    if ((nextEmbeddedParameter != embeddedParameters.end())
            && ((*nextEmbeddedParameter)[0] == 's'))
    {
        ODEBUG(DEBUG_PARSE, 2, cerr << "Got Embedded SignalRequest: " << (*nextEmbeddedParameter) << "\n"; );
        // embedded SignalRequest

        // now we need to parse the embedded requested event

        // something like

        // S(dl)

        //	cout << "getting signals: " << (*nextEmbeddedParameter) << endl;

        string embeddedSignals
        = str_paren_match(*nextEmbeddedParameter);

        split_t signalList
        = split_paren_match(embeddedSignals, string(","));

        split_t::iterator nextSignal
        = signalList.begin();
        while (nextSignal !=
                signalList.end())
        {
            // for each of these requested events, you can call another parser

            string tmp = (*nextSignal);

            Sptr < MgcpParameterSignal > newSignal
            = parseSignals(tmp);

	    if ( newSignal != 0 )
	    {
                insertSignal(newSignal);
	    }
            nextSignal++;
        }

        nextEmbeddedParameter++;
    }

    if ((nextEmbeddedParameter != embeddedParameters.end())
            && ((*nextEmbeddedParameter)[0] == 'd'))
    {
        ODEBUG(DEBUG_PARSE, 2, cerr << "Got Embedded DigitMap: " << (*nextEmbeddedParameter) << "\n"; );
        // embedded DigitMap

        string embeddedDigitMap
        = str_paren_match(*nextEmbeddedParameter);

        myMap.parseEmbedded(embeddedDigitMap);

        nextEmbeddedParameter++;
    }

    if (nextEmbeddedParameter != embeddedParameters.end())
    {
        // TODO: non-fatal error
        cerr << "warning: additional unknown parameters in embedded parameters" << endl;
    }
}


string
MgcpEmbeddedRequest_impl_::encode()
{
    string encoding;


    // encode the embedded RequestedEvents

    {
        encoding += "r(";

        vector < Sptr < MgcpParameterRequestedEvent > > ::const_iterator iter
        = myEvents.begin();
        bool prevFlag = false;

        while (iter != myEvents.end())
        {
            if (!*iter)
            {
                CERR(cerr << "error encoding because null...\n"; );
            }
            else
            {
                Sptr < MgcpParameterRequestedEvent > x = *iter;

                if (prevFlag)
                {
                    encoding += ",";
                }
                encoding += (x->encode());
                prevFlag = true;
            }
            ++iter;
        }

        encoding += ")";
    }

    encoding += ",";

    // encode the embedded Signals

    {
        encoding += "s(";

        vector < Sptr < MgcpParameterSignal > > ::const_iterator iter =
            mySignals.begin();
        bool prevFlag = false;

        while (iter != mySignals.end())
        {
            if (prevFlag)
            {
                encoding += ",";
            }
            encoding += ((*iter)->encode());
            prevFlag = true;
            ++iter;
        }

        encoding += ")";
    }

    encoding += ",";

    // encode the embedded DigitMap

    encoding += "d(";
    encoding += myMap.encodeEmbedded();
    encoding += ")";

    return encoding;
}


bool
MgcpEmbeddedRequest_impl_::insertEvent(
    Sptr < MgcpParameterRequestedEvent > newEvent
)
{
    assert( newEvent != 0 );
    myEvents.push_back(newEvent);
    return true;
}


bool
MgcpEmbeddedRequest_impl_::insertSignal(
    Sptr < MgcpParameterSignal > newSignal
)
{
    assert( newSignal != 0 );
    mySignals.push_back(newSignal);
    return true;
}


///
bool
MgcpEmbeddedRequest_impl_::insertMap(
    MgcpParameterDigitMap newMap
)
{
    myMap = newMap;
    return true;
}


void
MgcpEmbeddedRequest_impl_::clearEvents()
{
    myEvents.clear();

}


void
MgcpEmbeddedRequest_impl_::clearSignals()
{
    mySignals.clear();
}



///
int MgcpEmbeddedRequest_impl_::getDecodedNumEvents()
{
    return myEvents.size();
}


///
Sptr<MgcpParameterRequestedEvent> 
MgcpEmbeddedRequest_impl_::getDecodedEvent(int i)
{
    assert(static_cast < unsigned int > (i) < myEvents.size());
    return myEvents[i].getPtr();
}


///
int MgcpEmbeddedRequest_impl_::getDecodedNumSignals()
{
    return mySignals.size();
}


///
Sptr<MgcpParameterSignal> 
MgcpEmbeddedRequest_impl_::getDecodedSignal(int i)
{
    assert(static_cast < unsigned int > (i) < mySignals.size());
    return mySignals[i].getPtr();
}


///
MgcpParameterDigitMap MgcpEmbeddedRequest_impl_::getDecodedMap()
{
    return myMap;
}


///
ostream& MgcpEmbeddedRequest_impl_::printData(ostream& s) const
{
    s << "EmbeddedRequest *** start ***" << endl;

    vector < Sptr < MgcpParameterRequestedEvent > > ::const_iterator iter
    = myEvents.begin();

    s << "EmbeddedRequest Events:" << endl;

    while (iter != myEvents.end())
    {
        if (!*iter)
        {
            s << "    error while encoding" << endl;
        }
        else
        {
            s << "    ";
            (*iter)->printData(s);
        }
        ++iter;
    }

    s << "EmbeddedRequest Signals:" << endl;

    vector < Sptr < MgcpParameterSignal > > ::const_iterator signalIterator
    = mySignals.begin();

    while (signalIterator != mySignals.end())
    {
        if (!*signalIterator)
        {
            s << "    error while encoding" << endl;
        }
        else
        {
            s << "    ";
            (*signalIterator)->printData(s);
        }
        ++signalIterator;
    }

    s << endl;
    s << "EmbeddedRequest DigitMap:" << endl;
    myMap.printData(s);
    s << endl;
    s << "EmbeddedRequest *** end   ***" << endl;

    return s;
}


/**********************************************************************
 
	       MGCP generic actions
 
 **********************************************************************/


ostream&
MgcpRequestedAction::printData(ostream& s) const
{
    s << "Generic Action\n" << endl;
    s << "Type: ";

    switch (myActionType)
    {
        case MgcpRequestedActionTypeNotify:
        s << "Notify" << endl;
        break;
        case MgcpRequestedActionTypeAccumulate:
        s << "Accumulate" << endl;
        break;
        case MgcpRequestedActionTypeUseDigitMap:
        s << "UseDigitMap" << endl;
        break;
        case MgcpRequestedActionTypeSwap:
        s << "Swap" << endl;
        break;
        case MgcpRequestedActionTypeIgnore:
        s << "Ignore" << endl;
        break;
        case MgcpRequestedActionTypeKeepActive:
        s << "KeepActive" << endl;
        break;
        case MgcpRequestedActionTypeEmbedNotificationRequest:
        //	    cerr << "Embedded Notifications not implemented yet\n";
        s << "EmbeddedNotification: ";
        if (myEmbeddedRequest != 0)
        {
            myEmbeddedRequest->printData(s);
        }
        else
        {
            s << "none";
        }
        s << endl;
        break;
    }
    return s;
}


string
MgcpRequestedAction::encode()
{
    string myEncoding;
    switch (myActionType)
    {
        case MgcpRequestedActionTypeNotify:
        myEncoding = "(n)";
        break;
        case MgcpRequestedActionTypeAccumulate:
        myEncoding = "(a)";
        break;
        case MgcpRequestedActionTypeUseDigitMap:
        myEncoding = "(d)";
        break;
        case MgcpRequestedActionTypeSwap:
        myEncoding = "(s)";
        break;
        case MgcpRequestedActionTypeIgnore:
        myEncoding = "(i)";
        break;
        case MgcpRequestedActionTypeKeepActive:
        myEncoding = "(k)";
        break;
        case MgcpRequestedActionTypeEmbedNotificationRequest:
        //	    cerr << "Embedded Notifications not implemented yet\n";
        myEncoding = "(e(";

        myEncoding += myEmbeddedRequest->encode();

        myEncoding += "))";

        break;
    }
    return myEncoding;
}


bool
MgcpRequestedAction::parse(const string& inputString)
{
    if (inputString == "n")
    {
        myActionType = MgcpRequestedActionTypeNotify;
    }
    else if (inputString == "a")
    {
        myActionType = MgcpRequestedActionTypeAccumulate;
    }
    else if (inputString == "d")
    {
        myActionType = MgcpRequestedActionTypeUseDigitMap;
    }
    else if (inputString == "s")
    {
        myActionType = MgcpRequestedActionTypeSwap;
    }
    else if (inputString == "i")
    {
        myActionType = MgcpRequestedActionTypeIgnore;
    }
    else if (inputString == "k")
    {
        myActionType = MgcpRequestedActionTypeKeepActive;
    }
    else if (inputString[0] == 'e')
    {
        myActionType = MgcpRequestedActionTypeEmbedNotificationRequest;

        // now, we need to parse the action.  extract anything in
        // parentheses in the action

        string embeddedBehavior = str_paren_match(inputString);

        // in the embeddedBehavior we need to split it into
        // parameters

        string embeddedBehaviorStr = embeddedBehavior;

        ODEBUG(DEBUG_PARSE, 2, cerr << "EmbeddedRequest: " << embeddedBehaviorStr << "\n"; );

        myEmbeddedRequest = new MgcpEmbeddedRequest;

        myEmbeddedRequest->parse(embeddedBehaviorStr);

        ODEBUG(DEBUG_PARSE, 2, cerr << "EmbeddedRequestPrint: "; );
        ODEBUG(DEBUG_PARSE, 2, myEmbeddedRequest->printData(cerr); );

        // doh! need to copy this to the inside embedded request

    }

    else
    {
        // default case

        myActionType = MgcpRequestedActionTypeNotify;
    }
    return true;
}


bool
MgcpRequestedAction::insertEmbedded(Sptr < MgcpEmbeddedRequest > embedded)
{
    myActionType = MgcpRequestedActionTypeEmbedNotificationRequest;
    myEmbeddedRequest = embedded;
    return true;
}


