#ifndef MGCP_PARAMETER_EVENT_GENERIC_HXX
#define MGCP_PARAMETER_EVENT_GENERIC_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpParameterEventGeneric_hxx_Version =
    "$Id: MgcpParameterEventGeneric.hxx,v 1.37 2001/02/01 01:22:56 icahoon Exp $";


#include "global.h"
#include <string>
#include <vector>
#include <deque>
#include <typeinfo>
#include "SdpSession.hxx"
#include "MgcpParameter.hxx"
#include "MgcpGenericEvent.hxx"


///
typedef enum MgcpRequestedActionType
{
    MgcpRequestedActionTypeNotify,
    MgcpRequestedActionTypeAccumulate,
    MgcpRequestedActionTypeUseDigitMap,
    MgcpRequestedActionTypeSwap,
    MgcpRequestedActionTypeIgnore,
    MgcpRequestedActionTypeKeepActive,
    MgcpRequestedActionTypeEmbedNotificationRequest
}MgcpRequestedActionType;



class MgcpParameterRequestedEventGeneric;
class MgcpEmbeddedRequest_impl_;

class MgcpRequestedAction;
extern ostream& operator<<(ostream& s, const MgcpRequestedAction& x);

/***********************************************************************
 
       Events
 
**********************************************************************/

///
class MgcpParameterEventGeneric : public MgcpParameterEvent
{
    public:
        ///
        MgcpParameterEventGeneric(MgcpGenericEvent x)
        : myGenericEvent(x)
        {}

        ///
        MgcpParameterEventGeneric(
            const MgcpParameterEventGeneric& e
        )
        : myGenericEvent(e.myGenericEvent)
        { }


        ///
        string encode()
        {
            return myGenericEvent.getEncoding();
        }

        ///
        ostream& printData(ostream& s) const
        {
            return s << "Generic event";
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterEventGeneric(*this);
        }

        MgcpGenericEvent getGenericEvent()
        {
            return myGenericEvent;
        }

    private:
        ///
        MgcpGenericEvent myGenericEvent;
};


///
class MgcpEmbeddedRequest
{
    public:

        ///
        MgcpEmbeddedRequest();
        ///
        MgcpEmbeddedRequest( const MgcpEmbeddedRequest& e);
        ///
        ~MgcpEmbeddedRequest();
        ///
        void parse(string myString);
        ///
        string encode();
        ///
        bool insertEvent(Sptr < MgcpParameterRequestedEvent > newEvent);
        ///
        bool insertSignal(Sptr < MgcpParameterSignal > newSignal);
        ///
        bool insertMap( MgcpParameterDigitMap newMap );

        ///
        void clearEvents();
        ///
        void clearSignals();

        ///
        int getDecodedNumEvents();
        ///
        Sptr < MgcpParameterRequestedEvent > getDecodedEvent(int i);
        ///
        int getDecodedNumSignals();
        ///
        Sptr < MgcpParameterSignal > getDecodedSignal(int i);
        ///
        MgcpParameterDigitMap getDecodedMap();

        ///
        ostream& printData(ostream& s) const;

    private:
        ///
        MgcpEmbeddedRequest_impl_* Pimpl_;
};


///
class MgcpRequestedAction
{
    public:
        ///
        MgcpRequestedAction()
        : myActionType(MgcpRequestedActionTypeNotify),
        myEmbeddedRequest(0)

        {}


        /** Constructor which parses a string to build the action type
        */
  
        MgcpRequestedAction(const string& s)
        : myActionType(MgcpRequestedActionTypeNotify),
        myEmbeddedRequest(0)
        {
            this->parse(s);
        }
  

        ///
        MgcpRequestedAction(const MgcpRequestedActionType& actionType)
        : myActionType(actionType) ,
        myEmbeddedRequest(0)
        {}


        /// copy constructor

        MgcpRequestedAction(const MgcpRequestedAction& e)
        : myActionType(e.myActionType),
        myEmbeddedRequest(e.myEmbeddedRequest)
        {}


        /// = operator

        MgcpRequestedAction& operator= (const MgcpRequestedAction& aRequestedAction)
        {
	    if ( this != &aRequestedAction )
	    {
                this->myActionType = aRequestedAction.myActionType;
                this->myEmbeddedRequest = aRequestedAction.myEmbeddedRequest;
	    }
            return *this;
        }

        ///
        bool insertEmbedded(Sptr < MgcpEmbeddedRequest > embedded);

        ///
        string encode();

        ///
        MgcpRequestedActionType getActionType() const
        {
            return myActionType;
        }

        ///
        ostream& printData(ostream& s) const;

        friend ostream& operator<<(ostream& s, const MgcpRequestedAction& x);


    protected:

        /// this function parses a string and converts it into an action
        bool parse(const string& inputString);

    private:

        ///
        MgcpRequestedActionType myActionType;

        /// this is a slot for an embeddedrequest
        Sptr < MgcpEmbeddedRequest > myEmbeddedRequest;
};


///
class MgcpParameterRequestedEventGeneric : public MgcpParameterRequestedEvent
{
    public:
        ///
        MgcpParameterRequestedEventGeneric(MgcpGenericEvent x)
        : myGenericEvent(x)
        {}


        ///
        MgcpParameterRequestedEventGeneric(const MgcpGenericEvent& x,
                                           const MgcpRequestedAction& y)
        : myGenericEvent(x),
        myAction(y)
        {}


        ///
        MgcpParameterRequestedEventGeneric(
            const MgcpParameterRequestedEventGeneric& e
        )
        : myGenericEvent(e.myGenericEvent),
        myAction(e.myAction)
        { }


        ///
        string encode()
        {
            string myEncoding = myGenericEvent.getEncoding() + myAction.encode();
            return myEncoding;
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterRequestedEventGeneric(*this);
        }

        MgcpGenericEvent getGenericEvent()
        {
            return myGenericEvent;
        }

        MgcpRequestedAction getRequestedAction()
        {
            return myAction;
        }

        ///
        ostream& printData(ostream& s);

    private:

        ///
        MgcpGenericEvent myGenericEvent;

        ///
        MgcpRequestedAction myAction;
};


/***********************************************************************
 
			       Signals
 
 **********************************************************************/

///
class MgcpParameterSignalGeneric : public MgcpParameterSignal
{
    public:
        ///
        MgcpParameterSignalGeneric(MgcpGenericEvent x)
        : myGenericEvent(x)
        {}


        ///
        MgcpParameterSignalGeneric(
            const MgcpParameterSignalGeneric& e
        )
        : myGenericEvent(e.myGenericEvent)
        { }


        ///
        string encode()
        {
            return myGenericEvent.getEncoding();
            //	return myEncoding;
        }

        ///

        ostream& printData(ostream& s) const
        {
            return s << "Generic signal";
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterSignalGeneric(*this);
        }

        MgcpGenericEvent getGenericEvent()
        {
            return myGenericEvent;
        }

    private:
        ///
        //    string myEncoding;
        ///
        MgcpGenericEvent myGenericEvent;
};


#endif
