
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpParse_cxx_Version =
    "$Id: MgcpParse.cxx,v 1.30 2001/02/01 01:22:56 icahoon Exp $" ;


#include "global.h"
#include "MgcpParse.hxx"
#include "Sptr.hxx"
#include "support.hxx"
#include "MgcpCommand.hxx"
#include "TokenRecognizer.hxx"

/**********************************************************************
 
			  Parsing Functions
 
 **********************************************************************/

MgcpResponseCode mapRespCode(int code)
{
    MgcpResponseCode returnCode = ReasonCodeUndefined;

    switch ( code )
    {
        //#ifndef NCS_PROFILE
        //  case 0:
        //  returnCode = ReasonEndpointStateNominal;
        // break;
        //#else // NCS_PROFILE
        case 0:
        returnCode = ReasonResponseAcknowledgement;
        break;
        //#endif
        case 100:
        returnCode = ReasonTransactionInProgress;
        break;
        case 200:
        returnCode = ReasonTransactionExecuted;
        break;
        case 250:
        returnCode = ReasonConnectionDeleted;
        break;
        case 400:
        returnCode = ReasonTransientError;
        break;
        case 401:
        returnCode = ReasonTransientAlreadyOffHook;
        break;
        case 402:
        returnCode = ReasonTransientAlreadyOnHook;
        break;
        case 403:
        returnCode = ReasonTransientInsufficientResources;
        break;
        case 404:
        returnCode = ReasonTransientInsufficientBandwidth;
        break;
        case 500:
        returnCode = ReasonEndpointUnknown;
        break;
        case 501:
        returnCode = ReasonEndpointUnready;
        break;
        case 502:
        returnCode = ReasonPermanentInsufficientResources;
        break;
        case 510:
        returnCode = ReasonProtocolError;
        break;
        case 511:
        returnCode = ReasonUnrecognizedExtension;
        break;
        case 512:
        returnCode = ReasonUnsupportedEvent;
        break;
        case 513:
        returnCode = ReasonUnsupportedSignal;
        break;
        case 514:
        returnCode = ReasonUnsupportedAnnouncement;
        break;
        case 515:
        returnCode = ReasonIncorrectConnectionId;
        break;
        case 516:
        returnCode = ReasonUnknownCallId;
        break;
        case 517:
        returnCode = ReasonInvalidMode;
        break;
        case 518:
        returnCode = ReasonInvalidPackage;
        break;
        case 519:
        returnCode = ReasonNoDigitMap;
        break;
        case 520:
        returnCode = ReasonRestarting;
        break;
        case 521:
        returnCode = ReasonEndpointRedirected;
        break;
        case 522:
        returnCode = ReasonNoSuchEvent;
        break;
        case 523:
        returnCode = ReasonUnknownAction;
        break;
        case 524:
        returnCode = ReasonBadConnectionOption;
        break;
        case 525:
        returnCode = ReasonUnknownConnectionOption;
        break;
        case 526:
        returnCode = ReasonPermanentInsufficientBandwidth;
        break;
        case 527:
        returnCode = ReasonMissingRemoteDescriptor;
        break;
        case 528:
        returnCode = ReasonIncompatibleVersion;
        break;
        case 529:
        returnCode = ReasonHardwareFailure;
        break;
        case 530:
        returnCode = ReasonCASProtocolError;
        break;
        case 531:
        returnCode = ReasonFacilityFailure;
        break;
        case 900:
        returnCode = ReasonEndpointMalfunctioning;
        break;
        case 901:
        returnCode = ReasonEndpointOOS;
        break;
        case 902:
        returnCode = ReasonLostConnectivity;
        break;
#ifdef NCS_PROFILE
        case 903:
        returnCode = ReasonQoSRSVLost;
        break;
#endif
        case 1024:
        returnCode = TimeoutNoResponse;
        break;
        default:
        returnCode = ReasonCodeUndefined;
        break;
    }

    return ( returnCode );
}


//                        parse events

Sptr < MgcpParameterEvent >
parseEvents(const string& s)
{
    MgcpGenericEvent genEvent(s);

#ifndef NCS_PROFILE
    if (genEvent.getPackageType() != MgcpNoPackageType)
#else
    if (genEvent.getPackageType() != MgcpNCSUnknownPackageType)
#endif
    {
        return Sptr < MgcpParameterEvent > (new MgcpParameterEventGeneric(genEvent));
    }
    return Sptr < MgcpParameterEvent > (0);
}


// TODO: need to parse multiple actions in requested events

// TODO: need to allow multiple actions in the external API

// TODO: need to support multiple actions in the RequestedEvent objects




Sptr < MgcpParameterRequestedEvent >
parseEventsRequested(const string& s)
{
    // split the string into the actual event and the action

    string action = str_paren_match(s);
    string::size_type startAction = s.find_first_of("(");

    bool noActionListed = false;

    if (startAction == string::npos)
    {
        // no action found... not strictly an error
        ODEBUG(DEBUG_PARSE, 0, cerr << "PARSE: No action found\n"; );
        startAction = s.length();
        noActionListed = true;
    }

    string event = s.substr(0, startAction);

    ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: Event: " << event << "\n"; );
    ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: Action: " << action << "\n"; );

    MgcpGenericEvent genEvent(event);

    // TODO: this assumes that the action is a SINGLE action, and not
    // a list of actions.  The correct behavior here is to loop based
    // on the action list

    MgcpRequestedAction genAction;

    if (action.length() > 0)
    {
        genAction = MgcpRequestedAction(action);
    }

#ifndef NCS_PROFILE
    if (genEvent.getPackageType() != MgcpNoPackageType)
#else
    if (genEvent.getPackageType() != MgcpNCSUnknownPackageType)
#endif
    {
        MgcpParameterRequestedEventGeneric* p = 0;
        p = new MgcpParameterRequestedEventGeneric(genEvent, genAction);
        return Sptr < MgcpParameterRequestedEvent > (p);
    }
    return Sptr < MgcpParameterRequestedEvent > (0);
}


//                        parse signals


Sptr < MgcpParameterSignal >
parseSignals(const string& s)
{
    MgcpGenericEvent genEvent(s);

#ifndef NCS_PROFILE
    if (genEvent.getPackageType() != MgcpNoPackageType)
#else
    if (genEvent.getPackageType() != MgcpNCSUnknownPackageType)
#endif
    {
        return Sptr < MgcpParameterSignal > (new MgcpParameterSignalGeneric(genEvent));
    }

    return Sptr < MgcpParameterSignal > (0);
}


Sptr < MgcpParameter >
parseParameter(const string& refString)
{
    Sptr < MgcpParameter > ptr;

    split_t parameters(split(refString, " \t\r\n,"));


    if (!parameters.empty())
    {
        string strTemp = parameters[0];
        if (parameters[0] == "b:")
        {
            if (parameters[1] == "e:mu")
            {
                ptr = new MgcpParameterBearerInformation(EncodingMethoduLaw);
            }
            else if (parameters[1] == "e:a")
            {
                ptr = new MgcpParameterBearerInformation(EncodingMethodALaw);
            }
        }
        else if (parameters[0] == "n:")
        {
            ptr = new MgcpParameterNotifiedEntity(parameters[1]);
        }
        else if (parameters[0] == "d:")
        {
            MgcpParameterDigitMap* digitPtr = new MgcpParameterDigitMap();
 
            digitPtr->parse(parameters);
            ptr = digitPtr;
            
        }
        else if (parameters[0] == "r:")
        {
            MgcpParameterRequestedEvents* requestedPtr = new MgcpParameterRequestedEvents();

            // in the case of requested events this must be reparsed
            // due to the fact that you can have commas inside of the
            // paraentheses for the embedded requests.  Thus, the
            // parsing here must match parentheses above matching
            // commas.

            split_t parametersParenMatched(
                split_paren_match(refString, " \t\r\n,"));

           
            requestedPtr->parse(parametersParenMatched);
            ptr = requestedPtr;
            
        }
        else if (parameters[0] == "o:")
        {
            MgcpParameterObservedEvents* observedPtr = new MgcpParameterObservedEvents();
            
            observedPtr->parse(parameters);
            ptr = observedPtr;
       
        }
        else if (parameters[0] == "s:")
        {
            MgcpParameterSignalRequests* p = new MgcpParameterSignalRequests();
           
            p->parse(parameters);
            ptr = p;
            
        }
        else if (parameters[0] == "q:")
        {
            Sptr < MgcpParameterQuarantineHandling > p = new MgcpParameterQuarantineHandling();
           
            split_t parameterItems(split(refString, " ,"));

            p->parse( parameterItems );
            ptr = p;
            
        }
        else if (parameters[0] == "f:")
        {
            Sptr < MgcpParameterRequestedInfo > p = new MgcpParameterRequestedInfo();
            
            split_t parameterItems(split(refString, " ,"));

            p->parse( parameterItems );
            ptr = p;
            
        }
        else if (parameters[0] == "t:")
        {
            Sptr < MgcpParameterDetectEvents > p = new MgcpParameterDetectEvents();
            
            split_t parameterItems(split(refString, " ,"));

            p->parse( parameterItems );
            ptr = p;
            
        }
        /*
           else if (parameters[0] == "z:")
    {
            Sptr<MgcpParameterEndpointIdList> p = new MgcpParameterEndpointIdList();
           
            split_t parameterItems(split(refString, " ,"));

            p->parse( parameterItems );
            ptr = p;
            
    } 
          */
        else if (parameters[0] == "z:")
        {
            static bool visited = false;
            static Sptr < MgcpParameterEndpointIdList > p;

            if ( !visited )
            {
                p = new MgcpParameterEndpointIdList();
            }
            if ( p != 0 )
            {
                split_t parameterItems(split(refString, " ,"));
                p->parse( parameterItems );
                ptr = p;
                visited = true;
            }
        }
        else if (parameters[0] == "zm:")
        {
            Sptr < MgcpParameterMaxEndPointIds > p = new MgcpParameterMaxEndPointIds();
           
            split_t parameterItems(split(refString, " ,"));
            p->parse( parameterItems);
            ptr = p;
        
        }
        else if (parameters[0] == "zn:")
        {
            Sptr < MgcpParameterNumEndPoints > p = new MgcpParameterNumEndPoints();
            
            split_t parameterItems(split(refString, " ,"));
            p->parse( parameterItems);
            ptr = p;
            
        }
        else if (parameters[0] == "a:")
        {
            Sptr < MgcpParameterCapabilities > p = new MgcpParameterCapabilities();
           
            split_t parameterItems(split(refString, " ,"));
            p->parse( parameterItems);
            ptr = p;
         
        }
        else if (parameters[0] == "p:")
        {
            Sptr < MgcpParameterConnectionParameters > p = new MgcpParameterConnectionParameters();
          
            split_t parameterItems(split(refString, " ,"));
            p->parse( parameterItems);
            ptr = p;
          
        }

        else if (parameters[0] == "c:")
        {
            MgcpParameterCallId* callPtr = new MgcpParameterCallId();
           
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
        else if (parameters[0] == "i:")
        {
            MgcpParameterConnectionId* callPtr = new MgcpParameterConnectionId();
           
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
        else if (parameters[0] == "m:")
        {
            MgcpParameterConnectionMode* callPtr = new MgcpParameterConnectionMode();
           
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
        else if (parameters[0] == "x:")
        {
            MgcpParameterRequestId* callPtr = new MgcpParameterRequestId();
           
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
        else if (parameters[0] == "k:")
        {
            MgcpParameterResponseAcknowledgement* callPtr = new MgcpParameterResponseAcknowledgement();
           
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
        else if (parameters[0] == "l:")
        {
            MgcpParameterLocalConnectionOptions* callPtr
            = new MgcpParameterLocalConnectionOptions();
          
            callPtr->parse(parameters);
            ptr = callPtr;
            
        }
    }
    else
    {
        return 0 ;
    }

    return ptr ;
}


MgcpCommand*
getCommandFromToken( MgcpTokenType myToken)
{

    MgcpCommand* newMessage = 0;
    switch (myToken)
    {
        case MgcpTokenTypeEPCF:
        {
            newMessage = new MgcpCommandEndpointConfiguration;
        }
        break;

        case MgcpTokenTypeRQNT:
        {
            newMessage = new MgcpCommandNotificationRequest;
        }
        break;

        case MgcpTokenTypeNTFY:
        {
            newMessage = new MgcpCommandNotify;
        }
        break;

        case MgcpTokenTypeCRCX:
        {
            newMessage = new MgcpCommandCreateConnection;
        }
        break;

        case MgcpTokenTypeMDCX:
        {
            newMessage = new MgcpCommandModifyConnection;
        }
        break;

        case MgcpTokenTypeDLCX:
        {
            newMessage = new MgcpCommandDeleteConnection;
        }
        break;

        case MgcpTokenTypeRSIP:
        {
            newMessage = new MgcpCommandRestartInProgress;
        }
        break;

        case MgcpTokenTypeAUEP:
        {
            newMessage = new MgcpCommandAuditEndpoint;
        }

        case MgcpTokenTypeAUCX:
        {
            newMessage = new MgcpCommandAuditConnection;
        }
        break;

        default:
        {
            CERR(cerr << "could not find out what type of message this is\n"; );
        }
        break;
    }

    return newMessage;
}


MgcpCommand*
parseResponseMessage(split_t parms)
{
    // it's a digit so it's a response!
    MgcpResponse* msg = new MgcpResponse;

    split_t::iterator responseIterator = parms.begin();

    int responsePart = 0;

    while (responseIterator != parms.end())
    {
        switch (responsePart)
        {
            case 0:
            {
                msg->setResponseCode(
                    mapRespCode( atoi((*responseIterator).c_str())));
            }
            break;
            case 1:
            {
                msg->setTransactionId(
                    atoi((*responseIterator).c_str()));
            }
            break;
            case 2:
            {
                msg->setComment((*responseIterator));
            }
            break;
            default:
            {
                CERR(cerr << "warning:  to many parameters in the response\n"; );
            }
        }
        responsePart++;
        responseIterator++;
    }

    return msg;
}



MgcpCommand*
parseMessage( const char* buffer, unsigned int length )
{
    ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: begin *** parseMessage ***" << endl; );
   
    string buf(c2lower_s(buffer));
  
    string mixedBuffer(buffer);  // the mixed case buffer


    int diff = buf.length() - length;

    if (diff > 0)
    {
        ODEBUG(DEBUG_PARSE, 0, cerr << "PARSE: warning--trimmed excess\n"; );
        buf.erase(length, diff);
    }

    split_t::iterator iter;
    split_t::iterator mixedIter;

    ODEBUG(DEBUG_PARSE, 2, cout << "PARSE: buf: " << buf << endl; );
    ODEBUG(DEBUG_PARSE, 3, cout << "PARSE: buffer: " << buffer << endl; );

    split_t lines(split(buf, "\r\n"));

    split_t mixedLines(split(mixedBuffer, "\r\n"));


    iter = lines.begin();
    mixedIter = mixedLines.begin();

    if (iter == lines.end())
    {
        CERR(cerr << "empty message!" << endl; );
        return 0;
    }

    ++iter;
    ++mixedIter;

    MgcpCommand* newMessage = 0;

    ODEBUG(DEBUG_PARSE, 2, cout << "Line 0: " << lines[0] << endl; );
    ODEBUG(DEBUG_PARSE, 2, cout << "Line 0: " << lines[0] << endl; );

    string s = lines[0];

    split_t parms(split(s, " "));

    split_t::iterator parmIterator = parms.begin();

    if ((parmIterator != parms.end()) &&
            (parms[0].length() > 0))
    {
        char c = parms[0][0];

        if ((c >= '0') && (c <= '9'))
        {
            newMessage = parseResponseMessage(parms);
        }
        else
        {
            ODEBUG(DEBUG_PARSE, 2, cout << "Type: " << parms[0] << endl; );

            newMessage =
                getCommandFromToken(recognizerCommands.matchToken(parms[0]));

            if (!newMessage)
            {
                CERR(cerr << "unable to decode message\n"; );
                CERR(cerr << "text: \n"; );
                CERR(cerr << buf << endl; );
                return 0;
            }

            parmIterator++;
            if (parmIterator != parms.end())
            {
                newMessage->setTransactionId(
                    strtol(parms[1].c_str(), (char **)0, 10));
                ODEBUG(DEBUG_PARSE, 2, cout << "PARSE: tid: " << parms[1].c_str() << " : " << newMessage->getTransactionId() << "\n"; );

            }

            parmIterator++;

            if (parmIterator != parms.end())
            {
                newMessage->setEndpointId(parms[2]);
                Sptr < MgcpParameterEndpointId > end
                = new MgcpParameterEndpointId(parms[2]);
                newMessage->insert(end);
            }
        }
    }

    bool sessionDescriptors(false);

    // TODO: session descriptions need to be case insensitive

    // TODO: session descriptions should be refactored into their own fn

    while (iter != lines.end())
    {
        if ((*iter).find("v=") == 0)
        {
            sessionDescriptors = true;
            ODEBUG(DEBUG_PARSE, 1, cerr << "PARSE: Got SDP\n"; );
        }

        if (sessionDescriptors)
        {
            // do nothing

            ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: entering sessionDescriptors\n"; );
            // the next n lines are session-descriptors;
            //    split_t::iterator descStart = iter;
            split_t::iterator descEnd = iter;
            split_t::iterator mixedDescStart = mixedIter;
            split_t::iterator mixedDescEnd = mixedIter;

            ++descEnd;
            ++mixedDescEnd;

            while ((descEnd != lines.end()) &&
                    ((*descEnd).find("v=") != 0))
            {
                // keep iterating
                ODEBUG(DEBUG_PARSE, 3, cerr << "PARSE: next SDP"; );

                ++descEnd;
                ++mixedDescEnd;
            }
            iter = descEnd;
            mixedIter = mixedDescEnd;

            --iter;  /* remember that iter will be incremented at the
            		       bottom of the loop */
            --mixedIter;  /* remember that iter will be incremented at the
            		       bottom of the loop */

            split_t tempSd(mixedDescStart, mixedDescEnd);

            SdpSession tempDescriptor;
            if (!tempDescriptor.decode(tempSd))
            {
                CERR(cerr << "PARSE: Error decoding SDP\n"; );
            }

            // TODO: this is not right -- in some cases you can pass
            // both a local and a remote session descriptor, and that
            // requires correctness here.

            if (dynamic_cast < MgcpResponse* > (newMessage))
            {
                Sptr < MgcpParameterLocalConnectionDescriptor > y
                = new
                  MgcpParameterLocalConnectionDescriptor(tempDescriptor);
                newMessage->insert(y);
            }
            else
            {
                Sptr < MgcpParameterRemoteConnectionDescriptor > y
                = new
                  MgcpParameterRemoteConnectionDescriptor(tempDescriptor);
                newMessage->insert(y);
            }
        }
        else
        {
            ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: entering parseParameter\n"; );
            Sptr < MgcpParameter > parm = parseParameter((*iter));

            if (parm != 0)
            {
                newMessage->insert(parm);
            }
        }
        XDEBUG(1, cerr << "+"; );
        ++iter;
        ++mixedIter;
    }

    if (!newMessage)
    {
        CERR(cerr << "no message found!\n"; );
    }

    ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: begin *** parseMessage ***" << endl; );
    ODEBUG(DEBUG_PARSE, 2, cerr << "PARSE: End *** parseMessage ***" << endl; );

    return newMessage;
}
